@@ -13,3 +13,7 @@
 #undef ENABLE_APPLET
 #undef HAVE__NL_PAPER_WIDTH_HEIGHT
 #undef ENABLE_GNOME_FILE_SELECTOR
+#undef ENABLE_EXPERIMENTAL_FILTERS
+#undef HAVE_XINERAMA
+#undef XPCOM_SHARED_LIB
+#undef GCC_VERSION
