@@ -21,6 +21,7 @@
 #include "bookmarks.h"
 #include "misc.h"
 #include "prefs.h"
+#include "mozilla.h"
 
 #include <math.h>
 #include <libgnome/gnome-config.h>
@@ -89,9 +90,11 @@ autobookmarks_generate (void)
 		/* delete the folder if it exists */
 		if (autobookmarks_root != NULL)
 		{
+			bookmarks_toolbars_check_update (autobookmarks_root);
 			bookmarks_editor_remove_tree_items(autobookmarks_root);
 			bookmarks_remove_recursively (autobookmarks_root);
 			autobookmarks_root = NULL;
+			bookmarks_updated ();
 		}
 		return;
 	}
@@ -111,9 +114,11 @@ autobookmarks_generate (void)
 	else
 	{
 		/* no, create the folder */
+		gchar* utfstr = mozilla_locale_to_utf8(_("AutoBookmarks"));
 		autobookmarks_root = bookmarks_new_bookmark 
-			(BM_AUTOBOOKMARKS, TRUE, _("AutoBookmarks"), 
+			(BM_AUTOBOOKMARKS, TRUE, utfstr,
 			 NULL, NULL, NULL, NULL);
+		g_free(utfstr);
 
 		/* add it to the main folder */
 		g_return_if_fail(bookmarks_root != NULL);
@@ -141,7 +146,7 @@ autobookmarks_generate (void)
 	{
 		HistoryHost *host = (HistoryHost *)item->data;
 		HistoryItem *hi = (HistoryItem *)host->dominant_item;
-		gchar *title_locale, *name;
+		gchar *title_utf8, *name;
 		BookmarkItem *b;
 
 		/* add it if within bounds */
@@ -151,12 +156,12 @@ autobookmarks_generate (void)
 			/* note that shorten_name takes care of
 			 * duplicating the title string for us, even 
 			 * when it's not changed */
-			title_locale = (hi->title_locale != NULL ? 
-					hi->title_locale : hi->title_utf8);
-			name = shorten_name (title_locale, shorten);
+			title_utf8 = (hi->title_utf8 != NULL ? 
+				      hi->title_utf8 : hi->title_locale);
+			name = shorten_name (title_utf8, shorten);
 			b = bookmarks_new_bookmark (BM_SITE, TRUE, name, 
 						    hi->url, NULL, NULL, NULL);
-			g_free(name);
+			g_free (name);
 			
 			/* add it to the autobookmarks folder */
 			b->parent = autobookmarks_root;
@@ -167,6 +172,10 @@ autobookmarks_generate (void)
 
 	/* free the list */
 	g_list_free(list);
+
+	if (autobookmarks_root)
+		bookmarks_toolbars_check_update (autobookmarks_root);
+	bookmarks_updated ();
 }
 
 /**
