@@ -74,8 +74,6 @@ const gchar *common_prefixes[] =
 static GList *auto_completion_insert_sorted_nodup (GList *l, char *url,
 						   int type);
 static GList *auto_completion_concat (GList *l1, GList *l2);
-static void clist_select_row_cb (GtkCList *clist, gint row, gint column,
-				 GdkEventButton *event, GtkWidget *entry);
 static gchar *auto_completion_suggest (gchar *prefix);
 static gchar *auto_completion_simplify_url(gchar *url);
 static gboolean alternatives_win_button_press_cb (GtkWidget *widget,
@@ -84,6 +82,14 @@ static gboolean alternatives_win_button_press_cb (GtkWidget *widget,
 static gboolean alternatives_win_key_press_cb (GtkWidget *widget,
 					       GdkEventKey *event,
 					       GtkWidget *location_entry);
+static gboolean alternatives_clist_key_press_cb (GtkWidget *widget,
+						 GdkEventKey *event,
+						 GtkWidget *location_entry);
+static gboolean alternatives_clist_button_press_cb (GtkWidget *widget,
+						    GdkEventButton *event,
+						    GtkWidget *location_entry);
+static void alternatives_jump_to_clist_selection (GtkWidget *entry, 
+						  GtkCList *clist);
 
 
 /**
@@ -412,6 +418,66 @@ auto_completion_display (GaleonWindow *window, GtkWidget *entry)
 	g_free (text);
 }
 
+static void
+alternatives_jump_to_clist_selection (GtkWidget *entry, GtkCList *clist)
+{
+	if (GTK_IS_ENTRY (entry)
+	    && clist->selection) {
+		GdkEventKey tmp_event;
+		gchar *text;
+		int row = GPOINTER_TO_INT (clist->selection->data);
+	
+		gtk_clist_get_text (clist, row, 0, &text);
+		gtk_entry_set_text (GTK_ENTRY (entry), text);
+		auto_completion_reset();
+		
+		/* send a synthetic return keypress to the entry */
+		tmp_event.type = GDK_KEY_PRESS;
+		tmp_event.window = entry->window;
+		tmp_event.send_event = TRUE;
+		tmp_event.time = GDK_CURRENT_TIME;
+		tmp_event.state = 0;
+		tmp_event.keyval = GDK_Return;
+		gtk_widget_event(entry, (GdkEvent *)&tmp_event);
+	}
+}
+
+static gboolean 
+alternatives_clist_button_press_cb (GtkWidget *widget, GdkEventButton  *event, 
+				    GtkWidget *entry)
+{
+	GtkCList *clist = GTK_CLIST (widget);
+	if (event->button == 1)
+	{
+		gint row, column;
+		gtk_clist_get_selection_info (clist, event->x, event->y, 
+					      &row, &column);
+		gtk_clist_select_row (clist, row, 0);
+		alternatives_jump_to_clist_selection (entry, clist);
+	}
+	return TRUE;
+}
+
+static gboolean
+alternatives_clist_key_press_cb  (GtkWidget *widget, GdkEventKey *event,
+				  GtkWidget *entry)
+{
+	GtkCList *clist = GTK_CLIST (widget);
+	if (clist->focus_row >= 0)
+	{
+		gtk_clist_select_row (clist, clist->focus_row, 0);
+	}
+
+	if (((event->keyval == GDK_Return) 
+	     || event->keyval == GDK_space)
+	    && (clist->selection))
+	{
+		alternatives_jump_to_clist_selection (entry, clist);
+	}
+	return FALSE;
+}
+
+
 /**
  * Displays the list of possible completions under the location box of the
  * given browser
@@ -435,12 +501,11 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 	alternatives_window = gtk_window_new (GTK_WINDOW_POPUP);
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
-		       			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+		       			GTK_POLICY_NEVER, 
+					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (alternatives_window), scroll);
 	gtk_container_add (GTK_CONTAINER (scroll), clist);
 
-	gtk_signal_connect (GTK_OBJECT(clist), "select_row", 
-			    GTK_SIGNAL_FUNC (clist_select_row_cb), entry);
 	
 	for (l = possible_completions; (l!=NULL) && (count < MAX_ALTERNATIVES);
 	     l = g_list_next(l), count++) 
@@ -489,6 +554,19 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 			    "key-press-event",
 			    GTK_SIGNAL_FUNC(alternatives_win_key_press_cb),
 			    entry);
+	gtk_signal_connect_after (GTK_OBJECT (clist),
+				  "key-press-event",
+				  GTK_SIGNAL_FUNC
+				  (alternatives_clist_key_press_cb),
+				  entry);
+	gtk_signal_connect (GTK_OBJECT (clist),
+				  "button-press-event",
+				  GTK_SIGNAL_FUNC
+				  (alternatives_clist_button_press_cb),
+				  entry);
+
+	gtk_container_set_focus_child (GTK_CONTAINER (alternatives_window), 
+				       GTK_WIDGET (clist));
 
 	gdk_pointer_grab (alternatives_window->window, TRUE,
 			  GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK |
@@ -497,29 +575,6 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 	gtk_grab_add (alternatives_window);
 }
 
-static void 
-clist_select_row_cb (GtkCList *clist, gint row, gint column,
-		     GdkEventButton *event, GtkWidget *entry)
-{
-	GdkEventKey tmp_event;
-
-	if (GTK_IS_ENTRY (entry)) {
-		gchar *text;
-		gtk_clist_get_text (clist, row, column, &text);
-		gtk_entry_set_text (GTK_ENTRY (entry), text);
-		auto_completion_reset();
-
-		/* send a synthetic return keypress to the entry */
-		tmp_event.type = GDK_KEY_PRESS;
-		tmp_event.window = entry->window;
-		tmp_event.send_event = TRUE;
-		tmp_event.time = GDK_CURRENT_TIME;
-		tmp_event.state = 0;
-		tmp_event.keyval = GDK_Return;
-		gtk_widget_event(entry, (GdkEvent *)&tmp_event);
-	}
-}
-
 static gboolean
 alternatives_win_button_press_cb (GtkWidget *widget,
 				  GdkEventButton *event,
@@ -527,7 +582,7 @@ alternatives_win_button_press_cb (GtkWidget *widget,
 {
 	GtkWidget *event_widget;
 
-	event_widget = gtk_get_event_widget ((GdkEvent *)event);
+	event_widget = gtk_get_event_widget ((GdkEvent *) event);
 
 	/* Check to see if button press happened inside the alternatives
 	   window.  If not, destroy the window. */
@@ -551,26 +606,19 @@ alternatives_win_key_press_cb (GtkWidget *widget,
 			       GtkWidget *location_entry)
 {
 	GdkEventKey tmp_event;
+	GtkWidget *dest_widget = location_entry;
 	GtkCList *clist = GTK_CLIST (GTK_BIN (GTK_BIN (widget)->child)->child);
 
 	/* allow keyboard navigation in the alternatives clist */
-	if (event->keyval == GDK_Up || event->keyval == GDK_Down ||
-	    event->keyval == GDK_Page_Up ||  event->keyval == GDK_Page_Down ||
-	    event->keyval == GDK_space)
+	if (event->keyval == GDK_Up || event->keyval == GDK_Down 
+	    || event->keyval == GDK_Page_Up ||  event->keyval == GDK_Page_Down
+	    || ((event->keyval == GDK_space || event->keyval == GDK_Return)
+		&& clist->selection))
 	{
-		return FALSE;
+		/* forward the event to the clist, instead of the entry */
+		dest_widget = GTK_WIDGET (clist);
 	}
 	
-	if (event->keyval == GDK_Return)
-	{
-		if (clist->focus_row >= 0)
-		{
-			event->keyval = GDK_space;
-			return FALSE;
-		}
-		auto_completion_reset();
-	}
-
 	/* else send the key event to the location entry */
 	tmp_event.type = event->type;
 	tmp_event.window = location_entry->window;
@@ -580,7 +628,7 @@ alternatives_win_key_press_cb (GtkWidget *widget,
 	tmp_event.keyval = event->keyval;
 	tmp_event.length = event->length;
 	tmp_event.string = event->string;
-	gtk_widget_event(location_entry, (GdkEvent *)&tmp_event);
+	gtk_widget_event (dest_widget, (GdkEvent *)&tmp_event);
 
 	/* hmm... something seems to have changed in the GTK+ signal handling
 	   code.  Returning true no longer seems to stop emission of the key
