@@ -25,8 +25,11 @@
 #include "window.h"
 #include "prefs.h"
 #include "favicon.h"
+#include "mozilla.h"
+#include "themes.h"
 
 /* GNOME includes */
+#include <time.h>
 #include <stdlib.h>
 #include <string.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -34,8 +37,12 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomeui/gnome-dialog-util.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkdnd.h>
+#include <gtkmozembed.h>
+
+#define NUM_BOOKMARK_BACKUPS 3
 
 /** The pixmaps for the bookmarks */
 PixmapData *folder_pixmap_data = NULL;
@@ -54,6 +61,9 @@ BookmarkItem *default_bookmarks_root = NULL;
 /** The bookmarks need to be saved */
 gboolean bookmarks_dirty = FALSE;
 
+/** Do we need to recreate the bookmarks toolbars? */
+gboolean bookmarks_tb_dirty = FALSE;
+
 /** The next id to assign (for alias saving) */
 static long maxid = 0;
 
@@ -65,6 +75,9 @@ GList *unresolved_aliases = NULL;
 /** The favicon pixmap data cache */
 GHashTable *favicons_cache = NULL;
 
+/* the filename of the bookmarks */
+gchar *bookmarks_filename = NULL;
+
 /* DnD */
 const GtkTargetEntry bookmarks_dnd_targets [] = {
 	{ "GALEON_BOOKMARK", 0, DND_TARGET_GALEON_BOOKMARK },
@@ -77,9 +90,7 @@ const gint bookmarks_dnd_targets_num_items = (sizeof (bookmarks_dnd_targets) /
 					      sizeof (GtkTargetEntry));
 
 /* Private prototypes */
-static void bookmarks_backup (void);
 static void bookmarks_save (void);
-static void bookmarks_load_icons (void);
 static BookmarkItem *bookmarks_xml_read_item (xmlNodePtr item);
 static BookmarkItem *bookmarks_xml_read (xmlNodePtr item);
 static void bookmarks_add_completion_recursively (BookmarkItem *b);
@@ -91,9 +102,11 @@ static BookmarkItem *bookmarks_find_by_id (long id,
 					   BookmarkItem *root);
 static BookmarkItem *bookmarks_merge_trees_look_for_equal 
 	(BookmarkItem *folder, BookmarkItem *bookmark);
+static gint bookmarks_compare_name_and_type (gconstpointer a, gconstpointer b);
 static gint bookmarks_compare_name (gconstpointer a, gconstpointer b);
 static gchar *bookmarks_get_bookmark_path (BookmarkItem *b);
-
+static void bookmarks_ensure_unique_ids (BookmarkItem *root);
+static void bookmarks_ensure_unique_ids_rec (BookmarkItem *b, gulong *maxid);
 
 /**
  * bookmarks_init: intialise bookmarks for the first time
@@ -101,6 +114,8 @@ static gchar *bookmarks_get_bookmark_path (BookmarkItem *b);
 void
 bookmarks_init (void)
 {
+	bookmarks_filename = g_strconcat (g_get_home_dir (),
+			                  "/.galeon/bookmarks.xml", NULL);
 	bookmarks_load_icons ();
 	bookmarks_load ();
 }
@@ -114,7 +129,6 @@ bookmarks_exit (void)
 	GList *l = g_list_copy (bookmarks_editors);
 	GList *li;
 
-	bookmarks_backup ();
 	bookmarks_save ();
 
 	/* close all editors */
@@ -147,17 +161,9 @@ bookmarks_exit (void)
 void
 bookmarks_load (void)
 {
-	gchar *bookfile;
-	
 	default_bookmarks_root = NULL;
 
-	bookfile = g_concat_dir_and_file (g_get_home_dir (), 
-					  "/.galeon/bookmarks.xml");
-	
-	bookmarks_set_root (bookmarks_load_from (bookfile));
-	
-
-	g_free (bookfile);
+	bookmarks_set_root (bookmarks_load_from (bookmarks_filename));
 }
 
 /**
@@ -172,9 +178,6 @@ bookmarks_set_root (BookmarkItem *new_root)
 	for (li = bookmarks_editors; li != NULL; li = li->next) {
 		BookmarksEditorControls *current_controls = li->data;
 		gtk_clist_freeze (GTK_CLIST (current_controls->ctree));
-		/* delete old bookmarks view */
-		//bookmarks_editor_remove_tree_item 
-		//(current_controls, current_controls->root_bookmark);
 	}
 	
 	/* delete the old bookmarks */
@@ -182,9 +185,11 @@ bookmarks_set_root (BookmarkItem *new_root)
 	
 	bookmarks_root = new_root;
 	if (!bookmarks_root) {
+		gchar* utfstr = mozilla_locale_to_utf8(_("Bookmarks"));
 		bookmarks_root = bookmarks_new_bookmark 
-			(BM_FOLDER, TRUE, _("Bookmarks"), NULL, NULL,
+			(BM_FOLDER, TRUE, utfstr, NULL, NULL,
 			 NULL, NULL);
+		g_free(utfstr);
 	}
 	
 	if (default_bookmarks_root == NULL) {
@@ -212,51 +217,20 @@ bookmarks_set_root (BookmarkItem *new_root)
 /**
  * bookmarks_load_icons: load the bookmarks icons
  */
-static void 
+void 
 bookmarks_load_icons (void) 
 {
-	folder_pixmap_data = 
-		pixmap_data_new_from_file (SHARE_DIR "/dir.xpm");
-	folder_open_pixmap_data = 
-		pixmap_data_new_from_file (SHARE_DIR "/dir_open.xpm");
-	default_folder_pixmap_data = 
-		pixmap_data_new_from_file (SHARE_DIR "/default.xpm");
-	default_folder_open_pixmap_data = 
-		pixmap_data_new_from_file (SHARE_DIR "/default_open.xpm");
-	site_pixmap_data = 
-		pixmap_data_new_from_file (SHARE_DIR "/i-bookmark.xpm");
-	smart_bm_tray_pixmap_data = 
-		pixmap_data_new_from_file (SHARE_DIR "/smart-bm-tray.xpm");
+	/* load pixmaps */
+	folder_pixmap_data = get_theme_pixmap ("dir.xpm", FALSE);
+	folder_open_pixmap_data = get_theme_pixmap ("dir_open.xpm", FALSE);
+	default_folder_pixmap_data = get_theme_pixmap ("default.xpm", FALSE);
+	default_folder_open_pixmap_data = get_theme_pixmap ("default_open.xpm",
+							    FALSE);
+	site_pixmap_data = get_theme_pixmap ("i-bookmark.xpm", FALSE);
+	smart_bm_tray_pixmap_data = get_theme_pixmap ("smart-bm-tray.xpm",
+						      FALSE);
 
 	/* TODO: add icons for alias */
-
-	/* if we didn't manage to load the icons, make an empty structure
-	 * anyway, this allows the browser to run even when not fully 
-	 * installed.... */
-	if (folder_pixmap_data == NULL)
-	{
-		folder_pixmap_data = g_new0 (PixmapData, 1);
-	}
-	if (folder_open_pixmap_data == NULL)
-	{
-		folder_open_pixmap_data = g_new0 (PixmapData, 1);
-	}
-	if (default_folder_pixmap_data == NULL)
-	{
-		default_folder_pixmap_data = g_new0 (PixmapData, 1);
-	}
-	if (default_folder_open_pixmap_data == NULL)
-	{
-		default_folder_open_pixmap_data = g_new0 (PixmapData, 1);
-	}
-	if (site_pixmap_data == NULL)
-	{
-		site_pixmap_data = g_new0 (PixmapData, 1);
-	}
-	if (smart_bm_tray_pixmap_data == NULL)
-	{
-		smart_bm_tray_pixmap_data = g_new0 (PixmapData,1);
-	}
 }
 
 /**
@@ -300,10 +274,16 @@ bookmarks_xml_read (xmlNodePtr item)
 	item = item->childs;
 	unresolved_aliases = NULL;
 	
-	if (!error_bookmark) 
+	if (!error_bookmark) {
+		gchar* eutfstr = mozilla_locale_to_utf8(_("Error"));
+		gchar* nutfstr = mozilla_locale_to_utf8(
+			_("An error ocurred loading your bookmarks."));
 		error_bookmark = bookmarks_new_bookmark 
-			(BM_SITE, TRUE, _("Error"), "about:blank", NULL,
-			 _("An error ocurred loading your bookmarks."), NULL);
+			(BM_SITE, TRUE, eutfstr, "about:blank", NULL,
+			 nutfstr, NULL);
+		g_free(eutfstr);
+		g_free(nutfstr);
+	}
 	
 	/* old format, no longer supported */
 	/*
@@ -347,11 +327,11 @@ bookmarks_xml_read_item (xmlNodePtr item)
 {
 	BookmarkItem *b;
 	BookmarkItem *b2;
-	gchar *ct, *expanded, *ccm, *default_root, *tbs;
+	gchar *ct, *expanded, *ccm, *default_root, *tbs, *ak, *am;
 
-	gchar *name = xmlGetRawPropLocale (item, "name");	
-	gchar *notes = xmlGetRawPropLocale (item, "notes");
-	gchar *nick = xmlGetRawPropLocale (item, "nick");
+	gchar *name = xmlGetRawProp (item, "name");	
+	gchar *notes = xmlGetRawProp (item, "notes");
+	gchar *nick = xmlGetRawProp (item, "nick");
 	gchar *pixmap_file = xmlGetProp (item, "pixmap");
 	gchar *idstr = xmlGetProp (item, "id");
 	gchar *time_added_str = xmlGetProp (item, "time_added");
@@ -431,6 +411,32 @@ bookmarks_xml_read_item (xmlNodePtr item)
 	{
 		b = bookmarks_new_alias (NULL);
 		unresolved_aliases = g_list_prepend (unresolved_aliases, b);
+		ak = xmlGetProp (item, "accel_key");
+		if (ak)
+			b->accel_key = atoi (ak);
+		xmlFree (ak);
+		am = xmlGetProp (item, "accel_mods");
+		if (am)
+			b->accel_mods = atoi (am);
+		xmlFree (am);
+
+		ct = xmlGetProp (item, "create_toolbar");
+		tbs = xmlGetProp (item, "toolbar_style");
+		ccm = xmlGetProp (item, "create_context_menu");
+
+		b->create_toolbar = (ct && !strcmp (ct, "TRUE") ? 
+				     TRUE : FALSE);
+		if (tbs)
+			b->toolbar_style = atoi(tbs);
+		if (b->toolbar_style > 2) /* backwards compatiblity FIXME */
+			b->toolbar_style = TOOLBAR_STYLE_HORIZONTAL;
+		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE") ? 
+					  TRUE : FALSE);
+
+		xmlFree (ct);
+		xmlFree (tbs);
+		xmlFree (ccm);
+
 	} 
 	else 
 	{
@@ -443,6 +449,14 @@ bookmarks_xml_read_item (xmlNodePtr item)
 					  TRUE : FALSE);
 		xmlFree (ccm);
 		if (url) g_free (url);
+		ak = xmlGetProp (item, "accel_key");
+		if (ak)
+			b->accel_key = atoi (ak);
+		xmlFree (ak);
+		am = xmlGetProp (item, "accel_mods");
+		if (am)
+			b->accel_mods = atoi (am);
+		xmlFree (am);
 	}
 	if (time_added_str)
 	{
@@ -482,7 +496,7 @@ bookmarks_updated (void)
 
 	/* force a save */
 	bookmarks_dirty = TRUE;
-	bookmarks_backup ();
+	/* FIXME save is very expensive because of the backups ! */
 	bookmarks_save ();
 
 	/* update each window */
@@ -490,6 +504,13 @@ bookmarks_updated (void)
 	{
 		GaleonWindow *window = (GaleonWindow *)(l->data);
 
+		if (!window || (window->magic != GALEON_WINDOW_MAGIC))
+		{
+			g_warning ("bookmaks_updated found unexpected things "
+				   " in window list!");
+			continue;
+		}
+		
 		/* save toolbar layout */
 		window_save_layout (window);
 	
@@ -497,123 +518,145 @@ bookmarks_updated (void)
 		bookmarks_menu_recreate (window);
 	
 		/* update bookmarks toolbars */
-		bookmarks_toolbars_recreate (window);
+		if (bookmarks_tb_dirty)
+			bookmarks_toolbars_recreate (window);
 
 		/* restore toolbar layout */
 		window_restore_layout (window);
 	}
 
+	bookmarks_tb_dirty = FALSE;
+
+	/* reload any myportals */
+	for (l = all_embeds; l; l = g_list_next (l))
+	{
+		GaleonEmbed *embed = (GaleonEmbed *) l->data;
+
+		if (embed->site_location 
+		    && strncmp (embed->site_location, "myportal:", 9) == 0)
+		{
+			embed_reload (embed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
+		}
+	}
+
 	/* FIXME: update applet... */
 }
 
 /**
- * bookmarks_backup: backup old bookmarks file
+ * bookmarks_save: save all bookmarks to bookmarks_file, after backing up
+ * the previous versions 
  */
-/* FIXME: check for errors (permissions...) */
-void
-bookmarks_backup (void)
-{
-	int count = 0;
-
-	/* check if it's not enabled */
-	if (eel_gconf_get_boolean (CONF_BOOKMARKS_BACKUP_ENABLED) == FALSE)
-	{
-		return;
-	}
-
-	if (!bookmarks_dirty)
-	{
-		return;
-	}
-
-	for (count = eel_gconf_get_integer (CONF_BOOKMARKS_BACKUP_COUNT); 
-	     count >= 1 ; count--) {
-		gchar *str;
-		gchar *str2;
-		gchar *filename;
-		gchar *filename2;
-		GnomeVFSURI *filename_uri;
-		GnomeVFSURI *filename2_uri;
-		GnomeVFSResult rc;
-
-		str = g_strdup_printf ("%d", count);
-		str2 = g_strdup_printf ("%d", count - 1);
-
-		if (count == 1) {
-			/* FIXME: refactor into bookmarks_filename ... */
-			filename = g_strconcat 
-				(g_get_home_dir (),
-				 "/.galeon/bookmarks.xml", NULL);
-		} else {
-			/* FIXME: refactor into bookmarks_filename ... */
-			filename = g_strconcat 
-				(g_get_home_dir (), 
-				 "/.galeon/bookmarks.xml.", str2, NULL);
-		}
-
-		/* FIXME: refactor into bookmarks_filename ... */
-		filename2 = g_strconcat 
-			(g_get_home_dir (), "/.galeon/bookmarks.xml.", str,
-			 NULL);
-
-		/* give it head :) now before we leave the bookmarks keep at */
-		/* least one backup of last saved file for emergency reasons */
-		filename_uri = gnome_vfs_uri_new (filename);
-		filename2_uri = gnome_vfs_uri_new (filename2);
-
-		rc = gnome_vfs_move_uri (filename_uri, filename2_uri, TRUE);
-
-		gnome_vfs_uri_unref (filename_uri);
-		gnome_vfs_uri_unref (filename2_uri);
-
-		/* free allocated strings */
-		g_free (str);
-		g_free (str2);
-		g_free (filename);
-		g_free (filename2);
-	}
-}
-
-/**
- * bookmarks_save: save all bookmarks to the xml file
- */
-/* FIXME: check for errors (permissions...) */
 void
 bookmarks_save (void)
 {
-	gchar *filename;
+	gint size;
+	gchar *temp_filename;
+	GnomeVFSURI *temp_uri;
 
+	/* make sure the bookmarks have been changed before we do anything */
 	if (!bookmarks_dirty)
 	{
 		return;
 	}
 
-	/* FIXME: refactor into bookmarks_filename ... */
-	filename = g_strconcat (g_get_home_dir (), 
-				"/.galeon/bookmarks.xml", NULL);
-	
-	bookmarks_save_as (filename);
+	/* write the temp file */
+	temp_filename = g_strconcat (bookmarks_filename, ".temp", NULL);
+	size = bookmarks_save_as (temp_filename);
+	temp_uri = gnome_vfs_uri_new (temp_filename);
 
-	/* now clean */
-	bookmarks_dirty = FALSE;
+	/* if the file was written... */
+	if (size > 0 && gnome_vfs_uri_exists (temp_uri))
+	{
+		gint count;
+		GnomeVFSResult result;
 
-	/* free allocated string */
-	g_free (filename);
+		/* backup/rotate the bookmarks */
+		for (count = NUM_BOOKMARK_BACKUPS - 1; count >= 0; count--)
+		{
+			gchar *src_ext, *dest_ext;
+			gchar *src_filename, *dest_filename;
+			GnomeVFSURI *src_uri, *dest_uri;
+
+			/* get the extensions */
+			if (count > 0) src_ext = g_strdup_printf (
+							".%d", count - 1);
+			else src_ext = NULL;
+			dest_ext = g_strdup_printf (".%d", count);
+
+			/* create the filenames */
+			src_filename = g_strconcat (bookmarks_filename,
+						    src_ext, NULL);
+			dest_filename = g_strconcat (bookmarks_filename,
+						     dest_ext, NULL);
+
+			/* create uris from the filenames */
+			src_uri = gnome_vfs_uri_new (src_filename);
+			dest_uri = gnome_vfs_uri_new (dest_filename);
+
+			/* move the file */
+			if (gnome_vfs_uri_exists (src_uri))
+			{
+				gnome_vfs_move_uri (src_uri,
+						    dest_uri, TRUE);
+			}
+
+			/* free stuff */
+			g_free (src_ext);
+			g_free (dest_ext);
+			g_free (src_filename);
+			g_free (dest_filename);
+			gnome_vfs_uri_unref (src_uri);
+			gnome_vfs_uri_unref (dest_uri);
+		}
+
+		/* move the temp file to the normal location */
+		result = gnome_vfs_move (temp_filename,
+					 bookmarks_filename, TRUE);
+
+		/* make sure the move went okay */
+		if (result == GNOME_VFS_OK)
+		{
+			/* bookmarks have been synched now */
+			bookmarks_dirty = FALSE;
+		}
+		else
+		{
+			gnome_error_dialog (
+				_("Unable to save bookmark file."));
+		}
+	}
+
+	/* otherwise... */
+	else
+	{
+		gnome_error_dialog (_("Unable to save bookmark file."));
+	}
+
+	/* free the temp file stuff */
+	gnome_vfs_uri_unref (temp_uri);
+	g_free (temp_filename);
 }
 
-void 
+/**
+ * bookmarks_save_as: attempts to write the bookmarks to the given
+ * filename, and returns the number of bytes actually written */
+int
 bookmarks_save_as (const gchar *filename)
 {
 	xmlNodePtr node;
 	xmlDocPtr doc;
+	int size;
 	
 	/* build an XML document and save it to the filename */
 	doc = xmlNewDoc ("1.0");
 	node = xmlNewDocNode (doc, NULL, "bookmarks", NULL);
 	xmlDocSetRootElement (doc, node);
+	bookmarks_ensure_unique_ids (bookmarks_root);
 	bookmarks_save_recursively (doc, node, bookmarks_root);
-	xmlSaveFile (filename, doc);
+	size = xmlSaveFile (filename, doc);
 	xmlFreeDoc (doc);     
+
+	return size;
 }
 
 /**
@@ -626,12 +669,13 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 	BookmarkItem *b2;
 	xmlNodePtr node;
 	GList *li;
-	gchar *idstr = g_strdup_printf ("%ld", b->id);
+	gchar *idstr;
 	GString *s = g_string_new ("");
 	gchar *tmp;
 
 	if (b->alias_of)
 	{
+		idstr = g_strdup_printf ("%ld", b->id);
 		node = xmlNewDocNode (doc, NULL, "alias", NULL);
 		xmlSetProp (node, "id", idstr);
 		if (b->create_toolbar)
@@ -644,6 +688,14 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 		}
 		if (b->create_context_menu)
 			xmlSetProp (node, "create_context_menu", "TRUE");
+		if (b->accel_key) {
+			g_string_sprintf (s, "%d", b->accel_key);
+			xmlSetProp (node, "accel_key", s->str);
+		}
+		if (b->accel_mods) {
+			g_string_sprintf (s, "%d", b->accel_mods);
+			xmlSetProp (node, "accel_mods", s->str);
+		}
 		if (b->time_added) {
 			g_string_sprintf (s, "%d", b->time_added);
 			xmlSetProp (node, "time_added", s->str);
@@ -652,12 +704,13 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 	} 
 	else
 	{
+		idstr = g_strdup_printf ("%ld", b->id);
 		switch (b->type)
 		{
 		case BM_FOLDER:
 			node = xmlNewDocNode (doc, NULL, "folder", NULL);
 			xmlSetProp (node, "id", idstr);
-			xmlSetRawPropLocale (node, "name", b->name);
+			xmlSetRawProp (node, "name", b->name);
 			if (b->create_toolbar)
 				xmlSetProp (node, "create_toolbar", "TRUE");
 			if (b->toolbar_style != TOOLBAR_STYLE_HORIZONTAL)
@@ -667,11 +720,12 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 				g_free (tmp);
 			}
 			if (b->create_context_menu)
-				xmlSetProp (node, "create_context_menu",  "TRUE");
+				xmlSetProp (node, "create_context_menu", 
+					    "TRUE");
 			if (b->expanded)
 				xmlSetProp (node, "expanded", "TRUE");
 			if (*b->notes)
-				xmlSetRawPropLocale (node, "notes", b->notes);
+				xmlSetRawProp (node, "notes", b->notes);
 			if (default_bookmarks_root == b) {
 				xmlSetProp (node, "default_bookmarks_root", 
 					    "TRUE");
@@ -706,14 +760,23 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 		case BM_SITE:
 			node = xmlNewDocNode(doc, NULL, "site", NULL);
 			xmlSetProp (node, "id", idstr);
-			xmlSetRawPropLocale (node, "name", b->name);
+			xmlSetRawProp (node, "name", b->name);
 			xmlSetProp (node, "url", b->url);    
 			if (*b->nick)
-				xmlSetRawPropLocale (node, "nick", b->nick);
+				xmlSetRawProp (node, "nick", b->nick);
 			if (*b->notes)
-				xmlSetRawPropLocale (node, "notes", b->notes);
+				xmlSetRawProp (node, "notes", b->notes);
 			if (b->create_context_menu)
-				xmlSetProp (node, "create_context_menu", "TRUE");
+				xmlSetProp (node, "create_context_menu",
+					    "TRUE");
+			if (b->accel_key) {
+				g_string_sprintf (s, "%d", b->accel_key);
+				xmlSetProp (node, "accel_key", s->str);
+			}
+			if (b->accel_mods) {
+				g_string_sprintf (s, "%d", b->accel_mods);
+				xmlSetProp (node, "accel_mods", s->str);
+			}
 			if (b->pixmap_file && *b->pixmap_file)
 				xmlSetProp (node, "pixmap", b->pixmap_file);
 			if (b->time_added) {
@@ -734,7 +797,7 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 		case BM_AUTOBOOKMARKS:
 			node = xmlNewDocNode(doc, NULL, "autobookmarks", NULL);
 			xmlSetProp (node, "id", idstr);
-			xmlSetRawPropLocale (node, "name", b->name);
+			xmlSetRawProp (node, "name", b->name);
 			if (b->create_toolbar)
 				xmlSetProp (node, "create_toolbar", "TRUE");
 			if (b->toolbar_style != TOOLBAR_STYLE_HORIZONTAL)
@@ -744,11 +807,12 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 				g_free (tmp);
 			}
 			if (b->create_context_menu)
-				xmlSetProp (node, "create_context_menu", "TRUE");
+				xmlSetProp (node, "create_context_menu", 
+					    "TRUE");
 			if (b->expanded)
 				xmlSetProp (node, "expanded", "TRUE");
 			if (*b->notes)
-				xmlSetRawPropLocale (node, "notes", b->notes);
+				xmlSetRawProp (node, "notes", b->notes);
 			if (b->pixmap_file && *b->pixmap_file)
 				xmlSetProp(node, "pixmap", b->pixmap_file);
 			xmlAddChild(root, node);
@@ -761,7 +825,6 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 
 /**
  * bookmarks_remove_recursively: recursively remove a tree of bookmarks items
- * Doesn't update tree items
  */
 void 
 bookmarks_remove_recursively (BookmarkItem *b) {
@@ -789,8 +852,7 @@ bookmarks_remove_recursively (BookmarkItem *b) {
 		   it's next, if any */
 		b->alias_of->alias = b->alias;
 	
-	if (((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) && 
-	    !b->alias_of)
+	if (BOOKMARK_ITEM_IS_FOLDER (b) && !b->alias_of)
 	{
 		while (b->list)
 		{
@@ -814,10 +876,12 @@ bookmarks_remove_recursively (BookmarkItem *b) {
  * @nick: the nickname for the bookmark, if any
  * @notes: the comments aout the bookmark
  * 
- * This function allocates a BookmarkItem and initializes its fields to sane values.
- * Use it instead of g_new (BookmarkItem, 1) and manually initializing the fields. 
- * The returned BookmarkItem has it's parent set to NULL, so you have to add it to 
- * the bookmarks list yourself (if you need it)
+ * This function allocates a BookmarkItem and initializes its fields to sane 
+ * values.
+ * Use it instead of g_new (BookmarkItem, 1) and manually initializing the 
+ * fields. 
+ * The returned BookmarkItem has it's parent set to NULL, so you have to add it
+ * to the bookmarks list yourself (if you need it)
  * All the strings passed to this function are g_strduped
  * 
  * Return value: The new BookmarkItem
@@ -856,6 +920,7 @@ bookmarks_new_bookmark (BookmarkType type, gboolean escape_name,
 		else
 			bookmark->name = escape_uline_accel (url);
 		bookmark->url = g_strdup (url);
+		auto_completion_add_url (bookmark->url);
 		break;
 	default:
 		bookmark->name = bookmark->url = NULL;
@@ -878,7 +943,8 @@ bookmarks_new_bookmark (BookmarkType type, gboolean escape_name,
 	if (pixmap_file != NULL && strlen(pixmap_file) != 0)
 	{
 		bookmark->pixmap_file = g_strdup (pixmap_file);
-		bookmark->pixmap_data = pixmap_data_new_from_file (pixmap_file);
+		bookmark->pixmap_data = pixmap_data_new_from_file
+			(pixmap_file, FALSE);
 	}
 	else
 	{
@@ -892,6 +958,7 @@ bookmarks_new_bookmark (BookmarkType type, gboolean escape_name,
 	bookmark->time_added = 0;
 	bookmark->time_modified = 0;
 	bookmark->time_visited = 0;
+	
 	return bookmark;
 }
 
@@ -988,11 +1055,12 @@ void bookmarks_set_visited (BookmarkItem *b)
  * 
  * Return value: the new bookmark
  **/
+/* FIXME: This fails miserably wrt aliases */
 BookmarkItem *
 bookmarks_copy_bookmark (BookmarkItem *b)
 {
 	GList *l;
-	/* TODO: if it an alias... */
+	/* TODO: if it is an alias... */
 	BookmarkItem *new = bookmarks_new_bookmark (b->type, FALSE, b->name,
 						    b->url, b->nick, b->notes,
 						    b->pixmap_file);
@@ -1006,14 +1074,15 @@ bookmarks_copy_bookmark (BookmarkItem *b)
 	new->time_added = b->time_added;
 	new->time_modified = b->time_modified;
 	new->time_visited = b->time_visited;
-	if (b->pixmap_data != NULL)
+
+	if (BOOKMARK_ITEM_IS_FOLDER (b)) 
 	{
-		new->pixmap_data = b->pixmap_data;
-	}
-	if (b->type == BM_FOLDER) {
 		for (l = b->list; l != NULL; l = g_list_next (l)) {
 			BookmarkItem *child = l->data;
-			BookmarkItem *new_child = bookmarks_copy_bookmark (child);
+			BookmarkItem *new_child;
+			if (BOOKMARK_ITEM_IS_FOLDER (b) && (b->alias_of))
+				continue;
+			new_child = bookmarks_copy_bookmark (child);
 			new_child->parent = new;
 			new->list = g_list_append (new->list, new_child);
 		}
@@ -1025,8 +1094,9 @@ bookmarks_copy_bookmark (BookmarkItem *b)
  * bookmarks_free_bookmark: free a bookmark
  * @b: the bookmark to be freed, if NULL thenb do nothing
  * 
- * Frees the bookmarks and all the strings referenced by it and the list of children 
- * if it's a BM_FOLDER (but does not free the children). Use it instead of g_free. 
+ * Frees the bookmarks and all the strings referenced by it and the list 
+ * of children if it's a BM_FOLDER (but does not free the children). Use it 
+ * instead of g_free. 
  **/
 void
 bookmarks_free_bookmark (BookmarkItem *b)
@@ -1075,8 +1145,7 @@ bookmarks_insert_bookmark (BookmarkItem *b, BookmarkItem *near,
 	}
 	else {
 		if (insert_pos == GTK_CLIST_DRAG_INTO 
-		    && (near->type == BM_FOLDER 
-			|| near->type == BM_AUTOBOOKMARKS)
+		    && BOOKMARK_ITEM_IS_FOLDER (near) 
 		    && !near->alias_of)
 			parent = near;
 		else
@@ -1099,8 +1168,7 @@ bookmarks_insert_bookmark (BookmarkItem *b, BookmarkItem *near,
 		case GTK_CLIST_DRAG_INTO:
 			if (near == NULL)
 				break;
-			if (near->type == BM_SITE ||
-			    near->type == BM_SEPARATOR)
+			if (BOOKMARK_ITEM_IS_FOLDER (near))
 				position++;
 			break;				
 		case GTK_CLIST_DRAG_AFTER:
@@ -1112,6 +1180,8 @@ bookmarks_insert_bookmark (BookmarkItem *b, BookmarkItem *near,
 		parent->list = g_list_insert (parent->list, b, position);
 	}
 	bookmarks_update_alias (parent);
+
+	bookmarks_toolbars_check_update (b);
 }
 
 /**
@@ -1219,7 +1289,8 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
 	else 
 	{ 
 		g_free (b->name);
-		b->name = string;
+		b->name = mozilla_locale_to_utf8 (string);
+		g_free (string);
 		
 		if (strlen (b->name) == 0)
 		{
@@ -1230,10 +1301,12 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
 		if (b->url &&
 		    eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED))
 		{
-			favicon_get_siteicon (NULL, b->url);
+			if (b->url)
+				favicon_get_siteicon (NULL, b, NULL);
 		}
 		
 		bookmarks_editor_place_tree_items (b);
+		bookmarks_toolbars_check_update (b);
 		bookmarks_updated (); 
 	}
 }
@@ -1494,10 +1567,22 @@ bookmarks_find_by_nick (const gchar *nick, BookmarkItem *r, gboolean wantsmart)
 	BookmarkItem *b;
 	if (!r) r = bookmarks_root;
 
-	if ((r->nick) && !strcmp (r->nick, nick) &&
-	    (wantsmart ^ (r->url == NULL || strstr(r->url,"%s") == NULL)))
+	if (((r->nick) && r->nick[0])
+	    && (wantsmart ^ (r->url == NULL || strstr(r->url,"%s") == NULL)))
 	{
-		return r;
+		/* maybe the bookmark defines several nicks, separated
+		 * by commas or spaces */
+		gchar **nicks = g_strsplit (r->nick, ", ", 999);
+		gint i;
+		gboolean found = FALSE;
+		for (i = 0; nicks[i] != NULL; i++)
+			if (!strcmp (nicks[i], nick))
+			{
+				found = TRUE;
+				break;
+			}
+			g_strfreev (nicks);
+			if (found) return r;
 	}
 
 	if ((r->type == BM_FOLDER) && !r->alias_of)
@@ -1527,6 +1612,7 @@ bookmarks_item_to_string (BookmarkItem *bookmark)
 
 	doc = xmlNewDoc ("1.0");
 	node = xmlNewDocNode (doc, NULL, "single_bookmark", NULL);
+	bookmarks_ensure_unique_ids (bookmark);
 	bookmarks_save_recursively (doc, node, bookmark);
 	xmlDocSetRootElement (doc, node);
 	xmlDocDumpMemory (doc, &mem, &size);
@@ -1568,6 +1654,7 @@ bookmarks_item_from_string (char *string)
  *   to the form %{hex key code}
  *   The returned string should be freed.
  **/
+/* FIXME: audit this for utf8 */
 gchar *
 bookmarks_translate_string (gchar *smart_bm_string)
 {
@@ -1638,8 +1725,6 @@ bookmarks_folder_open_all_items (GaleonEmbed *embed, BookmarkItem *bi,
 	}
 }
 
-
-/* FIXME: Unused? */
 /**
  * bookmarks_add_completion_recursively: recursively add bookmarks to the
  * auto completions table
@@ -1697,7 +1782,7 @@ static BookmarkItem *bookmarks_find_by_id (long id, BookmarkItem *r)
 	/* Don't return bookmarks that may be not yet created aliases */
 	if ((r->id == id) && (r->name != NULL)) 
 		return bookmarks_find_real_bookmark (r);
-	if ((r->type == BM_FOLDER) && !r->alias_of)
+	if (BOOKMARK_ITEM_IS_FOLDER (r) && !r->alias_of)
 		for (l = r->list; l != NULL; l = g_list_next (l)) {
 			b = bookmarks_find_by_id (id, l->data);
 			if (b) return b;
@@ -1715,11 +1800,6 @@ bookmarks_get_siteicon (const gchar *url)
 	gchar *favicon_url, *favicon_path;
 	PixmapData *result = NULL;
 
-	if (!eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED))
-	{
-		return site_pixmap_data;
-	}
-
 	if (url == NULL)
 	{
 		return site_pixmap_data;
@@ -1738,7 +1818,7 @@ bookmarks_get_siteicon (const gchar *url)
 	{
 		return site_pixmap_data;
 	} 
-	
+
 	if (gnome_vfs_uri_has_parent (vfs_uri))
 	{
 		vfs_parent_uri = gnome_vfs_uri_get_parent (vfs_uri);
@@ -1753,11 +1833,11 @@ bookmarks_get_siteicon (const gchar *url)
 	favicon_url = gnome_vfs_uri_to_string (vfs_parent_uri, 0);
 
 	gnome_vfs_uri_unref (vfs_parent_uri);
-	gnome_vfs_uri_unref (favicon_uri);
+	if (favicon_uri) gnome_vfs_uri_unref (favicon_uri);
 	
 	favicon_path = favicon_filename (favicon_url);
 	
-	if (g_file_exists (favicon_path))
+	if (favicon_path && g_file_exists (favicon_path))
 	{
 		GdkPixbuf *pixbuf;
 		
@@ -1834,11 +1914,14 @@ void bookmarks_set_tree_item (BookmarkItem *b, BookmarksEditorControls *c,
 			return;
 		}
 	}
-	/* not found, add */
-	ti = g_new0 (BookmarkTreeItem, 1);
-	ti->tree_item = n;
-	ti->controls = c;
-	b->tree_items = g_list_prepend (b->tree_items, ti);
+	/* not found, add if needed */
+	if (n)
+	{
+		ti = g_new0 (BookmarkTreeItem, 1);
+		ti->tree_item = n;
+		ti->controls = c;
+		b->tree_items = g_list_prepend (b->tree_items, ti);
+	}
 }
 
 /**
@@ -1890,9 +1973,9 @@ bookmarks_merge_trees (BookmarkItem *root1, BookmarkItem *root2)
 				b->parent = root1;
 				last_one_added = TRUE;
 			}
-		} else if ((item2->type == BM_FOLDER)
-			   || (item2->type == BM_AUTOBOOKMARKS)) {
-			bookmarks_merge_trees (item2, item);
+		} else if (BOOKMARK_ITEM_IS_FOLDER (item2)) {
+			if (!(item2->alias_of || item->alias_of))
+				bookmarks_merge_trees (item2, item);
 			last_one_added = FALSE;
 		} else {
 			last_one_added = FALSE;
@@ -1912,11 +1995,11 @@ bookmarks_merge_trees_look_for_equal (BookmarkItem *folder,
 		    && (bookmark->type == BM_SITE)) {
 			if (!strcmp (item->url, bookmark->url)) 
 				return item;
-		} else if (((bookmark->type == BM_FOLDER)
-			    || (bookmark->type == BM_AUTOBOOKMARKS)) 
-			   && ((item->type == BM_FOLDER)
-			       || (item->type == BM_AUTOBOOKMARKS))) {
-			if (!strcmp (item->name, bookmark->name)) 
+		} else if ((BOOKMARK_ITEM_IS_FOLDER (bookmark))
+			   && (BOOKMARK_ITEM_IS_FOLDER (item))) {
+			if (!strcmp (item->name, bookmark->name)
+			    && (!(item->alias_of || bookmark->alias_of)
+				|| (item->alias_of && bookmark->alias_of))) 
 				return item;
 		}
 	}
@@ -1924,12 +2007,15 @@ bookmarks_merge_trees_look_for_equal (BookmarkItem *folder,
 }
 
 
-void bookmarks_sort_norec (BookmarkItem *b, gboolean update_edit)
+void bookmarks_sort_norec (BookmarkItem *b, gboolean folders_first, 
+			   gboolean update_edit)
 {
-	if ((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) {
+	if (BOOKMARK_ITEM_IS_FOLDER (b)) {
 		if (!b->alias_of) {
 			b->list = g_list_sort 
-				(b->list, bookmarks_compare_name);
+				(b->list, folders_first 
+				 ? bookmarks_compare_name_and_type
+				 : bookmarks_compare_name);
 			bookmarks_update_alias (b);
 		}
 		if (update_edit)
@@ -1937,15 +2023,16 @@ void bookmarks_sort_norec (BookmarkItem *b, gboolean update_edit)
 	}
 }
 
-void bookmarks_sort_rec (BookmarkItem *b, gboolean update_edit)
+void bookmarks_sort_rec (BookmarkItem *b, gboolean folders_first,
+			 gboolean update_edit)
 {
 	GList *l;
-	bookmarks_sort_norec (b, FALSE);
-	if ((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) 
+	bookmarks_sort_norec (b, folders_first, FALSE);
+	if (BOOKMARK_ITEM_IS_FOLDER (b))
 		for (l = b->list; l != NULL; l = l->next) {
 			BookmarkItem *b2 = l->data;
 			if (!b2->alias_of) {
-				bookmarks_sort_rec (b2, FALSE);
+				bookmarks_sort_rec (b2, folders_first, FALSE);
 			}
 		}
 	if (update_edit)
@@ -1962,6 +2049,20 @@ bookmarks_compare_name (gconstpointer a, gconstpointer b)
 	return g_strcasecmp (b1->name, b2->name);
 }
 
+static gint 
+bookmarks_compare_name_and_type (gconstpointer a, gconstpointer b)
+{
+	BookmarkItem *b1 = (BookmarkItem *) a;
+	BookmarkItem *b2 = (BookmarkItem *) b;
+
+	if (BOOKMARK_ITEM_IS_FOLDER (b1) && !BOOKMARK_ITEM_IS_FOLDER (b2))
+		return -1;
+	if (!BOOKMARK_ITEM_IS_FOLDER (b1) && BOOKMARK_ITEM_IS_FOLDER (b2))
+		return 1;
+
+	return g_strcasecmp (b1->name, b2->name);
+}
+
 /**
  * bookmarks_folder_open_in_myportal: Open the MyPortal page for a category
  **/
@@ -1974,7 +2075,7 @@ bookmarks_folder_open_in_myportal (GaleonEmbed *embed, BookmarkItem *bi,
 
 	g_return_if_fail (bi != NULL);
 
-	if ((bi->type != BM_FOLDER) && (bi->type != BM_AUTOBOOKMARKS))
+	if (!BOOKMARK_ITEM_IS_FOLDER (bi))
 	{
 		bi = bi->parent;
 	}
@@ -2019,3 +2120,43 @@ bookmarks_get_bookmark_path (BookmarkItem *b)
 	return path;
 }
 
+/**
+ * Makes sure that all the bookmarks descending from root have unique ids.
+ * The ids may change even if not necesary (because they are assigned in 
+ * growing order). Aliases are updated.
+ */
+static void
+bookmarks_ensure_unique_ids (BookmarkItem *root)
+{
+	gulong maxid = root->id;
+	bookmarks_ensure_unique_ids_rec (root, &maxid);
+}
+
+static void
+bookmarks_ensure_unique_ids_rec (BookmarkItem *b, gulong *maxid)
+{
+	GList *li;
+	for (li = b->list; li != NULL; li = li->next)
+	{
+		BookmarkItem *bi = li->data;
+
+		/* aliases are updated when they real bookmarks is updated */
+		if (bi->alias_of) continue;
+		
+		if (bi->id <= *maxid)
+		{
+			bi->id = ++(*maxid);
+			bookmarks_update_alias (bi);
+		}
+		else
+		{
+			*maxid = bi->id;
+		}
+
+		if (BOOKMARK_ITEM_IS_FOLDER (bi))
+		{
+			bookmarks_ensure_unique_ids_rec (bi, maxid);
+		}
+	}
+}
+
