@@ -45,6 +45,12 @@ extern GtkTargetEntry bookmarks_dnd_targets [];
 /** The bookmarks editing controls list */
 GList *bookmarks_editors = NULL;
 
+/* block any search position updates when true */
+gboolean block_search_position = FALSE;
+
+/* block popping up the dock menu */
+static gboolean block_dock_popup_menu = FALSE;
+
 /* Function prototypes */
 static BookmarksEditorControls *bookmarks_editor_controls_init
                                (BookmarkItem *root,
@@ -66,9 +72,10 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 				BookmarksEditorControlsType type)
 {
 	BookmarksEditorControls *editor;
-	GtkWidget *close, *detach, *show_menu;
+	GtkWidget *close, *detach, *show_menu, *dummy;
 	GladeXML *gxml = NULL;
 	GList *toolbar_items, *tlist;
+	GtkStyle *style, *togglestyle;
 		
 	editor = g_new0 (BookmarksEditorControls, 1);
 
@@ -79,7 +86,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 	switch (type)
 	{
 	case STANDARD_BOOKMARKS_EDITOR:
-		gxml = glade_widget_new ("bookmarks.glade", "bookmarks_editor", 
+		gxml = glade_widget_new ("bookmarks.glade", "bookmarks_editor",
 					 &(editor->dialog), editor);
 		editor->menubar = glade_xml_get_widget 
 			(gxml, "bookmarks_editor_menubar");
@@ -116,6 +123,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(gxml, "bookmarks_editor_time_modified_entry");
 		editor->time_visited_entry = glade_xml_get_widget
 			(gxml, "bookmarks_editor_time_visited_entry");
+		editor->search_entry = NULL;
 		bookmarks_editor_add_menubar (editor);
 		g_assert (editor->dialog != NULL);
 		g_assert (editor->toolbar != NULL);
@@ -150,6 +158,8 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(gxml, "bookmarks_dock_nick_entry");
 		editor->notes_text = glade_xml_get_widget
 			(gxml, "bookmarks_dock_notes_text");
+		editor->search_entry = glade_xml_get_widget
+			(gxml, "bookmarks_dock_search_entry");
 		editor->pixmap_entry = NULL;
 		editor->pixmap_file_entry = NULL;
 		editor->create_toolbar_toggle = NULL;
@@ -159,6 +169,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 		editor->time_added_entry = NULL;
 		editor->time_modified_entry = NULL;
 		editor->time_visited_entry = NULL;
+		editor->search_position = NULL;
 
 		g_assert (editor->dialog != NULL);
 		g_assert (editor->toolbar != NULL);
@@ -166,6 +177,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 		g_assert (editor->url_entry != NULL);
 		g_assert (editor->nick_entry != NULL);
 		g_assert (editor->notes_text != NULL);
+		g_assert (editor->search_entry != NULL);
 
 		/* fix up broken button relief (bug in libglade) */
 		close = glade_xml_get_widget 
@@ -188,6 +200,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 	editor->last_pressed = NULL;
 	editor->lock_edit_controls = FALSE;
 	editor->window = NULL;
+	editor->find_dialog = NULL;
 
 	/* get the widgets we need to work with later*/
 	editor->edit_button = glade_xml_get_widget
@@ -210,7 +223,8 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 
 	gtk_selection_add_target (GTK_WIDGET (editor->ctree),
 				  GDK_SELECTION_PRIMARY,
-				  GDK_SELECTION_TYPE_STRING, /* FIXME, use a correct type*/
+				  /* FIXME, use a correct type*/
+				  GDK_SELECTION_TYPE_STRING,
 				  1);
 	/* copy & paste callbacks */
 	gtk_signal_connect 
@@ -229,20 +243,64 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(GNOME_PIXMAP_ENTRY (editor->pixmap_entry), SHARE_DIR);
 	}
 
-	/* setup the context menu callback for the toolbar */
-	toolbar_items = gtk_container_children 
-		(GTK_CONTAINER (editor->toolbar));
-	tlist = toolbar_items;
-	while (tlist != NULL)
+	/* setup the context menu callback and the styles for the toolbar */
+	toolbar_items = GTK_TOOLBAR (editor->toolbar)->children; 
+	dummy = gtk_button_new ();
+	style = gtk_rc_get_style (dummy);
+	gtk_widget_destroy (dummy);
+
+	if (style == NULL)
 	{
+		style = gtk_style_new ();
+	}
+	else
+	{
+		gtk_style_ref (style);
+	}
+
+	dummy = gtk_toggle_button_new ();
+	togglestyle = gtk_rc_get_style (dummy);
+	gtk_widget_destroy (dummy);
+
+	if (togglestyle == NULL)
+	{
+		togglestyle = gtk_style_new ();
+	}
+	else
+	{
+		gtk_style_ref (togglestyle);
+	}
+
+	for (tlist = toolbar_items; tlist; tlist = g_list_next (tlist))
+	{
+		if (!GTK_IS_WIDGET (((GtkToolbarChild*)(tlist->data))->widget))
+			continue;
+
 		gtk_signal_connect 
-			(GTK_OBJECT (tlist->data), "button-press-event",
+			(GTK_OBJECT (((GtkToolbarChild*)(tlist->data))->widget),
+			"button-press-event",
 			GTK_SIGNAL_FUNC 
 			 (bookmarks_editor_toolbar_button_press_event_cb),
 			editor);
-		tlist = tlist->next;
+
+		if (!GTK_IS_TOGGLE_BUTTON
+			(((GtkToolbarChild*)(tlist->data))->widget))
+		{
+			gtk_widget_set_style
+				(((GtkToolbarChild*)(tlist->data))->label,
+				 style);
+		}
+		else if (GTK_IS_TOGGLE_BUTTON
+			(((GtkToolbarChild*)(tlist->data))->widget))
+		{
+			gtk_widget_set_style
+				(((GtkToolbarChild*)(tlist->data))->label,
+				 togglestyle);
+		}
 	}
-	g_list_free (toolbar_items);
+
+	gtk_style_unref (style);
+	gtk_style_unref (togglestyle);
 
 	/* callback to make sure that everything is cleaned before destroying 
 	   the widgets */
@@ -260,6 +318,7 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	gboolean menubar_relief;
 	gboolean toolbar_detachable;
 	gboolean menubar_detachable;
+	gboolean toolbar_lines;
 	BookmarksEditorControls *controls = NULL;
 
 	/* read preferences */
@@ -267,6 +326,7 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	menubar_detachable = gnome_preferences_get_menubar_detachable ();
 	toolbar_relief = gnome_preferences_get_toolbar_relief ();
 	menubar_relief = gnome_preferences_get_menubar_relief ();
+	toolbar_lines = gnome_preferences_get_toolbar_lines ();
 
 	controls = bookmarks_editor_controls_init
 		(bookmarks_root, STANDARD_BOOKMARKS_EDITOR);
@@ -275,6 +335,7 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	/* 630 is chosen since thats wide enough for the toolbar in
 	 * with all the standard GNOME defaults */
 	state_load_window_state (controls->dialog, "bookmarks_editor");
+	state_load_dock_layout (controls->dialog, "bookmarks_editor");
 	state_load_column_widths (controls->ctree, "bookmarks_editor");
 	state_load_pane_pos (controls->vpane,
 			     "bookmarks_editor_vpane=-1");
@@ -302,6 +363,10 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	gnome_dock_item_set_shadow_type
 		(GNOME_DOCK_ITEM (controls->menubar_dock),
 		 menubar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
+
+	gtk_toolbar_set_space_style (GTK_TOOLBAR (controls->toolbar),
+				     toolbar_lines ? GTK_TOOLBAR_SPACE_LINE :
+						     GTK_TOOLBAR_SPACE_EMPTY);
 		 
 	gtk_widget_show (controls->dialog);
 
@@ -337,6 +402,8 @@ bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 		{
 			state_save_window_state (controls->dialog->window,
 						 "bookmarks_editor");
+			state_save_dock_layout (controls->dialog,
+						 "bookmarks_editor");
 			state_save_column_widths (controls->ctree, 
 						  "bookmarks_editor");
 			state_save_pane_pos (controls->vpane,
@@ -346,6 +413,8 @@ bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 		{
 			state_save_window_state (controls->dialog->window,
 						 "compact_bookmarks_editor");
+			state_save_dock_layout (controls->dialog,
+						 "compact_bookmarks_editor");
 		}
 	}
 	
@@ -366,6 +435,13 @@ bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 		gtk_widget_destroy (controls->dialog);
 	}
 
+	if (controls->find_dialog)
+	{
+		gtk_widget_destroy (controls->find_dialog->dialog);
+		g_free (controls->find_dialog->str);
+		g_free (controls->find_dialog);
+	}
+	
 	/* scrub control structure and free */
 	memset (controls, 0, sizeof (BookmarksEditorControls));
 	g_free (controls);
@@ -375,6 +451,7 @@ BookmarksEditorControls *
 bookmarks_editor_show_dock (GaleonWindow *window)
 {
 	BookmarksEditorControls *controls;
+	gboolean toolbar_lines = gnome_preferences_get_toolbar_lines ();
 
 	controls = bookmarks_editor_controls_init (bookmarks_root, 
 						   DOCKED_BOOKMARKS_EDITOR);
@@ -388,6 +465,9 @@ bookmarks_editor_show_dock (GaleonWindow *window)
 		(GTK_TOOLBAR (controls->toolbar), 
 		eel_gconf_get_integer 
 		       (CONF_TOOLBAR_BOOKMARKS_EDITOR_DOCKED_STYLE));
+	gtk_toolbar_set_space_style (GTK_TOOLBAR (controls->toolbar),
+				     toolbar_lines ? GTK_TOOLBAR_SPACE_LINE :
+						     GTK_TOOLBAR_SPACE_EMPTY);
 
 	window_dock (window, controls->dialog, 
 		     CONF_STATE_BOOKMARKS_DOCK_WIDTH);
@@ -423,6 +503,7 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 	GdkBitmap *mask;
 	GtkCTreeNode *parent_node;
 	GtkCTreeNode *sibling_node;
+	gchar *loc_str, *tmp;
 
 	text[2] = NULL;
 	switch (b->type) {
@@ -436,7 +517,10 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		text[0] = strip_uline_accel (b->name);
+		tmp = strip_uline_accel (b->name);
+		loc_str = mozilla_utf8_to_locale(tmp);
+		text[0] = loc_str;
+		g_free (tmp);
 		text[1] = g_strdup (b->url);
 		isleaf = TRUE;
 
@@ -452,7 +536,10 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		text[0] = strip_uline_accel (b->name);
+		tmp = strip_uline_accel (b->name);
+		loc_str = mozilla_utf8_to_locale(tmp);
+		text[0] = loc_str;
+		g_free (tmp);
 		text[1] = g_strdup ("");
 		isleaf = FALSE;
 		if (b == default_bookmarks_root) 
@@ -578,7 +665,8 @@ bookmarks_editor_update_tree_items (BookmarkItem *b)
  * 
  **/
 void
-bookmarks_editor_remove_tree_item (BookmarksEditorControls *controls, BookmarkItem *b)
+bookmarks_editor_remove_tree_item (BookmarksEditorControls *controls,
+				   BookmarkItem *b)
 {
 	if (b == controls->root_bookmark) {
 		GList *l;
@@ -590,7 +678,7 @@ bookmarks_editor_remove_tree_item (BookmarksEditorControls *controls, BookmarkIt
 			/* this removes the treeitem and its children */
 			gtk_ctree_remove_node (GTK_CTREE (controls->ctree),
 					       GTK_CTREE_NODE (node));
-			/* now, let's NULL the fields */
+			/* now, let's NULL the fields of the children */
 			bookmarks_editor_remove_tree_item_null (b, controls);
 		}
 	}
@@ -634,6 +722,7 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 	GtkCTreeNode *parent_node;
+	gchar *loc_str, *tmp;
 		
 	if (!bookmarks_get_tree_item (b, controls)) {
 		bookmarks_editor_place_tree_item (controls, b);
@@ -652,7 +741,10 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		text[0] = strip_uline_accel (b->name);
+		tmp = strip_uline_accel (b->name);
+		loc_str = mozilla_utf8_to_locale(tmp);
+		text[0] = loc_str;
+		g_free (tmp);
 		text[1] = g_strdup (b->url);
 		isleaf = TRUE;
 		bm_icon = bookmarks_get_siteicon (b->url);
@@ -664,7 +756,10 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		text[0] = strip_uline_accel (b->name);
+		tmp = strip_uline_accel (b->name);
+		loc_str = mozilla_utf8_to_locale(tmp);
+		text[0] = loc_str;
+		g_free (tmp);
 		text[1] = g_strdup ("");
 		isleaf = FALSE;
 		if (b == default_bookmarks_root)
@@ -697,10 +792,12 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 			pixmap = mask = NULL;
 		}
 		gtk_ctree_node_set_pixtext
-			(GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls), 0, text[0],
+			(GTK_CTREE (controls->ctree), 
+			 bookmarks_get_tree_item (b, controls), 0, text[0],
 			2, pixmap, mask);
 		gtk_ctree_node_set_text
-			(GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls), 1, text[1]);
+			(GTK_CTREE (controls->ctree), 
+			 bookmarks_get_tree_item (b, controls), 1, text[1]);
 	}
 	g_free (text[0]);
 	g_free (text[1]);
@@ -711,8 +808,9 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
  * 	at a given bookmark item & focus the name entry
  * @b : bookmark item to focus
  */
-void bookmarks_editor_select_bookmark (BookmarksEditorControls *controls,
-				       BookmarkItem *bi)
+void
+bookmarks_editor_select_bookmark (BookmarksEditorControls *controls,
+				  BookmarkItem *bi)
 {
 	GtkCTreeNode *node, *parent;
 	gint row;
@@ -778,17 +876,26 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 }
 
 void 
-bookmarks_editor_popup_menu (BookmarksEditorControls *controls)
+bookmarks_editor_popup_menu (BookmarksEditorControls *controls, GtkWidget *w)
 {
 	gint ret;
-	GtkWidget *popup = gtk_menu_new ();
+	GtkWidget *popup;
+
+	if (block_dock_popup_menu) return;
+
+	popup = gtk_menu_new ();
 
 	bookmarks_editor_fill_menu (GTK_MENU_SHELL (popup),
 				    NULL, controls,
 				    NULL, NULL);
 
-	ret = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
+	block_dock_popup_menu = TRUE;
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
+	ret = gnome_popup_menu_do_popup_modal (popup,
+					       menu_position_under_widget, w,
 					       NULL, NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), FALSE);
+	block_dock_popup_menu = FALSE;
 
 	/* Wait for the menu to be destroyed if a menu item is selected */
 	if (ret != -1) 
@@ -829,20 +936,20 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		{ GNOME_APP_UI_ITEM, N_("New _item"), NULL,
 		  bookmarks_editor_new_item_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_FILENAME, "../galeon/new_item_small.xpm",
-		  GDK_n, GDK_CONTROL_MASK, NULL },
+		  0, 0, NULL },
 		{ GNOME_APP_UI_ITEM, N_("New _separator"), NULL,
 		  bookmarks_editor_new_separator_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_FILENAME,
 		  "../galeon/new_separator_small.xpm",
-		  GDK_e, GDK_CONTROL_MASK, NULL },
+		  0, 0, NULL },
 		{ GNOME_APP_UI_ITEM, N_("New _folder"), NULL,
 		  bookmarks_editor_new_folder_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_FILENAME, "../galeon/new_folder_small.xpm",
-		  GDK_F, GDK_CONTROL_MASK, NULL },
+		  0, 0, NULL },
 		{ GNOME_APP_UI_ITEM, N_("New _alias"), NULL,
 		  bookmarks_editor_add_alias_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_FILENAME, "../galeon/new_alias_small.xpm",
-		  GDK_a, GDK_CONTROL_MASK, NULL },
+		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
 		GNOMEUIINFO_MENU_OPEN_ITEM
                         (bookmarks_editor_open_clicked_cb, controls),
@@ -863,10 +970,19 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		  bookmarks_editor_sort_norec_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("_Recursive"), NULL,
+		{ GNOME_APP_UI_ITEM, N_("_Recursively"), NULL,
 		  bookmarks_editor_sort_rec_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
+		GNOMEUIINFO_SEPARATOR,
+		{ GNOME_APP_UI_ITEM, N_("One _Level (folders first)"), NULL,
+		  bookmarks_editor_sort_ff_norec_clicked_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
+		{ GNOME_APP_UI_ITEM, N_("Recursively (_folders first)"), NULL,
+		  bookmarks_editor_sort_ff_rec_clicked_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
 		GNOMEUIINFO_END
 	};
 	GnomeUIInfo edit_menu_uiinfo[] =
@@ -898,9 +1014,9 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		  bookmarks_editor_move_down_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_DOWN,
 		  GDK_d, GDK_CONTROL_MASK, NULL },
-		{ GNOME_APP_UI_ITEM, N_("_Refresh icon"), NULL,
+		{ GNOME_APP_UI_ITEM, N_("Fetch bookmark _icon"), NULL,
 		  bookmarks_editor_fetch_icon_clicked_cb, controls, NULL,
-		  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH,
+		  GNOME_APP_PIXMAP_FILENAME, "../galeon/fetch_favicon_small.xpm",
 		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
 		{ GNOME_APP_UI_ITEM, N_("Add alias for r_oot bookmark"), NULL,
@@ -908,8 +1024,13 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_SUBTREE (N_("Sort folder"), 
+		GNOMEUIINFO_SUBTREE (N_("Sort fold_er"), 
 				     sort_folder_menu_uiinfo),
+		GNOMEUIINFO_SEPARATOR,
+		{ GNOME_APP_UI_ITEM, N_("_Find bookmarks..."), NULL,
+		  bookmarks_editor_find_bookmarks_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
+		  GDK_f, GDK_CONTROL_MASK, NULL },
 		GNOMEUIINFO_END
 	};
 	GnomeUIInfo view_menu_uiinfo[] =
@@ -953,6 +1074,10 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 	};
 	GnomeUIInfo import_menu_uiinfo[] =
 	{
+		{ GNOME_APP_UI_ITEM, N_("Import from _Galeon's own format..."),
+		  NULL, bookmarks_import_own_activate_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
 		{ GNOME_APP_UI_ITEM, N_("Import from _XBEL..."), NULL,
 		  bookmarks_import_xbel_activate_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
@@ -1060,3 +1185,134 @@ bookmarks_editors_refresh (void)
 		bookmarks_editor_set_root (controls, controls->root_bookmark);
 	}
 }
+
+gboolean
+bookmarks_editor_search (BookmarksEditorControls *controls, gchar *str,
+			 BookmarksSearchIn search_in,
+			 BookmarksSearchMethod search_method,
+			 gboolean backwards, gboolean wrap)
+{
+	GtkCList *clist = GTK_CLIST (controls->ctree);
+	GtkCTree *ctree = GTK_CTREE (controls->ctree);
+	gboolean found = FALSE;
+	BookmarkItem *b = NULL;
+
+	g_assert (controls != NULL);
+	g_assert (str != NULL);
+
+	/* reset to start */
+	if (wrap && !backwards)
+	{
+		controls->search_position = clist->row_list;
+	}
+	/* reset to end */
+	else if (wrap && backwards)
+	{
+		controls->search_position = clist->row_list_end;
+	}
+	/* no search position; set */
+	else if (controls->search_position == NULL)
+	{
+		controls->search_position = clist->row_list;
+	}
+	/* we need to start at the next/prev position, otherwise we'll "find"
+	 * the bookmark at the current pos again */
+	else if (backwards)
+	{
+		controls->search_position =
+			g_list_previous (controls->search_position);
+	}
+	else
+	{
+		controls->search_position =
+			g_list_next (controls->search_position);
+	}
+
+	/* go through the clist until we find a match */
+	for (; controls->search_position; controls->search_position =
+	     backwards ? g_list_previous (controls->search_position) : 
+	                 g_list_next (controls->search_position))
+	{
+		b = gtk_ctree_node_get_row_data (ctree,
+				GTK_CTREE_NODE (controls->search_position));
+		found = bookmarks_editor_search_matches (b, str, search_in,
+							 search_method);
+	
+		/* select bookmark, if found */
+		if (found == TRUE)
+		{
+			block_search_position = TRUE;
+			bookmarks_editor_select_bookmark (controls, b);
+			block_search_position = FALSE;
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+gboolean
+bookmarks_editor_search_matches (BookmarkItem *b, gchar *str,
+				 BookmarksSearchIn search_in,
+				 BookmarksSearchMethod search_method)
+{
+	gchar *tmp = NULL;
+	gboolean found = FALSE;
+
+	if (b == NULL) return FALSE;
+	
+	/* select where to search in */
+	switch (search_in)
+	{
+	case BOOKMARKS_SEARCH_IN_NAME:
+		tmp = b->name;
+		break;
+	case BOOKMARKS_SEARCH_IN_NICKS:
+		tmp = b->nick;
+		break;
+	case BOOKMARKS_SEARCH_IN_URL:
+		tmp = b->url;
+		break;
+	case BOOKMARKS_SEARCH_IN_NOTES:
+		tmp = b->notes;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	/* select search method */
+	switch (search_method)
+	{
+	case BOOKMARKS_SEARCH_METHOD_CONTAINS:
+		found = tmp && g_strcasestr (tmp, str) != NULL;
+		break;
+	case BOOKMARKS_SEARCH_METHOD_STARTS_WITH:
+		found = tmp &&
+			g_strncasecmp (tmp, str, strlen (str)) == 0;
+		break;
+	case BOOKMARKS_SEARCH_METHOD_ENDS_WITH:
+		found = tmp &&
+			g_strcasecmp (tmp + strlen (tmp) - strlen (str),
+				      str) == 0;
+		break;
+	case BOOKMARKS_SEARCH_METHOD_IS:
+		found = tmp && g_strcasecmp (tmp, str) == 0;
+		break;
+	case BOOKMARKS_SEARCH_METHOD_IS_NOT:
+		found = TRUE;
+		if (tmp)
+			found = g_strcasecmp (tmp, str) != 0;
+		break;
+	case BOOKMARKS_SEARCH_METHOD_DOES_NOT_CONTAIN:
+		found = TRUE;
+		if (tmp)
+			found = g_strcasestr (tmp, str) == NULL;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	return found;
+}
