@@ -27,6 +27,7 @@
 #include "context.h"
 #include "dialog.h"
 #include "filepicker.h"
+#include "autocompletion.h"
 
 #include <string.h>
 #include <time.h>
@@ -73,11 +74,38 @@ static void bookmarks_editor_set_times (BookmarksEditorControls *controls,
 					BookmarkItem *bookmark,
 					BookmarkItem *bookmarkreal);
 static void bookmarks_selection_prepare (GList *l,
-					BookmarksEditorControls *controls);
+					 BookmarksEditorControls *controls);
 static gint bookmarks_row_compare (GList *a, GList *b);
 void
 bookmarks_editor_key_press_event_cb (GtkWidget *widget,GdkEventKey *event,
 				     BookmarksEditorControls *controls);
+static void
+bookmarks_editor_sort_cb_generic (BookmarksEditorControls *controls, 
+				  gboolean recursive, gboolean folders_first);
+static gboolean bookmarks_find_search (BookmarksFindDialog *dialog,
+				       gboolean prev, gboolean wrap);
+void bookmarks_find_prev_clicked_cb (GtkWidget *widget,
+				     BookmarksFindDialog *dialog);
+void bookmarks_find_next_clicked_cb (GtkWidget *widget,
+				     BookmarksFindDialog *dialog);
+void bookmarks_find_cancel_clicked_cb (GtkWidget *widget,
+				       BookmarksFindDialog *dialog);
+void bookmarks_find_entry_changed_cb (GtkEditable *editable,
+				      BookmarksFindDialog *dialog);
+void bookmarks_find_criterium1_deactivate_cb (GtkMenuShell *menushell,
+					      BookmarksFindDialog *dialog);
+void bookmarks_find_criterium2_deactivate_cb (GtkMenuShell *menushell,
+					      BookmarksFindDialog *dialog);
+#if 0
+static gboolean 
+bookmarks_find_can_search_prevnext (BookmarksFindDialog *dialog,
+				    gboolean backward);
+static void
+bookmarks_find_update_prevnext_sensitivity (BookmarksFindDialog *dialog);
+#endif
+
+extern gboolean block_search_position;
+						
 void
 bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 					   gint column,
@@ -90,10 +118,10 @@ bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 
 	if (controls->set_default_folder_menuitem != NULL)
 	{
-		gtk_widget_set_sensitive (controls->set_default_folder_menuitem,
-					  (b != default_bookmarks_root) &&
-					   (b->type == BM_AUTOBOOKMARKS ||
-					    b->type == BM_FOLDER));
+		gtk_widget_set_sensitive
+			(controls->set_default_folder_menuitem,
+			 (b != default_bookmarks_root) 
+			 && BOOKMARK_ITEM_IS_FOLDER (b));
 	}
 
 	if (controls->notes_text)
@@ -138,6 +166,10 @@ bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 	bookmarks_editor_set_times (controls, b, 
 				    bookmarks_find_real_bookmark (b));
 	controls->lock_edit_controls = FALSE;
+	
+	/* update search stuff */
+	if (!block_search_position && controls->find_dialog)
+		controls->search_position = node;	
 }
 
 void
@@ -174,36 +206,58 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 		gboolean create_toolbar = FALSE;
 		gboolean create_context_menu = FALSE;
 		gchar *notes = NULL;
+		gchar *utf8_str;
 		/* note that b_alias and b_real may be the same bookmark */
 		b_alias = controls->selection;
 		b_real = bookmarks_find_real_bookmark (controls->selection);
 		if (controls->name_entry)
 		{
-			gchar *tmp;
-			tmp = gtk_editable_get_chars 
+			gchar *tmp1, *tmp2;
+			tmp1 = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->name_entry), 0, -1);
-			name = escape_uline_accel (tmp);
-			g_free (tmp);
+			tmp2 = escape_uline_accel (tmp1);
+			utf8_str = mozilla_locale_to_utf8 (tmp2);
+			g_free (tmp2);
+			g_free (tmp1);
+			name = utf8_str;
 		}
 		if (controls->url_entry)
 			url = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->url_entry), 0, -1);
 		if (controls->nick_entry)
-			nick = gtk_editable_get_chars 
+		{
+			gchar *tmp1, *tmp2;
+			tmp1 = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->nick_entry), 0, -1);
+			tmp2 = escape_uline_accel (tmp1);
+			utf8_str = mozilla_locale_to_utf8 (tmp2);
+			g_free (tmp2);
+			g_free (tmp1);
+			nick = utf8_str;
+		}
 		if (controls->pixmap_file_entry)
 			pixmap_file = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->pixmap_file_entry), 
 				 0, -1);
 		if (controls->create_toolbar_toggle)
 			create_toolbar = gtk_toggle_button_get_active 
-				(GTK_TOGGLE_BUTTON (controls->create_toolbar_toggle));
+				(GTK_TOGGLE_BUTTON (
+					controls->create_toolbar_toggle));
 		if (controls->create_context_menu_toggle)
 			create_context_menu = gtk_toggle_button_get_active 
-				(GTK_TOGGLE_BUTTON (controls->create_context_menu_toggle));
-		if (controls->notes_text)
-			notes = gtk_editable_get_chars 
+				(GTK_TOGGLE_BUTTON (
+					controls->create_context_menu_toggle));
+		if (controls->notes_text) 
+		{
+			gchar *tmp1, *tmp2;
+			tmp1 = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->notes_text), 0, -1);
+			tmp2 = escape_uline_accel (tmp1);
+			utf8_str = mozilla_locale_to_utf8 (tmp2);
+			g_free (tmp2);
+			g_free (tmp1);
+			notes = utf8_str;
+		}
 		switch (b_real->type) {
 		case BM_SITE:
 			if (controls->name_entry) {
@@ -213,6 +267,7 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 			if (controls->url_entry) {
 				if (b_real->url) g_free (b_real->url);
 				b_real->url = url;
+				auto_completion_add_url (b_real->url);
 			}
 			if (controls->nick_entry) {
 				if (b_real->nick) g_free (b_real->nick);
@@ -224,15 +279,18 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 			}
 			/* FIXME: remove duplication */
 			if (controls->pixmap_file_entry) {
-				if (strcmp (pixmap_file, b_real->pixmap_file) != 0)
+				if (strcmp (pixmap_file, b_real->pixmap_file)
+				    != 0)
 				{
 					if (b_real->pixmap_data)
 						g_free (b_real->pixmap_data);
 					b_real->pixmap_data = 
 						pixmap_data_new_from_file 
-								(pixmap_file);
+								(pixmap_file,
+								 FALSE);
 				}
-				if (b_real->pixmap_file) g_free (b_real->pixmap_file);
+				if (b_real->pixmap_file) 
+					g_free (b_real->pixmap_file);
 				b_real->pixmap_file = pixmap_file;
 			}
 			if (controls->notes_text) {
@@ -249,22 +307,27 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 			if (url) g_free (url);
 			if (nick) g_free (nick);
 			if (controls->create_toolbar_toggle) {
+				bookmarks_toolbars_check_update (b_alias);
 				b_alias->create_toolbar = create_toolbar;
 			}
 			if (controls->create_context_menu_toggle) {
-				b_alias->create_context_menu = create_context_menu;
+				b_alias->create_context_menu = 
+					create_context_menu;
 			}
 			/* FIXME: remove duplication */
 			if (controls->pixmap_file_entry) {
-				if (strcmp (pixmap_file, b_real->pixmap_file) != 0)
+				if (strcmp (pixmap_file, b_real->pixmap_file) 
+				    != 0)
 				{
 					if (b_real->pixmap_data)
 						g_free (b_real->pixmap_data);
 					b_real->pixmap_data = 
 						pixmap_data_new_from_file 
-								  (pixmap_file);
+								  (pixmap_file,
+								   FALSE);
 				}
-				if (b_real->pixmap_file) g_free (b_real->pixmap_file);
+				if (b_real->pixmap_file) 
+					g_free (b_real->pixmap_file);
 				b_real->pixmap_file = pixmap_file;
 			}
 			if (controls->notes_text) {
@@ -275,12 +338,13 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 		default:
 			break;
 		}
-
+		
 		b_real->time_modified = b_alias->time_modified = time (NULL);
 		bookmarks_editor_set_times (controls, b_alias, b_real);
 		bookmarks_editor_update_tree_items (b_real);
 		bookmarks_update_alias (b_real);
 		bookmarks_dirty = TRUE;
+		bookmarks_toolbars_check_update (b_real);
 	}
 }
 
@@ -336,6 +400,7 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 	GtkCTree *ctree = GTK_CTREE(controls->ctree);
 	BookmarkItem *next_to_select = NULL;
 	gboolean autobookmarks_deleted = FALSE;
+	gboolean need_to_ask = FALSE;
 
 	selection_list = bookmarks_get_selections_sorted(clist);
 
@@ -348,12 +413,35 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 		if (next_to_select_pos)
 			next_to_select = next_to_select_pos->data;
 	}
+	
+	bookmarks_selection_prepare (selection_list, controls);
+	
+	/* check if we are trying to remove any non empty folder 
+	   and bug the user to make sure he does want it */
+	for (l = selection_list; l != NULL; l = g_list_next (l)) {
+		BookmarkItem *b = l->data;
+		need_to_ask = need_to_ask || ((b->type == BM_FOLDER)
+					      && (b->list != NULL)
+					      && (b->alias_of == NULL));
+	}
+
+	if (need_to_ask) {
+		GtkWidget *parent = (controls->dialog &&
+				     GTK_IS_WINDOW (controls->dialog)) ? 
+			controls->dialog : controls->window->WMain;
+		if (!dialog_ask_question 
+		    (_("You are going to remove (at least) a non empty \n"
+		       "bookmark folder. All its bookmarks will be deleted.\n"
+		       "Do you want to continue?"), parent)) {
+			g_list_free (selection_list);
+			return;
+		}
+	}
 
 	gtk_clist_freeze (clist);
 	for (l = selection_list; l != NULL; l = g_list_next (l)) {
-		BookmarkItem *b;
+		BookmarkItem *b = l->data;
 
-		b =  gtk_ctree_node_get_row_data(ctree, l->data);
 		if (b->type == BM_AUTOBOOKMARKS)
 			autobookmarks_deleted = TRUE;
 
@@ -368,6 +456,7 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 			}
 		}
 		
+		bookmarks_toolbars_check_update (b);
 		bookmarks_remove_recursively (b);
 		bookmarks_dirty = TRUE;
         }
@@ -402,19 +491,24 @@ bookmarks_editor_move_up_clicked_cb (GtkButton *button,
 	gtk_clist_freeze(clist);
 	while (selection)
 	{
-		b = gtk_ctree_node_get_row_data(GTK_CTREE(controls->ctree),
-						GTK_CTREE_NODE(selection->data));
-		if (bookmarks_move_bookmark(controls, b, 0))
+		b = gtk_ctree_node_get_row_data
+			(GTK_CTREE (controls->ctree),
+			 GTK_CTREE_NODE (selection->data));
+		bookmarks_toolbars_check_update (b);
+		if (bookmarks_move_bookmark (controls, b, 0))
 		{
 			bookmarks_dirty = TRUE;
 			break;
 		}
+		/* need to call bookmarks_toolbars_check_update twice because 
+		   the parent of the bookmark may change */
+		bookmarks_toolbars_check_update (b);
 		bookmarks_dirty = TRUE;
 		selection = selection->next;
 	}
-	gtk_clist_thaw(clist);
+	gtk_clist_thaw (clist);
 	
-	g_list_free(selections_sorted);
+	g_list_free (selections_sorted);
 }
 
 void 
@@ -427,25 +521,30 @@ bookmarks_editor_move_down_clicked_cb (GtkButton *button,
 
 	clist = GTK_CLIST(controls->ctree);
 	
-	selections_sorted = bookmarks_get_selections_sorted(clist);
-	selection = selections_sorted = g_list_reverse(selections_sorted);
+	selections_sorted = bookmarks_get_selections_sorted (clist);
+	selection = selections_sorted = g_list_reverse (selections_sorted);
 
-	gtk_clist_freeze(clist);
+	gtk_clist_freeze (clist);
 	while (selection)
 	{
-		b = gtk_ctree_node_get_row_data(GTK_CTREE(controls->ctree),
-						GTK_CTREE_NODE(selection->data));
+		b = gtk_ctree_node_get_row_data
+			(GTK_CTREE(controls->ctree),
+			 GTK_CTREE_NODE(selection->data));
+		bookmarks_toolbars_check_update (b);
 		if (bookmarks_move_bookmark(controls, b, 1))
 		{
 			bookmarks_dirty = TRUE;
 			break;
 		}
+		/* need to call bookmarks_toolbars_check_update twice because 
+		   the parent of the bookmark may change */
+		bookmarks_toolbars_check_update (b);
 		bookmarks_dirty = TRUE;
 		selection = selection->next;
 	}
-	gtk_clist_thaw(clist);
+	gtk_clist_thaw (clist);
 	
-	g_list_free(selections_sorted);
+	g_list_free (selections_sorted);
 }
 
 void 
@@ -456,20 +555,21 @@ bookmarks_editor_fetch_icon_clicked_cb (GtkButton *button,
 	GList *selections_sorted, *selection;
 	BookmarkItem *b;
 
-	clist = GTK_CLIST(controls->ctree);
+	clist = GTK_CLIST (controls->ctree);
 	
 	selections_sorted = bookmarks_get_selections_sorted(clist);
 	selection = selections_sorted;
 
 	while (selection != NULL)
 	{
-		b = gtk_ctree_node_get_row_data(GTK_CTREE(controls->ctree),
-						GTK_CTREE_NODE(selection->data));
+		b = gtk_ctree_node_get_row_data
+			(GTK_CTREE(controls->ctree),
+			 GTK_CTREE_NODE(selection->data));
 		fetch_icon_recursively (b, controls);
 		selection = selection->next;
 	}
 	
-	g_list_free(selections_sorted);
+	g_list_free (selections_sorted);
 }
 
 static
@@ -484,14 +584,15 @@ void fetch_icon_recursively (BookmarkItem *b,
 	case BM_AUTOBOOKMARKS:
 		for (l = b->list; l != NULL; l = g_list_next (l))
 		{
-			fetch_icon_recursively ((BookmarkItem *)(l->data),
-						controls);
+			if (!b->alias_of)
+				fetch_icon_recursively 
+					((BookmarkItem *)(l->data),
+					 controls);
 		}
 		break;
 		
 	case BM_SITE:
-		favicon_get_siteicon (NULL, b->url);
-		bookmarks_dirty = TRUE;
+		favicon_get_siteicon (NULL, b, controls);
 		break;
 
 	case BM_SEPARATOR:
@@ -539,15 +640,13 @@ bookmarks_editor_add_alias_cb (GtkButton *button,
 			       BookmarksEditorControls *controls) 
 {
 	BookmarkItem *orig = controls->selection;
-	BookmarkItem *new = bookmarks_new_alias (orig);
 	BookmarkItem *near = controls->selection;
-	/* if the original is a folder, add the new boookmark before to avoid loops */
+
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
-	if (new->type != BM_SITE)
-		insert_pos = GTK_CLIST_DRAG_BEFORE;
 
 	if (near)
 	{
+		BookmarkItem *new = bookmarks_new_alias (orig);
 		new->time_added = time (NULL);
 		bookmarks_insert_bookmark (new, near, insert_pos);
 		bookmarks_editor_place_tree_items (new);
@@ -576,7 +675,7 @@ bookmarks_editor_new_item_clicked_cb (GtkButton *button,
 				      BookmarksEditorControls *controls) 
 {
 	BookmarkItem *new = bookmarks_new_bookmark 
-		(BM_SITE, TRUE, _("New bookmark"),  "http://",
+		(BM_SITE, TRUE, "",  "http://",
 		 NULL, NULL, NULL);
 	BookmarkItem *near = controls->selection;
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
@@ -595,7 +694,10 @@ bookmarks_editor_new_item_clicked_cb (GtkButton *button,
 		if (!GTK_WIDGET_VISIBLE (controls->edit_frame))
 			return;
 	}
-	gtk_widget_grab_focus (controls->name_entry);
+	if (controls->name_entry) 
+	{
+		gtk_widget_grab_focus (controls->name_entry);
+	}
 }
 
 void 
@@ -603,7 +705,7 @@ bookmarks_editor_new_folder_clicked_cb (GtkButton *button,
 					  BookmarksEditorControls *controls) 
 {
 	BookmarkItem *new = bookmarks_new_bookmark 
-		(BM_FOLDER, TRUE, _("New bookmark folder"), NULL, NULL, NULL, NULL);
+		(BM_FOLDER, TRUE, "", NULL, NULL, NULL, NULL);
 	BookmarkItem *near = controls->selection;
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
 
@@ -621,17 +723,22 @@ bookmarks_editor_new_folder_clicked_cb (GtkButton *button,
 		if (!GTK_WIDGET_VISIBLE (controls->edit_frame))
 			return;
 	}
-	gtk_widget_grab_focus (controls->name_entry);
+	if (controls->name_entry)
+	{
+		gtk_widget_grab_focus (controls->name_entry);
+	}
 }
 
 void 
 bookmarks_editor_new_separator_clicked_cb (GtkButton *button,
 					   BookmarksEditorControls *controls) 
 {
+	gchar* utfstr = mozilla_locale_to_utf8(_("Separator"));
 	BookmarkItem *new = bookmarks_new_bookmark 
-		(BM_SEPARATOR, TRUE, _("Separator"), NULL, NULL, NULL, NULL);
+		(BM_SEPARATOR, TRUE, utfstr, NULL, NULL, NULL, NULL);
 	BookmarkItem *near = controls->selection;
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
+	g_free(utfstr);
 
 	new->time_added = time (NULL);
 
@@ -666,6 +773,11 @@ bookmarks_editor_handle_link_clicked (BookmarksEditorControls *controls,
 		embed_activate_link (embed, &newembed,
 				     controls->selection->url, state);
 	}
+	if ((controls->window) && (controls->window->active_embed == newembed))
+		/* only grab the focus when the bookmark is opened in the 
+		   active embed */
+		embed_grab_focus (controls->window->active_embed);
+
 	/* remember created window if necessary */
 	if (!controls->window && newembed)
 		controls->window = newembed->parent_window;
@@ -679,11 +791,13 @@ bookmarks_editor_go_clicked_cb (GtkButton *button,
 }
 
 gboolean
-bookmarks_editor_copy_button_press_cb (GtkWidget *widget, GdkEventButton *event,
+bookmarks_editor_copy_button_press_cb (GtkWidget *widget, 
+				       GdkEventButton *event,
 				       BookmarksEditorControls *controls)
 {
 	if (event->button == 3) {
-		bookmarks_editor_copy_clicked_cb (GTK_BUTTON (widget), controls);
+		bookmarks_editor_copy_clicked_cb
+			(GTK_BUTTON (widget), controls);
 	}
 	return FALSE;
 }
@@ -698,10 +812,12 @@ bookmarks_editor_copy_clicked_cb (GtkButton *button,
 		destination_menu = 
 			bookmarks_create_copy_menu (controls->selection);
 		gnome_popup_menu_do_popup_modal
-			(GTK_WIDGET (destination_menu), menu_position_under_widget, 
+			(GTK_WIDGET (destination_menu), 
+			 menu_position_under_widget, 
 			 GTK_WIDGET (button), NULL, NULL);
 		gtk_object_unref (GTK_OBJECT (destination_menu));
 		bookmarks_dirty = TRUE;
+		bookmarks_tb_dirty = TRUE;
 	}
 }
 
@@ -712,13 +828,17 @@ bookmarks_editor_ctree_tree_move_cb (GtkCTree *ctree, GtkCTreeNode *node,
 				     BookmarksEditorControls *controls)
 {
 	BookmarkItem *source = gtk_ctree_node_get_row_data (ctree, node);
-	BookmarkItem *new_parent = gtk_ctree_node_get_row_data (ctree, new_parent_node);
-	BookmarkItem *new_sib = gtk_ctree_node_get_row_data (ctree, new_sibling_node);
+	BookmarkItem *new_parent = gtk_ctree_node_get_row_data 
+		(ctree, new_parent_node);
+	BookmarkItem *new_sib = gtk_ctree_node_get_row_data
+		(ctree, new_sibling_node);
 	GList *new_sib_list_node;
 	gint new_pos = 0;
 	
 	g_return_if_fail (source && source->parent);
 
+	bookmarks_toolbars_check_update (source);
+
 	if (!new_parent)
 		new_parent = controls->root_bookmark;
 		
@@ -728,14 +848,18 @@ bookmarks_editor_ctree_tree_move_cb (GtkCTree *ctree, GtkCTreeNode *node,
 	
 	new_sib_list_node = g_list_find (new_parent->list, new_sib);
 	if (new_sib_list_node) {
-		new_pos = g_list_position (new_parent->list, new_sib_list_node);
-		new_parent->list = g_list_insert (new_parent->list, source, new_pos);
+		new_pos = g_list_position (new_parent->list, 
+					   new_sib_list_node);
+		new_parent->list = g_list_insert (new_parent->list, source,
+						  new_pos);
 	} else
 		new_parent->list = g_list_append (new_parent->list, source);
 
 	bookmarks_update_alias (source->parent);
 	bookmarks_editor_place_tree_items (source);
 
+	bookmarks_toolbars_check_update (source);
+
 	/* DEBUG only */
 #if 0
 	g_assert (g_list_find (source->parent->list, source));
@@ -767,7 +891,8 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 	gtk_clist_get_selection_info (GTK_CLIST (widget), event->x, event->y,
 				      &row, &column);
 	controls->last_pressed = gtk_ctree_node_get_row_data 
-		(GTK_CTREE (widget), gtk_ctree_node_nth (GTK_CTREE (widget), row));
+		(GTK_CTREE (widget), gtk_ctree_node_nth (
+			GTK_CTREE (widget), row));
 
 	if (controls->last_pressed == NULL)
 		return FALSE;
@@ -787,7 +912,7 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 			window = g_list_last (all_windows)->data;
 		return_val_if_not_window (window, FALSE);
 
-		if (bi->type == BM_AUTOBOOKMARKS || bi->type == BM_FOLDER)
+		if (BOOKMARK_ITEM_IS_FOLDER (bi))
 		{
 			popup = gtk_menu_new ();
 			bookmarks_create_edit_items (GTK_MENU (popup),
@@ -827,7 +952,8 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 	{
 		/* make sure that this entry is selected */
 		gtk_ctree_select (GTK_CTREE (controls->ctree),
-				  bookmarks_get_tree_item (controls->last_pressed, controls));
+				  bookmarks_get_tree_item (
+					  controls->last_pressed, controls));
 		bookmarks_editor_handle_link_clicked (controls, 
 				mouse_state_to_link_state (event->button,
 							event->state));
@@ -894,8 +1020,9 @@ bookmarks_editor_expand_all_cb (GtkButton *button,
 	gtk_clist_freeze (GTK_CLIST (controls->ctree));
 	for (l = controls->root_bookmark->list; l != NULL; l = l->next) {
 		BookmarkItem *b = l->data;
-		gtk_ctree_expand_recursive (GTK_CTREE (controls->ctree), 
-					    bookmarks_get_tree_item (b, controls));
+		gtk_ctree_expand_recursive 
+			(GTK_CTREE (controls->ctree), 
+			 bookmarks_get_tree_item (b, controls));
 	}
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
@@ -908,8 +1035,9 @@ bookmarks_editor_collapse_all_clicked_cb (GtkButton *button,
 	gtk_clist_freeze (GTK_CLIST (controls->ctree));
 	for (l = controls->root_bookmark->list; l != NULL; l = l->next) {
 		BookmarkItem *b = l->data;
-		gtk_ctree_collapse_recursive (GTK_CTREE (controls->ctree), 
-					      bookmarks_get_tree_item (b, controls));
+		gtk_ctree_collapse_recursive
+			(GTK_CTREE (controls->ctree), 
+			 bookmarks_get_tree_item (b, controls));
 	}
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
@@ -939,14 +1067,18 @@ bookmarks_editor_set_name (BookmarksEditorControls *controls,
 			   BookmarkItem *bookmark, gboolean active)
 {
 	gchar *tmp = NULL;
+	gchar *loc_str = NULL;
 
-	if (bookmark && bookmark->name)
+	if (bookmark && bookmark->name) {
 		tmp = strip_uline_accel (bookmark->name);
+		loc_str = mozilla_utf8_to_locale (tmp);
+	}
 
 	bookmarks_editor_set_text (controls->name_entry, 
-				   bookmark ? tmp : NULL, 
+				   bookmark ? loc_str : NULL, 
 				   active);
-	if (tmp) g_free (tmp);
+	g_free (tmp);
+	g_free (loc_str);
 }
 
 /**
@@ -968,9 +1100,16 @@ static void
 bookmarks_editor_set_nick (BookmarksEditorControls *controls, 
 			   BookmarkItem *bookmark, gboolean active)
 {
+	gchar *loc_str = NULL;
+
+	if (!controls->nick_entry) return;
+
+	if (bookmark)
+		loc_str = mozilla_utf8_to_locale (bookmark->nick);
+
 	bookmarks_editor_set_text (controls->nick_entry, 
-				   bookmark ? bookmark->nick : NULL, 
-				   active);
+				   loc_str, active);
+	g_free (loc_str);
 }
 
 /**
@@ -1037,8 +1176,10 @@ bookmarks_editor_set_notes (BookmarksEditorControls *controls,
 	gtk_editable_delete_text (GTK_EDITABLE (control), 0, -1);
 	if (active && bookmark && bookmark->notes)
 	{
+		gchar *loc_str = mozilla_utf8_to_locale (bookmark->notes);
 		gtk_text_insert (GTK_TEXT (control), NULL, NULL, NULL,
-				 bookmark->notes, strlen (bookmark->notes));
+				 loc_str, strlen (bookmark->notes));
+		g_free (loc_str);
 	}
 
 	/* set widget sensitivity */
@@ -1063,32 +1204,38 @@ bookmarks_editor_set_times (BookmarksEditorControls *controls,
 {
 	gchar *text;
 	if (controls->time_added_entry) {
-		text = bookmark ? time_to_string (bookmark->time_added) : g_strdup ("");
-		gtk_entry_set_text (GTK_ENTRY (controls->time_added_entry), text);
+		text = bookmark ? time_to_string (bookmark->time_added) 
+			: g_strdup ("");
+		gtk_entry_set_text (GTK_ENTRY (controls->time_added_entry),
+				    text);
 		g_free (text);
 	}
 	if (controls->time_modified_entry) {
-		text = bookmarkreal ? time_to_string (bookmarkreal->time_modified) : g_strdup ("");
-		gtk_entry_set_text (GTK_ENTRY (controls->time_modified_entry), text);
+		text = bookmarkreal ? time_to_string
+			(bookmarkreal->time_modified) : g_strdup ("");
+		gtk_entry_set_text (GTK_ENTRY (controls->time_modified_entry),
+				    text);
 		g_free (text);
 	}
 	if (controls->time_visited_entry) {
-		text = bookmarkreal ? time_to_string (bookmarkreal->time_visited) : g_strdup ("");
-		gtk_entry_set_text (GTK_ENTRY (controls->time_visited_entry), text);
+		text = bookmarkreal 
+			? time_to_string (bookmarkreal->time_visited) 
+			: g_strdup ("");
+		gtk_entry_set_text (GTK_ENTRY (controls->time_visited_entry),
+				    text);
 		g_free (text);
 	}
 }
 
 void 
 bookmarks_editor_show_only_this_folder_cb (GtkButton *button,
-				             BookmarksEditorControls *controls) 
+				             BookmarksEditorControls *controls)
 {
 	if (controls->selection) {
 		BookmarkItem *new_root = 
-			((controls->selection->type == BM_FOLDER)
-			 || (controls->selection->type == BM_AUTOBOOKMARKS)) ?
-			controls->selection :
-			controls->selection->parent;
+			BOOKMARK_ITEM_IS_FOLDER (controls->selection)
+			? controls->selection
+			: controls->selection->parent;
 		if (new_root) bookmarks_editor_set_root (controls, new_root);
 			
 	}
@@ -1284,10 +1431,10 @@ bookmarks_editor_set_default_folder_cb (GtkWidget *control,
 	g_return_if_fail (controls);
 
 	if (controls->selection != NULL && 
-	    (controls->selection->type == BM_FOLDER || 
-	     controls->selection->type == BM_AUTOBOOKMARKS))
+	    BOOKMARK_ITEM_IS_FOLDER (controls->selection))
 	{
-		bookmarks_set_as_default_activate_cb (NULL, controls->selection);
+		bookmarks_set_as_default_activate_cb 
+			(NULL, controls->selection);
 	}
 }
 
@@ -1305,7 +1452,8 @@ static void
 bookmarks_editor_toolbar_appearance_menu_cb (GtkRadioMenuItem *menuitem, 
 					     BookmarksEditorControls *controls)
 {
-	GSList *list = g_slist_reverse (g_slist_copy (gtk_radio_menu_item_group (menuitem)));
+	GSList *list = g_slist_reverse
+		(g_slist_copy (gtk_radio_menu_item_group (menuitem)));
 	gint index = g_slist_index (list, menuitem);
 	
 	g_slist_free (list);
@@ -1386,7 +1534,7 @@ bookmarks_editor_toolbar_button_press_event_cb
 
 void
 bookmarks_editor_save_as_clicked_cb (GtkWidget *widget,
-					  BookmarksEditorControls *controls)
+				     BookmarksEditorControls *controls)
 {
 	gchar *file = NULL;
 
@@ -1419,6 +1567,7 @@ bookmarks_editor_open_clicked_cb (GtkWidget *widget,
 	}
 	g_free (file);
 	bookmarks_dirty = TRUE;
+	bookmarks_tb_dirty = TRUE;
 	/* refresh everywhere else */
 	bookmarks_editor_apply (controls);
 }
@@ -1427,37 +1576,53 @@ void
 bookmarks_editor_sort_norec_clicked_cb (GtkWidget *widget, 
 					BookmarksEditorControls *controls)
 {
-	GList *l = bookmarks_get_selections_sorted 
-		(GTK_CLIST (controls->ctree));
-	GList *li;
-	bookmarks_selection_prepare (l, controls);
-	for (li = l; li != NULL; li = li->next) {
-		BookmarkItem *sel = li->data;
-		BookmarkItem *sort_root = 
-			((sel->type == BM_FOLDER)
-			 || (sel->type == BM_AUTOBOOKMARKS)) 
-			? sel 
-			: sel->parent;
-		if (sort_root) bookmarks_sort_norec (sort_root, TRUE);
-	}
-	g_list_free (l);
+	bookmarks_editor_sort_cb_generic (controls, FALSE, FALSE);
 }
 
 void
 bookmarks_editor_sort_rec_clicked_cb (GtkWidget *widget, 
 				      BookmarksEditorControls *controls)
 {
-	GList *l = g_list_copy (GTK_CLIST (controls->ctree)->selection);
+	bookmarks_editor_sort_cb_generic (controls, TRUE, FALSE);
+}
+
+void
+bookmarks_editor_sort_ff_norec_clicked_cb (GtkWidget *widget, 
+					BookmarksEditorControls *controls)
+{
+	bookmarks_editor_sort_cb_generic (controls, FALSE, TRUE);
+}
+
+void
+bookmarks_editor_sort_ff_rec_clicked_cb (GtkWidget *widget, 
+				      BookmarksEditorControls *controls)
+{
+	bookmarks_editor_sort_cb_generic (controls, TRUE, TRUE);
+}
+
+static void
+bookmarks_editor_sort_cb_generic (BookmarksEditorControls *controls, 
+				  gboolean recursive, gboolean folders_first)
+{
+	GList *l = recursive 
+		? g_list_copy (GTK_CLIST (controls->ctree)->selection)
+		: bookmarks_get_selections_sorted (GTK_CLIST(controls->ctree));
 	GList *li;
 	bookmarks_selection_prepare (l, controls);
 	for (li = l; li != NULL; li = li->next) {
 		BookmarkItem *sel = li->data;
-		BookmarkItem *sort_root = 
-			((sel->type == BM_FOLDER)
-			 || (sel->type == BM_AUTOBOOKMARKS)) 
+		BookmarkItem *sort_root = BOOKMARK_ITEM_IS_FOLDER (sel)
 			? sel 
 			: sel->parent;
-		if (sort_root) bookmarks_sort_rec (sort_root, TRUE);
+		if (sort_root) 
+		{
+			if (recursive) 
+				bookmarks_sort_rec (sort_root, folders_first,
+						    TRUE);
+			else
+				bookmarks_sort_norec (sort_root, folders_first,
+						      TRUE);
+		}
 	}
 	g_list_free (l);
 }
@@ -1547,7 +1712,7 @@ void
 bookmarks_dock_menu_button_clicked_cb (GtkButton *button,
 				       BookmarksEditorControls *controls)
 {
-	bookmarks_editor_popup_menu (controls);
+	bookmarks_editor_popup_menu (controls, GTK_WIDGET (button));
 }
 
 
@@ -1557,3 +1722,238 @@ bookmarks_editor_destroy_cb (GtkWidget *widget,
 {
 	bookmarks_editor_hide_dialog (controls);
 }
+
+void
+bookmarks_dock_search_entry_activate_cb (GtkWidget *widget,
+					 BookmarksEditorControls *controls)
+{
+	gchar *text = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
+	gchar *utext = mozilla_locale_to_utf8(text);
+
+	if (bookmarks_editor_search (controls, utext, BOOKMARKS_SEARCH_IN_NAME,
+	    BOOKMARKS_SEARCH_METHOD_CONTAINS, FALSE, FALSE) == FALSE)
+	{
+		bookmarks_editor_search (controls, utext,
+			 		 BOOKMARKS_SEARCH_IN_NAME,
+					 BOOKMARKS_SEARCH_METHOD_CONTAINS,
+					 FALSE, TRUE);
+	}
+
+	gtk_widget_grab_focus (widget);
+
+	g_free (utext);
+	g_free (text);
+}
+
+void
+bookmarks_editor_find_bookmarks_cb (GtkWidget *widget,
+				    BookmarksEditorControls *controls)
+{
+	BookmarksFindDialog *dialog;
+	GladeXML *gxml;
+
+	if (controls->find_dialog)
+	{
+		gdk_window_raise (controls->find_dialog->dialog->window);
+		return;
+	}
+	
+	/* build dialog */
+	dialog = g_new0 (BookmarksFindDialog, 1);
+	gxml = glade_widget_new ("bookmarks.glade", 
+				 "bookmarks_find_dialog",
+				 &(dialog->dialog), dialog);
+	dialog->prev_button           = glade_xml_get_widget (gxml,
+		"bookmarks_find_prev_button");
+	dialog->next_button           = glade_xml_get_widget (gxml,
+		"bookmarks_find_next_button");
+	dialog->criterium1_optionmenu = glade_xml_get_widget (gxml,
+		"bookmarks_find_criterium1_optionmenu");
+	dialog->criterium2_optionmenu = glade_xml_get_widget (gxml,
+		"bookmarks_find_criterium2_optionmenu");
+	dialog->gnomeentry            = glade_xml_get_widget (gxml,
+		"bookmarks_find_gnomeentry");
+	dialog->entry                 = glade_xml_get_widget (gxml,
+		"bookmarks_find_entry");
+
+	g_assert (dialog->prev_button           != NULL);
+	g_assert (dialog->next_button           != NULL);
+	g_assert (dialog->criterium1_optionmenu != NULL);
+	g_assert (dialog->criterium2_optionmenu != NULL);
+	g_assert (dialog->gnomeentry            != NULL);
+	g_assert (dialog->entry                 != NULL);
+
+	/* connect signals */
+	gtk_signal_connect (GTK_OBJECT
+		(GTK_OPTION_MENU (dialog->criterium1_optionmenu)->menu),
+		 "deactivate", bookmarks_find_criterium1_deactivate_cb,
+		 dialog);
+	gtk_signal_connect (GTK_OBJECT
+		(GTK_OPTION_MENU (dialog->criterium2_optionmenu)->menu),
+		 "deactivate", bookmarks_find_criterium2_deactivate_cb,
+		 dialog);
+
+	/* set data */
+	dialog->controls = controls;
+	
+	/* this is done so that the enter key is not captured by the entry */
+	gnome_dialog_editable_enters (GNOME_DIALOG (dialog->dialog),
+				      GTK_EDITABLE (dialog->entry));
+
+	/* parent dialog correctly */
+	if (controls->type == STANDARD_BOOKMARKS_EDITOR)
+		dialog_set_parent (dialog->dialog, controls->dialog);
+	else
+		dialog_set_parent (dialog->dialog, controls->window->WMain);
+
+	/* when docked, make docked search entry unsensitive */
+	/* if (controls->search_entry)
+		gtk_widget_set_sensitive (controls->search_entry, FALSE); */
+
+	/* update controls structure */
+	controls->find_dialog = dialog;
+
+	/* set default values */
+	dialog->str = g_strdup ("");
+	dialog->search_in = BOOKMARKS_SEARCH_IN_NAME;
+	dialog->search_method = BOOKMARKS_SEARCH_METHOD_CONTAINS;
+
+	gtk_object_unref (GTK_OBJECT (gxml));
+}
+
+void
+bookmarks_find_prev_clicked_cb (GtkWidget *widget, BookmarksFindDialog *dialog)
+{
+	bookmarks_find_search (dialog, TRUE, FALSE);
+}
+
+void
+bookmarks_find_next_clicked_cb (GtkWidget *widget, BookmarksFindDialog *dialog)
+{
+	bookmarks_find_search (dialog, FALSE, FALSE);
+}
+
+static gboolean
+bookmarks_find_search (BookmarksFindDialog *dialog, gboolean prev,
+		       gboolean wrap)
+{
+	gboolean found = FALSE;
+	GList *backup = dialog->controls->search_position;
+	gchar *utfstr = mozilla_locale_to_utf8(dialog->str);
+
+	/* do the actual search */
+	found = bookmarks_editor_search (dialog->controls, utfstr,
+					 dialog->search_in,
+					 dialog->search_method, prev, wrap);
+
+	if (found == FALSE)
+	{
+		/* not found, wrap, and try again */
+		found = bookmarks_editor_search (dialog->controls, utfstr,
+					         dialog->search_in,
+					         dialog->search_method,
+						 prev, TRUE);
+	}
+
+	if (found == FALSE)
+	{
+		/* it's really not there; set search pos back to the previous
+		 * search */
+		dialog->controls->search_position = backup;
+	}
+	
+	g_free (utfstr);
+
+	return found;
+}
+
+void
+bookmarks_find_cancel_clicked_cb (GtkWidget *widget,
+				  BookmarksFindDialog *dialog)
+{
+	BookmarksEditorControls *controls = dialog->controls;
+	gtk_widget_destroy (dialog->dialog);
+	g_free (dialog->str);
+	g_free (dialog);
+	controls->find_dialog = NULL;
+	
+	/*if (controls->search_entry)
+		gtk_widget_set_sensitive (controls->search_entry, TRUE);*/
+}
+
+void
+bookmarks_find_entry_changed_cb (GtkEditable *editable,
+				 BookmarksFindDialog *dialog)
+{
+	if (dialog->str) g_free (dialog->str);
+	dialog->str = gtk_editable_get_chars (editable, 0, -1);
+}
+
+void
+bookmarks_find_criterium1_deactivate_cb (GtkMenuShell *menushell,
+					 BookmarksFindDialog *dialog)
+{
+	GtkWidget *active_item;
+
+	/* find which item has been selected */
+	active_item = gtk_menu_get_active (GTK_MENU (menushell));
+	dialog->search_in = g_list_index (GTK_MENU_SHELL (menushell)->children,
+					  active_item);
+}
+
+void
+bookmarks_find_criterium2_deactivate_cb (GtkMenuShell *menushell,
+					 BookmarksFindDialog *dialog)
+{
+	GtkWidget *active_item;
+
+	/* find which item has been selected */
+	active_item = gtk_menu_get_active (GTK_MENU (menushell));
+	dialog->search_method =
+		g_list_index (GTK_MENU_SHELL (menushell)->children,
+			      active_item);
+}
+
+/* not used anymore, but kept in case we want to change the behaviour back */
+#if 0
+static void
+bookmarks_find_update_prevnext_sensitivity (BookmarksFindDialog *dialog)
+{
+	gboolean prev_found, next_found;
+
+	prev_found =
+		bookmarks_find_can_search_prevnext (dialog, TRUE);
+	gtk_widget_set_sensitive (dialog->prev_button, prev_found);
+	next_found =
+	 	bookmarks_find_can_search_prevnext (dialog, FALSE);
+	gtk_widget_set_sensitive (dialog->next_button, next_found);
+}
+
+static gboolean
+bookmarks_find_can_search_prevnext (BookmarksFindDialog *dialog,
+				    gboolean backward)
+{
+	GtkCTree *ctree = GTK_CTREE (dialog->controls->ctree);
+ 	GList *l = backward ?
+		g_list_previous (dialog->controls->search_position) :
+		g_list_next (dialog->controls->search_position);
+
+	if (dialog->controls->search_position == NULL)
+		l = GTK_CLIST (dialog->controls->ctree)->row_list;
+	
+	for (; l; l = backward ? g_list_previous (l) : g_list_next (l))
+	{
+		BookmarkItem *b = gtk_ctree_node_get_row_data (ctree,
+						GTK_CTREE_NODE (l));
+
+		if (bookmarks_editor_search_matches (b, dialog->str,
+		  				     dialog->search_in,
+		   				     dialog->search_method))
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+#endif
