@@ -18,8 +18,11 @@
 
 #include "galeon.h"
 #include "bookmarks.h"
+#include "misc.h"
+#include "mozilla.h"
 
 #include <string.h>
+#include <libgnome/gnome-i18n.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkdnd.h>
 
@@ -39,7 +42,9 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 					      gint x, gint y,
 					      GtkSelectionData *selection_data,
 					      guint info, guint time, 
-					      BookmarksEditorControls *controls);
+					      BookmarksEditorControls 
+					      *controls);
+
 gboolean bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 					     GdkDragContext *context,
 					     gint x,
@@ -53,6 +58,7 @@ void bookmarks_editor_ctree_drag_leave (GtkWidget *widget,
 static gboolean vscroll_timeout (gint direction, BookmarksEditorControls *controls);
 static gboolean vscroll_timeout_up (BookmarksEditorControls *controls);
 static gboolean vscroll_timeout_down (BookmarksEditorControls *controls);
+static void selection_set_string (GtkSelectionData *sd, const gchar *str);
 
 /* Internal helper functions from gtkctree.c */
 #define GTK_CLIST_CLASS_FW(_widget_) \
@@ -159,12 +165,12 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 	drag_target = GTK_CTREE_NODE(g_list_nth (clist->row_list,
 				     new_info.cell.row));
 
-	if (widget == gtk_drag_get_source_widget(context) &&
-	    !check_drag(ctree, drag_source, drag_target, new_info.insert_pos))
+	if (widget == gtk_drag_get_source_widget (context) &&
+	    !check_drag (ctree, drag_source, drag_target, new_info.insert_pos))
 	{
 		if (dest_info->cell.row < 0)
 		{
-			gdk_drag_status(context, GDK_ACTION_DEFAULT, time);
+			gdk_drag_status (context, GDK_ACTION_DEFAULT, time);
 			return FALSE;
 		}
 		return TRUE;
@@ -198,7 +204,8 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 	{
 		if (!clist->vtimer)
 		{
-			DRAW_DRAG_HIGHLIGHT (clist, dest_info);
+			if (dest_info->cell.row != -1)
+				DRAW_DRAG_HIGHLIGHT (clist, dest_info);
 			clist->vtimer = g_timeout_add (50, (GSourceFunc)
 						       vscroll_timeout_down,
 						       controls);
@@ -208,7 +215,8 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 	{
 		if (!clist->vtimer)
 		{
-			DRAW_DRAG_HIGHLIGHT (clist, dest_info);
+			if (dest_info->cell.row != -1)
+				DRAW_DRAG_HIGHLIGHT (clist, dest_info);
 			clist->vtimer = g_timeout_add (50, (GSourceFunc)
 						       vscroll_timeout_up,
 						       controls);
@@ -218,7 +226,8 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 	{
 		if (clist->vtimer)
 		{
-			DRAW_DRAG_HIGHLIGHT (clist, dest_info);
+			if (dest_info->cell.row != -1)
+				DRAW_DRAG_HIGHLIGHT (clist, dest_info);
 			g_source_remove (clist->vtimer);
 			clist->vtimer = 0;
 		}
@@ -246,7 +255,7 @@ vscroll_timeout (gint direction, BookmarksEditorControls *controls)
 
 	g_return_val_if_fail (clist != NULL, FALSE);
 
-	scroll_step = clist->row_height + 1;
+	scroll_step = (clist->row_height / 2) + 1;
 
 	if (direction < 0)
 	{
@@ -287,64 +296,65 @@ bookmarks_editor_ctree_drag_data_get_cb (GtkWidget *widget,
 					 guint info, guint time, 
 					 BookmarksEditorControls *controls)
 {
-	if (controls->last_pressed) {
+	if (controls->last_pressed) 
+	{
 		gchar *mem = NULL;
 		switch (info) {
 		case DND_TARGET_GALEON_BOOKMARK:
-			if (controls->last_pressed) {
-				mem = bookmarks_item_to_string (controls->last_pressed);
-				gtk_selection_data_set 
-					(selection_data, selection_data->target,
-					 8, mem, strlen (mem));
-			}
+			mem = bookmarks_item_to_string 
+				(controls->last_pressed);
+			selection_set_string (selection_data, mem);
 			break;
 		case DND_TARGET_STRING:
-			if (controls->last_pressed->type == BM_FOLDER) {
-				if (controls->last_pressed->name
-				    && (strlen (controls->last_pressed->name) > 0)) {
-					gtk_selection_data_set 
-						(selection_data, selection_data->target,
-						 8, controls->last_pressed->name, 
-						 strlen (controls->last_pressed->name));
-				}
+			if (((controls->last_pressed->type == BM_AUTOBOOKMARKS)
+			     || (controls->last_pressed->type == BM_FOLDER))
+			    && (controls->last_pressed->name)
+			    && (strlen (controls->last_pressed->name) > 0)) 
+			{
+				selection_set_string 
+					(selection_data,
+					 controls->last_pressed->name);
 				break;
-			}
+			} /* if not a folder, pass url instead of name */
 		case DND_TARGET_NETSCAPE_URL:
 		case DND_TARGET_GALEON_URL:
-			if (controls->last_pressed->url 
-			    && (strlen (controls->last_pressed->url) > 0)) {
-				gtk_selection_data_set 
-					(selection_data, selection_data->target,
-					 8, controls->last_pressed->url, 
-					 strlen (controls->last_pressed->url));
+			if ((controls->last_pressed->url)
+			    && (strlen (controls->last_pressed->url) > 0)) 
+			{
+				selection_set_string 
+					(selection_data, 
+					 controls->last_pressed->url);
 			}
 			break;
 		case DND_TARGET_TEXT_URI_LIST:
-			if (controls->last_pressed->type == BM_FOLDER) {
+			if (controls->last_pressed->type == BM_FOLDER) 
+			{
 				GList *l;
 				gchar *text = g_strdup ("");
 				for (l = controls->last_pressed->list; 
 				     l != NULL; 
-				     l = g_list_next(l)) {
+				     l = g_list_next (l)) {
 					BookmarkItem *item = l->data;
-					if (item->url && (strlen (item->url) > 0)) {
+					if (item->url 
+					    && (strlen (item->url) > 0)) 
+					{
 						gchar *prevtext = text;
-						text = g_strconcat (text, item->url, 
-								    "\r\n", NULL);
+						text = g_strconcat (text,
+								    item->url, 
+								    "\r\n",
+								    NULL);
 						g_free (prevtext);
 					}
 				}
-				gtk_selection_data_set 
-					(selection_data, selection_data->target,
-					 8, text, strlen (text));
+				selection_set_string (selection_data, text);
 				g_free (text);
-			} else if (controls->last_pressed->url 
-				   && (strlen (controls->last_pressed->url) > 0)) {
+			} 
+			else if (controls->last_pressed->url) 
+			{
 				gchar *text = g_strconcat 
-					(controls->last_pressed->url, "\r\n", NULL);
-				gtk_selection_data_set 
-					(selection_data, selection_data->target,
-					 8, text, strlen (text));
+					(controls->last_pressed->url, "\r\n",
+					 NULL);
+				selection_set_string (selection_data, text);
 				g_free (text);
 			}
 			break;
@@ -363,58 +373,26 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 					      gint x, gint y,
 					      GtkSelectionData *selection_data,
 					      guint info, guint time, 
-					      BookmarksEditorControls *controls)
+					      BookmarksEditorControls
+					      *controls)
 {
 	GtkCTree *ctree;
 	GtkCList *clist;
-	BookmarkItem *b = NULL;
 	gchar *s, *t;
 	GtkCListDestInfo dest_info;
+	char **tmp;
 	 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_CTREE (widget));
 	g_return_if_fail (context != NULL);
 	g_return_if_fail (selection_data != NULL);
 
-	switch(info)
-	{
-	case DND_TARGET_GALEON_BOOKMARK:
-		b = bookmarks_item_from_string (selection_data->data);
-		break;
-	case DND_TARGET_TEXT_URI_LIST:
-		b = bookmarks_new_bookmark 
-			(BM_FOLDER, TRUE, "New folder", NULL, NULL, NULL, NULL);
-		s = selection_data->data;
-		while ((t = strstr (s, "\r\n"))) {
-			gchar *url = g_strndup (s, t-s);
-			BookmarkItem *b2 = bookmarks_new_bookmark 
-				(BM_SITE, TRUE, NULL, url, NULL, NULL, NULL);
-			b2->parent = b;
-			b->list = g_list_append (b->list, b2);
-			g_free (url);
-			s = t+1;
-		};
-		break;
-	case DND_TARGET_NETSCAPE_URL:
-	case DND_TARGET_GALEON_URL:
-	case DND_TARGET_STRING:
-		b = bookmarks_new_bookmark 
-			(BM_SITE, TRUE, NULL, selection_data->data, NULL, NULL, NULL);
-		break;
-	default:
-		g_warning ("Unknown DND type");
-		break;
-	}
-
-	if (!b)
-		return;
-
 	ctree = GTK_CTREE (widget);
 	clist = GTK_CLIST (widget);
-
+	
 	/* Get the destination node info */
 	drag_dest_cell (clist, x, y, &dest_info);
-
+	
 	/* use the saved insert position, since the mouse position
 	   may have moved after the button was released */
 	dest_info.insert_pos = insert_pos;
@@ -427,30 +405,27 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 
 		if (clist->click_cell.row < 0)
 		{
-			bookmarks_free_bookmark (b);
 			return;
 		}
 
-		selections_sorted = bookmarks_get_selections_sorted(clist);
+		selections_sorted = bookmarks_get_selections_sorted (clist);
 
 		/* move the first selection to the insertion point */
 
-		source_node = GTK_CTREE_NODE(selections_sorted->data);
-		dest_node = GTK_CTREE_NODE(g_list_nth(clist->row_list,
-					   dest_info.cell.row));
+		source_node = GTK_CTREE_NODE (selections_sorted->data);
+		dest_node = GTK_CTREE_NODE (g_list_nth (clist->row_list,
+							dest_info.cell.row));
 
-		if (!source_node || !dest_node)
+		if (!source_node || !dest_node
+		    || !check_drag (ctree, source_node, dest_node,
+				    dest_info.insert_pos))
 		{
-			bookmarks_free_bookmark (b);
-			return;
-		}
-		if (!check_drag(ctree, source_node, dest_node,
-				dest_info.insert_pos))
-		{
-			bookmarks_free_bookmark (b);
 			return;
 		}
 
+		/* Just moving the tree nodes is enough.
+		   bookmarks_editor_ctree_tree_move_cb will take care of 
+		   everything */
 		switch (dest_info.insert_pos)
 		{
 		case GTK_CLIST_DRAG_NONE:
@@ -480,7 +455,6 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 
 			if (!source_node || !dest_node)
 			{
-				bookmarks_free_bookmark (b);
 				return;
 			}
 
@@ -490,20 +464,74 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 			selections = selections->next;
 		}
 		g_list_free(selections_sorted);
-		/* we just moved the clist around, and did not acually use this
-		   atleast I think so =) MattM */
-		bookmarks_free_bookmark (b);
 	}
 	else
 	{
+		BookmarkItem *b = NULL;
 		BookmarkItem *near = NULL;
 		gint row = -1;
+		gchar* utfstr;
+		
+		switch(info)
+		{
+		case DND_TARGET_GALEON_BOOKMARK:
+			b = bookmarks_item_from_string (selection_data->data);
+			break;
+		case DND_TARGET_TEXT_URI_LIST:
+			utfstr = mozilla_locale_to_utf8(_("New folder"));
+			b = bookmarks_new_bookmark 
+				(BM_FOLDER, TRUE, utfstr, NULL,
+				 NULL, NULL, NULL);
+			g_free(utfstr);
+			s = selection_data->data;
+			while ((t = strstr (s, "\r\n"))) {
+				gchar *url = g_strndup (s, t-s);
+				BookmarkItem *b2 = bookmarks_new_bookmark 
+					(BM_SITE, TRUE, NULL, url, NULL,
+					 NULL, NULL);
+				b2->parent = b;
+				b->list = g_list_append (b->list, b2);
+				g_free (url);
+				s = t+1;
+			};
+			break;
+		case DND_TARGET_NETSCAPE_URL:
+			tmp = g_strsplit ((gchar *) selection_data->data, 
+					  "\n", 2);
+			if (tmp && tmp[0])
+			{
+				b = bookmarks_new_bookmark (BM_SITE, TRUE,
+							    tmp[1], tmp[0], 
+							    NULL, NULL, NULL);
+			}
+			else
+			{
+				b = bookmarks_new_bookmark 
+					(BM_SITE, TRUE, NULL,
+					 (gchar *) selection_data->data,
+					 NULL, NULL, NULL);
+			}
+			g_strfreev (tmp);
+			break;
+		case DND_TARGET_GALEON_URL:
+		case DND_TARGET_STRING:
+			b = bookmarks_new_bookmark 
+				(BM_SITE, TRUE, NULL, 
+				 selection_data->data, NULL, NULL, NULL);
+			break;
+		default:
+			g_warning ("Unknown DND type");
+			break;
+		}
+		
+		if (!b)
+			return;
 
 		near = gtk_ctree_node_get_row_data 
 			(GTK_CTREE (controls->ctree), gtk_ctree_node_nth
 			 (GTK_CTREE (widget), dest_info.cell.row));
 		if (!near) near = controls->root_bookmark;
-
+		
 		/* this happens if the bookmarks clist is empty */
 		if (dest_info.insert_pos == GTK_CLIST_DRAG_NONE)
 			dest_info.insert_pos = GTK_CLIST_DRAG_INTO;
@@ -511,23 +539,33 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 		bookmarks_insert_bookmark (b, near, dest_info.insert_pos);
 		bookmarks_update_alias (b->parent);
 		bookmarks_editor_place_tree_items (b);
-
+		bookmarks_toolbars_check_update (b);
+		bookmarks_toolbars_check_update (near);
+		
 		/* set the focus so it matches the selection */
 		if (gtk_ctree_node_nth (GTK_CTREE (controls->ctree),
 				        dest_info.cell.row)
 		    != bookmarks_get_tree_item (b, controls))
 		{
-			if (GTK_CTREE_NODE_NEXT (bookmarks_get_tree_item (b, controls)) == NULL)
-				row = g_list_length(g_list_first(
-				                    (GList *) bookmarks_get_tree_item (b, controls))) - 1;
+			if (GTK_CTREE_NODE_NEXT (bookmarks_get_tree_item 
+						 (b, controls)) == NULL)
+				row = g_list_length
+					(g_list_first
+					 ((GList *) bookmarks_get_tree_item
+					  (b, controls))) - 1;
 			else
-				row = g_list_position(g_list_first(
-						      (GList *)bookmarks_get_tree_item (b, controls)),
-						      (GList *)bookmarks_get_tree_item (b, controls));
+				row = g_list_position
+					(g_list_first
+					 ((GList *)bookmarks_get_tree_item 
+					  (b, controls)),
+					 (GList *) bookmarks_get_tree_item
+					 (b, controls));
 		}
 		if (row >= -1)
 			GTK_CLIST(controls->ctree)->focus_row = row;
-		gtk_ctree_select (GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls));
+		gtk_ctree_select (GTK_CTREE (controls->ctree),
+				  bookmarks_get_tree_item (b, controls));
+		bookmarks_toolbars_check_update (b);
 	}
 	bookmarks_dirty = TRUE;
 }
@@ -572,7 +610,8 @@ drag_dest_cell (GtkCList         *clist,
 	if (dest_info->cell.row >= clist->rows)
 	{
 		dest_info->cell.row = clist->rows - 1;
-		y = ROW_TOP_YPIXEL(clist, dest_info->cell.row) + clist->row_height;
+		y = ROW_TOP_YPIXEL(clist, dest_info->cell.row) 
+			+ clist->row_height;
 	}
 	if (dest_info->cell.row < -1)
 		dest_info->cell.row = -1;
@@ -632,33 +671,42 @@ check_drag (GtkCTree        *ctree,
 			return FALSE;
 		case GTK_CLIST_DRAG_AFTER:
 			if (GTK_CTREE_ROW(drag_target)->sibling != drag_source)
-				return (!ctree->drag_compare ||
-					ctree->drag_compare(
-					  ctree,
-					  drag_source,
-					  GTK_CTREE_ROW(drag_target)->parent,
-					  GTK_CTREE_ROW(drag_target)->sibling));
+				return (!ctree->drag_compare 
+					|| ctree->drag_compare
+					(ctree,
+					 drag_source,
+					 GTK_CTREE_ROW(drag_target)->parent,
+					 GTK_CTREE_ROW(drag_target)->sibling));
 			break;
 		case GTK_CLIST_DRAG_BEFORE:
-			if (GTK_CTREE_ROW (drag_source)->sibling != drag_target)
-				return (!ctree->drag_compare ||
-		 			ctree->drag_compare(
-		 			  ctree,
-					  drag_source,
-					  GTK_CTREE_ROW(drag_target)->parent,
-					  drag_target));
+			if (GTK_CTREE_ROW (drag_source)->sibling 
+			    != drag_target)
+				return (!ctree->drag_compare 
+					|| ctree->drag_compare
+					(ctree,
+					 drag_source,
+					 GTK_CTREE_ROW(drag_target)->parent,
+					 drag_target));
 			break;
 		case GTK_CLIST_DRAG_INTO:
-			if (!GTK_CTREE_ROW (drag_target)->is_leaf &&
-			    GTK_CTREE_ROW (drag_target)->children != drag_source)
-				return (!ctree->drag_compare ||
-					ctree->drag_compare(
-					  ctree,
-					  drag_source,
-					  drag_target,
-					  GTK_CTREE_ROW (drag_target)->children));
+			if (!GTK_CTREE_ROW (drag_target)->is_leaf 
+			    && GTK_CTREE_ROW (drag_target)->children 
+			    != drag_source)
+				return (!ctree->drag_compare
+					|| ctree->drag_compare
+					(ctree,
+					 drag_source,
+					 drag_target,
+					 GTK_CTREE_ROW 
+					 (drag_target)->children));
 			break;
 		}
 	}
 	return FALSE;
 }
+
+static void 
+selection_set_string (GtkSelectionData *sd, const gchar *str)
+{
+	gtk_selection_data_set (sd, sd->target, 8, str, strlen (str));
+}
