@@ -30,9 +30,11 @@
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gtkpixmapmenuitem.h>
+#include <gdk/gdkkeysyms.h>
 
 static void bookmarks_menu_destroy (GaleonWindow *window);
-static GtkWidget *bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu);
+static GtkWidget *bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
+					      gboolean lock_accels);
 static void bookmarks_create_file_menu (GtkMenuItem *file_menuitem,
 					BookmarkItem *bookmark);
 static void bookmarks_create_file_submenu (GtkMenuItem *file_menuitem, 
@@ -53,6 +55,12 @@ static gint bookmarks_smart_bm_dialog_multi (GtkWidget *dialog,
 					     BookmarkItem *bi,
 					     LinkState state);
 static gboolean bookmark_has_subcategories (BookmarkItem *b);
+static void bookmarks_menu_sync_accel_labels_item (GtkMenuItem *item);
+static void bookmarks_init_menu_accels_recursively (BookmarkItem *b,
+						    GaleonWindow *window);
+
+/* when not zero, accel update signals will be ignored */
+gint bookmarks_block_accel_update = 0;
 
 /**
  * bookmarks_menu_create: create the bookmarks menu for a single window
@@ -62,15 +70,22 @@ bookmarks_menu_create (GaleonWindow *window)
 {
 	GtkMenuItem *add_menuitem;
 	GtkMenu *menu;
-	
+
 	return_if_not_window (window);
 
-	/* skip if no menu on this window */
-	if (!(window->menuBarOn))
+	/* free dummywidgets */
+	if (window->dummy_widgets) 
 	{
-		return;
+		/* free dummywidgets */
+		g_list_foreach (window->dummy_widgets, 
+				(GFunc)gtk_widget_unref, NULL);
+		g_list_free (window->dummy_widgets);
 	}
 
+	/* setup bookmark accel dummywidgets (also do this when no menubar) */
+	window->dummy_widgets = NULL;
+	bookmarks_init_menu_accels_recursively (bookmarks_root, window);
+
 	menu = (GtkMenu *) window->bookmarks_menu;
 	add_menuitem = (GtkMenuItem *) window->add_bookmark;
 
@@ -78,7 +93,7 @@ bookmarks_menu_create (GaleonWindow *window)
 	bookmarks_create_file_submenu (add_menuitem, bookmarks_root);
 	bookmarks_menu_create_recursively (bookmarks_root, menu,
 		add_menuitem, GTK_TOOLTIPS(window->bookmarks_tooltips), 
-		TRUE, FALSE);
+		TRUE, FALSE, FALSE);
 }
 
 /**
@@ -98,12 +113,6 @@ bookmarks_menu_destroy (GaleonWindow *window)
 	GtkWidget *separator;
 	GList *li, *l;
 	
-	/* skip if no menu on this window */
-	if (!(window->menuBarOn))
-	{
-		return;
-	}
-	
 	separator = window->bookmarks_separator;
 	bookmarks_menu = window->bookmarks_menu;
 
@@ -122,17 +131,17 @@ bookmarks_menu_destroy (GaleonWindow *window)
 	for (li = li->next; li != NULL; li = li->next)
 	{
 		if (GTK_IS_WIDGET (li->data))
-		{
-			gtk_widget_destroy (GTK_WIDGET (li->data));
-		}
+			gtk_container_remove (GTK_CONTAINER (bookmarks_menu),
+				GTK_WIDGET (li->data));
 	}
 
 	/* destroy old tooltip */
 	if (window->bookmarks_tooltips)
 	{
 		gtk_object_destroy (GTK_OBJECT (window->bookmarks_tooltips));
+		window->bookmarks_tooltips = NULL;
 	}
-	
+
 	g_list_free (l);
 }
 
@@ -152,7 +161,8 @@ void
 bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu, 
 				   GtkMenuItem *file_menuitem,
 				   GtkTooltips *tooltips, gboolean options, 
-				   gboolean disable_tearoffs)
+				   gboolean disable_tearoffs,
+				   gboolean lock_accels)
 {
 	GList *l; 
 	GtkWidget *mi;
@@ -161,7 +171,12 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 
 	g_return_if_fail (bookmark->type == BM_FOLDER || 
 			  bookmark->type == BM_AUTOBOOKMARKS);
+
+	gtk_object_set_data (GTK_OBJECT (menu), "disable_tearoffs",
+			     GINT_TO_POINTER (disable_tearoffs));
 	
+	bookmarks_block_accel_update++;
+
 	if (file_menuitem != NULL) 
 		bookmarks_create_file_menu (file_menuitem, bookmark);
 			
@@ -175,21 +190,24 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 		else if (bookmark == bookmarks_root &&
 				opts_location == EXTRA_ITEM_LOCATION_CONTEXT)
 		{
-			mi = bookmarks_menu_create_item (bookmark, menu);
-			gtk_signal_connect (GTK_OBJECT(mi),
-					    "button-press-event",
-					    GTK_SIGNAL_FUNC (
-					     bookmarks_button_press_event_cb),
-					    bookmark);
-			gtk_signal_connect (GTK_OBJECT(mi),
-					    "button-release-event",
-					    GTK_SIGNAL_FUNC (
-					     bookmarks_button_release_event_cb),
-					    bookmark);
+			mi = bookmarks_menu_create_item (bookmark, menu,
+							 lock_accels);
+			gtk_signal_connect 
+				(GTK_OBJECT(mi),
+				 "button-press-event",
+				 GTK_SIGNAL_FUNC (
+					 bookmarks_button_press_event_cb),
+				 bookmark);
+			gtk_signal_connect 
+				(GTK_OBJECT(mi),
+				 "button-release-event",
+				 GTK_SIGNAL_FUNC (
+					 bookmarks_button_release_event_cb),
+				 bookmark);
 			gtk_widget_show (mi);
 			gtk_menu_append (menu, mi);
 			mi = gtk_menu_item_new ();
-			gtk_widget_set_sensitive(mi, FALSE);
+			gtk_widget_set_sensitive (mi, FALSE);
 			gtk_widget_show (mi);
 			gtk_menu_append (menu, mi);
 		}
@@ -199,14 +217,19 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 	{
 		BookmarkItem *b = l->data;
 		GtkWidget *submenu;
+		gchar *loc_str;
 
 		switch (b->type)
 		{
 		case BM_FOLDER:
 		case BM_AUTOBOOKMARKS:
 			submenu = gtk_menu_new ();
-			gtk_menu_set_title (GTK_MENU (submenu), b->name);
-			mi = bookmarks_menu_create_item (b, menu);
+			gtk_menu_set_accel_group (GTK_MENU (submenu),
+						  menu->accel_group);
+			loc_str = mozilla_utf8_to_locale (b->name);
+			gtk_menu_set_title (GTK_MENU (submenu), loc_str);
+			g_free (loc_str);
+			mi = bookmarks_menu_create_item (b, menu, lock_accels);
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), submenu);
 			gtk_signal_connect 
 				(GTK_OBJECT (submenu), "button-press-event", 
@@ -216,21 +239,28 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 				(GTK_OBJECT (submenu), "button-release-event", 
 				 GTK_SIGNAL_FUNC (
 					bookmarks_button_release_event_cb), b);
-
+			
 			gtk_object_set_data (GTK_OBJECT (mi), "bookmark", b);
-			gtk_object_set_data (GTK_OBJECT (mi), "submenu", submenu);
-			gtk_object_set_data (GTK_OBJECT (mi), "tooltips", tooltips);
+			gtk_object_set_data (GTK_OBJECT (mi), "submenu",
+					     submenu);
+			gtk_object_set_data (GTK_OBJECT (mi), "tooltips",
+					     tooltips);
 			gtk_object_set_data (GTK_OBJECT (mi), "options", 
 					     GINT_TO_POINTER (options));
-			gtk_object_set_data (GTK_OBJECT (mi), "disable_tearoffs", 
-					     GINT_TO_POINTER (disable_tearoffs));
+			gtk_object_set_data (GTK_OBJECT (mi), 
+					     "disable_tearoffs", 
+					     GINT_TO_POINTER (
+						     disable_tearoffs));
+			gtk_object_set_data (GTK_OBJECT (mi), "lock_accels", 
+					     GINT_TO_POINTER (lock_accels));
 			
-			gtk_signal_connect (GTK_OBJECT (mi), "map", 
-					    bookmarks_menu_create_recursively_cb,
-					    NULL);
+			gtk_signal_connect 
+				(GTK_OBJECT (mi), "map", 
+				 bookmarks_menu_create_recursively_cb,
+				 NULL);
 			break;
 		case BM_SITE:
-			mi = bookmarks_menu_create_item (b, menu);
+			mi = bookmarks_menu_create_item (b, menu, lock_accels);
 			gtk_signal_connect_after
 				(GTK_OBJECT (mi), "button_press_event",
 				 GTK_SIGNAL_FUNC (
@@ -238,18 +268,20 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 			gtk_signal_connect_after
 				(GTK_OBJECT (mi), "button_release_event", 
 				 GTK_SIGNAL_FUNC (
-					 bookmarks_button_release_event_cb), b);
+					 bookmarks_button_release_event_cb),
+				 b);
 			gtk_signal_connect_after
 				(GTK_OBJECT (mi), "activate", 
 				 GTK_SIGNAL_FUNC (bookmarks_activate_cb), b);
 			if (tooltips)
-				gtk_tooltips_set_tip (tooltips, GTK_WIDGET (mi),
-						      b->url, NULL);
+				gtk_tooltips_set_tip
+					(tooltips, GTK_WIDGET (mi),
+					 b->url, NULL);
 			break;
 		case BM_SEPARATOR:
 		default:
 			mi = gtk_menu_item_new ();
-			gtk_widget_set_sensitive(mi, FALSE);
+			gtk_widget_set_sensitive (mi, FALSE);
 			break;
 		}
 		gtk_widget_show (mi);
@@ -273,6 +305,8 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 					bookmarks_button_release_event_cb),
 				    NULL);
 	}
+
+	bookmarks_block_accel_update--;
 }
 
 static void
@@ -281,6 +315,8 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 	GList *l;
 	GtkWidget *file_menu;
 
+	bookmarks_block_accel_update++;
+
 	if (!GTK_MENU_ITEM (file_menuitem)->submenu) {
 		gtk_signal_disconnect_by_data
 			(GTK_OBJECT (file_menuitem),
@@ -289,22 +325,24 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 			(file_menuitem, bookmark);
 	}
 	
-	file_menu = 
-		GTK_MENU_ITEM (file_menuitem)->submenu;
+	file_menu = GTK_MENU_ITEM (file_menuitem)->submenu;
 	
 	for (l = bookmark->list; l ; l = l->next) {
 		BookmarkItem *b = l->data;
 		if ((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) {
 			GtkWidget *file = bookmarks_menu_create_item (b,
-							GTK_MENU (file_menu));
+							GTK_MENU (file_menu),
+							FALSE);
 			gtk_widget_show (file);
 			gtk_menu_append (GTK_MENU (file_menu), file);
 			if (bookmark_has_subcategories (b)) {
-				bookmarks_create_file_submenu (GTK_MENU_ITEM (file), b);
-				gtk_signal_connect (GTK_OBJECT(file),
-						    "map",
-						    bookmarks_create_file_menu_cb,
-						    b);
+				bookmarks_create_file_submenu
+					(GTK_MENU_ITEM (file), b);
+				gtk_signal_connect
+					(GTK_OBJECT(file),
+					 "map",
+					 bookmarks_create_file_menu_cb,
+					 b);
 			} else {
 				gtk_signal_connect (GTK_OBJECT(file),
 						    "activate",
@@ -319,33 +357,41 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 					 GTK_WIDGET (mi));
 		}
 	}
+	gtk_menu_reposition (GTK_MENU (file_menu));
+
+	bookmarks_block_accel_update--;
+
 }
 
 static void
 bookmarks_menu_create_recursively_cb (GtkMenuItem *mi, gpointer data)
 {
 	BookmarkItem *b = gtk_object_get_data (GTK_OBJECT (mi), "bookmark");
-	GtkMenu *submenu = gtk_object_get_data (GTK_OBJECT (mi), "submenu");
+	GtkWidget *submenu = gtk_object_get_data (GTK_OBJECT (mi), "submenu");
 	GtkTooltips *tooltips = gtk_object_get_data (GTK_OBJECT (mi), "tooltips");
 	gboolean options = 
 		GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "options"));
 	gboolean disable_tearoffs = 
 		GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), 
 						      "disable_tearoffs"));
+	gboolean lock_accels =
+		GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), 
+						      "lock_accels"));
+	
+	bookmarks_block_accel_update++;
 
 	gtk_signal_disconnect_by_func (GTK_OBJECT (mi),  
-			    bookmarks_menu_create_recursively_cb,
-			    NULL);
-
-	gtk_object_remove_data (GTK_OBJECT (mi), "bookmark");
-	gtk_object_remove_data (GTK_OBJECT (mi), "submenu");
-	gtk_object_remove_data (GTK_OBJECT (mi), "tooltips");
-	gtk_object_remove_data (GTK_OBJECT (mi), "options");
-	gtk_object_remove_data (GTK_OBJECT (mi), "disable_tearoffs");
-
+				       bookmarks_menu_create_recursively_cb,
+				       NULL);
+	
 	bookmarks_menu_create_recursively (b, GTK_MENU (submenu), NULL, 
-					   tooltips, options, disable_tearoffs);
+					   tooltips, options,
+					   disable_tearoffs,
+					   lock_accels);
 
+	gtk_menu_reposition (GTK_MENU (submenu));
+
+	bookmarks_block_accel_update--;
 }
 
 static void
@@ -354,7 +400,9 @@ bookmarks_create_file_menu_cb (GtkMenuItem *mi, BookmarkItem *b)
 	gtk_signal_disconnect_by_func (GTK_OBJECT (mi),  
 			    bookmarks_create_file_menu_cb,
 			    b);
+	bookmarks_block_accel_update++;
 	bookmarks_create_file_menu (mi, b);
+	bookmarks_block_accel_update--;
 }
 
 static gboolean 
@@ -389,6 +437,10 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 	if (opts_location == EXTRA_ITEM_LOCATION_TOP_SUBMENU || 
 			opts_location == EXTRA_ITEM_LOCATION_BOTTOM_SUBMENU)
 	{
+		gboolean disable_tearoffs = 
+			GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (menu),
+						      "disable_tearoffs"));
+
 		extra_menu = GTK_MENU (gtk_menu_new ());
 		extra_menu_item = gtk_menu_item_new_with_label ("");
 		label_set_accel_text (_("_Folder actions"), 
@@ -397,7 +449,8 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (extra_menu_item),
 				GTK_WIDGET (extra_menu));
 
-		if (gnome_preferences_get_menus_have_tearoff ()){
+		if (gnome_preferences_get_menus_have_tearoff () &&
+		    !disable_tearoffs){
 			GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
 			gtk_menu_prepend (extra_menu, tearoff);
 			gtk_widget_show (tearoff);
@@ -434,28 +487,59 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 void
 bookmarks_menu_create_base_extra_items (GtkMenu *menu, BookmarkItem *bookmark)
 {
+	GList *item;
 	context_menu_add_item (menu,
 			_("Add _bookmark here"),
 			bookmarks_add_activate_cb, bookmark, NULL, TRUE);
+	context_menu_add_item (menu,
+			_("Add _folder here"),
+			bookmarks_add_folder_activate_cb, bookmark, NULL,
+			TRUE);
 	context_menu_add_item (menu,
 			_("_Open whole folder in tabs"),
 			bookmarks_open_all_activate_cb, bookmark, NULL, TRUE);
+	
+	/* connect release signal for catching middle mousebutton for
+	 * opening in new tab */
+	item = g_list_nth (GTK_MENU_SHELL (menu)->children,
+		           g_list_length (GTK_MENU_SHELL (menu)->children) - 1);
+	gtk_signal_connect (GTK_OBJECT (item->data), "button-release-event",
+			    GTK_SIGNAL_FUNC (bookmarks_open_all_release_cb),
+			    bookmark);
+	
 	context_menu_add_item (menu,
 			_("Open whole folder in _windows"),
 			bookmarks_open_all_in_wins_activate_cb, bookmark, 
 			NULL, TRUE);
+	
+	/* connect release signal for catching middle mousebutton for
+	 * opening in new window */
+	item = g_list_nth (GTK_MENU_SHELL (menu)->children,
+		           g_list_length (GTK_MENU_SHELL (menu)->children) - 1);
+	gtk_signal_connect (GTK_OBJECT (item->data), "button-release-event",
+			    GTK_SIGNAL_FUNC
+			   	(bookmarks_open_all_in_wins_release_cb),
+			    bookmark);
+	
 	context_menu_add_item (menu, 
 			_("Open folder in My _Portal"),
 			bookmarks_open_in_myportal_cb, bookmark,
 			NULL, TRUE);
 	context_menu_add_check_item (menu,
-			_("_Create toolbar"),
+			_("_Show as toolbar"),
 			bookmarks_create_toolbar_activate_cb, bookmark,
 			bookmark->create_toolbar);
 	context_menu_add_item (menu,
 			_("_Set as default folder"),
 			bookmarks_set_as_default_activate_cb, bookmark,
 			NULL, !(default_bookmarks_root == bookmark));
+	if (bookmark->type == BM_AUTOBOOKMARKS)
+	{
+		context_menu_add_item (menu,
+			_("Regenerate _autobookmarks"),
+			bookmarks_regenerate_autobookmarks_cb, bookmark,
+			NULL, TRUE);
+	}
 }
 
 /**
@@ -501,8 +585,15 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 {
 	GtkWidget *w;
 	GtkWidget *file_menu = gtk_menu_new();
+	gchar *loc_str;
+	gchar *stripped;
+
+	loc_str = mozilla_utf8_to_locale (bookmark->name);
+	stripped = strip_uline_accel (loc_str);
+	g_free (loc_str);
+
+	bookmarks_block_accel_update++;
 
-	gchar *stripped = strip_uline_accel (bookmark->name);
 	gtk_menu_set_title (GTK_MENU (file_menu), stripped);
 	g_free(stripped);
 
@@ -510,7 +601,7 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 		(GTK_MENU_ITEM (file_menuitem),
 		 file_menu);
 
-	w = bookmarks_menu_create_item (bookmark, GTK_MENU (file_menu));
+	w = bookmarks_menu_create_item (bookmark, GTK_MENU (file_menu), FALSE);
 	gtk_signal_connect (GTK_OBJECT (w), "activate", 
 			    bookmarks_file_bookmark_cb, bookmark);
 	gtk_widget_show (w);
@@ -526,6 +617,9 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 		gtk_menu_prepend (GTK_MENU (file_menu), tearoff);
 		gtk_widget_show (tearoff);
 	}
+
+	bookmarks_block_accel_update--;
+
 }
 
 /* FIXME: obsoleted
@@ -540,7 +634,8 @@ bookmarks_create_copy_menu (BookmarkItem *source)
 						      bookmarks_root);
 	if (! res) {
 		BookmarkItem *item = bookmarks_root;
-		GtkWidget *menu_item = bookmarks_menu_create_item (item, res);
+		GtkWidget *menu_item = bookmarks_menu_create_item (item, res,
+				  				   FALSE);
 		res = GTK_MENU (gtk_menu_new ());
 		gtk_widget_show (menu_item);
 		gtk_menu_prepend (res, menu_item);
@@ -550,7 +645,8 @@ bookmarks_create_copy_menu (BookmarkItem *source)
 					  source);
 	} else {
 		BookmarkItem *item = bookmarks_root;
-		GtkWidget *menu_item = bookmarks_menu_create_item (item, res);
+		GtkWidget *menu_item = bookmarks_menu_create_item (item, res,
+								   FALSE);
 		GtkWidget *sep = gtk_menu_item_new ();
 		gtk_widget_set_sensitive(sep, FALSE);
 		gtk_widget_show (menu_item);
@@ -576,29 +672,40 @@ bookmarks_create_copy_menu_recursively (BookmarkItem *source, BookmarkItem *b)
 		BookmarkItem *item = l->data;
 		GtkWidget *menu_item, *sub_menu_item, *sep;
 		if (item->type == BM_FOLDER && !(item->alias_of)) {
-			GtkMenu *submenu = bookmarks_create_copy_menu_recursively (source, item);
+			GtkMenu *submenu = bookmarks_create_copy_menu_recursively 
+				(source, item);
 			has_sub_categories = TRUE;
-			sub_menu_item = bookmarks_menu_create_item (item, res);
+			sub_menu_item = bookmarks_menu_create_item (item, res, 
+								    FALSE);
 			if (submenu) {
 				gtk_menu_item_set_submenu 
 					(GTK_MENU_ITEM (sub_menu_item), 
 					 GTK_WIDGET (submenu));
-				menu_item = bookmarks_menu_create_item (item, submenu);
+				menu_item = bookmarks_menu_create_item 
+					(item, submenu, FALSE);
 				gtk_widget_show (menu_item);
 				sep = gtk_menu_item_new ();
 				gtk_widget_set_sensitive(sep, FALSE);
 				gtk_widget_show (sep);
 				gtk_menu_prepend (submenu, sep);
 				gtk_menu_prepend (submenu, menu_item);
-				gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-						    bookmarks_editor_bookmark_copy_cb, item);
-				gtk_object_set_user_data (GTK_OBJECT (menu_item), 
-							  source);
+				gtk_signal_connect 
+					(GTK_OBJECT (menu_item),
+					 "activate",
+					 bookmarks_editor_bookmark_copy_cb, 
+					 item);
+				gtk_object_set_user_data 
+					(GTK_OBJECT (menu_item), 
+					 source);
 			} else {
-				gtk_signal_connect (GTK_OBJECT (sub_menu_item), 
-						    "activate", bookmarks_editor_bookmark_copy_cb, item);
-				gtk_object_set_user_data (GTK_OBJECT (sub_menu_item), 
-							  source);
+				gtk_signal_connect 
+					(GTK_OBJECT (sub_menu_item), 
+					 "activate", 
+					 bookmarks_editor_bookmark_copy_cb,
+					 item);
+				gtk_object_set_user_data 
+					(GTK_OBJECT (sub_menu_item), 
+					 source);
 			}
 			gtk_widget_show (sub_menu_item);
 			gtk_menu_append (res, sub_menu_item);
@@ -620,25 +727,30 @@ bookmarks_create_copy_menu_recursively (BookmarkItem *source, BookmarkItem *b)
  */
 /* This may be improved */
 static GtkWidget *
-bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
+bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
+			    gboolean lock_accels)
 {
+	gchar *loc_str;
 	GtkWidget *pixmap;
 	const PixmapData *bm_icon;
 	
 	GtkWidget *m = gtk_pixmap_menu_item_new ();
 	GtkWidget *l;
-	GtkWidget *hb = gtk_hbox_new(FALSE, 0);
+	GtkWidget *hb = gtk_hbox_new (FALSE, 0);
 
 	g_return_val_if_fail (bm->name != NULL, NULL);
 
+	bookmarks_block_accel_update++;
+
 	if (bm->pixmap_data == NULL || bm->pixmap_data->pixmap == NULL)
 	{
+		loc_str = mozilla_utf8_to_locale (bm->name);
 		l = gtk_label_new ("");
-		if (strlen (bm->name) > BOOKMARKS_MENU_MAX_LENGTH)
+		if (strlen (loc_str) > BOOKMARKS_MENU_MAX_LENGTH)
 		{
 			gchar *s, *t;
 
-			s = g_strndup(bm->name, BOOKMARKS_MENU_MAX_LENGTH);
+			s = g_strndup(loc_str, BOOKMARKS_MENU_MAX_LENGTH);
 			t = g_strconcat (s, "...", NULL);
 			label_set_accel_text (t, l, GTK_WIDGET (menu), m);
 			g_free (t);
@@ -646,9 +758,11 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
 		} 
 		else
 		{
-			label_set_accel_text (bm->name, l, GTK_WIDGET (menu), m);
+			label_set_accel_text
+				(loc_str, l, GTK_WIDGET (menu), m);
 		}
 		gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 2);
+		g_free (loc_str);
 	}
 	else
 	{
@@ -670,19 +784,65 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
 	}
 	else
 	{
+		GtkWidget *accel_label;
+		gchar *str;
+
+		if (lock_accels == FALSE)
+		{
+			/* as GtkAccelLabels don't seem to work properly, we 
+			 * use a standard label and our own string creation 
+			 * function */
+			accel_label = gtk_label_new ("");
+			gtk_box_pack_end (GTK_BOX (hb), accel_label, FALSE,
+					  FALSE, 2);
+
+			/* set object data */
+			gtk_object_set_data (GTK_OBJECT (m), "accel_label",
+					     accel_label);
+		
+		
+			/* connect accel signals */
+			gtk_signal_connect_after (GTK_OBJECT (m), 
+						  "add_accelerator",
+					          bookmarks_add_accel_cb, bm);
+			gtk_signal_connect (GTK_OBJECT (m), 
+					    "remove_accelerator",
+					    bookmarks_remove_accel_cb, bm);
+			gtk_signal_connect (GTK_OBJECT (m), "select",
+					    bookmarks_ensure_accel_cb, bm);
+		
+			/* add accelerator */
+			gtk_widget_add_accelerator (m, "activate",
+						    menu->accel_group, 
+						    bm->accel_key, 
+						    bm->accel_mods, 
+						    GTK_ACCEL_VISIBLE);
+			
+			str = string_from_accel_data 
+				(m, bm->accel_key, bm->accel_mods);
+			gtk_label_set_text (GTK_LABEL (accel_label), str);
+			g_free (str);
+		}
+		else
+		{
+			gtk_widget_lock_accelerators (m);
+		}
+
 		bm_icon = bookmarks_get_siteicon (bm->url);
 		pixmap = gtk_pixmap_new (bm_icon->pixmap, bm_icon->mask);
 	}
 
-	gtk_container_add (GTK_CONTAINER (m), hb);
-	gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM (m), pixmap);
+	gtk_object_set_data (GTK_OBJECT (m), "bookmark", bm);
+
 	gtk_widget_show_all (hb);
+	gtk_container_add (GTK_CONTAINER (m), hb);
+
+	gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM (m), pixmap);
 	if (gnome_preferences_get_menus_have_icons ())
 		gtk_widget_show (pixmap);
-	/* TODO: we would like to allow bookmark menu item keystroke binding,
-	 * but for now just disable it since it only acts weird and confusing
-	 * without full support for it. */
-	gtk_widget_lock_accelerators (m);
+
+	bookmarks_block_accel_update--;
+
 	return m;
 }
 
@@ -693,12 +853,14 @@ void
 bookmarks_smart_bm_dialog (GaleonEmbed *embed, GaleonEmbed **newembed,
 			   BookmarkItem *bi, LinkState state)
 {	
+	gchar *loc_str;
 	GtkWidget *dialog;
 	gchar *s = bi->url, *strippedname;
 	int num_parameters = 0;
 	int dlg_status = 0;
 
-	strippedname = strip_uline_accel (bi->name);
+	loc_str = bi->name;
+	strippedname = strip_uline_accel (loc_str);
 	dialog = gnome_dialog_new (strippedname,
 				   GNOME_STOCK_BUTTON_OK,
 				   GNOME_STOCK_BUTTON_CANCEL,
@@ -721,6 +883,9 @@ bookmarks_smart_bm_dialog (GaleonEmbed *embed, GaleonEmbed **newembed,
 	}
 	g_free (strippedname);
 
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), "bookmarks_getstring",
+				"galeon_browser");
+	
 	/* count %s in the URL */
 	s = bi->url;
 	while ((s = strstr(s, "%s"))) {
@@ -768,8 +933,8 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonEmbed *embed,
 
 	/* set parent and level in case of fulscreeen */
 	dialog_set_parent (dialog, embed->parent_window->WMain);
-	
-	if ((dlg_status = gnome_dialog_run (GNOME_DIALOG (dialog))) == 0) {
+
+	if ((dlg_status = dialog_run (GNOME_DIALOG (dialog), FALSE)) == 0) {
 		s = strstr (bi->url, "%s");
 		g_return_val_if_fail (s, 0);
 
@@ -863,7 +1028,7 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonEmbed *embed,
 	/* set parent and level in case of fulscreeen */
 	dialog_set_parent (dialog, embed->parent_window->WMain);
 
-	if ((dlg_status = gnome_dialog_run (GNOME_DIALOG (dialog))) == 0)
+	if ((dlg_status = dialog_run (GNOME_DIALOG (dialog), FALSE)) == 0)
 	{
 		/* user selected OK, construct URL and go there */
 		result = g_strnfill(0,0);
@@ -898,3 +1063,157 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonEmbed *embed,
 	g_ptr_array_free (entries, FALSE);
 	return dlg_status;
 }
+
+/**
+ * bookmarks_init_menu_accels_recursively: recursively add accelerators
+ * for menuitems. We use a dummy widget as the real menus are created
+ * on the fly, not on startup.
+ */
+static void
+bookmarks_init_menu_accels_recursively (BookmarkItem *b, GaleonWindow *window)
+{
+	GList *item;
+	GtkWidget *dummy;
+ 
+	if (b->alias_of)
+		return; /* the real one should have been added already */
+	
+	switch (b->type)
+	{
+	case BM_FOLDER:
+	case BM_AUTOBOOKMARKS:
+		for (item = b->list; item != NULL; item = item->next)
+		{
+			bookmarks_init_menu_accels_recursively (item->data,
+								window);
+		}
+		break;
+
+	case BM_SITE:  
+		if (!((b->accel_key == GDK_VoidSymbol) || (b->accel_key == 0)))
+		{
+			dummy = gtk_menu_item_new ();
+			
+			window->dummy_widgets =
+				g_list_append (window->dummy_widgets, dummy);
+			gtk_widget_ref (dummy);
+			gtk_object_set_data 
+				(GTK_OBJECT (dummy), "GaleonWindow", 
+				 window);
+			gtk_object_set_data 
+				(GTK_OBJECT (dummy), "bookmark", b);
+			
+			gtk_signal_connect (GTK_OBJECT (dummy), "activate",
+					    GTK_SIGNAL_FUNC 
+					    (bookmarks_activate_cb), b);
+			
+			gtk_widget_add_accelerator 
+				(dummy, "activate",
+				 GNOME_APP (window->WMain)->accel_group,
+				 b->accel_key, b->accel_mods,
+				 GTK_ACCEL_VISIBLE);
+
+			gtk_signal_connect (GTK_OBJECT (dummy), 
+					    "remove_accelerator",
+					    bookmarks_remove_dummy_accel_cb,
+					    b);
+		}
+		break;
+
+	case BM_SEPARATOR:
+		break;
+	}
+}
+
+/**
+ * Updates the accel labels of the current window
+ * The menus of other windows are recreated
+ */
+void 
+bookmarks_menu_sync_accel_labels (void)
+{
+	GList *li;
+
+	bookmarks_block_accel_update++;
+	for (li = all_windows; li != NULL; li = li->next)
+	{
+		GaleonWindow *window = (GaleonWindow *)(li->data);
+		GtkMenuShell *menu;
+
+		menu = GTK_MENU_SHELL (window->bookmarks_menu);
+
+		if (!menu)
+			continue;
+
+		if (menu->active)
+		{
+			/* this menu is currently being browsed */
+			GList *lj, *l;
+			GtkWidget *separator;
+	
+			separator = window->bookmarks_separator;
+
+			l = gtk_container_children (GTK_CONTAINER (menu));
+	
+			/* look for the separator */
+			for (lj = l; lj != NULL; lj = lj->next)
+			{
+				if (lj->data == separator)
+					break;
+			}
+	
+			/* update the submenu of every widget after it */
+			for (lj = lj->next; lj != NULL; lj = lj->next)
+				if (GTK_IS_MENU_ITEM (lj->data))
+					bookmarks_menu_sync_accel_labels_item
+						(lj->data);
+			
+			g_list_free (l);
+		}
+		else
+		{
+			/* we can safely recreate it */
+			bookmarks_menu_recreate (window);
+		}
+	}
+	bookmarks_block_accel_update--;
+}
+
+static void
+bookmarks_menu_sync_accel_labels_item (GtkMenuItem *item)
+{
+	GList *l, *li;
+	GtkWidget *submenu = item->submenu;
+	BookmarkItem *bm;
+	bm = gtk_object_get_data (GTK_OBJECT (item), "bookmark");
+
+	if (bm == NULL)
+		/* this is not a bookmark menu item */
+		return;
+	
+	if (!submenu) 
+	{
+		/* does not have submenus to clean */
+		/* resync the label ... */
+		GtkWidget *l;
+		gchar *str;
+		
+		l = gtk_object_get_data (GTK_OBJECT (item), "accel_label");
+		if (l == NULL)
+			/* nothing to update */
+			return;
+		
+		str = string_from_accel_data (GTK_WIDGET (item), bm->accel_key, 
+					      bm->accel_mods);
+		gtk_label_set_text (GTK_LABEL (l), str);
+		g_free (str);
+	
+		return;
+	}
+
+	l = gtk_container_children (GTK_CONTAINER (submenu));
+	for (li = l; li != NULL; li = li->next)
+		if (GTK_IS_MENU_ITEM (li->data))
+			bookmarks_menu_sync_accel_labels_item (li->data);
+	g_list_free (l);
+}
