@@ -25,6 +25,7 @@
 #include "dialog.h"
 #include "favicon.h"
 #include "filepicker.h"
+#include "mozilla.h"
 
 #include <string.h>
 #include <time.h>
@@ -35,17 +36,24 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
 #include <gtk/gtkcheckmenuitem.h>
+#include <gdk/gdkkeysyms.h>
 
 void bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
 					   GaleonWindow *window);
 void bookmarks_import_other_activate_cb (GtkMenuItem *menuitem, 
 					 BookmarksEditorControls *controls);
 static void bookmarks_import_netscape_style (GaleonWindow *window,
-					     GString *fname);
+					     GString *fname,
+					     gboolean use_locale);
 void bookmark_export_netscape_message (GaleonWindow *window, gchar *fname);
-void bookmark_export_netscape_do (gchar *fname);
+static void bookmark_export_netscape_do (gchar *fname, gboolean use_locale);
 static void bookmarks_import_xbel_style (GaleonWindow *window,
 					 const gchar *fname);
+static void bookmarks_import_own_style (GaleonWindow *window, 
+					const gchar *fname);
+
+/* from bookmarks_menus.c */
+extern gint bookmarks_block_accel_update;
 
 static void 
 bookmarks_handle_link_clicked (GaleonWindow *window, BookmarkItem *bi,
@@ -64,6 +72,7 @@ bookmarks_handle_link_clicked (GaleonWindow *window, BookmarkItem *bi,
 		embed_activate_link (window->active_embed, NULL, 
 				     bi->url, state);
 	}
+	embed_grab_focus (window->active_embed);
 }
 
 
@@ -85,6 +94,9 @@ bookmarks_button_release_event_cb (GtkWidget *widget, GdkEventButton *event,
 
 	return_val_if_not_window (window, FALSE);
 
+	/* don't handle button 3 */
+	if (event->button == 3) return FALSE;
+
 	/* Check to see if we're supposed to handle this event */
 	/* These checks are here just to make sure tearoff menus get
 	   activated correctly */
@@ -109,7 +121,8 @@ bookmarks_button_release_event_cb (GtkWidget *widget, GdkEventButton *event,
 
 	if (bi->type == BM_SITE)
 	{
-		gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
+		if (GTK_IS_MENU_SHELL (menu))
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
 		bookmarks_handle_link_clicked (window, bi, 
 				mouse_state_to_link_state (event->button,
 							   event->state));
@@ -202,6 +215,11 @@ bookmarks_button_press_event_cb (GtkWidget *widget, GdkEventButton *event,
 			gtk_widget_unref (GTK_WIDGET(popup));
 			gtk_menu_shell_deactivate (GTK_MENU_SHELL (root));
 
+			/* deactivate the bookmark menu too, may be neccesary
+			   in some cases if root != window->bookmarks_menu */
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL 
+						   (window->bookmarks_menu));
+
 			/* stop event from causing ui freezage/weirdness */
 			gtk_signal_emit_stop_by_name (GTK_OBJECT(widget),
 					"button-press-event");
@@ -263,6 +281,37 @@ bookmarks_add_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 	bookmarks_file_bookmark_cb (widget, bi);
 }
 
+void
+bookmarks_add_folder_activate_cb (GtkWidget *widget, BookmarkItem *bi)
+{
+	BookmarkItem *new;
+	GaleonWindow *par_gwin = window_from_widget (widget);
+	GtkWidget *par_win = par_gwin ? par_gwin->WMain : NULL;
+	gchar* new_folder_name = NULL;
+	
+	g_return_if_fail (bi != NULL);
+	g_return_if_fail ((bi->type == BM_FOLDER) 
+			  || (bi->type == BM_AUTOBOOKMARKS));
+
+	new_folder_name = mozilla_locale_to_utf8(_("New folder"));
+	new = bookmarks_new_bookmark (BM_FOLDER, TRUE, new_folder_name,
+				      NULL, NULL, NULL, NULL);
+	g_free(new_folder_name);
+	new->time_added = time (NULL);
+
+	bi->list = g_list_append (bi->list, new);
+	new->parent = bi;
+
+	bookmarks_dirty = TRUE;
+	bookmarks_toolbars_check_update (new);
+
+	gnome_request_dialog 
+		(FALSE, _("Please enter the name of the folder"),
+		 _("New folder"), 100,
+		 GTK_SIGNAL_FUNC (bookmarks_string_request_callback),
+		 new, par_win ? GTK_WINDOW (par_win) : NULL);
+}
+
 void
 bookmarks_open_all_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 {
@@ -274,6 +323,22 @@ bookmarks_open_all_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 					 bi, FALSE, TRUE);
 }
 
+void
+bookmarks_open_all_release_cb (GtkWidget *widget, GdkEventButton *event,
+			       BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (widget);
+	GtkMenu *menu = GTK_MENU (widget->parent);
+
+	return_if_not_window (window);
+
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu));
+	
+	g_return_if_fail (bi != NULL);
+	bookmarks_folder_open_all_items (window->active_embed,
+					 bi, FALSE, (event->button != 2));
+}
+
 void
 bookmarks_open_all_in_wins_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 {
@@ -285,11 +350,30 @@ bookmarks_open_all_in_wins_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 					 bi, TRUE, TRUE);
 }
 
+void
+bookmarks_open_all_in_wins_release_cb (GtkWidget *widget, GdkEventButton *event,
+			               BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (widget);
+	GtkMenu *menu = GTK_MENU (widget->parent);
+
+	return_if_not_window (window);
+
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu));
+	
+	g_return_if_fail (bi != NULL);
+	bookmarks_folder_open_all_items (window->active_embed,
+					 bi, TRUE, (event->button != 2));
+}
+
 void
 bookmarks_create_toolbar_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 {
 	g_return_if_fail (bi != NULL);
+	bookmarks_toolbars_check_update (bi);
 	bi->create_toolbar = !bi->create_toolbar;
+	/* called twice intentionally */
+	bookmarks_toolbars_check_update (bi);
 	bookmarks_updated ();
 }
 
@@ -300,6 +384,7 @@ bookmarks_set_as_default_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 	gint i = 2;
 	g_return_if_fail (bi != NULL);
 	
+	bookmarks_toolbars_check_update (default_bookmarks_root);
 	tree_items = default_bookmarks_root->tree_items;
 	while (i > 0)
 	{
@@ -333,10 +418,17 @@ bookmarks_set_as_default_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 		tree_items = bi->tree_items;
 		i--;
 		default_bookmarks_root = bi;
+		bookmarks_toolbars_check_update (bi);
 	}
 	bookmarks_updated ();
 }
 
+void
+bookmarks_regenerate_autobookmarks_cb (GtkWidget *widget, BookmarkItem *bi)
+{
+	autobookmarks_generate ();
+}
+
 /**
  * bookmarks_toolbar_menu_edit_cb: called by the context menu on the toolbar
  * and menus.  Open the edit bookmarks dialog and select the rigth folder.
@@ -363,10 +455,38 @@ bookmarks_edit_activate_cb (GtkWidget *menuitem, BookmarkItem *bm)
  */
 void
 bookmarks_remove_activate_cb (GtkWidget *menuitem, 
-				  BookmarkItem *bm)
+			      BookmarkItem *bm)
 {
 	gboolean autobookmarks_deleted = FALSE;
+	
+	GaleonWindow *window = window_from_widget (menuitem);
 
+	/* Make sure that the bookmarks menu has disapeared from the
+	   screen, otherwise the menu can hide the dialog (and it would be 
+	   very bad) */
+	if (window) 
+	{
+		GtkWidget *item = gtk_menu_get_attach_widget 
+			(GTK_MENU (window->bookmarks_menu));		
+		gtk_menu_popdown (GTK_MENU (window->bookmarks_menu));
+		gtk_menu_item_deselect (GTK_MENU_ITEM (item));
+		/* the menubar */ 
+		if (GTK_IS_MENU_SHELL (item->parent))
+			gtk_menu_shell_deactivate 
+				(GTK_MENU_SHELL (item->parent));
+	}
+
+	if (bm->list /* don't ask about deleting an empty folder or an alias */
+	    && !bm->alias_of
+	    && !dialog_ask_question 
+	    (_("Removing this category will remove all its children.\n"
+	       "Are you sure that you want to continue?"),
+	     window->WMain))
+	{
+		return;
+	}
+
+	bookmarks_toolbars_check_update (bm);
 	if (bm->type == BM_AUTOBOOKMARKS)
 	{
 		autobookmarks_deleted = TRUE;
@@ -383,6 +503,7 @@ bookmarks_remove_activate_cb (GtkWidget *menuitem,
 void
 bookmarks_file_bookmark_cb (GtkWidget *widget, BookmarkItem *p)
 {
+	gchar *loc_str;
 	BookmarkItem *new;	
 	GaleonEmbed *embed;
 	GaleonWindow *window;
@@ -393,40 +514,45 @@ bookmarks_file_bookmark_cb (GtkWidget *widget, BookmarkItem *p)
 	embed = window->active_embed;
 	return_if_not_embed (embed);
 
-	new = bookmarks_new_bookmark (BM_SITE, TRUE, embed->site_title, 
+	new = bookmarks_new_bookmark (BM_SITE, TRUE, embed->site_title_utf8, 
 				      embed->site_location, NULL, NULL, NULL);
 	new->time_added = time (NULL);
 
 	p->list = g_list_append (p->list, new);
 	new->parent = p;
 	bookmarks_dirty = TRUE;
+	bookmarks_toolbars_check_update (new);
 
 	if (eel_gconf_get_integer(CONF_BOOKMARKS_TITLE) == 
 	    BOOKMARK_PAGE_TITLE)
 	{
 		if (new->url && 
 		    eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED))
-			favicon_get_siteicon (embed, new->url);
+			favicon_get_siteicon (embed, new, NULL);
 
 		bookmarks_editor_place_tree_items (new);
+		bookmarks_toolbars_check_update (new);
 		bookmarks_updated ();
 	} else {
+		loc_str = mozilla_utf8_to_locale (new->name);
 		gnome_request_dialog 
 			(FALSE, _("Please enter the name of the bookmark"),
-			 new->name, 100,
+			 loc_str, 100,
 			 GTK_SIGNAL_FUNC (bookmarks_string_request_callback),
 			 new, GTK_WINDOW (window->WMain));
+		g_free (loc_str);
 	}
 }
 
 /* import given netscape-compatible bookmark filename with pass/fail dialog */
 static void
-bookmarks_import_netscape_style (GaleonWindow *window, GString *fname)
+bookmarks_import_netscape_style (GaleonWindow *window, GString *fname,
+				 gboolean use_locale)
 {
 	/* NB: used to use gnome_app_message here, but there's not a 
 	 * good choice of window to associate the message with, so 
 	 * forcing a dialog seems like a better idea -- MattA 30-12-2000 */
-	if (netscape_import_bookmarks(fname))
+	if (netscape_import_bookmarks(fname, use_locale))
 	{
 		gnome_ok_dialog (_("Importing bookmarks failed"));
 	}
@@ -434,6 +560,7 @@ bookmarks_import_netscape_style (GaleonWindow *window, GString *fname)
 	{
 		gnome_ok_dialog (_("Bookmarks successfully imported/merged"));
 	}
+	bookmarks_tb_dirty = TRUE;
 	bookmarks_updated ();
 }
 
@@ -448,6 +575,22 @@ bookmarks_import_xbel_style (GaleonWindow *window, const gchar *fname)
 	} else {
 		gnome_ok_dialog (_("Importing bookmarks failed"));
 	}
+	bookmarks_tb_dirty = TRUE;
+	bookmarks_updated ();
+	bookmarks_editors_refresh ();
+}
+
+static void
+bookmarks_import_own_style (GaleonWindow *window, const gchar *fname)
+{
+	BookmarkItem *imported = bookmarks_load_from ((gchar *) fname);
+	if (imported) {
+		bookmarks_merge_trees (bookmarks_root, imported);
+		bookmarks_free_bookmark (imported);
+		gnome_ok_dialog (_("Bookmarks successfully imported/merged"));
+	} else {
+		gnome_ok_dialog (_("Importing bookmarks failed"));
+	}
 	bookmarks_updated ();
 	bookmarks_editors_refresh ();
 }
@@ -461,7 +604,7 @@ bookmarks_import_netscape_activate_cb (GtkMenuItem *menuitem,
 	g_string_assign (fname, g_get_home_dir ());
 	g_string_append (fname, NETSCAPE_BOOKMARKS);
        	
-	bookmarks_import_netscape_style (window, fname);
+	bookmarks_import_netscape_style (window, fname, TRUE);
 
 	g_string_free (fname, TRUE);
 }
@@ -478,7 +621,7 @@ bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
 	g_string_assign (fname, prefs_dir);
 	g_string_append (fname, "/bookmarks.html");
 
-	bookmarks_import_netscape_style (window, fname);
+	bookmarks_import_netscape_style (window, fname, FALSE);
 
 	g_string_free (fname, TRUE);
 	g_free (prefs_dir);
@@ -494,7 +637,7 @@ bookmarks_import_other_activate_cb (GtkMenuItem *menuitem,
 			      g_get_home_dir(), NULL, modeOpen,
 			      &file, NULL)) {
 		GString *temp = g_string_new (file);
-		bookmarks_import_netscape_style (NULL, temp);
+		bookmarks_import_netscape_style (NULL, temp, FALSE);
 		g_string_free (temp, TRUE);
 	}
 
@@ -515,6 +658,20 @@ bookmarks_import_xbel_activate_cb (GtkMenuItem *menuitem,
 	g_free (file);
 }
 
+void
+bookmarks_import_own_activate_cb (GtkMenuItem *menuitem, 
+				  BookmarksEditorControls *controls)
+{
+	gchar *file = NULL;
+
+	if (show_file_picker (controls->dialog, _("Import from..."),
+			      g_get_home_dir(), NULL, modeOpen,
+			      &file, NULL))
+		bookmarks_import_own_style (NULL, file);
+	
+	g_free (file);
+}
+
 void
 bookmarks_export_netscape_activate_cb (GtkMenuItem *menuitem,
 				       GtkWidget *widget)
@@ -523,7 +680,7 @@ bookmarks_export_netscape_activate_cb (GtkMenuItem *menuitem,
 				    NULL);
        	
 	if (dialog_check_overwrite_file (fname, widget))
-		bookmark_export_netscape_do (fname);
+		bookmark_export_netscape_do (fname, TRUE);
 
 	g_free (fname);
 }
@@ -537,7 +694,7 @@ bookmarks_export_mozilla_activate_cb (GtkMenuItem *menuitem,
 
 	fname = g_strconcat (prefs_dir,"/bookmarks.html", NULL);
 	if (dialog_check_overwrite_file (fname, widget))
-		bookmark_export_netscape_do (fname);
+		bookmark_export_netscape_do (fname, FALSE);
 
 	g_free (fname);
 	g_free (prefs_dir);
@@ -554,7 +711,7 @@ bookmarks_export_other_activate_cb (GtkMenuItem *menuitem,
 			      &file, NULL))
 	{
 		if (dialog_check_overwrite_file (file, NULL))
-			bookmark_export_netscape_do (file);
+			bookmark_export_netscape_do (file, FALSE);
 		g_free (file);
 	}
 }
@@ -575,10 +732,10 @@ bookmarks_export_xbel_activate_cb (GtkMenuItem *menuitem,
 	}
 }
 
-void 
-bookmark_export_netscape_do (gchar *fname)
+static void 
+bookmark_export_netscape_do (gchar *fname, gboolean use_locale)
 {
-	if (netscape_export_bookmarks (fname, bookmarks_root))
+	if (netscape_export_bookmarks (fname, bookmarks_root, use_locale))
 		gnome_ok_dialog (_("Exporting bookmarks failed"));
 	else 
 		gnome_ok_dialog (_("Bookmarks successfully exported"));
@@ -595,3 +752,86 @@ bookmarks_open_in_myportal_cb (GtkWidget *widget, BookmarkItem *bi)
 			bi, TRUE, TRUE);
 }
 
+void
+bookmarks_add_accel_cb (GtkWidget *widget, guint accel_signal_id,
+			GtkAccelGroup *accel_group, guint accel_key,
+			GdkModifierType accel_mods, GtkAccelFlags accel_flags,
+			BookmarkItem *bi)
+{
+	if (bookmarks_block_accel_update > 0) return;
+
+	/* if nothing is to be added, don't do anything */
+	if (accel_key == GDK_VoidSymbol || accel_key == 0) return;
+
+	/* check whether bookmarks will need to be updated */
+	if (!(bi->accel_key == accel_key && bi->accel_mods == accel_mods))
+		bookmarks_dirty = TRUE;
+
+	/* update bookmark */
+	bi->accel_key = accel_key;
+	bi->accel_mods = accel_mods;
+
+	/* sync */
+	bookmarks_menu_sync_accel_labels ();
+}
+
+void
+bookmarks_remove_accel_cb (GtkWidget *widget, GtkAccelGroup *accel_group, 
+			   guint accel_key, GdkModifierType accel_mods, 
+			   BookmarkItem *bi)
+{
+	if (bookmarks_block_accel_update > 0) return;
+
+	/* bookmarks will need to be updated */
+	bookmarks_dirty = TRUE;
+
+	/* update bookmark */
+	bi->accel_key = 0;
+	bi->accel_mods = 0;
+
+	/* sync */
+	bookmarks_menu_sync_accel_labels ();
+}
+
+void
+bookmarks_ensure_accel_cb (GtkWidget *widget, BookmarkItem *bm)
+{
+	GtkMenu *menu;
+	GtkWidget *l;
+	gchar *str;
+
+	if (bookmarks_block_accel_update > 0) return;
+
+	g_return_if_fail (GTK_IS_MENU_ITEM (widget));
+	g_return_if_fail (GTK_IS_MENU (widget->parent));
+
+	menu = GTK_MENU (widget->parent);
+
+	bookmarks_block_accel_update++;
+	gtk_widget_add_accelerator (widget, "activate", menu->accel_group, 
+				    bm->accel_key, bm->accel_mods, 
+				    GTK_ACCEL_VISIBLE);
+	bookmarks_block_accel_update--;
+	
+	/* update accel label too, 
+	 * FIXME: this is not really neccesary 90% of time */
+	
+	l = gtk_object_get_data (GTK_OBJECT (widget), "accel_label");
+	g_return_if_fail (l != NULL);
+
+	str = string_from_accel_data (widget, bm->accel_key, bm->accel_mods);
+	gtk_label_set_text (GTK_LABEL (l), str);
+	g_free (str);
+}
+
+void
+bookmarks_remove_dummy_accel_cb (GtkWidget *widget, GtkAccelGroup *accel_group, 
+			         guint accel_key, GdkModifierType accel_mods, 
+			         BookmarkItem *bi)
+{
+	if (bookmarks_block_accel_update > 0) return;
+
+	/* update bookmark */
+	bi->accel_key = 0;
+	bi->accel_mods = 0;
+}
