@@ -25,7 +25,7 @@
 #include "mozilla.h"
 #include "downloader.h"
 #include "context.h"
-#include "prefs.h"
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
@@ -56,13 +56,13 @@ static char BACK_STR[]          = N_("Back");
 static char FORWARD_STR[]       = N_("Forward");
 static char UP_STR[]            = N_("Up");
 static char RELOAD_STR[]        = N_("Reload");
-static char OPEN_NEW_WIN_STR[]  = N_("Open in a new window");
-static char OPEN_NEW_TAB_STR[]  = N_("Open in a new tab");
+static char OPEN_NEW_WIN_STR[]  = N_("Open in new window");
+static char OPEN_NEW_TAB_STR[]  = N_("Open in new tab");
 static char DOWNLOAD_FILE_STR[] = N_("Download link");
 static char COPY_LINK_LOC_STR[] = N_("Copy link location");
 static char ADD_BM_STR[]        = N_("Add bookmark");
-static char OPEN_IMG_WIN_STR[]  = N_("Open image in a new window");
-static char OPEN_IMG_TAB_STR[]  = N_("Open image in a new tab");
+static char OPEN_IMG_WIN_STR[]  = N_("Open image in new window");
+static char OPEN_IMG_TAB_STR[]  = N_("Open image in new tab");
 static char SAVE_IMG_STR[]      = N_("Save image");
 static char SAVE_IMG_AS_STR[]   = N_("Save image as...");
 static char USE_IMG_AS_BG_STR[] = N_("Use image as background");
@@ -75,7 +75,12 @@ static char COPY_FRAME_LOC_STR[]     = N_("Copy frame location");
 static char ADD_BM_FRAME_STR[]       = N_("Add bookmark for frame");
 static char VIEW_SOURCE_STR[]        = N_("View source");
 static char SAVE_AS_STR[]            = N_("Save As...");
-static char FULLSCREEN_VIEW_STR[]    = N_("Full Screen View");
+static char FULLSCREEN_STR[]    = N_("Full screen");
+static char MENUBAR_VIEW_STR[]  = N_("Show menubar");
+static char BGIMG_STR[]         = N_("Save background as...");
+static char CUT_STR[]           = N_("Cut");
+static char COPY_STR[]          = N_("Copy");
+static char PASTE_STR[]         = N_("Paste");
 
 #define NAVIGATION_CONTEXT_MENU \
   CONTEXT_MENU_ITEM(BACK_STR,    GNOME_STOCK_MENU_BACK,    GALEON_CONTEXT_BACK),    \
@@ -111,10 +116,16 @@ static char FULLSCREEN_VIEW_STR[]    = N_("Full Screen View");
 #define SHORT_CONTEXT_MENU \
   CONTEXT_MENU_ITEM(COPY_LINK_LOC_STR, GNOME_STOCK_MENU_COPY,  GALEON_CONTEXT_COPYDOCLOC),    \
   CONTEXT_MENU_ITEM(SAVE_AS_STR,               GNOME_STOCK_MENU_SAVE_AS, GALEON_CONTEXT_SAVEAS),     \
+  CONTEXT_MENU_ITEM(BGIMG_STR,		       GNOME_STOCK_MENU_SAVE_AS, GALEON_CONTEXT_BGIMG),	\
   CONTEXT_MENU_ITEM(VIEW_SOURCE_STR,           GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_VIEWSOURCE),     \
-  CONTEXT_MENU_TOGGLEITEM(FULLSCREEN_VIEW_STR, GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_FULLSCREEN), \
+  CONTEXT_MENU_TOGGLEITEM(FULLSCREEN_STR,      GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_FULLSCREEN), \
+  CONTEXT_MENU_ITEM(MENUBAR_VIEW_STR,          GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_MENUBAR), \
   CONTEXT_MENU_ITEM(ADD_BM_STR,                GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_ADDTEMPBM),
-
+#define CLIPBOARD_CONTEXT_MENU \
+  CONTEXT_MENU_ITEM(CUT_STR,   GNOME_STOCK_MENU_CUT,    GALEON_CONTEXT_CUT),    \
+  CONTEXT_MENU_ITEM(COPY_STR,  GNOME_STOCK_MENU_COPY,   GALEON_CONTEXT_COPY),    \
+  CONTEXT_MENU_ITEM(PASTE_STR, GNOME_STOCK_MENU_PASTE,  GALEON_CONTEXT_PASTE),  
+  
 /* local data structures */
 typedef enum
 {
@@ -125,6 +136,7 @@ typedef enum
 	GALEON_CONTEXT_RELOAD,
 	GALEON_CONTEXT_SAVEAS,
 	GALEON_CONTEXT_FULLSCREEN,
+	GALEON_CONTEXT_MENUBAR,
 	GALEON_CONTEXT_ADDTEMPBM,
 	GALEON_CONTEXT_OPENWINDOW,
 	GALEON_CONTEXT_OPENTAB,
@@ -146,7 +158,11 @@ typedef enum
 	GALEON_CONTEXT_RELOADFRAME,
 	GALEON_CONTEXT_COPYFRAMELOC,
 	GALEON_CONTEXT_ADDTEMPBMFRAME,
-	GALEON_CONTEXT_VIEWSOURCE	
+	GALEON_CONTEXT_VIEWSOURCE,
+	GALEON_CONTEXT_BGIMG,
+	GALEON_CONTEXT_CUT,
+	GALEON_CONTEXT_COPY,
+	GALEON_CONTEXT_PASTE
 } GaleonContextAction;
 
 typedef enum
@@ -163,7 +179,8 @@ static GnomeUIInfo *context_make_menu 	     (WrapperContextInfo *info);
 static guint context_show_context_menu 	     (GaleonEmbed *embed, 
 					      GnomeUIInfo *menu,
 					      int button,
-					      guint timestamp);
+					      guint timestamp,
+					      WrapperContextInfo *info);
 static void context_do_action 		     (GaleonContextAction action, 
 					      WrapperContextInfo *info,
 					      GaleonEmbed *embed);
@@ -173,6 +190,15 @@ static void context_set_navigation_buttons   (GaleonEmbed *embed,
 static void context_set_full_screen_check    (GaleonWindow *window,
 					      GnomeUIInfo *menu,
 					      GtkWidget *popup);
+static void context_set_menu_bar_check       (GaleonWindow *window,
+					      GnomeUIInfo *menu,
+					      GtkWidget *popup);
+static void context_set_background_image     (GaleonWindow *window,
+					      GnomeUIInfo *menu,
+					      WrapperContextInfo *info);
+static void context_set_clipboard_buttons (GaleonEmbed *embed, 
+					   GnomeUIInfo *uiinfo,
+					   GtkWidget *popup);
 static void context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed);
 static BookmarkItem *context_menu_create_fake_bookmarks_root (void);
 static void context_menu_create_fake_bookmarks_root_recursively 
@@ -183,6 +209,8 @@ static void fill_in_image_context_menu (gchar *imagename,
 /* local variables */
 static gboolean found_bookmarks = FALSE;
 
+extern gboolean fullscreen_active;
+
 /**
  * context_set_full_screen_check: searches for a check button and sets it
  **/
@@ -193,10 +221,6 @@ static void context_set_full_screen_check (GaleonWindow *window,
 	gint i;
 
 	return_if_not_window (window);
-	if (!window->menuBarOn)
-	{
-		return;
-	}
 
 	for (i = 0; menu[i].type != GNOME_APP_UI_ENDOFINFO; i++)
 	{
@@ -207,8 +231,61 @@ static void context_set_full_screen_check (GaleonWindow *window,
 				GTK_CHECK_MENU_ITEM (menu[i].widget);
 			g_return_if_fail (menu_view_fullscreen != NULL); 
 			gtk_check_menu_item_set_active (menu_view_fullscreen,
-				(GTK_CHECK_MENU_ITEM 
-					(window->view_fullscreen))->active);
+							fullscreen_active);
+		}
+	}
+}
+
+/**
+ * context_set_menu_bar_check: searches for a check button and sets it
+ **/
+static void context_set_menu_bar_check (GaleonWindow *window,
+					GnomeUIInfo *menu,
+					GtkWidget *popup)
+{
+	gint i;
+
+	return_if_not_window (window);
+
+	for (i = 0; menu[i].type != GNOME_APP_UI_ENDOFINFO; i++)
+	{
+		if (GPOINTER_TO_INT (menu[i].user_data) == 
+		    GALEON_CONTEXT_MENUBAR)
+		{
+			GtkMenuItem *menu_view_menubar = 
+				GTK_MENU_ITEM (menu[i].widget);
+			g_return_if_fail (menu_view_menubar != NULL); 
+			if (GTK_CHECK_MENU_ITEM (window->view_menubar)->active)
+			{
+				gtk_widget_hide
+					(GTK_WIDGET (menu_view_menubar));
+			}
+		}
+	}
+}
+
+/**
+ * context_set_menu_bar_check: searches for a check button and sets it
+ **/
+static void context_set_background_image (GaleonWindow *window,
+					  GnomeUIInfo *menu,
+					  WrapperContextInfo *info)
+{
+	gint i;
+
+	return_if_not_window (window);
+
+	for (i = 0; menu[i].type != GNOME_APP_UI_ENDOFINFO; i++)
+	{
+		if (GPOINTER_TO_INT (menu[i].user_data) == 
+		    GALEON_CONTEXT_BGIMG)
+		{
+			GtkMenuItem *m = GTK_MENU_ITEM (menu[i].widget);
+			g_return_if_fail (m != NULL); 
+			if (info->bgimg == NULL)
+			{
+				gtk_widget_hide (GTK_WIDGET (m));
+			}
 		}
 	}
 }
@@ -244,14 +321,17 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 			go_forward_pos = i + 1;
 			go_up_pos      = i + 2;
 
+			gtk_widget_set_sensitive(uiinfo[go_back_pos].widget, 
+						 can_go_back);
+			gtk_widget_set_sensitive(uiinfo[go_forward_pos].widget, 
+						 can_go_forward);
+			gtk_widget_set_sensitive (uiinfo[go_up_pos].widget, 
+						  can_go_up);
+
 			break;
 		}
 	}
 
-	gtk_widget_set_sensitive(uiinfo[go_back_pos].widget, can_go_back);
-	gtk_widget_set_sensitive(uiinfo[go_forward_pos].widget, can_go_forward);
-	gtk_widget_set_sensitive (uiinfo[go_up_pos].widget, can_go_up);
-
 	if (show_history_popup)
 	{
 		if (can_go_back && 
@@ -294,6 +374,43 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 	}
 }
 
+/**
+ * context_set_clipboard_buttons: sets up cut/copy/paste buttons
+ */
+static void context_set_clipboard_buttons (GaleonEmbed *embed, 
+					   GnomeUIInfo *uiinfo,
+					   GtkWidget *popup)
+{
+	gboolean can_copy, can_cut, can_paste;
+	gint context_action, i;
+	gint copy_pos = 1, cut_pos = 0, paste_pos = 2;
+	
+	can_cut = mozilla_can_cut_selection (embed);
+	can_copy = mozilla_can_copy_selection (embed);
+	can_paste = mozilla_can_paste (embed);
+	
+	for (i = 0; uiinfo[i].type != GNOME_APP_UI_ENDOFINFO; i++)
+	{
+		context_action = GPOINTER_TO_INT (uiinfo[i].user_data);
+
+		if (context_action == GALEON_CONTEXT_CUT)
+		{
+			cut_pos = i;
+			copy_pos = i + 1;
+			paste_pos = i + 2;
+
+			gtk_widget_set_sensitive(uiinfo[cut_pos].widget, 
+						 can_cut);
+			gtk_widget_set_sensitive(uiinfo[copy_pos].widget, 
+						 can_copy);
+			gtk_widget_set_sensitive (uiinfo[paste_pos].widget, 
+						  can_paste);
+
+			break;
+		}
+	}
+}
+
 /**
  * context_make_menu: helper func to create the correct menu
  **/
@@ -368,12 +485,12 @@ static GnomeUIInfo *context_make_menu (WrapperContextInfo *info)
                 return g_memdup (&image_uiinfo, sizeof(image_uiinfo));
 	}
 	case GALEON_CONTEXT_INPUT:
-	{ /* FIXME: maybe make this context have cut/copy/paste etc. */
+	{
 		GnomeUIInfo input_uiinfo[] =
 		{
-			NAVIGATION_CONTEXT_MENU
+			CLIPBOARD_CONTEXT_MENU
 			GNOMEUIINFO_SEPARATOR,
-			SHORT_CONTEXT_MENU
+			NAVIGATION_CONTEXT_MENU
 			GNOMEUIINFO_END
 		};
 		return g_memdup (&input_uiinfo, sizeof(input_uiinfo));
@@ -390,11 +507,13 @@ static GnomeUIInfo *context_make_menu (WrapperContextInfo *info)
 static guint context_show_context_menu (GaleonEmbed *embed, 
 					GnomeUIInfo *menu,
 					int button,
-					guint timestamp)
+					guint timestamp,
+					WrapperContextInfo *info)
 {
 	guint ret;
 	GtkWidget *popup;
 	GdkEventButton *event = NULL;
+	GList *l;
 	/* Create a fake GdkEventButton struct for gnome_popup_menu_new so that
 	 * we can get standard gtk popup menu behavior.
 	 * button and time are the only fields it needs. */
@@ -406,8 +525,15 @@ static guint context_show_context_menu (GaleonEmbed *embed,
 	}
 	popup = gnome_popup_menu_new (menu);
 	context_set_navigation_buttons (embed, menu, popup);
+	context_set_clipboard_buttons (embed, menu, popup);
 	context_set_full_screen_check (embed->parent_window, menu, popup);
+	context_set_menu_bar_check (embed->parent_window, menu, popup);
+	context_set_background_image (embed->parent_window, menu, info);
 	context_menu_add_bookmarks (GTK_MENU (popup), embed);
+	for (l = GTK_MENU (popup)->menu_shell.children; l; l = g_list_next (l))
+	{
+		gtk_widget_lock_accelerators (GTK_WIDGET (l->data));
+	}
 	ret = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
 					       event , NULL);
 	/* Wait for the menu to be destroyed if a menu item is selected */
@@ -445,7 +571,7 @@ static void context_do_action (GaleonContextAction action,
 		embed_go_up (embed, 0, 0);
 		break;
 	case GALEON_CONTEXT_RELOAD:
-		embed_reload (embed);
+		embed_reload (embed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
 		break;
 	case GALEON_CONTEXT_SAVEAS:
 		embed_save_document (embed, FALSE);
@@ -453,9 +579,17 @@ static void context_do_action (GaleonContextAction action,
 	case GALEON_CONTEXT_FULLSCREEN:
 		window_toggle_fullscreen_mode (embed->parent_window);
 		break;
+	case GALEON_CONTEXT_MENUBAR:
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM 
+			(embed->parent_window->view_menubar), TRUE);
+		break;
 	case GALEON_CONTEXT_ADDTEMPBM:
-		add_bookmark_default (BM_SITE, embed->site_title, 
-				      embed->site_location, NULL);
+		if (default_bookmarks_root) 
+			bookmarks_file_bookmark_cb (embed->parent_window->WMain,
+						    default_bookmarks_root);
+		break;
+	case GALEON_CONTEXT_BGIMG:
+		embed_save_image (embed, info->bgimg, TRUE);
 		break;
 	case GALEON_CONTEXT_OPENWINDOW:
 		embed_create_from_url (embed, info->link, FALSE, TRUE);
@@ -497,7 +631,7 @@ static void context_do_action (GaleonContextAction action,
 		copy_to_clipboard (g_strdup (info->img), embed);
 		break;
 	case GALEON_CONTEXT_BLOCKIMAGESOURCE:
-		mozilla_block_url (info->img);
+		embed_toggle_image_blocking (info->img);
 		break;
 	case GALEON_CONTEXT_OPENFRAME:
 		embed_open_frame (embed, TRUE, FALSE);
@@ -523,14 +657,26 @@ static void context_do_action (GaleonContextAction action,
 		url = mozilla_get_document_url (embed);
 		if (url)
 		{
-			add_bookmark_default (BM_SITE, embed->site_title, 
-					      url, NULL);
+			add_bookmark_default
+				(BM_SITE, (embed->site_title_utf8 
+					   ? embed->site_title_utf8
+					   : embed->site_title), 
+				 url, NULL);
 			g_free (url);
 		}
 		break;
 	case GALEON_CONTEXT_VIEWSOURCE:
 		embed_view_source (embed, FALSE, !tabbed_mode);
 		break;
+	case GALEON_CONTEXT_CUT:
+		mozilla_cut_selection (embed);
+		break;
+	case GALEON_CONTEXT_COPY:
+		mozilla_copy_selection (embed);
+		break;
+	case GALEON_CONTEXT_PASTE:
+		mozilla_paste (embed);
+		break;
 	default:
 		/* nothing happened, most likely the user did not end up
 		   selecting an item from the popup */
@@ -552,7 +698,7 @@ void context_show_menu (GaleonEmbed *embed, WrapperContextInfo *info,
 	if (menu != NULL)
 	{
 		guint result = context_show_context_menu (embed, menu, button,
-							  timestamp);
+							  timestamp, info);
 		if (result != -1)
 		{
 			GaleonContextAction action = 
@@ -579,7 +725,7 @@ void context_show_bookmark_menu (GaleonEmbed *embed)
 	menu = gtk_menu_new ();
 	tips = gtk_tooltips_new ();
 	bookmarks_menu_create_recursively (bookmarks_root, GTK_MENU (menu),
-					   NULL, tips, TRUE, TRUE);
+					   NULL, tips, TRUE, TRUE, TRUE);
 	
 	/* warning: ugly code */
 	gtk_object_set_data (GTK_OBJECT (menu), "GaleonWindow", window); 
@@ -621,13 +767,6 @@ context_show_appearance_menu (GaleonWindow *window, GtkMenu *menu,
 		appearance_menu = menu;
 	}
 
-	if (gnome_preferences_get_menus_have_tearoff ())
-	{
-		GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
-		gtk_menu_prepend (appearance_menu, tearoff);
-		gtk_widget_show (tearoff);
-	}
-
 	if (bi)
 	{
 		/* we're on a bookmark toolbar */
@@ -707,10 +846,11 @@ context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed)
 	}
 	
 	bookmarks_menu_create_recursively (fake_bookmarks_root, 
-					   menu, NULL, tips, FALSE, TRUE);
+					   menu, NULL, tips, FALSE,
+					   TRUE, TRUE);
 	
 	bookmarks_free_bookmark (fake_bookmarks_root);
-	
+
 	/* warning: ugly code */
 	gtk_object_set_data (GTK_OBJECT (menu), "GaleonWindow", window); 
 	gtk_signal_connect_object (GTK_OBJECT (menu), "destroy", 
@@ -726,9 +866,11 @@ fill_in_image_context_menu (gchar *imagename, gint pos,
 			GnomeUIInfo *uiinfo)
 {
 	gchar *name, *s;
-	const gchar *host;
+	const gchar *host = NULL;
 	gint length;
 	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (imagename);
+	GList *permissions, *l;
+	gboolean blocked = FALSE;
 
 	g_assert (imagename != NULL);
 	
@@ -751,20 +893,54 @@ fill_in_image_context_menu (gchar *imagename, gint pos,
 	g_free (s);
 	g_free (name);
 
-	g_free(uiinfo[pos+7].label);
-	host = gnome_vfs_uri_get_host_name(vfs_uri);
-	if (host)
+	if (vfs_uri)
+		host = gnome_vfs_uri_get_host_name (vfs_uri);
+	permissions = mozilla_get_permissions (IMAGEPERMISSION);
+
+	for (l = permissions; l && blocked == FALSE; l = g_list_next (l))
 	{
-		uiinfo[pos+7].label = 
-			g_strdup_printf (_("Block images from %s"), host);
+		CookieBase *b = (CookieBase *) l->data;
+		if (host && strcmp (host, b->domain) == 0)
+			blocked = TRUE;
+		g_free (b->type);
+		g_free (b->domain);
+		g_free (b);
+	}
+
+	g_list_free (permissions);
+	
+	g_free (uiinfo[pos+7].label);
+
+	if (blocked == FALSE)
+	{
+		if (host)
+		{
+			uiinfo[pos+7].label = 
+				g_strdup_printf (_("Block images from %s"),
+						 host);
+		}
+		else
+		{
+			uiinfo[pos+7].label = 
+				g_strdup (_("Block image source"));
+		}
 	}
 	else
 	{
-		uiinfo[pos+7].label = 
-			g_strdup (_("Block image source"));
+		if (host)
+		{
+			uiinfo[pos+7].label = 
+				g_strdup_printf (_("Allow images from %s"),
+						 host);
+		}
+		else
+		{
+			uiinfo[pos+7].label = 
+				g_strdup (_("Allow image source"));
+		}	
 	}
 
-	gnome_vfs_uri_unref(vfs_uri);
+	if (vfs_uri) gnome_vfs_uri_unref(vfs_uri);
 }
 
 /**
