@@ -22,10 +22,15 @@
 #include "mozilla.h"
 #include "downloader.h"
 #include "filepicker.h"
+#include "misc.h"
 #include "prefs.h"
+#include "embed.h"
 
 /* GNOME includes */
-#include <libgnorba/gnorba.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-exec.h>
+#include <libgnomeui/gnome-dialog-util.h>
+#include <liboaf/liboaf.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 static gboolean downloader_gtm_add_url (const gchar *url, gchar *dir, 
@@ -49,7 +54,13 @@ downloader_save_link (GaleonEmbed *embed, const gchar *url)
 	switch (eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER))
 	{
 	case DOWNLOADER_BUILTIN:
+#if MOZILLA_VERSION > VERSION3(0,9,4)
+		save_url (embed, url, CONF_STATE_SAVE_DIR,
+			  eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR),
+			  FALSE, FALSE);
+#else
 		mozilla_save_link (embed, url);
+#endif
 		break;
 		
 	case DOWNLOADER_GTM:
@@ -134,7 +145,12 @@ downloader_save_url_without_dir (const gchar *url)
 
 	/* get directory to save to */
 	dir = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-	if (dir)
+
+	if (dir && strcmp(dir,"") == 0)
+	{
+		exp_dir = g_strdup (dir);
+	}
+	else if (dir)
 	{
 		exp_dir = gnome_vfs_expand_initial_tilde (dir);
 		g_free (dir);
@@ -180,12 +196,13 @@ static void
 downloader_save_url_with_command_line (const gchar *url, const gchar *command,
 				       const gchar *directory)
 {
-	GtkWidget *dialog;
-	gchar *full_command;
+	gchar *partial_command, *full_command;
 	gchar *text;	
 	gint pid;
 
-	full_command = g_strdup_printf (command, url);
+	partial_command = str_dup_replace (command, "%s", url);
+	full_command = str_dup_replace (partial_command, "%f", directory);
+	g_free (partial_command);
 	
 	if (eel_gconf_get_boolean (CONF_DOWNLOADING_EXTERNAL_TERMINAL))
 	{
@@ -210,8 +227,7 @@ downloader_save_url_with_command_line (const gchar *url, const gchar *command,
 	{
 		text = g_strdup_printf (_("Failed to execute download "
 					  "command:\n\"%s\""), full_command);
-		dialog = gnome_error_dialog (text);
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		g_warning (text);
 		g_free (text);
 	}
 
@@ -240,27 +256,19 @@ downloader_gtm_add_url (const gchar *url, gchar *dir, gboolean disable_proxy,
 			gboolean disable_auto_dl)
 {
 	CORBA_Environment ev;
+	extern CORBA_Environment corba_env;
 	GTM_Download gtm_download_client;
-	static gboolean gnorba_init = FALSE;
 	gboolean result;
 
 	/* exception catching environment */
 	CORBA_exception_init (&ev);
 
-	/* initialise gnorba */
-	if (!galeon_panel_mode && !gnorba_init)
-	{
-		/* use dummy environment */
-		int argc = 0;
-		char *argv[1] = { NULL };
-		gnorba_CORBA_init (&argc, argv, GNORBA_INIT_SERVER_FUNC, &ev);
-		gnorba_init = TRUE;
-	}
-
 	result = FALSE;
-	gtm_download_client = 
-	goad_server_activate_with_repo_id (NULL, "IDL:gtm/download:1.0",
-					   0, NULL);
+
+	ev = corba_env;
+	gtm_download_client =   
+		oaf_activate_from_id ("OAFIID:GNOME_GTM", 
+				      0, NULL, &ev);
 
 	if (gtm_download_client != NULL)
 	{
