@@ -32,6 +32,8 @@
 #include "filepicker.h"
 #include "dialog.h"
 #include "mozilla_notifiers.h"
+#include "themes.h"
+#include "tabbutton.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -39,6 +41,7 @@
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
+#include <libgnomeui/gnome-preferences.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -52,13 +55,11 @@ static void add_bf_menu (GtkWidget *menu, char* label, int index,
 static GtkWidget * new_num_accel_menu_item (gint num, gchar *origtext, 
 					    gboolean lettersok, 
 					    GtkWidget *menu);
-static void save_url (GaleonEmbed *embed, char *target, 
-		      char* default_dir_pref, gboolean ask,
-		      gboolean main, gboolean ask_content);
 static void save_url_message (GaleonEmbed *embed, char *path,
 			      gboolean result);
 static void embed_select_new_page (GaleonEmbed *embed);
 static gboolean save_with_content (GaleonEmbed *embed, char *path, gboolean main);
+static void embed_free_stylesheet_menu_item (GtkMenuItem *mi, gpointer data);
 
 /** The global list of all GaleonEmbed structures */
 GList *all_embeds = NULL;
@@ -91,7 +92,8 @@ const gint embed_drop_types_num_items = (sizeof (embed_drop_types) /
  */
 GaleonEmbed *
 embed_create (GaleonEmbed *previous, gboolean new_window, 
-	      gboolean force_jump, guint32 chrome_mask)
+	      gboolean force_jump, gboolean raise_window,
+	      guint32 chrome_mask)
 {
 	GaleonWindow *window = NULL;
 	GaleonEmbed *embed;
@@ -117,7 +119,7 @@ embed_create (GaleonEmbed *previous, gboolean new_window,
 	}
 
 	/* add to window */
-	window_add_embed (window, embed, force_jump);
+	window_add_embed (window, embed, force_jump, raise_window);
 
 	/* return completed embed */
 	return embed;
@@ -135,7 +137,34 @@ embed_create_in_window (GaleonWindow *window)
 	embed = embed_create_no_window ();
 
 	/* add to window */
-	window_add_embed (window, embed, FALSE);
+	window_add_embed (window, embed, FALSE, TRUE);
+
+	/* return completed embed */
+	return embed;
+}
+
+/**
+ * embed_create_from_url_noraise: create a browser from a given url string
+ * without raising the window
+ */
+GaleonEmbed *
+embed_create_from_url_noraise (GaleonEmbed *previous, const gchar *url, 
+			       gboolean force_jump, gboolean new_window)
+{
+	GaleonEmbed *embed;
+
+	/* check argument */
+	g_assert(url != NULL);
+
+	/* create a window */
+	embed = embed_create (previous, new_window, force_jump, 
+			      FALSE, GTK_MOZ_EMBED_FLAG_ALLCHROME);
+
+	/* show the window */
+	embed_set_visibility (embed, TRUE);
+
+	/* load the url */
+	embed_load_url (embed, url);
 
 	/* return completed embed */
 	return embed;
@@ -155,7 +184,7 @@ embed_create_from_url (GaleonEmbed *previous, const gchar *url,
 
 	/* create a window */
 	embed = embed_create (previous, new_window, force_jump, 
-			      GTK_MOZ_EMBED_FLAG_ALLCHROME);
+			      TRUE, GTK_MOZ_EMBED_FLAG_ALLCHROME);
 
 	/* show the window */
 	embed_set_visibility (embed, TRUE);
@@ -235,16 +264,16 @@ embed_create_no_window (void)
 	/* add to global list of all embeds */
 	all_embeds = g_list_prepend (all_embeds, embed);
 
-	/* not closing */
-	embed->is_closing = FALSE;
-
 	/* no content for the location or title */
 	embed->site_location = NULL;
 	embed->site_title = g_strdup (_("Untitled"));
-	embed->site_title_utf8 = locale_to_utf8 (embed->site_title);
+	embed->site_title_utf8 = mozilla_locale_to_utf8 (embed->site_title);
 
 	embed->notebook_close_button = NULL;
+	embed->notebook_close_pixmap = NULL;
 
+	embed->usersheet = NULL;
+	
 	/* initial zoom level */
 	embed->zoom = 100;
 	
@@ -383,20 +412,31 @@ embed_load_url (GaleonEmbed *embed, const gchar *url)
 	embed_update_page_location (embed);
 
 	/* reset title */
-	g_free (embed->site_title);
+	/* Why is this necessary ? it causes
+	   bug 59577 -- Marco */
+/*	g_free (embed->site_title);
 	g_free (embed->site_title_utf8);
+	embed->site_title = NULL;
+	embed->site_title_utf8 = NULL;
 	embed->site_title = g_strdup (_("Untitled"));
-	embed->site_title_utf8 = locale_to_utf8 (embed->site_title);
-	embed_update_page_title (embed);
-
-	/* load the URL */
-	gtk_moz_embed_load_url (GTK_MOZ_EMBED(embed->mozEmbed), url);
+	embed->site_title_utf8 = mozilla_locale_to_utf8 (embed->site_title);
+	embed_update_page_title (embed);*/
 
 	/* initialise embed whenever a document is first loaded */
 	if (embed->wrapper == NULL)
 	{
 		embed_wrapper_init (embed);
 	}
+
+	/* load the URL */
+	gtk_moz_embed_load_url (GTK_MOZ_EMBED(embed->mozEmbed), url);
+}
+
+void
+embed_grab_focus (GaleonEmbed *embed)
+{
+	return_if_not_embed (embed);
+	gtk_widget_grab_focus (GTK_BIN(embed->mozEmbed)->child);
 }
 
 /**
@@ -553,9 +593,29 @@ embed_startup_init ()
 
 	/* only register the external ftp handler if it isn't galeon */
 	if (eel_gconf_get_integer(CONF_DOWNLOADING_EXTERNAL_FTP) == 1 &&
-		!(preferences_galeon_used_by_gnome_for_ftp ()))	{
+	    !preferences_galeon_used_by_gnome_for_protocol ("ftp"))
+	{
 		mozilla_register_FtpProtocolHandler ();
 	}
+
+	/* only register the external mailto handler if it isn't galeon.
+	   Note that this is startup only, as gnome-config does not
+	   support notifiers. If the user does the stupid thing while galeon
+	   is running, they're SOL. */
+	if (!preferences_galeon_used_by_gnome_for_protocol ("mailto"))
+	{
+		mozilla_register_MailtoProtocolHandler ();
+	}
+
+	/* optionally register galeon to handle ghelp protocols */
+	if (eel_gconf_get_boolean(CONF_GHELP_USE_GALEON) == TRUE ||
+	    preferences_galeon_used_by_gnome_for_protocol ("man") ||
+	    preferences_galeon_used_by_gnome_for_protocol ("info") ||
+	    preferences_galeon_used_by_gnome_for_protocol ("ghelp") ||
+	    preferences_galeon_used_by_gnome_for_protocol ("gnome-help"))
+	{
+		mozilla_register_gHelpProtocolHandlers ();
+	}
 }
 
 /**
@@ -582,7 +642,6 @@ embed_connect_signals (GaleonEmbed *embed)
 		{ "progress",        mozembed_progress_change_cb   },
 		{ "link_message",    mozembed_link_message_cb      },
 		{ "js_status",       mozembed_js_status_cb         },
-		{ "open_uri",        mozembed_open_uri_cb          },
 		{ "visibility",      mozembed_visibility_cb        },
 		{ "destroy_browser", mozembed_destroy_brsr_cb      },
 		{ "dom_mouse_down",  mozembed_dom_mouse_down_cb    },	
@@ -590,6 +649,7 @@ embed_connect_signals (GaleonEmbed *embed)
 		{ "dom_key_press",   mozembed_dom_key_press_cb     },
 		{ "size_to",         mozembed_size_to_cb           },
 		{ "new_window",      mozembed_new_window_cb        },
+		{ "security_change", mozembed_security_change_cb   },
 
 		/* terminator -- must be last in the list! */
 		{ NULL, NULL } 
@@ -708,37 +768,31 @@ void embed_set_image_as_background (GaleonEmbed *embed, gchar *url)
  */
 void embed_save_document (GaleonEmbed *embed, gboolean main)
 {
-	gchar *target = NULL;
+	gchar *url = NULL;
 		
 	/* get either frame or main document */
-	if (main)
-	{
-		target = mozilla_get_main_document_url (embed);
-	}
-	else 
-	{
-		target = mozilla_get_document_url (embed);
-	}
+	url = main ? mozilla_get_main_document_url (embed) :
+		mozilla_get_document_url (embed);
 
-	if (!target)
+	if (!url)
 	{
 		gnome_error_dialog (_("No document is loaded"));
 		return;
 	}
 
 	/* save it */
-	save_url (embed, target,  CONF_STATE_SAVE_DIR, 
+	save_url (embed, url, CONF_STATE_SAVE_DIR, 
 		  TRUE, TRUE, main);
 
 	/* free */
-	g_free (target);
+	g_free (url);
 }
 
 /**
  * save_url: save a given URL
  * FIXME pass main here is weird
  */
-static void 
+void 
 save_url (GaleonEmbed *embed, gchar *target, 
 	  gchar *default_dir_pref, gboolean ask,
 	  gboolean ask_content, gboolean main)
@@ -782,6 +836,7 @@ save_url (GaleonEmbed *embed, gchar *target,
 	if (ret == TRUE)
 	{
 		uri = gnome_vfs_uri_new (retPath);
+
 		if (uri)
 		{
 			retDir = gnome_vfs_uri_extract_dirname (uri);
@@ -924,19 +979,6 @@ embed_close (GaleonEmbed *embed)
 {
 	GaleonWindow *window;
 
-	/* already closing */
-	if (embed->is_closing)
-	{
-		return;
-	}
-	embed->is_closing = TRUE;
-	
-	/* remove the close timeout, if its active */
-	if (embed->close_timeout_id)
-	{
-		gtk_timeout_remove (embed->close_timeout_id);
-	}
-
 	/* check args */
 	window = embed->parent_window;
 	return_if_not_window (window);
@@ -947,6 +989,12 @@ embed_close (GaleonEmbed *embed)
 	/* stop any pending loads */
 	gtk_moz_embed_stop_load (GTK_MOZ_EMBED (embed->mozEmbed));
 
+	/* Give the focus to the new active embed */
+	if (window->active_embed && (window->active_embed != embed))
+	{
+		embed_grab_focus (window->active_embed);
+	}
+
 	/* destroy the embedding widget -- this will also destroy 
 	 * the notebook tab and its label, and remove it from 
 	 * the relevant lists */
@@ -954,7 +1002,7 @@ embed_close (GaleonEmbed *embed)
 }
 
 /**
- * embed_open_frame: open the frame pointed by the event target 
+ * embed_open_frame: open the focused frame
  */
 void 
 embed_open_frame (GaleonEmbed *embed, gboolean same_embed, gboolean new_window)
@@ -986,10 +1034,10 @@ embed_open_frame (GaleonEmbed *embed, gboolean same_embed, gboolean new_window)
  * embed_reload: call gtk_moz_embed_reload.
  */
 void
-embed_reload (GaleonEmbed *embed)
+embed_reload (GaleonEmbed *embed, int flags)
 {
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed->mozEmbed), 
-			      GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
+			      flags);
 }
 
 /**
@@ -1013,7 +1061,7 @@ embed_update_page_location (GaleonEmbed *embed)
 	return_if_not_window (window);
 
 	/* get the location string */
-	new_location = window->active_embed->site_location;
+	new_location = embed->site_location;
 
 	/* clear text */
 	window_clear_url_entry (window);
@@ -1024,7 +1072,6 @@ embed_update_page_location (GaleonEmbed *embed)
 	
 		/* update the drag location pixmap */
 		if (eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED) &&
-		    embed->parent_window->toolBarOn && 
 		    window->drag_pixmap != NULL)
 		{
 			drag_pixmap = bookmarks_get_siteicon (new_location);
@@ -1042,8 +1089,7 @@ embed_update_page_location (GaleonEmbed *embed)
 void
 embed_update_page_title (GaleonEmbed *embed)
 {
-	gchar *full_title;
-	gchar *title_string;
+	gchar *title_string, *full_title;
 	GaleonWindow *window;
 
 	/* set notebook label (although this might not be visible) */
@@ -1064,7 +1110,7 @@ embed_update_page_title (GaleonEmbed *embed)
 	if (!title_string) return;
 
 	/* format the full title */
-	full_title = g_strdup_printf (title_string, embed->site_title);
+	full_title = str_dup_replace (title_string, "%s", embed->site_title);
 
 	/* set the toplevel window title to the document title */
 	gtk_window_set_title (GTK_WINDOW (window->WMain), full_title);
@@ -1097,19 +1143,15 @@ embed_update_tab_title (GaleonEmbed *embed)
         switch (style)
         {
         case 0:
-		/* abbreviate the text label */
-                gtk_widget_set_usize (GTK_WIDGET (tab), -1, -1);
-                length = eel_gconf_get_integer (
-			CONF_TABS_TABBED_SHORTEN_CHARS);   
-                shortened = shorten_name (embed->site_title, length);
+		/* set fixed size of the label widget */
+               	shortened = embed->site_title;
                 break;
 
         case 1: 
-		/* set fixed size of the label widget */
-		length = eel_gconf_get_integer 
-			(CONF_TABS_TABBED_SHORTEN_POINTS);
-                gtk_widget_set_usize (GTK_WIDGET (tab), length, -2);
-               	shortened = embed->site_title;
+		/* abbreviate the text label */
+                length = eel_gconf_get_integer (
+			CONF_TABS_TABBED_SHORTEN_CHARS);   
+                shortened = shorten_name (embed->site_title, length);
                 break;
 
         default:
@@ -1122,21 +1164,61 @@ embed_update_tab_title (GaleonEmbed *embed)
 	{
 		gtk_label_set_text (GTK_LABEL (embed->notebook_label),
 				    shortened);
+		embed_update_tab_size (embed);
 	}
 
 	/* the menu text is the full text */
 	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (window->notebook),
 					  GTK_WIDGET (embed->mozEmbed),
 					  embed->site_title);
-	gtk_widget_queue_resize (window->notebook);
 
 	/* free allocated strings */
-	if (style == 0)
+	if (style == 1)
 	{
 		g_free (shortened);
 	}
 }
 
+/**
+ * embed_update_tab_size: updates tab sizing policies and size
+ */
+void
+embed_update_tab_size (GaleonEmbed *embed)
+{
+	GaleonWindow *window;
+	GtkWidget *tab;
+	gint length;
+	gint style;
+
+	/* get the parent window */
+	return_if_not_embed (embed);
+	window = embed->parent_window;
+	return_if_not_window (window);
+
+        /* shorten notebook's label */
+	tab = embed->notebook_hbox;
+	style = eel_gconf_get_integer (CONF_TABS_TABBED_SHORTEN_STYLE);
+        switch (style)
+        {
+        case 0:
+		length = eel_gconf_get_integer 
+			(CONF_TABS_TABBED_SHORTEN_POINTS);
+                gtk_widget_set_usize (GTK_WIDGET (tab), length, -2);
+                break;
+
+        case 1: 
+                gtk_widget_set_usize (GTK_WIDGET (tab), -1, -1);
+                break;
+
+        default:
+                g_assert_not_reached ();
+                return;
+        }
+
+	/* make sure everything is resized */
+	gtk_widget_queue_resize (window->notebook);
+}
+
 /**
  * embed_update_tab_status: sets the status for the notebook tab
  * label belonging to embed.  this means changing the color of the tab
@@ -1172,7 +1254,8 @@ embed_update_tab_status (GaleonEmbed *embed)
 	}
 
 	/* update closebutton status */
-	if (embed->notebook_close_button != NULL)
+	if (embed->notebook_close_button != NULL &&
+	    embed->notebook_close_pixmap != NULL)
 	{
 		gboolean make_close_insensitive;
 
@@ -1181,15 +1264,18 @@ embed_update_tab_status (GaleonEmbed *embed)
 
 		if (embed->is_active || !make_close_insensitive)
 		{
-			gtk_widget_set_sensitive (embed->notebook_close_button,
-						  TRUE);
+			gtk_widget_hide (embed->notebook_close_pixmap);
+			gtk_widget_show (embed->notebook_close_button);
 		}
 		else if (make_close_insensitive)
 		{
-			gtk_widget_set_sensitive (embed->notebook_close_button,
-						  FALSE);
+			gtk_widget_hide (embed->notebook_close_button);
+			gtk_widget_show (embed->notebook_close_pixmap);
 		}
 	}
+
+	gtk_object_set_data (GTK_OBJECT (embed->notebook_close_button),
+			     "tab_active", GINT_TO_POINTER (embed->is_active));
 }
 
 /**
@@ -1213,31 +1299,33 @@ embed_update_tab_close_button (GaleonEmbed *embed)
 	show = eel_gconf_get_boolean (CONF_TABS_TABBED_CLOSEBUTTON);
 
 	/* check for case when this is being destroyed */
-	if (!show && embed->notebook_close_button != NULL)
+	if (!show && (embed->notebook_close_button != NULL ||
+		      embed->notebook_close_pixmap != NULL))
 	{
 		gtk_widget_destroy (embed->notebook_close_button);
 		embed->notebook_close_button = NULL;
+		gtk_widget_destroy (embed->notebook_close_pixmap);
+		embed->notebook_close_pixmap = NULL;
 	}
 	/* check for case where button is being created */
-	else if (show && embed->notebook_close_button == NULL)
+	else if (show)
 	{
-		/* load the close pixmap if it's not already loaded */
-		if (close_pix == NULL)
+		GtkRequisition r;
+		if (embed->notebook_close_button != NULL)
 		{
-			/* load it */
-			close_pix = pixmap_data_new_from_file
-				(SHARE_DIR "/small-close.xpm");
-
-			/* cope with uninstalled case cleanly */
-			if (close_pix == NULL)
-			{
-				close_pix = g_new0 (PixmapData, 1);
-			}
+			gtk_widget_destroy (embed->notebook_close_button);
 		}
+		
+		if (embed->notebook_close_pixmap != NULL)
+		{
+			gtk_widget_destroy (embed->notebook_close_pixmap);
+		}
+		
+		close_pix = get_theme_pixmap ("small-close.xpm", FALSE);
 
 		/* build button */
-		button = embed->notebook_close_button = gtk_button_new ();
-		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+		button = embed->notebook_close_button = tab_button_new ();
+		tab_button_set_relief (TAB_BUTTON (button), GTK_RELIEF_NONE);
 		pixmap = gtk_pixmap_new (close_pix->pixmap, close_pix->mask);
 		gtk_widget_show (pixmap);
 		gtk_container_add (GTK_CONTAINER (button), pixmap);
@@ -1252,8 +1340,24 @@ embed_update_tab_close_button (GaleonEmbed *embed)
 				    	embed_tab_close_button_clicked_cb),
 				    embed);
 
-		/* show the button */
-		gtk_widget_show (button);
+		/* get button size */
+		gtk_widget_size_request (button, &r);
+		
+		/* build inactive close pixmap */
+		pixmap = embed->notebook_close_pixmap = 
+			gtk_pixmap_new (close_pix->pixmap, close_pix->mask);
+
+		/* give it the same size as the button, to prevent tab
+		 * resizing */
+		gtk_widget_set_usize (pixmap, r.width, r.height);
+
+		/* it's only used in inactive situations, so make it
+		 * insensitive */
+		gtk_widget_set_sensitive (pixmap, FALSE);
+
+		/* pack button into tab */
+		gtk_box_pack_start (GTK_BOX (embed->notebook_hbox), pixmap,
+				    FALSE, FALSE, 0);
 	}
 }
 
@@ -1685,3 +1789,152 @@ embed_select_new_page (GaleonEmbed *embed)
 	} 
 }
 
+void
+embed_init_css_menu (GaleonEmbed *embed)
+{
+	GaleonWindow *window = embed->parent_window;
+	GSList *radiogroup = NULL;
+	GList *node, *list = NULL;
+	GtkWidget *css_menu, *item;
+	gboolean sheets = FALSE, ticked = FALSE;
+	AlternateStyleSheet *current;
+	gint i;
+
+	return_if_not_embed (embed);
+	return_if_not_window (window);
+	if (!embed->mozEmbed) return;
+	if (embed->load_started > 0) return;
+
+	/* get stylesheet data */
+	list = mozilla_get_alternate_stylesheets (embed);
+	current = mozilla_get_selected_stylesheet (embed);
+
+	/* create menu */
+	css_menu = gtk_menu_new ();
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->csslist), css_menu);
+
+	/* tearoff.. */
+	if (gnome_preferences_get_menus_have_tearoff ())
+	{
+		GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
+		gtk_menu_append (GTK_MENU (css_menu), tearoff);
+		gtk_widget_show (tearoff);
+	}
+
+	/* process all sheets */
+	for (i = 1, node = list; node; node = node->next, i++)
+	{
+		AlternateStyleSheet *a = (AlternateStyleSheet *) node->data;
+		sheets = TRUE;
+
+		if (strcmp (a->name, "") == 0)
+		{
+			item = gtk_radio_menu_item_new_with_label
+					(radiogroup, _("Untitled"));
+		}
+		else
+		{
+			item = gtk_radio_menu_item_new_with_label
+					(radiogroup, a->name);
+		}
+		radiogroup = gtk_radio_menu_item_group 
+				(GTK_RADIO_MENU_ITEM (item));
+		gtk_menu_append (GTK_MENU(css_menu), item);
+		gtk_widget_lock_accelerators (item);
+		gtk_widget_show (item);
+		
+		gtk_object_set_data (GTK_OBJECT (item), "embed", embed);
+		gtk_object_set_data (GTK_OBJECT (item), "index",
+				     GINT_TO_POINTER (i));
+		gtk_signal_connect (GTK_OBJECT (item), "activate", 
+				    embed_change_stylesheet_cb, a);
+		gtk_signal_connect (GTK_OBJECT (item), "destroy", 
+				    embed_free_stylesheet_menu_item, a);
+		  
+		/* tick if this is the current sheet */
+		if      (current != NULL && a->sheet == current->sheet &&
+		         a->type == STYLESHEET_ALTERNATE && ticked == FALSE)
+		{
+			GTK_CHECK_MENU_ITEM (item)->active = ticked = TRUE;
+		}
+		else if (current != NULL && a->type == STYLESHEET_DEFAULT &&
+		         ticked == FALSE)
+		{
+			GTK_CHECK_MENU_ITEM (item)->active = ticked = TRUE;
+		}
+		else if (current == NULL && a->type == STYLESHEET_NONE &&
+		         ticked == FALSE)
+		{
+			GTK_CHECK_MENU_ITEM (item)->active = ticked = TRUE;
+		}
+		else
+		{
+			GTK_CHECK_MENU_ITEM (item)->active = FALSE;
+		}
+	}
+
+	/* set sensitivity according to whether any sheets are found */
+	gtk_widget_set_sensitive (window->csslist, sheets);
+
+	/* cleanup */
+	if (current)
+	{
+		g_free (current->name);
+		g_free (current);
+	}
+	g_list_free (list);
+}
+
+static void
+embed_free_stylesheet_menu_item (GtkMenuItem *mi, gpointer data)
+{
+	AlternateStyleSheet *a = (AlternateStyleSheet *) data;
+	g_free (a->name);
+	g_free (a);
+}
+
+void
+embed_toggle_image_blocking (const gchar *url)
+{
+	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (url);
+	const gchar *host;
+	GList *permissions, *l;
+	gboolean done = FALSE;
+
+	if (vfs_uri == NULL)
+	{
+		return;
+	}
+	
+	host = gnome_vfs_uri_get_host_name (vfs_uri);
+	if (host == NULL)
+	{
+		gnome_vfs_uri_unref (vfs_uri);
+		return;
+	}
+
+	permissions = mozilla_get_permissions (IMAGEPERMISSION);
+
+	for (l = permissions; l && done == FALSE; l = g_list_next (l))
+	{
+		CookieBase *b = (CookieBase *) l->data;
+		
+		if (strcmp (host, b->domain) == 0)
+		{
+			GList *remove = NULL;
+			remove = g_list_append (remove, b);
+			mozilla_remove_permissions (remove, IMAGEPERMISSION);
+			g_list_free (remove);
+			done = TRUE;
+		}
+
+		g_free (b->type);
+		g_free (b->domain);
+		g_free (b);
+	}
+
+	if (done == FALSE) mozilla_block_url (url);
+
+	gnome_vfs_uri_unref (vfs_uri);
+	g_list_free (permissions);
+}
