@@ -17,18 +17,17 @@
  */
 
 #include "galeon.h"
-#include "window.h"
 #include "mozilla.h"
 #include "embed.h"
 #include "misc.h"
-#include "prefs.h"
+#include "history.h"
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
-#include <libgnome/gnome-config.h>
-#include <libgnome/gnome-mime.h>
 #include <gtkmozembed.h>
 
 /* for nautilus dnd desktop items */
+#include <libgnome/gnome-mime.h>
 #include <gnome-xml/tree.h>
 #include <gnome-xml/parser.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
@@ -47,6 +46,7 @@ embed_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 	gchar *drop_uri;
 	gchar *parse_uri;
 	gchar *real_uri;
+	gchar **tmp;
 
 	/* get tab status */
 	tabbed = eel_gconf_get_boolean (CONF_TABS_TABBED);
@@ -54,6 +54,18 @@ embed_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 	switch (info)
 	{
 	case DND_TARGET_NETSCAPE_URL:
+		/* netscape format is: url \n title */
+		tmp = g_strsplit(data, "\n", 2);
+		if (tmp)
+		{
+			embed_load_url (embed, tmp[0]);
+		}
+		else
+		{
+			embed_load_url (embed, data);
+		}
+		g_strfreev (tmp);
+		break;
 	case DND_TARGET_STRING:
 	case DND_TARGET_GALEON_URL:
 		/* standard types are just strings */
@@ -120,9 +132,11 @@ save_site_location (GaleonEmbed *embed)
 {
 	GaleonWindow *window;
 	guchar *text, *stored;
+	int start, end, tmp;
 
 	/* get window */
 	window = embed->parent_window;
+	return_if_not_window (window);
 
 	/* if no location entry then don't change anything */
 	if (window->location_entry == NULL)
@@ -148,10 +162,21 @@ save_site_location (GaleonEmbed *embed)
 	}
 
 	/* get selected text */
+	start = GTK_EDITABLE (window->location_entry)->selection_start_pos;
+	end = GTK_EDITABLE (window->location_entry)->selection_end_pos;
+
+	/* switch them when they are in the opposite order,
+	   it happens if you select from right to left */
+	if (start > end)
+	{
+		tmp = start;
+		start = end;
+		end = tmp;
+	}
+
 	text = gtk_editable_get_chars 
 		(GTK_EDITABLE (window->location_entry), 
-		 GTK_EDITABLE (window->location_entry)->selection_start_pos,
-		 GTK_EDITABLE (window->location_entry)->selection_end_pos);
+		 start, end);
 
 	/* free up stored per-window selection */
 	stored = gtk_object_get_data (GTK_OBJECT (window->WMain), "selection");
@@ -165,3 +190,16 @@ save_site_location (GaleonEmbed *embed)
 	gtk_selection_owner_set (window->WMain, GDK_SELECTION_PRIMARY,
 				 GDK_CURRENT_TIME);
 }
+
+void
+embed_change_stylesheet_cb (GtkWidget *mi, AlternateStyleSheet *css)
+{
+	GaleonEmbed *embed =
+		(GaleonEmbed*) gtk_object_get_data (GTK_OBJECT (mi), "embed");
+	gint index =
+	       GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "index"));
+	return_if_not_embed (embed);
+
+	history_set_selected_authorcss (embed->site_location, index);
+	mozilla_set_alternate_stylesheet (embed, css);
+}
