@@ -17,7 +17,8 @@
  */
 
 void
-favicon_get_siteicon (GaleonEmbed *embed, gchar *url);
+favicon_get_siteicon (GaleonEmbed *embed, BookmarkItem *b,
+		      BookmarksEditorControls *editor);
 gchar *
 favicon_filename (gchar *favicon_url);
 
