@@ -18,16 +18,15 @@
 
 #include "galeon.h"
 #include "mozilla.h"
-#include "window.h"
-#include "misc.h"
 #include "find.h"
 #include "glade.h"
 #include "dialog.h"
+#include "prefs_strings.h"
+#include "eel-gconf-extensions.h"
 
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkentry.h>
 #include <libgnomeui/gnome-entry.h>
-#include <libgnomeui/gnome-winhints.h>
 
 /**
  * FindDialog: structure representing a "find in document" dialog
@@ -38,6 +37,9 @@ typedef struct
 	GtkWidget *entry;
 	GtkWidget *gnome_entry;
 	GtkWidget *case_checkbutton;
+	GtkWidget *word_checkbutton;
+	GtkWidget *wrap_checkbutton;
+	GtkWidget *frames_checkbutton;
 	GtkWidget *prev_button;
 	GtkWidget *next_button;
 	gboolean string_changed;
@@ -58,12 +60,14 @@ void find_next_button_clicked_cb (GtkButton *button, FindDialog *find_dialog);
 void find_prev_button_clicked_cb (GtkButton *button, FindDialog *find_dialog);
 void find_close_button_clicked_cb (GtkButton *button, FindDialog *find_dialog); 
 void find_destroy_cb (GtkObject *object, FindDialog *find_dialog);
-void find_case_check_toggled_cb (GtkToggleButton *togglebutton, FindDialog *find_dialog);
+void reset_find_on_check_toggled_cb (GtkToggleButton *togglebutton, FindDialog *find_dialog);
 void find_entry_activate_cb (GtkEditable *editable, FindDialog *find_dialog);
 void find_entry_changed_cb  (GtkEditable *editable, FindDialog *find_dialog);
-void find_reset (FindDialog *find_dialog);
-gboolean find_go (FindDialog *find_dialog, 
-		  gboolean backwards);
+static void find_reset (FindDialog *find_dialog);
+static gboolean find_go (FindDialog *find_dialog, 
+			 gboolean backwards);
+static void find_lookup_widgets (FindDialog *find, GladeXML *gxml);
+static void find_controls_set_sensitive (FindDialog *find);
 
 /**
  * embed_show_find_dialog: show the find dialog
@@ -77,23 +81,17 @@ void find_show_dialog (GaleonWindow *window)
 	
 	if (window->find_dialog == NULL)
 	{
+		/* Create the dialog and initialize data*/
 		find = g_new0 (FindDialog, 1);
-
 		find->window = window;
-		
 		gxml = glade_widget_new ("galeon.glade", "find_dialog", 
 					 &(find->dialog), find);
-		
-		find->gnome_entry = glade_xml_get_widget (gxml, 
-							  "find_gnome_entry");
-		find->entry = glade_xml_get_widget (gxml, "find_entry");
-		find->prev_button = glade_xml_get_widget (gxml, "BPrev");
-		find->next_button = glade_xml_get_widget (gxml, "BNext");
-		find->case_checkbutton = glade_xml_get_widget (gxml, 
-							       "case_check");
-
+		find_lookup_widgets (find, gxml);		
+		window->find_dialog = find->dialog;
+		find->string_changed = TRUE;
 		gtk_object_unref (GTK_OBJECT (gxml));
 
+		/* HACK Set last search text */
 		if (GNOME_ENTRY(find->gnome_entry)->items &&
 		    GNOME_ENTRY(find->gnome_entry)->items->data)
 		{
@@ -105,9 +103,7 @@ void find_show_dialog (GaleonWindow *window)
 					0, -1);
 		}
 
-		find->string_changed = TRUE;
-
-		window->find_dialog = find->dialog;
+		find_controls_set_sensitive (find);
 
 		dialog_set_parent (find->dialog, window->WMain);
 	}
@@ -118,6 +114,41 @@ void find_show_dialog (GaleonWindow *window)
 	}
 }
 
+static void
+find_lookup_widgets (FindDialog *find, GladeXML *gxml)
+{
+	find->gnome_entry = glade_xml_get_widget (gxml, 
+						  "find_gnome_entry");
+	find->entry = glade_xml_get_widget (gxml, "find_entry");
+	find->prev_button = glade_xml_get_widget (gxml, "BPrev");
+	find->next_button = glade_xml_get_widget (gxml, "BNext");
+	find->case_checkbutton = glade_xml_get_widget (gxml, 
+						       "case_check");
+	find->word_checkbutton = glade_xml_get_widget (gxml, 
+						       "word_check");
+	find->wrap_checkbutton = glade_xml_get_widget (gxml, 
+						       "wrap_check");
+	find->frames_checkbutton = glade_xml_get_widget (gxml, 
+							 "frames_check");
+}
+
+static void
+find_controls_set_sensitive (FindDialog *find)
+{
+	gtk_toggle_button_set_active
+		(GTK_TOGGLE_BUTTON (find->case_checkbutton),
+		 eel_gconf_get_boolean (CONF_FIND_MATCH_CASE));
+	gtk_toggle_button_set_active
+		(GTK_TOGGLE_BUTTON (find->word_checkbutton),
+		 eel_gconf_get_boolean (CONF_FIND_MATCH_WORD));
+	gtk_toggle_button_set_active
+		(GTK_TOGGLE_BUTTON (find->frames_checkbutton),
+		 eel_gconf_get_boolean (CONF_FIND_SEARCH_IN_FRAMES));
+	gtk_toggle_button_set_active
+		(GTK_TOGGLE_BUTTON (find->wrap_checkbutton),
+		 !eel_gconf_get_boolean (CONF_FIND_AUTOWRAP));
+}
+
 void find_destroy_dialog (GaleonWindow *window)
 {
 	gtk_widget_destroy (window->find_dialog);
@@ -166,15 +197,31 @@ find_close_button_clicked_cb (GtkButton *button, FindDialog *find_dialog)
 void
 find_destroy_cb (GtkObject *object, FindDialog *find_dialog)
 {
+	/* save options */
+	eel_gconf_set_boolean (CONF_FIND_MATCH_CASE,
+		gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (find_dialog->case_checkbutton)));
+	eel_gconf_set_boolean (CONF_FIND_MATCH_WORD,
+		gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (find_dialog->word_checkbutton)));
+	eel_gconf_set_boolean (CONF_FIND_SEARCH_IN_FRAMES,
+		gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (find_dialog->frames_checkbutton)));
+	eel_gconf_set_boolean (CONF_FIND_AUTOWRAP,
+		!gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (find_dialog->wrap_checkbutton)));
+
 	find_dialog->window->find_dialog = NULL;
 	g_free (find_dialog);
 }
 
 /**
- * find_case_check_toggled_cb: case sensitive check button clicked
+ * reset_find_on_check_toggled_cb: Reset the find process if one of the checkbox
+ * settings has changed.
  */
 void 
-find_case_check_toggled_cb (GtkToggleButton *togglebutton, FindDialog *find_dialog)
+reset_find_on_check_toggled_cb (GtkToggleButton *togglebutton,
+				FindDialog *find_dialog)
 {
 	find_reset (find_dialog);
 }
@@ -203,15 +250,16 @@ find_entry_changed_cb  (GtkEditable *editable, FindDialog *find_dialog)
 	find_reset (find_dialog);
 }
 
-void
+static void
 find_reset (FindDialog *find_dialog)
 {
 	gtk_widget_set_sensitive (find_dialog->next_button, TRUE);
 	gtk_widget_set_sensitive (find_dialog->prev_button, TRUE);
 }
 
-gboolean find_go (FindDialog *find_dialog, 
-		  gboolean backwards)
+static gboolean 
+find_go (FindDialog *find_dialog, 
+	 gboolean backwards)
 {
 	WrapperSearchProperties *properties;
 	gchar *search_string;
@@ -237,6 +285,12 @@ gboolean find_go (FindDialog *find_dialog,
 	properties->backwards =  backwards;
 	properties->match_case = gtk_toggle_button_get_active 
 		(GTK_TOGGLE_BUTTON (find_dialog->case_checkbutton));
+	properties->wrap = !gtk_toggle_button_get_active 
+		(GTK_TOGGLE_BUTTON (find_dialog->wrap_checkbutton));
+	properties->entire_word = gtk_toggle_button_get_active 
+		(GTK_TOGGLE_BUTTON (find_dialog->word_checkbutton));
+	properties->search_frames = gtk_toggle_button_get_active 
+		(GTK_TOGGLE_BUTTON (find_dialog->frames_checkbutton));
 
 	/* store the search string in the find history */
 	if (find_dialog->string_changed)
@@ -271,9 +325,17 @@ find_next (GaleonEmbed *embed, const gchar *string)
 	properties = g_new0 (WrapperSearchProperties, 1);
 	properties->search_string = g_strdup (string);
 	properties->backwards = FALSE;
-	properties->match_case = FALSE;
+	properties->match_case =
+		eel_gconf_get_boolean (CONF_FIND_MATCH_CASE);
+	properties->entire_word =
+		eel_gconf_get_boolean (CONF_FIND_MATCH_WORD);
+	properties->search_frames =
+		eel_gconf_get_boolean (CONF_FIND_SEARCH_IN_FRAMES);
+	properties->wrap =
+		eel_gconf_get_boolean (CONF_FIND_AUTOWRAP);
 
 	result = mozilla_find (embed, properties);
+
 	g_free (properties->search_string);
 	g_free (properties);
 
