@@ -33,11 +33,12 @@
 #include <gtk/gtkdnd.h>
 
 /* forward declarations of types */
-typedef struct _GaleonWindow GaleonWindow;
-typedef struct _GaleonEmbed  GaleonEmbed;
-typedef struct _BookmarkItem BookmarkItem;
-typedef struct _HistoryHost  HistoryHost;
-typedef struct _HistoryItem  HistoryItem;
+typedef struct _GaleonWindow             GaleonWindow;
+typedef struct _GaleonEmbed              GaleonEmbed;
+typedef struct _BookmarkItem             BookmarkItem;
+typedef struct _HistoryHost              HistoryHost;
+typedef struct _HistoryItem              HistoryItem;
+typedef struct _BookmarksEditorControls  BookmarksEditorControls;
 
 
 /**
@@ -78,6 +79,7 @@ struct _GaleonEmbed
 	GtkWidget *notebook_hbox;                /* packing box           */
 	GtkWidget *notebook_label;               /* the label             */
 	GtkWidget *notebook_close_button;        /* active close button   */
+	GtkWidget *notebook_close_pixmap;	 /* inactive close pixmap */
 
 	/* C++ helper, a "GaleonWrapper" */
         gpointer wrapper;
@@ -108,9 +110,6 @@ struct _GaleonEmbed
 	/* is this browser actually visible */
 	gboolean is_visible : 1;
 
-	/* is this embed in the process of closing? */
-	gboolean is_closing : 1;
-
 	/* ignore mime processing of next url */
 	gboolean mime_ignore_once : 1;
 
@@ -120,13 +119,24 @@ struct _GaleonEmbed
 	/* is this a xul dialog ? */
 	gboolean is_chrome : 1;
 
+	/* secure page ? */
+	gboolean secure : 1;
+
+	/* user stylesheet */
+	gpointer usersheet;
+	
+	/* has user stylesheet been applied ? */
+	gboolean usercss_applied;
+
 	/* how this embed was focused if applicable */
 	FocusType focus_type;
-
-	/* close timeout to ensure all mozembed callbacks have finished */
-	gint close_timeout_id;
 };
 
+#define	MAIN_TOOLBAR 1
+#define	BOOKMARKS_TOOLBAR 2
+#define	TABS_APPEAREANCE 4
+#define	AUTOBOOKMARKS 8
+
 /**
  * GaleonWindow: this is the type of Galeon windows, i.e. a wrapping of
  * the toplevel GnomeApp window. A single window can contain many 
@@ -144,8 +154,8 @@ struct _GaleonWindow
 	gint visible_embeds; /* number of visible GaleonEmbeds, used to
 			      * track whether or not the window is visible */
 
-	/* number of embeds which are running (for spinner) */
-	gint n_running;
+	/* true if the spinner is running; false otherwise */
+	gboolean spinner_running;
 
 	/* assigned size */
 	gboolean set_size : 1;
@@ -162,6 +172,13 @@ struct _GaleonWindow
 	GtkWidget *hbox;
 	GtkWidget *docked;                /* currently docked control      */
 	const gchar *docked_width_pref;   /* where width of dock is stored */
+	
+	/* These specify the non-fullscreen *bar state, so we can switch
+	 * back from fullscreen properly */
+	gboolean show_menubar   : 1;
+	gboolean show_toolbar   : 1;
+	gboolean show_bookmarks : 1;
+	gboolean show_statusbar : 1;
 
 	/* Toolbar components */
 	GtkWidget *dock;
@@ -190,6 +207,9 @@ struct _GaleonWindow
 	GtkWidget *spinner;               /* the GnomeAnimator spinner */
 	GtkWidget *history_button;
 	GtkWidget *bookmarks_button;
+	GtkWidget *back_history_button;
+	GtkWidget *forward_history_button;
+	GtkWidget *clear_location_button;
 	
 	/* menus */
 	GtkWidget *file_menu;
@@ -216,15 +236,25 @@ struct _GaleonWindow
 	GtkWidget *animate_always;
 	GtkWidget *animate_once_through;
 	GtkWidget *animate_never;
+	GtkWidget *no_proxy;
+	GtkWidget *manual_proxy;
+	GtkWidget *autoproxy;
 	GtkWidget *use_own_fonts;
 	GtkWidget *use_own_colors;
 	GtkWidget *enable_java;
 	GtkWidget *enable_javascript;
-	GtkWidget *enable_proxy;
-	GtkWidget *autojump;
+	GtkWidget *allow_popups;
 	GtkWidget *add_bookmark;
 	GtkWidget *bookmarks_separator;
 	GtkWidget *encoding;
+	GtkWidget *csslist;
+	GtkWidget *java_console;
+	GtkWidget *back_menuitem;
+	GtkWidget *forward_menuitem;
+	GtkWidget *up_menuitem;
+	GtkWidget *refresh_menuitem;
+	GtkWidget *homepage_menuitem;
+
 	/* edit menu menuitems */
 	GtkWidget *edit_cut;
 	GtkWidget *edit_copy;
@@ -235,6 +265,10 @@ struct _GaleonWindow
 	GtkWidget *find_dialog;
 	GtkWidget *open_url_entry;
 
+	/* security status */
+	GtkWidget *security_frame;
+	GtkWidget *security_icon;
+
 	/* list of toolbars */
 	GList *bookmarks_toolbars; 
 	GtkTooltips *bookmarks_tooltips;
@@ -263,6 +297,12 @@ struct _GaleonWindow
 	/* used for draggable tabs */
 	gint drag_motion_signal;
 	gint drag_release_signal;
+
+	/* list of dummy widgets used to bind accelerators to */
+	GList *dummy_widgets;
+
+	/* window appeareance settings that need UI update */
+	int to_update;
 };
 
 /**
@@ -307,7 +347,7 @@ typedef struct
 } WidgetLookup;
 
 /**
- * KeyCode: key pressed char codes, taken from mozilla
+ * MozillaKeyCode: key pressed char codes, taken from mozilla
  */
 typedef enum
 {
@@ -456,10 +496,10 @@ typedef enum
 	DOM_VK_LEFT_2 = 65361,   /* added */
 	DOM_VK_KP_RIGHT = 65363, /* added */
 	DOM_VK_KP_LEFT = 65430   /* added */
-} KeyCode;
+} MozillaKeyCode;
 
 /**
- * KeyModifier: key modifiers
+ * MozillaKeyModifier: key modifiers
  */
 typedef enum
 {
@@ -468,7 +508,7 @@ typedef enum
 	SHIFT_KEY = (1 << 2),
 	META_KEY  = (1 << 3),
 	KEY_CODE  = (1 << 4)
-} KeyModifier;
+} MozillaKeyModifier;
 
 /**
  * BookmarkTitle: type of behaviour when making a new bookmark
@@ -489,15 +529,6 @@ typedef enum
 	STARTPAGE_BLANK,
 } StartPageType;
 
-/**
- * HomePageType: home page type
- */
-typedef enum
-{
-	HOMEPAGE_URL,
-	HOMEPAGE_MYPORTAL
-} HomePageType;
-
 /**
  * MimeAction: types of actions to take for opening files
  */
@@ -569,10 +600,10 @@ typedef enum
  */
 typedef enum
 {
-	VIEWSTATE_NONE,
-	VIEWSTATE_DEFAULT,
+	VIEWSTATE_EXISTING,
 	VIEWSTATE_WINDOW,
-	VIEWSTATE_TAB
+	VIEWSTATE_TAB,
+	VIEWSTATE_NORAISE_TAB
 } ViewState;
 
 /**
@@ -639,8 +670,8 @@ typedef struct
 	GtkWidget *sidebar;
 	GtkWidget *themes_page;
 	GtkWidget *spinner_page;
-	GtkWidget *iconlist;
-	GtkWidget *themelist;
+	GtkWidget *spinner_iconlist;
+	GtkWidget *theme_iconlist;
 	GtkWidget *theme_dir_entry;
 	GtkWidget *smart_bm_example_entry;
 	GtkWidget *lang_entry;/*FIXME*/
@@ -659,6 +690,7 @@ typedef struct
 	GtkWidget *smart_bm_entry_width_spin;
 	GtkWidget *smart_bm_goto_button_checkbutton;
 	GtkWidget *smart_bm_history_checkbutton;
+	GtkWidget *smart_bm_clear_checkbutton;
 	GtkWidget *java_checkbutton;
 	GtkWidget *javascript_checkbutton;
 	GtkWidget *user_agent_entry;
@@ -677,17 +709,21 @@ typedef struct
 	gint mime_last_sort_column;
 	GtkSortType mime_last_sort_type;
 	GtkWidget *mime_list;
+	GtkWidget *mime_delete_button;
 	GtkWidget *mime_save_to_disk;
 	GtkWidget *mime_helper_frame;
-	GtkWidget *mime_name_entry;
+	GtkWidget *mime_type_label;
+	GtkWidget *mime_type_entry;
 	GtkWidget *mime_helper_entry;
 	GtkWidget *mime_always_use;
 	GtkWidget *mime_terminal;
 	GtkWidget *mime_url_helper;
+	GtkWidget *mime_action_frame;
 
 	/* downloader page */
 	GtkWidget *download_use_builtin;
 	GtkWidget *download_command_entry;
+	GtkWidget *download_command_label;
 	GtkWidget *download_command_combo;
 	GtkWidget *download_run_in_terminal;
 	GtkWidget *download_autodownload;
@@ -703,10 +739,11 @@ typedef struct
 	GtkWidget *tabbed_always_show_check;
 	GtkWidget *tabbed_closebutton_check;
 	GtkWidget *tabbed_closebutton_insensitive_check;
+	GtkWidget *tabbed_shorten_name_vbox;
 	GtkWidget *tabbed_shorten_spin;
 	GtkWidget *tabbed_shorten_spin2;
 	GtkWidget *tabbed_position_optionmenu;
-	GtkWidget *tabbed_style_radiobutton1;
+	GtkWidget *shorten_target_radiobutton;
 	GtkWidget *tabbed_separator_check;
 	GtkWidget *tabbed_vowel_check;
 	GtkWidget *tabbed_prefix_check;
@@ -716,7 +753,11 @@ typedef struct
 	/* ... */
 	GtkWidget *autocompletion_enabled_checkbutton;
 	GtkWidget *autocompletion_sort_whole;
-        GtkWidget *proxy_use_checkbutton;
+        GtkWidget *proxy_none_radiobutton;
+        GtkWidget *proxy_manual_radiobutton;
+        GtkWidget *proxy_auto_radiobutton;
+        GtkWidget *proxy_auto_url_entry;
+        GtkWidget *proxy_auto_reload_button;
 	GtkWidget *http_proxy_entry;
 	GtkWidget *ssl_proxy_entry;
 	GtkWidget *ftp_proxy_entry;
@@ -732,6 +773,7 @@ typedef struct
 	GtkWidget *fantasy_font_fpicker;
 	GtkWidget *monospace_font_fpicker;
 	GtkWidget *minimum_font_size_spin;
+	GtkWidget *user_stylesheet_entry;
 	GtkWidget *default_font_optionmenu;
 	GtkWidget *history_expire_spin;
 	GtkWidget *use_own_fonts_checkbutton;
@@ -740,8 +782,16 @@ typedef struct
 	GtkWidget *show_bookmarks_in_fullscreen_check;
 	GtkWidget *show_menubar_in_fullscreen_check;
 	GtkWidget *show_statusbar_in_fullscreen_check;
+	GtkWidget *show_toolbars_check;
+	GtkWidget *show_bookmarks_check;
+	GtkWidget *show_menubar_check;
+	GtkWidget *show_statusbar_check;
+	GtkWidget *show_history_dock_check;
+	GtkWidget *show_bookmarks_dock_check;
 	GtkWidget *use_external_source_viewer_check;
+	GtkWidget *legacy_mailer_entry;
 	GtkWidget *external_source_viewer_entry;
+	GtkWidget *use_internal_ghelp_check;
 	GtkWidget *crash_optionmenu;
 	GtkWidget *allow_popups_checkbutton;
 	GtkWidget *statusbar_rewrite_check;
@@ -833,6 +883,9 @@ struct _HistoryHost
 	/* stored zoom for this host */
 	gint zoom;
 
+	/* stored selected author stylesheet for this host */
+	gint selected_authorcss;
+
 	/* dominant child -- the location visited the most of this server */
 	HistoryItem *dominant_item;
 };
@@ -849,6 +902,7 @@ struct _HistoryItem
 	GTime first;
 	GTime last;
 	gint visits;
+	gint selected_authorcss;
 
 	/* parent host */
 	HistoryHost *host;
@@ -876,6 +930,9 @@ typedef enum
 	TOOLBAR_ITEM_BOOKMARKS,
 	TOOLBAR_ITEM_HISTORY,
 	TOOLBAR_ITEM_PRINT,
+	TOOLBAR_ITEM_BACK_HISTORY,
+	TOOLBAR_ITEM_FORWARD_HISTORY,
+	TOOLBAR_ITEM_CLEAR_LOCATION,
 
 	/* must be last */
 	TOOLBAR_ITEM_COUNT
@@ -999,7 +1056,7 @@ struct _BookmarkItem
 	BookmarkItem *alias;
 
 	/* a unique identifier */
-	glong id;
+	gulong id;
 
 	/* time info (if they are 0, then they should be ignored) */
 	GTime time_added;
@@ -1008,8 +1065,54 @@ struct _BookmarkItem
 
 	/* toolbar style */
 	ToolbarStyle toolbar_style;
+
+	/* accel data */
+	guint accel_key;
+	GdkModifierType accel_mods;
 };
 
+/**
+ * BookmarksSearchIn: Search in bookmarks "search in"
+ */
+typedef enum {
+	BOOKMARKS_SEARCH_IN_NAME,
+	BOOKMARKS_SEARCH_IN_NICKS,
+	BOOKMARKS_SEARCH_IN_NOTES,
+	BOOKMARKS_SEARCH_IN_URL
+} BookmarksSearchIn;
+
+/**
+ * BookmarksSearchMethod: Search in bookmarks search method
+ */
+typedef enum {
+	BOOKMARKS_SEARCH_METHOD_CONTAINS,
+	BOOKMARKS_SEARCH_METHOD_STARTS_WITH,
+	BOOKMARKS_SEARCH_METHOD_ENDS_WITH,
+	BOOKMARKS_SEARCH_METHOD_IS,
+	BOOKMARKS_SEARCH_METHOD_IS_NOT,
+	BOOKMARKS_SEARCH_METHOD_DOES_NOT_CONTAIN
+} BookmarksSearchMethod;
+
+/**
+ * BookmarksFindDialog: Find in bookmarks dialog
+ */
+typedef struct {
+	/* widgets */
+	GtkWidget *dialog;
+	GtkWidget *prev_button;
+	GtkWidget *next_button;
+	GtkWidget *criterium1_optionmenu;
+	GtkWidget *criterium2_optionmenu;
+	GtkWidget *gnomeentry;
+	GtkWidget *entry;
+
+	/* data */
+	gchar *str;
+	BookmarksSearchIn search_in;
+	BookmarksSearchMethod search_method;
+	BookmarksEditorControls *controls;
+} BookmarksFindDialog;
+
 /**
  * BookmarksEditorControlsType: types of bookmark editor dialogs
  */
@@ -1022,7 +1125,7 @@ typedef enum
 /**
  * BookmarksEditorControls: representation of a bookmarks editor.
  */
-typedef struct
+struct _BookmarksEditorControls
 {
 	BookmarksEditorControlsType type;
 
@@ -1046,6 +1149,7 @@ typedef struct
 	GtkWidget *create_toolbar_toggle;
 	GtkWidget *create_context_menu_toggle;
 	GtkWidget *notes_text;
+	GtkWidget *search_entry;
 	GtkWidget *time_added_entry;
 	GtkWidget *time_modified_entry;
 	GtkWidget *time_visited_entry;
@@ -1066,7 +1170,13 @@ typedef struct
 	/* the clipboard data owned by this editor, if any */
 	gchar *clipboard;
 
-} BookmarksEditorControls;
+	/* the position in the ctree of the search */
+	GList *search_position;
+
+	/* find dialog */
+	BookmarksFindDialog *find_dialog;
+
+};
 
 /**
  * Pair of controls / tree_item for a bookmark
@@ -1101,7 +1211,8 @@ typedef enum
 } DnDType;
 
 /**
- * CookieBase: FIXME?
+ * CookieBase: FIXME
+ * WARNING: if you modify this you must modify also Password
  */
 typedef struct
 {
@@ -1132,6 +1243,7 @@ typedef struct
 	gchar *host;
 	gchar *username;
 	gboolean remove;
+	gboolean block;
 } Password;
 
 /**
@@ -1155,17 +1267,6 @@ typedef enum
 	PASSWORDSITES = 4
 } PDMPage;
 
-/**
- * WheelModKey: modifiers that can be used in conjuction with the mouse wheel
- */
-typedef enum
-{
-	WHEEL_MOD_NOKEY,
-	WHEEL_MOD_ALT,
-	WHEEL_MOD_CTRL,
-	WHEEL_MOD_SHIFT
-} WheelModKey;
-
 /**
  * Downloader: the choice between the builtin (mozilla) downloader, the
  * external GTM downloader (via CORBA), or using some external command.
@@ -1201,6 +1302,89 @@ typedef enum
 	PREFS_PAGE_COUNT /* must be last */
 } PreferencesPageId;
 
+/**
+ * AlternateStyleSheetType: Kind of stylesheet
+ */
+typedef enum
+{
+	STYLESHEET_NONE,
+	STYLESHEET_DEFAULT,
+	STYLESHEET_ALTERNATE
+} AlternateStyleSheetType;
+
+/**
+ * AlternateStyleSheet: An alternate stylesheet
+ */
+typedef struct
+{
+	gchar *name;    /* stylesheet name */
+	gpointer sheet; /* pointer to the mozilla sheet itself */
+	AlternateStyleSheetType type;
+} AlternateStyleSheet;
+
+/**
+ * FiltersDialog: A filters dialog
+ */
+typedef struct
+{
+	/* widgets */
+	GtkWidget *dialog;
+	GtkWidget *vbox;
+
+	/* list of filters */
+	GList *filters;
+
+	/* parent */
+	GaleonWindow *window;
+} FiltersDialog;
+
+/**
+ * FilterMode: Filtering mode
+ */
+typedef enum
+{
+	FILTER_ALLOW,
+	FILTER_BLOCK
+} FilterMode;
+
+/**
+ * Filter: A filter for the filters dialog
+ */
+typedef struct
+{
+	/* widgets */
+	GtkWidget *vbox;
+	GtkWidget *button;
+	GtkWidget *contents;
+	GtkWidget *allow_radiobutton;
+	GtkWidget *block_radiobutton;
+	GtkWidget *clist;
+
+	/* filter name */
+	gchar *name;
+
+	/* short filter name, for in filters.js */
+	gchar *jsname;
+
+	/* list of javascript commands in the form "command access" */
+	GList *jscommands;
+
+	/* filter mode */
+	FilterMode mode;
+	
+	/* list of blocked/allowed sites */
+	GList *sites;
+
+	/* selection */
+	GList *selection;
+
+	/* is filter open? */
+	gboolean shown;
+	
+	/* parent */
+	FiltersDialog *dialog;
+} Filter;
+
 /**
  * FileCallbackFunc: the type of functions used in file callbacks
  */
