@@ -28,6 +28,7 @@
 #include "glade.h"
 #include "dialog.h"
 #include "eel-gconf-extensions.h"
+#include "mozilla.h"
 
 #include <string.h>
 #include <gtk/gtkoptionmenu.h>
@@ -40,11 +41,11 @@
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
-
-/* GNOME includes */
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
+#include <time.h>
+
 /**
  * The history hash table. A hash table is used because you have to make 
  * an url lookup in history each time you try to add or update a record
@@ -129,7 +130,6 @@ static void  history_save_host (gpointer key, gpointer value,
 static void history_save_item (gpointer key, gpointer value,
 			       gpointer user_data);
 static void eputs (guchar *text, FILE *fp);
-static gchar *decode_entity (const unsigned char *encoded);
 static void history_add_server_to_list (gpointer key, HistoryHost *host,
 					GList **list);
 static void tree_sort (GtkCTree *ctree, GtkCTreeNode *node, gpointer data);
@@ -144,6 +144,9 @@ static void history_drag_data_get_cb (GtkWidget *widget,
 				      HistoryView *hv);
 void 
 window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data);
+void 
+window_menu_recent_release_cb (GtkMenuItem *menuitem, GdkEventButton *event,
+			       gpointer user_data);
 static void history_notifiers_init(HistoryView *hv);
 
 /**
@@ -598,6 +601,7 @@ history_update_item_view (HistoryView *hv, HistoryItem *hi)
 	gchar last[256], first[256], visits[30];
 	gchar *text[6] = { NULL, NULL, last, first, visits, NULL };
 	GtkCTreeNode *item_node, *host_node;
+	const PixmapData *bm_icon;
 	gint i;
 
 	/* freeze the tree */
@@ -629,11 +633,14 @@ history_update_item_view (HistoryView *hv, HistoryItem *hi)
 		/* make strings for item */
 		history_make_item_strings (hi, text);
 
+		/* make pixmap for item */
+		bm_icon = bookmarks_get_siteicon (hi->url);
+		
 		/* update in tree */
 		gtk_ctree_node_set_pixtext (GTK_CTREE (hv->ctree),
 					    item_node, 0, text[0], 2,
-					    site_pixmap_data->pixmap,
-					    site_pixmap_data->mask);
+					    bm_icon->pixmap,
+					    bm_icon->mask);
 		for (i = 1; i < 5; i++)
 		{
 			gtk_ctree_node_set_text (GTK_CTREE (hv->ctree),
@@ -701,16 +708,17 @@ history_add_host (const gchar *name)
 	host = g_new0 (HistoryHost, 1);
 
 	/* fill in structure */
-	host->title_locale  = g_strdup (name);
-	host->title_utf8    = g_strdup (name);
-	host->url           = g_strdup (name);
-	host->node          = NULL;
-	host->sibling       = NULL;
-	host->zoom          = 0;
-	host->first         = 0;
-	host->last          = 0;
-	host->visits        = 0;
-	host->dominant_item = NULL;
+	host->title_locale       = g_strdup (name);
+	host->title_utf8         = g_strdup (name);
+	host->url                = g_strdup (name);
+	host->node               = NULL;
+	host->sibling            = NULL;
+	host->zoom               = 0;
+	host->selected_authorcss = -1;
+	host->first              = 0;
+	host->last               = 0;
+	host->visits             = 0;
+	host->dominant_item      = NULL;
 
 	/* add to the hosts hashtable */
 	g_hash_table_insert (history_hosts, host->url, host);
@@ -855,6 +863,7 @@ history_show_dialog_add_item (gpointer key, gpointer value, gpointer user_data)
 	GtkCTreeNode *real_sibling;
 	HistoryHost *host = NULL;
 	gchar *title_locale;
+	const PixmapData *bm_icon;
 
 	/* check if this url passes the filter */
 	title_locale = (hi->title_locale != NULL ? 
@@ -918,13 +927,16 @@ history_show_dialog_add_item (gpointer key, gpointer value, gpointer user_data)
 	/* make strings */
 	history_make_item_strings (hi, text);
 
+	/* fetch icon */
+	bm_icon = bookmarks_get_siteicon (hi->url);
+	
 	/* add node to tree */
 	node = gtk_ctree_insert_node (GTK_CTREE (hv->ctree), parent_node,
 				      parent_node ? NULL : *sibling, text, 2, 
-				      site_pixmap_data->pixmap,
-				      site_pixmap_data->mask, 
-				      site_pixmap_data->pixmap,
-				      site_pixmap_data->mask, 
+				      bm_icon->pixmap,
+				      bm_icon->mask, 
+				      bm_icon->pixmap,
+				      bm_icon->mask, 
 				      TRUE, TRUE);
 
 	/* set data so we can find the item when we dbl click / do compares */
@@ -1029,16 +1041,17 @@ history_add_item (gchar *url, gchar *title_utf8, GTime first,
 	host = history_get_host (url);
 
 	/* fill in the fields */
-	hi->host       = host;
-	hi->url        = url;
-	hi->title_utf8 = title_utf8;
-	hi->first      = first;
-	hi->last       = last;
-	hi->visits     = visits;
+	hi->host               = host;
+	hi->url                = url;
+	hi->title_utf8         = title_utf8;
+	hi->first              = first;
+	hi->last               = last;
+	hi->visits             = visits;
+	hi->selected_authorcss = -1;
 
 	
 	/* convert to locale if different */
-	title_locale = utf8_to_locale (title_utf8);
+	title_locale = mozilla_utf8_to_locale (title_utf8);
 	if (strcmp (title_locale, title_utf8) != 0)
 	{
 		/* keep this */
@@ -1234,6 +1247,9 @@ history_create_recent_menu_item (HistoryItem *hi, gint num, GtkWidget *menu)
 	gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
 			    GTK_SIGNAL_FUNC (window_menu_recent_activate_cb),
 			    NULL);
+	gtk_signal_connect (GTK_OBJECT(menuitem), "button-release-event",
+			    GTK_SIGNAL_FUNC (window_menu_recent_release_cb),
+			    NULL);
 
 	return menuitem;
 }
@@ -1283,6 +1299,49 @@ history_get_zoom (const gchar *url)
 	return (host == NULL ? 0 : host->zoom);
 }
 
+/**
+ * history_set_selected_authorcss: set the selected author css of the server 
+ * referenced by the given url
+ */
+void
+history_set_selected_authorcss (const gchar *url, gint n)
+{
+	HistoryHost *host;
+
+	/* check URL is valid */
+	if (url == NULL)
+	{
+		return;
+	}
+
+	/* store selected css in host */
+	host = history_get_host (url);
+	if (host != NULL)
+		host->selected_authorcss = n;
+}
+
+/**
+ * history_get_selected_authorcss: get the selected author css of the server 
+ * referenced by the given url
+ */
+gint
+history_get_selected_authorcss (const gchar *url)
+{
+	HistoryHost *host;
+
+	/* check URL is valid */
+	if (url == NULL)
+	{
+		return -1;
+	}
+
+	/* find a host */
+	host = history_get_host (url);
+	
+	/* our result */
+	return (host == NULL ? -1 : host->selected_authorcss);
+}
+
 /**
  * history_get_host_list: get a fresh linked list of all the hosts
  */
@@ -1543,18 +1602,23 @@ history_parse_host_element (const xmlChar **attrs)
 	const xmlChar **attr;
 	gchar *name = NULL;
 	gint zoom = 0;
+	gint css = -1;
 
 	/* parse each attribute */
 	for (attr = attrs; *attr != NULL; attr += 2)
 	{
 		if (xmlStrcmp (attr[SAX_KEY], "name") == 0)
 		{
-			name = decode_entity (attr[SAX_VALUE]);
+			name = xml_decode_entity (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp (attr[SAX_KEY], "zoom") == 0)
 		{
 			zoom = atoi (attr[SAX_VALUE]);
 		}
+		else if (xmlStrcmp (attr[SAX_KEY], "authorcss") == 0)
+		{
+			css = atoi (attr[SAX_VALUE]);
+		}
 		else
 		{
 			g_warning ("Unsupported attribute for "
@@ -1566,6 +1630,7 @@ history_parse_host_element (const xmlChar **attrs)
 	/* create a host structure and store */
 	host = history_add_host (name);
 	host->zoom = zoom;
+	host->selected_authorcss = css;
 	g_free (name);
 }
 
@@ -1587,11 +1652,11 @@ history_parse_item_element (const xmlChar **attrs)
 	{
 		if (xmlStrcmp(attr[SAX_KEY], "title") == 0)
 		{
-			title_utf8 = decode_entity (attr[SAX_VALUE]);
+			title_utf8 = xml_decode_entity (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "url") == 0)
 		{
-			url = decode_entity (attr[SAX_VALUE]);
+			url = xml_decode_entity (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "first_time") == 0)
 		{
@@ -1664,12 +1729,26 @@ history_save_host (gpointer key, gpointer value, gpointer user_data)
         HistoryHost *hh = (HistoryHost *) value;
 	FILE *fp = (FILE *) user_data;
 
-	if (hh->zoom != 0)
+	if (hh->zoom != 0 || hh->selected_authorcss != -1)
 	{
 		fputs ("  <host name=\"", fp);
 		fputs (hh->url, fp);
+	}
+	
+	if (hh->zoom != 0)
+	{
 		fputs ("\" zoom=\"", fp);
 		fprintf (fp, "%d", hh->zoom);
+	}
+
+	if (hh->selected_authorcss != -1)
+	{
+		fputs ("\" authorcss=\"", fp);
+		fprintf (fp, "%d", hh->selected_authorcss);
+	}
+
+	if (hh->zoom != 0 || hh->selected_authorcss != -1)
+	{
 		fputs ("\"/>\n", fp);
 	}
 }
@@ -1759,77 +1838,6 @@ eputs (guchar *text, FILE *fp)
 	fwrite (text + last, i - last, 1, fp);
 }
 
-
-/**
- * decode_entity: translate &amp; et al and return a new expanded string 
- */
-static gchar *
-decode_entity (const unsigned char *encoded)
-{
-	gchar *buffer;
-	gint i, j, length;
-	
-	if (encoded == NULL)
-	{
-		return NULL;
-	}
-
-	buffer = g_strdup (encoded);
-
-	/* Optimize for case where no escape codes are found, otherwise
-	 * use the first found code as the starting index */
-	if ((i = (char *) strstr (buffer, "&") - buffer) < 0)
-	{
-		return buffer;
-	}
-	
-	length = strlen (buffer);
-	for (j = i; i < length; i++)
-	{
-		if (encoded[i] == '&')
-		{
-			if (strncmp (encoded + i + 1, "amp;", 4) == 0)
-			{
-				buffer[j++] = '&';
-				i += 4;
-			}
-			else if (strncmp (encoded + i + 1, "lt;", 3) == 0)
-			{
-				buffer[j++] = '<';
-				i += 3;
-			}
-			else if (strncmp (encoded + i + 1, "gt;", 3) == 0)
-			{
-				buffer[j++] = '>';
-				i += 3;
-			}
-			else if (strncmp (encoded + i + 1, "quot;", 5) == 0)
-			{
-				buffer[j++] = '"';
-				i += 5;
-			}
-			else if (strncmp (encoded + i + 1, "apos;", 5) == 0)
-			{
-				buffer[j++] = '\'';
-				i += 5;
-			}
-			else if (encoded[i + 1] == '#')
-			{
-				buffer[j++] = atoi (encoded + i + 2);
-				i += 5;
-			}
-		}
-		else
-		{
-			buffer[j++] = encoded[i];
-		}
-	}
-
-	buffer[j] = '\0';
-	return buffer;
-}
-
-
 static void
 history_drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
 			  GtkSelectionData *selection_data, guint info,
