@@ -38,6 +38,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-init.h>
 #include <libgnomeui/gnome-geometry.h>
+#include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <liboaf/liboaf.h>
@@ -60,8 +61,6 @@ static void create_default_browser (void);
 static inline ViewState get_viewstate (void);
 static gint read_mozilla_version (void);
 static gint check_mozilla_version (void);
-static GdkFilterReturn theme_changed_cb (GdkXEvent *xevent, GdkEvent *event, 
-					 gpointer data);
 
 /* import from Galeon-impl.c */
 Galeon_Browser impl_Galeon_Browser__create (PortableServer_POA poa, 
@@ -72,9 +71,10 @@ gboolean galeon_server_mode = FALSE;
 gboolean galeon_panel_mode = FALSE;
 
 /* file local variables */
-static CORBA_Environment corba_env;          /* corba environment for errors */
+CORBA_Environment corba_env;                 /* corba environment for errors */
 static gboolean open_in_existing   = FALSE;  /* load in existing window?     */
 static gboolean open_in_new_tab    = FALSE;  /* force open in a new tab?     */
+static gboolean noraise            = FALSE;  /* no raise                     */
 static gboolean open_in_new_window = FALSE;  /* force open in a new window?  */
 static gboolean open_fullscreen    = FALSE;  /* open galeon in full screen ? */
 static gchar *geometry_string      = NULL;   /* the geometry string          */
@@ -96,8 +96,11 @@ static struct poptOption popt_options[] =
 	{ "new-window", 'w', POPT_ARG_NONE, &open_in_new_window, 0,
 	  N_("Open a new window in an existing Galeon process"),
 	  NULL },
+	{ "noraise", '\0', POPT_ARG_NONE, &noraise, 0,
+	  N_("Do not raise the window when opening a page in an existing Galeon process"),
+	  NULL },
 	{ "fullscreen", 'f', POPT_ARG_NONE, &open_fullscreen, 0,
-	  N_("Run galeon in full screen mode"),
+	  N_("Run Galeon in full screen mode"),
 	  NULL },
 	{ "existing", 'x', POPT_ARG_NONE, &open_in_existing, 0,
 	  N_("Attempt to load URL in existing Galeon window"),
@@ -111,8 +114,8 @@ static struct poptOption popt_options[] =
 	  N_("Don't open any windows; instead act as a server "
 	     "for quick startup of new Galeon instances"),
 	  NULL },		  
-	{ "add-temp-bookmark", 't', POPT_ARG_STRING, &bookmark_url,
-	  0, N_("Add a temp bookmark (don't open any window)"), 
+	{ "add-bookmark", 't', POPT_ARG_STRING, &bookmark_url,
+	  0, N_("Add a bookmark (don't open any window)"), 
 	  N_("URL")},
 	{ "geometry", 'g', POPT_ARG_STRING, &geometry_string,
 	  0, N_("Create the initial window with the given geometry.\n"
@@ -151,9 +154,11 @@ main (int argc, char *argv[])
 	 /* make stdout line buffered - we only use it for debug info */
 	setvbuf (stdout, NULL, _IOLBF, 0);
 
+#ifdef ENABLE_NLS
 	/* initialise localisation */
 	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
 	textdomain (PACKAGE);
+#endif
 
 #ifdef ENABLE_APPLET
 	/* panel mode? this check has to be done here since we
@@ -210,6 +215,12 @@ main (int argc, char *argv[])
 		return 0;
 	}
 
+	/* start a new CORBA shell */
+	new_shell (orb);
+
+	/* initialise required external and internal Galeon services */
+	no_default = galeon_init (argc, argv);
+
 	/* add a temp bookmark and exit if needed */
 	if (bookmark_url != NULL)
 	{
@@ -219,12 +230,6 @@ main (int argc, char *argv[])
 		return 0;
 	}
 	
-	/* start a new CORBA shell */
-	new_shell (orb);
-
-	/* initialise required external and internal Galeon services */
-	no_default = galeon_init (argc, argv);
-
 	/* if we have command-line supplied URLs */
 	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED) ? 1 : 0;
 	for (i = 0; i < n_urls; i++)
@@ -293,7 +298,7 @@ read_mozilla_version (void)
 
 	if (p == NULL)
 	{
-		g_warning ("mozilla version not found in %s", preffile);
+		g_warning ("Mozilla version not found in %s", preffile);
 	}
 	else
 	{
@@ -359,10 +364,10 @@ check_mozilla_version (void)
 		}
 
 		questionStr = g_strdup_printf (_(
-"The version of mozilla galeon was compiled against ("VERSIONFMT") "
+"The version of Mozilla Galeon was compiled against ("VERSIONFMT") "
 "does not match the detected version ("VERSIONFMT").\n\n"
 "You may experience segfaults and other oddities, please test "
-"with a matching version of mozilla before reporting any bugs.\n\n"
+"with a matching version of Mozilla before reporting any bugs.\n\n"
 "Continue anyway?"),
 			VERSIONFMTARGS (MOZILLA_VERSION),
 			VERSIONFMTARGS (mozilla_version));
@@ -575,6 +580,11 @@ new_shell (CORBA_ORB orb)
 	reg_id = oaf_make_registration_id ("OAFIID:GNOME_Galeon_Automation",
 					   g_getenv ("DISPLAY"));
 	oaf_active_server_register (reg_id, client);
+
+	PortableServer_POAManager_activate
+		(PortableServer_POA__get_the_POAManager (poa, &corba_env), 
+		 &corba_env);
+
 	g_free (reg_id);
 }
 
@@ -592,57 +602,11 @@ galeon_init (int argc, char *argv[])
 	gchar *profile;
 	extern gboolean pushed_startup;
 	gboolean moz_version_checked = FALSE;
-	GdkAtom message_type;
+/*	GdkAtom message_type;*/
 	gchar *teststr;
 	gchar *mozprefs;
 	gboolean set_mozilla_prefs = FALSE;
 
-	/* initialise GNOME VFS */
-	vfs_success = gnome_vfs_init ();
-	g_assert (vfs_success);
-
-	/* initialise GLADE */
-	glade_gnome_init ();
-
-	/* ensure mozilla matches (and initialise mozilla_version var) */
-	if (g_getenv ("MOZILLA_FIVE_HOME") != NULL)
-	{
-		if (check_mozilla_version ())
-		{
-			exit (1);
-		}
-		moz_version_checked = TRUE;
-	}
-
-	/* preload the prefs */
-	/* it also enables notifiers support */
-	eel_gconf_monitor_add ("/apps/galeon");
-	eel_gconf_monitor_add ("/apps/nautilus/preferences");
-
-	/* check if gconf schemas are working */
-	teststr = gconf_client_get_string (eel_gconf_client_get_global(),
-					  "/apps/galeon/gconf_test", NULL);
-	if (!teststr)
-	{
-		GtkWidget *dialog;
-		dialog = gnome_error_dialog ("Cannot find a schema for galeon preferences. \nCheck your gconf setup, look at galeon FAQ for more info");
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-		exit (0);
-	}
-	else
-	{
-		g_free (teststr);
-	}
-
-	/* check if this is the first time this user has run Galeon */
-	new_user = newuser_check ();
-
-	/* initialise Galeon itself */
-	initialise_colours ();
-	preferences_load ();
-	history_init ();
-	bookmarks_init ();
-
 	/* init mozilla home */
 	gtk_moz_embed_set_comp_path (g_getenv ("MOZILLA_FIVE_HOME"));
 
@@ -673,6 +637,56 @@ galeon_init (int argc, char *argv[])
 		mozilla_prefs_set ();
 	}
 
+	/* ensure mozilla matches (and initialise mozilla_version var) */
+	if (g_getenv ("MOZILLA_FIVE_HOME") != NULL)
+	{
+		if (check_mozilla_version ())
+		{
+			exit (1);
+		}
+		moz_version_checked = TRUE;
+	}
+
+	/* set the user agent, this is always neccesary because the galeon
+	   version may change */
+	mozilla_prefs_set_user_agent ();
+
+	/* initialise GNOME VFS */
+	vfs_success = gnome_vfs_init ();
+	g_assert (vfs_success);
+
+	/* initialise GLADE */
+	glade_gnome_init ();
+
+	/* preload the prefs */
+	/* it also enables notifiers support */
+	eel_gconf_monitor_add ("/apps/galeon");
+	eel_gconf_monitor_add ("/apps/nautilus/preferences");
+
+	/* check if gconf schemas are working */
+	teststr = gconf_client_get_string (eel_gconf_client_get_global(),
+					  "/apps/galeon/gconf_test", NULL);
+	if (!teststr)
+	{
+		GtkWidget *dialog;
+		dialog = gnome_error_dialog ("Cannot find a schema for galeon preferences. \nCheck your gconf setup, look at galeon FAQ for more info");
+		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		exit (0);
+	}
+	else
+	{
+		g_free (teststr);
+	}
+
+	/* check if this is the first time this user has run Galeon */
+	new_user = newuser_check ();
+
+	/* initialise Galeon itself */
+	initialise_colours ();
+	preferences_load ();
+	history_init ();
+	bookmarks_init ();
+
 	/* if we couldn't check the first time, try now since initializing moz
 	 * will set the MOZILLA_FIVE_HOME in some configurations */
 	if (!moz_version_checked)
@@ -695,10 +709,14 @@ galeon_init (int argc, char *argv[])
 	gtk_signal_connect (GTK_OBJECT (session_client), "save_yourself",
 			    (GtkSignalFunc)client_save_yourself_cb, NULL);
 		
-	/* update tab styles when theme changed */
+	/* update tab styles and aa pixmaps when theme changed */
+	/* FIXME theme change is caught too many times, disabling for now
+	 *    -- Jorn 10/10/2001 */
+#if 0
 	message_type = gdk_atom_intern ("_GTK_READ_RCFILES", FALSE);
 	gdk_add_client_message_filter (message_type, (GdkFilterFunc)
-				       theme_changed_cb, NULL);
+				       gtk_theme_changed_cb, NULL);
+#endif
 
 	/* FIXME: catch GDK errors which happen on theme changes
 	 * (until we work out what causes this) */
@@ -757,6 +775,7 @@ create_default_browser (void)
 	/* build the default embed inside it */
 	embed = embed_create_in_window (window);
 	embed_set_visibility (embed, TRUE);
+	window_grab_location_focus (window);
 	url = embed_get_default_url (NULL);
 	embed_load_url (embed, url);
 	g_free (url);
@@ -766,44 +785,39 @@ create_default_browser (void)
  * get_viewstate:
  *
  * Determine the viewstate. A viewstate defines how the URI should be opened.
- * Seemingly cryptic. Might be redone if additional viewstates are added 
- * -Nate
- *
- * You're evil ;-)
- *   -- MattA
  * 
- * VIEWSTATE_NONE:    Open up in existing window if there is one
- * VIEWSTATE_DEFAULT: Use default (setting) viewstate
- * VIEWSTATE_WINDOW:  Open up in window
- * VIEWSTATE_TAB:     Open up in tab
+ * VIEWSTATE_EXISTING:    Open up in existing window if there is one
+ * VIEWSTATE_WINDOW:      Open up in window
+ * VIEWSTATE_TAB:         Open up in tab
+ * VIEWSTATE_NORAISE_TAB: Open up in tab without raising the window
  */
 static inline ViewState
 get_viewstate (void)
 {
-	return (((!open_in_existing & (open_in_new_tab ^ 
-					  open_in_new_window)) << 1) + 
-		(!open_in_new_window & !open_in_existing));
-}
+	ViewState result;
+	gboolean tabbed_mode;
 
-/**
- * theme_changed_cb: called when a Gtk+ theme change is detected, update the
- * tab label styles
- */
-static GdkFilterReturn
-theme_changed_cb (GdkXEvent *xevent, GdkEvent *event, gpointer data)
-{
-	GList *e;
-	
-	/* reload colour styles */
-	initialise_colours ();
+	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED);
 
-	/* set in all tabs */
-	for (e = all_embeds; e != NULL; e = g_list_next (e))
+	result = tabbed_mode ? VIEWSTATE_TAB : VIEWSTATE_WINDOW;
+
+	if ((noraise && open_in_new_tab) ||
+	    (noraise && tabbed_mode))
 	{
-		embed_update_tab_status ((GaleonEmbed *)(e->data));
+		result = VIEWSTATE_NORAISE_TAB;
+	}
+	else if (open_in_new_window)
+	{
+		result = VIEWSTATE_WINDOW;
+	}
+	else if (open_in_new_tab)
+	{
+		result = VIEWSTATE_TAB;
+	}
+	else if (open_in_existing)
+	{
+		result = VIEWSTATE_EXISTING;
 	}
-	
-	/* process other GDK events */
-	return GDK_FILTER_CONTINUE;
-}
 
+	return result;
+}
