@@ -26,12 +26,14 @@
 #include "bookmarks.h"
 #include "print.h"
 #include "find.h"
-#include "mozilla.h"
 #include "session.h"
 #include "xlink.h"
 #include "dialog.h"
 #include "filepicker.h"
 #include "mozilla.h"
+#ifdef ENABLE_EXPERIMENTAL_FILTERS
+#include "filters.h"
+#endif
 
 #include <string.h>
 #include <gtkmozembed.h>
@@ -44,6 +46,9 @@
 
 void 
 window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data);
+void 
+window_menu_recent_release_cb (GtkMenuItem *menuitem, GdkEventButton *event,
+			       gpointer user_data);
 
 /**
  * window_menu_prefs_activate_cb: show preferences dialog
@@ -70,6 +75,10 @@ window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
 		{
 			bookmarks_editor_show_dock (window);
 		}
+		else
+		{
+			window_undock (window);
+		}
 	}
 	else
 	{
@@ -258,14 +267,20 @@ window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
 	window_load_url(window, hi->url);
 }
 
-/**
- * window_spinner_button_clicked_cb: spinner clicked
- */
-void
-window_spinner_button_clicked_cb(GtkButton *button, GaleonWindow *window)
+void 
+window_menu_recent_release_cb (GtkMenuItem *menuitem, GdkEventButton *event,
+			       gpointer user_data)
 {
+	GaleonWindow *window = window_from_widget (GTK_WIDGET (menuitem));
+        HistoryItem *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));
+	GtkMenu *menu = GTK_MENU (GTK_WIDGET (menuitem)->parent);
+	
 	return_if_not_window (window);
-	window_load_url (window, GALEON_HOMEPAGE_URL);
+
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu));
+	
+	embed_activate_link (window->active_embed, NULL, hi->url,
+		mouse_state_to_link_state (event->button, event->state));
 }
 
 /**
@@ -334,6 +349,8 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 		N_("Yanko Kaneti <yaneti@declera.com>"),
 		"Philip Langdale <philipl@mail.utexas.edu>",
 		"Matthew Mueller <donut@azstarnet.com>",
+		"Daniel Erat  <dan_erat@pobox.com>",
+		"Jorn Baayen <jorn@nl.linux.org>",
 		" ", /* dont delete the space */
 		N_("Graphic arts: "),
 		/* if your encoding allows it, use eacute (U00E9)
@@ -355,10 +372,10 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 		
 		if (strcmp (_("Translation:"), "Translation:") == 0) {
 			bottom_line = g_strdup
-				("A GNOME browser based on mozilla");
+				("A GNOME browser based on Mozilla");
 		} else {
 			bottom_line = g_strconcat (
-				_("A GNOME browser based on mozilla"),
+				_("A GNOME browser based on Mozilla"),
 				"\n \n",
 			/* Translators should localize the following string
 			 * which will be displayed at the bottom of the about
@@ -378,6 +395,10 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 		file = user_file ("about_logo.png", FALSE);
 		about = gnome_about_new 
 			(_(PACKAGE), VERSION,
+			/* Translators: Please change the (C) to a real
+			 * copyright character if your character set allows it
+			 * (Hint: iso-8859-1 is one of the character sets that
+			 * has this symbol). */
 			 _("Copyright (C) 2000-2001 Marco Pesenti Gritti"),
 				 (const gchar **)translated_names, 
 				 bottom_line, file);
@@ -386,6 +407,8 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 
 		gtk_signal_connect (GTK_OBJECT (about), "destroy",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
+		gtk_window_set_wmclass (GTK_WINDOW (about), "dialog_about",
+					"galeon_browser");
 		gtk_widget_show (about);
 	}
 
@@ -400,32 +423,99 @@ window_menu_help_manual_activate_cb (GtkMenuItem *menuitem,
 				     GaleonWindow *window)
 {
 	return_if_not_window (window);
-	gnome_help_goto (NULL, "ghelp:galeon");
+	gnome_help_goto (NULL, "ghelp:galeon-manual");
 }
 
 /**
- * window_menu_help_homepage_activate: go to the Galeon homepage
+ * window_menu_help_homepage_activate_cb: go to the Galeon homepage
  */
 void
-window_menu_help_homepage_activate_cb (GtkMenuItem *menuitem, 
+window_menu_help_homepage_activate_cb (GtkWidget *menuitem, 
 				       GaleonWindow *window)
 {
 	return_if_not_window (window);
 	window_load_url (window, GALEON_HOMEPAGE_URL);
 }
 
-
 /**
- * window_menu_help_bugreport_activate: go to the Galeon bug report page
+ * window_menu_help_homepage_release_cb: go to the Galeon homepage
  */
 void
-window_menu_help_bugreport_activate_cb (GtkMenuItem *menuitem, 
+window_menu_help_homepage_release_cb (GtkWidget *menuitem, 
+				      GdkEventButton *event,
+				      GaleonWindow *window)
+{
+	GaleonEmbed *new = NULL;
+	GtkMenu *menu = GTK_MENU (menuitem->parent);
+	
+	return_if_not_window (window);
+
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
+	
+	embed_activate_link (window->active_embed, &new, GALEON_HOMEPAGE_URL,
+		mouse_state_to_link_state (event->button, event->state));
+}
+
+/**
+ * window_menu_help_bugreport_activate_cb: go to the Galeon bug report page
+ */
+void
+window_menu_help_bugreport_activate_cb (GtkWidget *menuitem, 
 					GaleonWindow *window)
 {
 	return_if_not_window (window);
 	window_load_url (window, GALEON_BUGREPORT_URL);
 }
 
+/**
+ * window_menu_help_bugreport_release_cb: go to the Galeon bug report page
+ */
+void
+window_menu_help_bugreport_release_cb (GtkWidget *menuitem, 
+				       GdkEventButton *event,
+				       GaleonWindow *window)
+{
+	GaleonEmbed *new = NULL;
+	GtkMenu *menu = GTK_MENU (menuitem->parent);
+	
+	return_if_not_window (window);
+
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
+	
+	embed_activate_link (window->active_embed, &new, GALEON_BUGREPORT_URL,
+		mouse_state_to_link_state (event->button, event->state));
+}
+
+/**
+ * window_menu_help_about_plugins_activate_cb: go to about:plugins
+ */
+void
+window_menu_help_about_plugins_activate_cb (GtkWidget *menuitem, 	
+					    GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, "about:plugins");
+}
+
+/**
+ * window_menu_help_about_plugins_release_cb: go to about:plugins
+ */
+void
+window_menu_help_about_plugins_release_cb (GtkWidget *menuitem, 	
+					   GdkEventButton *event,
+					   GaleonWindow *window)
+{
+	GaleonEmbed *new = NULL;
+	GtkMenu *menu = GTK_MENU (menuitem->parent);
+	
+	return_if_not_window (window);
+
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
+	
+	embed_activate_link (window->active_embed, &new, "about:plugins",
+		mouse_state_to_link_state (event->button, event->state));
+}
+
 /** 
  * window_menu_new_window_activate_cb: open a new window
  */
@@ -434,14 +524,14 @@ window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
 				    GaleonWindow *window)
 {
 	GaleonEmbed *embed;
+
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
+
 	embed = embed_create_default (window->active_embed, TRUE);
-	if (embed->parent_window->location_entry != NULL)
-	{
-		gtk_widget_grab_focus 
-			(GTK_WIDGET (embed->parent_window->location_entry));
-	}
+
+	return_if_not_window (embed->parent_window);
+	window_grab_location_focus (embed->parent_window);
 }
 
 /** 
@@ -454,12 +544,8 @@ window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
 	GaleonEmbed *embed;
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
+	window_grab_location_focus (window);
 	embed = embed_create_default (window->active_embed, FALSE);
-	if (embed->parent_window->location_entry != NULL)
-	{
-		gtk_widget_grab_focus 
-			(GTK_WIDGET (embed->parent_window->location_entry));
-	}
 }
 
 /** 
@@ -499,15 +585,7 @@ void
 window_menu_exit_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
-
-	/* We call the quit function indirectly, to make sure control returns
-	   to the main loop first.  This is because we might have been
-	   activated from within the mozembed_key_press callback, and if the
-	   widget is destroyed before returning from the callback, very bad
-	   things happen.  --Josh 
-	   Lets take care the timer is not triggered again - yaneti */
-	quit_timeout_id = gtk_timeout_add (50, (GtkFunction) session_quit_cb,
-					   GINT_TO_POINTER (FALSE));
+	session_quit (FALSE);
 }
 
 /** 
@@ -518,8 +596,7 @@ window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	quit_timeout_id = gtk_timeout_add (50, (GtkFunction) session_quit_cb,
-					   GINT_TO_POINTER (TRUE));
+	session_quit (TRUE);
 }
 
 /** 
@@ -541,9 +618,7 @@ window_menu_close_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	window->active_embed->close_timeout_id = 
-		gtk_timeout_add (50, (GtkFunction) embed_close,
-				 (gpointer)window->active_embed);
+	embed_close (window->active_embed);
 }
 
 /** 
@@ -598,9 +673,19 @@ window_menu_detach_tab_activate_cb (GtkMenuItem *menuitem,
 void
 window_menu_prev_tab_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
+	gint current, last;  
+
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_notebook_prev_page (GTK_NOTEBOOK (window->notebook));
+	
+	current =
+		gtk_notebook_get_current_page (GTK_NOTEBOOK (window->notebook));
+	last = g_list_length (GTK_NOTEBOOK (window->notebook)->children) - 1;
+	
+	if (current == 0)
+		gtk_notebook_set_page (GTK_NOTEBOOK (window->notebook), last);
+	else
+		gtk_notebook_prev_page (GTK_NOTEBOOK (window->notebook));
 }
 
 /** 
@@ -609,9 +694,19 @@ window_menu_prev_tab_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 void
 window_menu_next_tab_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
+	gint current, last;
+	
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_notebook_next_page (GTK_NOTEBOOK (window->notebook));
+	
+	current =
+		gtk_notebook_get_current_page (GTK_NOTEBOOK (window->notebook));
+	last = g_list_length (GTK_NOTEBOOK (window->notebook)->children) - 1;
+	
+	if (current == last)
+		gtk_notebook_set_page (GTK_NOTEBOOK (window->notebook), 0);
+	else
+		gtk_notebook_next_page (GTK_NOTEBOOK (window->notebook));
 }
 
 /** 
@@ -642,7 +737,8 @@ window_menu_shift_tab_right_activate_cb (GtkMenuItem *menuitem,
  * window_menu_stop_activate_cb: stop menu activated
  */
 void
-window_menu_stop_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+window_menu_stop_activate_cb (GtkMenuItem *menuitem,
+			      GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
@@ -653,11 +749,12 @@ window_menu_stop_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
  * window_menu_reload_activate_cb: reload menu activated
  */
 void
-window_menu_reload_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+window_menu_reload_activate_cb (GtkMenuItem *menuitem,
+				GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_reload (window->active_embed);
+	embed_reload (window->active_embed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
 }
 
 /** 
@@ -665,7 +762,7 @@ window_menu_reload_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
  */
 void
 window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
-				    GaleonWindow *window)
+				   GaleonWindow *window)
 {
 	return_if_not_window (window);
 	window_go_home (window, 0);
@@ -673,12 +770,14 @@ window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
 
 void
 window_menu_add_bookmark_activate_cb (GtkMenuItem *menuitem,
-					   GaleonWindow *window)
+				      GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	add_bookmark_default (BM_SITE, window->active_embed->site_title, 
-			      window->active_embed->site_location, NULL);
+	g_return_if_fail (default_bookmarks_root != NULL);
+
+	bookmarks_file_bookmark_cb (GTK_WIDGET (menuitem),
+				    default_bookmarks_root);
 }
 
 /** 
@@ -689,7 +788,6 @@ void window_menu_toolbar_activate_cb (GtkCheckMenuItem *menuitem,
 {
 	return_if_not_window (window);
 
-	/* FIXME handle this properly when the menubar does not exists */
 	g_return_if_fail (menuitem);
 
 	if (menuitem->active) 
@@ -725,7 +823,6 @@ void window_menu_menubar_activate_cb (GtkCheckMenuItem *menuitem,
 {
 	return_if_not_window (window);
 
-	/* FIXME handle this properly when the menubar does not exists */
 	g_return_if_fail (menuitem);
 
 	if (menuitem->active) 
@@ -746,7 +843,6 @@ void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
 {
 	return_if_not_window (window);
 
-	/* FIXME handle this properly when the menubar does not exists */
 	g_return_if_fail (menuitem);
 
 	if (menuitem->active) 
@@ -761,17 +857,16 @@ void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
 
 /** 
  * window_menu_fullscreen_activate_cb: set fullscreen mode
- *
  */
 void window_menu_fullscreen_activate_cb (GtkCheckMenuItem *menuitem,
 					 GaleonWindow *window)
 {
 	return_if_not_window (window);
 
-	/* FIXME handle this properly when the menubar does not exists */
-	g_return_if_fail (menuitem);
-
-	window_set_fullscreen_mode (window, menuitem->active);
+	if (menuitem)
+		window_set_fullscreen_mode (window, menuitem->active);
+	else
+		window_toggle_fullscreen_mode (window);
 }
 
 void
@@ -784,8 +879,8 @@ window_menu_own_colors_activate_cb (GtkCheckMenuItem *menuitem,
 }
 
 void
-window_menu_enable_javascript_activate_cb (GtkCheckMenuItem *menuitem,
-					   GaleonWindow *window)
+window_menu_allow_javascript_activate_cb (GtkCheckMenuItem *menuitem,
+					  GaleonWindow *window)
 {
 	return_if_not_window (window);
 	eel_gconf_set_boolean(CONF_FILTERING_JAVASCRIPT_ENABLED, 
@@ -793,21 +888,12 @@ window_menu_enable_javascript_activate_cb (GtkCheckMenuItem *menuitem,
 }
 
 void
-window_menu_enable_proxy_activate_cb (GtkCheckMenuItem *menuitem,
-                                           GaleonWindow *window)
-{
-        return_if_not_window (window);
-	eel_gconf_set_boolean(CONF_NETWORK_PROXY_ENABLED, menuitem->active);
-}
-
-
-void
-window_menu_autojump_activate_cb (GtkCheckMenuItem *menuitem,
-					   GaleonWindow *window)
+window_menu_allow_popups_activate_cb (GtkCheckMenuItem *menuitem,
+				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	eel_gconf_set_boolean(CONF_TABS_TABBED_AUTOJUMP, 
-			      menuitem->active);
+	eel_gconf_set_boolean (CONF_FILTERING_ALLOW_POPUPS, 
+			       menuitem->active);
 }
 
 void
@@ -819,7 +905,7 @@ window_menu_own_fonts_activate_cb (GtkCheckMenuItem *menuitem,
 }
 
 void
-window_menu_enable_java_activate_cb (GtkCheckMenuItem *menuitem,
+window_menu_allow_java_activate_cb (GtkCheckMenuItem *menuitem,
 				     GaleonWindow *window)
 {
 	return_if_not_window (window);
@@ -857,8 +943,8 @@ window_menu_images_always_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		eel_gconf_set_integer(CONF_FILTERING_IMAGE_LOADING_TYPE, 
-				      IMAGE_LOADING_ALWAYS);
+		eel_gconf_set_integer (CONF_FILTERING_IMAGE_LOADING_TYPE, 
+				       IMAGE_LOADING_ALWAYS);
 	}
 }
 
@@ -1047,7 +1133,7 @@ window_menu_encoding_activate_cb (GtkWidget *menuitem,
 	mozilla_force_character_set (window->active_embed,
 				     g_hash_table_lookup (charsets,
 							  menulabel));
-	embed_reload (window->active_embed);
+	embed_reload (window->active_embed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
 }
 
 /*
@@ -1163,3 +1249,44 @@ window_menu_show_javascript_console_cb (GtkWidget *meniutem,
 {
 	mozilla_show_javascript_console ();
 }
+
+#ifdef ENABLE_EXPERIMENTAL_FILTERS
+void
+window_menu_filters_activate_cb (GtkWidget *menuitem, GaleonWindow *window)
+{
+	filters_show_dialog (window);
+}
+#endif
+
+void
+window_menu_no_proxies_activate_cb (GtkCheckMenuItem *menuitem,
+				    GaleonWindow *window)
+{
+	return_if_not_window (window);
+	if (menuitem->active)
+	{
+		eel_gconf_set_integer (CONF_NETWORK_PROXY_MODE, 0);
+	}
+}
+
+void
+window_menu_manual_proxies_activate_cb (GtkCheckMenuItem *menuitem,
+					GaleonWindow *window)
+{
+	return_if_not_window (window);
+	if (menuitem->active)
+	{
+		eel_gconf_set_integer (CONF_NETWORK_PROXY_MODE, 1);
+	}
+}
+
+void
+window_menu_automatic_proxies_activate_cb (GtkCheckMenuItem *menuitem,
+					   GaleonWindow *window)
+{
+	return_if_not_window (window);
+	if (menuitem->active)
+	{
+		eel_gconf_set_integer (CONF_NETWORK_PROXY_MODE, 2);
+	}
+}
