@@ -47,49 +47,51 @@ static char *ugh[] = {
 /* The menubar GnomeUIInfo structs */ 
 GnomeUIInfo file_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_New window"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_New window"), N_("Create a new window"),
 	  window_menu_new_window_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW,
 	  GDK_n, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("New _tab"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("New _tab"), N_("Create a new tab"),
 	  window_menu_new_tab_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW,
 	  GDK_T, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_MENU_OPEN_ITEM (window_menu_open_activate_cb, NULL),
-	{ GNOME_APP_UI_ITEM, N_("Open _URL..."), N_("Open a new URL"),
+	{ GNOME_APP_UI_ITEM, N_("Open _URL..."), N_("Open a URL"),
 	  window_menu_open_url_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
 	  GDK_o, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Save _As..."),
-	  N_("Save the current page with a different name"),
+	{ GNOME_APP_UI_ITEM, N_("Save _as..."),
+	  N_("Save the current page"),
 	  window_menu_save_as_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
 	  GDK_S, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_Print..."), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Print..."), N_("Print the current page"),
 	  window_menu_print_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT,
 	  GDK_P, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_TOGGLEITEM, N_("Work O_ffline"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Work offline"),
+	  N_("Toggle offline mode"),
 	  window_menu_offline_mode_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("Sa_ve session as..."),
-	  N_("Save the location of each open window to a file"),
+	  N_("Save the current session to a file"),
 	  window_menu_save_session_as_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Op_en session..."),
-	  N_("Restore a previous browsing session from a file"),
+	  N_("Restore a previous session from a file"),
 	  window_menu_open_session_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_Close"), N_("Close the current browser"),
+	{ GNOME_APP_UI_ITEM, N_("_Close"),
+	  N_("Close the current tab or window"),
 	  window_menu_close_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
 	  GDK_w, GDK_CONTROL_MASK, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Exit with _session"),
-	  N_("Exit the program, saving the location of each open window"),
+	  N_("Exit the program, saving the current session"),
 	  window_menu_exit_with_session_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	GNOMEUIINFO_MENU_EXIT_ITEM (window_menu_exit_activate_cb, NULL),
@@ -114,16 +116,16 @@ GnomeUIInfo edit_menu_uiinfo[] =
 
 GnomeUIInfo zoom_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_In"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_In"), N_("Zoom in"),
 	  window_menu_zoom_in_activate_cb, NULL, NULL,
  	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_equal, GDK_CONTROL_MASK, NULL },
 	  /* xgettext:no-c-format */
-	{ GNOME_APP_UI_ITEM, N_("_100%"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_100%"), N_("Reset zoom level to 100%"),
 	  window_menu_zoom_100_activate_cb, NULL, NULL,
  	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_0, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Out"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Out"), N_("Zoom out"),
 	  window_menu_zoom_out_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_minus, GDK_CONTROL_MASK, NULL },
@@ -132,37 +134,50 @@ GnomeUIInfo zoom_menu_uiinfo[] =
 
 GnomeUIInfo view_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Menubar"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Menubar"),
+	  N_("Toggle menubar visibility"),
 	  window_menu_menubar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_M, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Toolbar"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Toolbar"),
+	  N_("Toggle toolbar visibility"),
 	  window_menu_toolbar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_T, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Bookmarks toolbars"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Bookmarks toolbars"),
+	  N_("Toggle bookmark toolbars visibility"),
 	  window_menu_bookmarks_toolbars_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_B, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Statusbar"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Statusbar"),
+	  N_("Toggle statusbar visiblity"),
 	  window_menu_statusbar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_S, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Full screen"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Full screen"),
+	  N_("Toggle full screen mode"),
 	  window_menu_fullscreen_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_F11, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_SUBTREE, N_("_Zoom"), NULL,
+	{ GNOME_APP_UI_SUBTREE, N_("_Zoom"),
+	  N_("Use a different font size"),
 	  zoom_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Encoding"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Encoding"),
+	  N_("Use a different character encoding"),
+	  NULL, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Stylesheet"),
+	  N_("Specify stylesheet settings for the current page"),
 	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_Page source"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Page source"),
+	  N_("View the source code of the current page"),
 	  window_menu_view_source_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_U, GDK_CONTROL_MASK, NULL },
@@ -171,46 +186,53 @@ GnomeUIInfo view_menu_uiinfo[] =
 
 GnomeUIInfo tab_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Detach tab"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Detach tab"),
+	  N_("Move the current tab to a new window"),
 	  window_menu_detach_tab_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_I, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Move tab to another window"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Move tab to another window"),
+	  N_("Move the current tab to a different window"),
 	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_Previous tab"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Previous tab"),
+	  N_("Switch to the previous tab"),
 	  window_menu_prev_tab_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
-	  GDK_comma, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Next tab"), NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK,
+	  GDK_Page_Up, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Next tab"),
+	  N_("Switch to the next tab"),
 	  window_menu_next_tab_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO,
-	  GDK_period, GDK_CONTROL_MASK, NULL },
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD,
+	  GDK_Page_Down, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("Move tab _left / up"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("Move tab _left / up"),
+	  N_("Move the current tab left / up"),
 	  window_menu_shift_tab_left_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK, GDK_less,
-	  GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Move tab _right / down"), NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO, GDK_Page_Up,
+	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Move tab _right / down"),
+	  N_("Move the current tab right / down"),
 	  window_menu_shift_tab_right_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD, GDK_greater,
-	  GDK_CONTROL_MASK, NULL },
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO, GDK_Page_Down,
+	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo load_images_always_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Always"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Always"), N_("Load all images"),
 	  window_menu_images_always_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("From _current server only"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_From current server only"),
+	  N_("Load images from current server only"),
 	  window_menu_images_current_server_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Never"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Never"), N_("Don't load any images"),
 	  window_menu_images_never_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
@@ -221,33 +243,23 @@ GnomeUIInfo load_images_menu_uiinfo[] =
 {
 	{ GNOME_APP_UI_RADIOITEMS, NULL, NULL, load_images_always_uiinfo,
 	  NULL, NULL, 0, NULL, 0, 0, NULL },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("A_llow images from this site"), NULL,
-	  window_allow_images_from_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Block images from this site"), NULL,
-	  window_block_images_from_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_View image sites..."), NULL,
-	  window_view_image_sites_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo animate_always_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Continuously"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Continuously"),
+	  N_("Animate images continuously"),
 	  window_menu_animate_always_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Once"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Once"),
+	  N_("Animate images once through"),
 	  window_menu_animate_once_through_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Never"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Never"),
+	  N_("Never animate images"),
 	  window_menu_animate_never_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
@@ -263,75 +275,133 @@ GnomeUIInfo animate_images_menu_uiinfo[] =
 
 GnomeUIInfo cookies_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("View _cookies..."), NULL,
+	{ GNOME_APP_UI_ITEM, N_("View _cookies..."),
+	  N_("View stored cookies"),
 	  window_view_cookies_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("View cookie _sites..."), NULL,
+	{ GNOME_APP_UI_ITEM, N_("View cookie _sites..."),
+	  N_("View sites from which cookies will be accepted or rejected"),
 	  window_view_cookie_sites_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_Allow cookies from this site"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Allow cookies from this site"),
+	  N_("Allow cookies from this site"),
 	  window_allow_cookies_from_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Block cookies from this site"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Block cookies from this site"),
+	  N_("Block cookies from this site"),
 	  window_block_cookies_from_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo block_images_menu_uiinfo[] =
+{
+	{ GNOME_APP_UI_ITEM, N_("_Allow images from this site"),
+	  N_("Allow images from this site to be loaded"),
+	  window_allow_images_from_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Block images from this site"),
+	  N_("Block images from this site from being loaded"),
+	  window_block_images_from_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_View image sites..."),
+	  N_("View sites for which images will not be loaded"),
+	  window_view_image_sites_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo passwords_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("View _passwords..."), NULL,
+	{ GNOME_APP_UI_ITEM, N_("View _passwords..."),
+	  N_("View stored passwords"),
 	  window_view_passwords_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("View password _sites..."), NULL,
+	{ GNOME_APP_UI_ITEM, N_("View password _sites..."),
+	  N_("View sites for which passwords will not be stored"),
 	  window_view_password_sites_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo proxies_always_uiinfo[] =
+{
+	{ GNOME_APP_UI_ITEM, N_("_Disabled"),
+	  N_("Never use proxies"),
+	  window_menu_no_proxies_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Manual"),
+	  N_("Use manual proxy configuration"),
+	  window_menu_manual_proxies_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Auto"),
+	  N_("Use automatic proxy configuration"),
+	  window_menu_automatic_proxies_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo proxies_menu_uiinfo[] =
+{
+	{ GNOME_APP_UI_RADIOITEMS, NULL, NULL, proxies_always_uiinfo,
+	  NULL, NULL, 0, NULL, 0, 0, NULL },
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo settings_menu_uiinfo[] =
 {
 	{ GNOME_APP_UI_SUBTREE, N_("_Load images"),
-	  N_("Display the images of the document"),
+	  N_("Control whether images are displayed"),
 	  load_images_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_SUBTREE, N_("_Animate images"), NULL,
+	{ GNOME_APP_UI_SUBTREE, N_("_Animate images"),
+	  N_("Control whether images are animated "),
 	  animate_images_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_SUBTREE, N_("_Proxy"),
+	  N_("Control how proxies should be used"),
+	  proxies_menu_uiinfo, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Use own _fonts"),
-	  N_("Use own fonts instead of document provided fonts"),
+	  N_("Use default fonts instead of page-requested fonts"),
 	  window_menu_own_fonts_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("Use own c_olors"),
-	  N_("Use own colors instead of document provided colors"),
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Use own _colors"),
+	  N_("Use default colors instead of page-requested colors"),
 	  window_menu_own_colors_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("Enable _Java"), NULL,
-	  window_menu_enable_java_activate_cb, NULL, NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Allow _Java"),
+	  N_("Allow Java code to be executed"),
+	  window_menu_allow_java_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("Enable Java_Script"), NULL,
-	  window_menu_enable_javascript_activate_cb, NULL, NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Allow Java_Script"),
+	  N_("Allow JavaScript code to be executed"),
+	  window_menu_allow_javascript_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-        { GNOME_APP_UI_TOGGLEITEM, N_("Enable pro_xy"), NULL,
-          window_menu_enable_proxy_activate_cb, NULL, NULL,
-          GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("Auto-jump to new _tabs"), NULL,
-	  window_menu_autojump_activate_cb, NULL, NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Allow popups"),
+	  N_("Allow pages to create popup windows"),
+	  window_menu_allow_popups_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
           0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
@@ -341,15 +411,18 @@ GnomeUIInfo settings_menu_uiinfo[] =
 
 GnomeUIInfo bookmarks_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("Add _bookmark"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("Add _bookmark"),
+	  N_("Add a bookmark for the current page to the default folder"),
 	  window_menu_add_bookmark_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_D, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Add bookmark to"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Add bookmark to"),
+	  N_("Add a bookmark for the current page to a specified folder"),
 	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_INDEX,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Edit bookmarks..."), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Edit bookmarks..."),
+	  N_("Open the bookmarks editor"),
 	  window_menu_bookmarks_edit_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
 	  GDK_B, GDK_CONTROL_MASK, NULL },
@@ -359,27 +432,32 @@ GnomeUIInfo bookmarks_menu_uiinfo[] =
 
 GnomeUIInfo go_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Back"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Back"),
+	  N_("Go back in history"),
 	  window_menu_back_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK,
 	  GDK_Left, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Forward"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Forward"),
+	  N_("Go forward in history"),
 	  window_menu_forward_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD,
 	  GDK_Right, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Up"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Up"),
+	  N_("Go up a level from the current location"),
 	  window_menu_up_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UP,
 	  GDK_Up, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Reload"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Reload"),
+	  N_("Reload the current page"),
 	  window_menu_reload_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH,
 	  GDK_r, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("H_ome page"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("H_ome page"), N_("Go to your home page"),
 	  window_menu_home_page_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
 	  GDK_Home, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Stop"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Stop"),
+	  N_("Stop the loading of the current page"),
 	  window_menu_stop_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_STOP,
 	  GDK_Escape, 0, NULL },
@@ -389,14 +467,24 @@ GnomeUIInfo go_menu_uiinfo[] =
 
 GnomeUIInfo tools_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_SUBTREE, N_("_Cookies"), NULL,
+	{ GNOME_APP_UI_SUBTREE, N_("_Cookies"), N_("Cookie management"),
 	  cookies_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_SUBTREE, N_("_Passwords"), NULL,
+	{ GNOME_APP_UI_SUBTREE, N_("_Images"), N_("Image blocking management"),
+	  block_images_menu_uiinfo, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_SUBTREE, N_("_Passwords"), N_("Password management"),
 	  passwords_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+#ifdef ENABLE_EXPERIMENTAL_FILTERS
+	{ GNOME_APP_UI_ITEM, N_("_Filters..."), N_("Open the filter editor"),
+	  window_menu_filters_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+#endif
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_History..."),
 	  N_("Log of visited pages"),
@@ -404,19 +492,19 @@ GnomeUIInfo tools_menu_uiinfo[] =
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_H, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR, 
-	{ GNOME_APP_UI_ITEM, N_("_Java Console..."),
-	  N_("Show the Java Console"),
+	{ GNOME_APP_UI_ITEM, N_("_Java console..."),
+	  N_("Open the Java console"),
 	  window_menu_show_java_console_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Javascript Console..."),
-	  N_("Show the Javascript Console"),
+	{ GNOME_APP_UI_ITEM, N_("Java_Script console..."),
+	  N_("Open the JavaScript console"),
 	  window_menu_show_javascript_console_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR, 
 	{ GNOME_APP_UI_ITEM, N_("_Rerun first time druid..."),
-	  N_("Lets run the first time druid again"),
+	  N_("Run the first time druid again"),
 	  window_run_first_time_druid_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
@@ -425,18 +513,26 @@ GnomeUIInfo tools_menu_uiinfo[] =
 
 GnomeUIInfo help_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Galeon manual"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("Galeon _manual"), 
+	  N_("Open the Galeon manual"),
 	  window_menu_help_manual_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_BLUE,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Galeon _homepage"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("Galeon _home page"),
+	  N_("Go to the Galeon homepage"),
 	  window_menu_help_homepage_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Report a bug"), NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Report a bug"),
+	  N_("Report a bug in Galeon"),
 	  window_menu_help_bugreport_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("About _plugins"),
+	  N_("Show information about the currently installed plugins"),
+	  window_menu_help_about_plugins_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
 	GNOMEUIINFO_MENU_ABOUT_ITEM (window_menu_help_about_activate_cb, NULL),
 	GNOMEUIINFO_END
 };
@@ -468,6 +564,9 @@ GnomeUIInfo *all_menus[] =
 	animate_images_menu_uiinfo,
 	cookies_menu_uiinfo,
 	passwords_menu_uiinfo,
+	block_images_menu_uiinfo,
+	proxies_always_uiinfo,
+	proxies_menu_uiinfo,
 	settings_menu_uiinfo,
 	bookmarks_menu_uiinfo,
 	go_menu_uiinfo,
@@ -488,6 +587,9 @@ gint menus_num_items[] =
 	sizeof(animate_images_menu_uiinfo) / sizeof(GnomeUIInfo),
 	sizeof(cookies_menu_uiinfo)        / sizeof(GnomeUIInfo),
 	sizeof(passwords_menu_uiinfo)	   / sizeof(GnomeUIInfo),
+	sizeof(block_images_menu_uiinfo)   / sizeof(GnomeUIInfo),
+	sizeof(proxies_always_uiinfo)      / sizeof(GnomeUIInfo),
+	sizeof(proxies_menu_uiinfo)        / sizeof(GnomeUIInfo),
 	sizeof(settings_menu_uiinfo)       / sizeof(GnomeUIInfo),
 	sizeof(bookmarks_menu_uiinfo)      / sizeof(GnomeUIInfo),
 	sizeof(go_menu_uiinfo)             / sizeof(GnomeUIInfo),
