@@ -32,14 +32,11 @@
 #include <errno.h>
 #include <dirent.h>
 #include <limits.h>
+#include <ctype.h>
 
 #include <time.h>
 #include <stdlib.h>
 #include <string.h>
-#include <iconv.h>
-#ifdef HAVE_CODESET
-#include <langinfo.h>
-#endif
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
 #include <gtk/gtkcheckmenuitem.h>
@@ -66,7 +63,7 @@ GtkStyle *new_text_style = NULL;
 /* Local prototypes */
 static gboolean is_punctuation (gchar c);
 static gchar* get_first_directory (const gchar *start_dir);
-static void pixbuf_threshold_alpha (GdkPixbuf *pixbuf);
+static void pixbuf_threshold_alpha (GdkPixbuf *pixbuf, gboolean prelight);
 
 /**
  * user_file: returns the pathname of galeon shared files (e.g., galeon.glade)
@@ -184,7 +181,8 @@ new_num_accel_str (gint num, gchar *text, gboolean lettersok)
 }
 
 /**
- * Sets a labels text with shortcut key handling in the standard _<key> accel way
+ * Sets a labels text with shortcut key handling in the 
+ * standard _<key> accel way
  */
 void
 label_set_accel_text (const gchar *text, GtkWidget *label, GtkWidget *menu, 
@@ -236,9 +234,11 @@ strip_uline_accel (const gchar *text)
 }
 
 /**
- * Escape _'s in string such that the gtk_label_parse_uline will display as normal
+ * Escape _'s in string such that the gtk_label_parse_uline will display as
+ *  normal
  * caller is responsible for freeing the returned string
  */
+/* FIXME: utf8 */
 gchar *
 escape_uline_accel (const gchar *text)
 {
@@ -341,6 +341,7 @@ is_punctuation (gchar c)
  * to quite small lengths, generally remaining comprehensible down to
  * around six to eight characters.
  */
+/* FIXME: utf8? */
 gchar *
 shorten_name(const gchar *input_name, gint target_length)
 {
@@ -466,11 +467,19 @@ shorten_name(const gchar *input_name, gint target_length)
 		for (i = length - 1, j = length - 1; i >= 0; i--)
 		{
 			c = name[i];
-			if (length <= target_length ||
-			    (c != 'a' && c != 'e' && c != 'i' &&
-			     c != 'o' && c != 'u'))
+			/* keep the letter if the string is short enough
+			 * already, if the letter is at the beginning of a
+			 * word, or if it's not a lowercase vowel */
+			if (length <= target_length || i == 0 ||
+ 			    (i > 0 && name[i - 1] == ' ') ||
+ 			    /* TRANSLATOR: Put in this string those characters
+ 			     that can be skipped without making a word totally
+ 			     irrecognizable. In most languages, those are the
+ 			     vowels (with and without accents) */
+ 			    !strchr (_("aeiouyAEIOUY"), c))
 			{
-				name[j--] = c;
+				name[j] = c;
+				j--;
 			}
 			else
 			{
@@ -613,6 +622,59 @@ g_strcasestr (const gchar *a, const gchar *b)
 	return ptr == NULL ? NULL : (a + (ptr - down_a));
 }
 
+/**
+ * str_dup_replace: returns a new string (which must be freed later)
+ * identical to str, but with all occurrences of a replaced by b.  if any
+ * of the given strings are null, or if a is empty, null is returned. */
+gchar
+*str_dup_replace (const gchar *str, const gchar *a, const gchar *b)
+{
+	gchar *new_str, *start_str, *end_str, *dest_str;
+	gint str_len, a_len, b_len, new_str_len;
+
+	/* return if any of the strings are null, or if a is empty */
+	if (!str || !a || !b || a[0] == '\0') return NULL;
+
+	/* get the lengths of the user-supplied strings */
+	str_len = new_str_len = strlen (str);
+	a_len = strlen (a);
+	b_len = strlen (b);
+
+	/* find the length of the new string */
+	start_str = (gchar *) str;
+	while ((start_str = strstr (start_str, a)))
+	{
+		new_str_len -= a_len;
+		new_str_len += b_len;
+		start_str = &(start_str[a_len]);
+	}
+
+	/* allocate memory for the new string */
+	new_str = g_new0 (gchar, new_str_len + 1);
+	g_return_val_if_fail (new_str, NULL);
+
+	/* walk through the string, replacing a with b */
+	start_str = (gchar *) str;
+	dest_str = new_str;
+	while ((end_str = strstr (start_str, a)))
+	{
+		/* copy the stuff preceding a */
+		memcpy (dest_str, start_str, end_str - start_str);
+		dest_str = &(dest_str[end_str - start_str]);
+		/* copy b in the place of a */
+		memcpy (dest_str, b, b_len);
+		dest_str = &(dest_str[b_len]);
+		/* move to the end of the occurrence of a */
+		start_str = &(end_str[a_len]);
+	}
+
+	/* copy the last part of the string and terminate it */
+	memcpy (dest_str, start_str, strlen (start_str));
+	dest_str[strlen (start_str)] = '\0';
+
+	return new_str;
+}
+
 static gchar*
 get_first_directory (const gchar *start_dir) 
 {
@@ -691,237 +753,6 @@ launch_external_viewer (const gchar *filename)
 	g_free (prog);
 }
 
-
-static gchar *get_locale_charset (void)
-{
-#ifdef HAVE_CODESET
-		return nl_langinfo (CODESET);
-#else
-		gchar *charset = g_getenv ("CHARSET");
-		if (charset == NULL)
-		{
-			return "US-ASCII";
-		}
-		return charset;
-#endif
-}
-
-/**
- * locale_to_utf8
- */
-gchar *locale_to_utf8 (const gchar *locstr)
-{
-	gchar *utfstr;
-	size_t loclen, utflen;
-	gchar *fsave, *tsave;
-	static char *charset = NULL;
-	size_t count;
-	static iconv_t cd = (iconv_t)(-1);
-	
-	if (charset == NULL)
-	{
-		charset = get_locale_charset ();
-	}
-
-	if (cd == (iconv_t)(-1))
-	{
-		cd = iconv_open ("UTF-8", charset);
-		if (cd == (iconv_t)(-1))
-		{
-			g_warning ("iconv_open: %s", strerror (errno));
-			cd = iconv_open ("UTF-8", "US-ASCII");
-			g_assert (cd != (iconv_t)(-1));
-		}
-	}
-
-	if (locstr == NULL)
-	{
-		return NULL;
-	}
-
-	loclen = strlen (locstr);
-	utflen = MB_LEN_MAX * loclen;
-	utfstr = g_new0 (gchar, utflen + 1);
-	fsave = (gchar *)locstr;
-	tsave = utfstr;
-	count = iconv (cd, &fsave, &loclen, &tsave, &utflen);
-	if (count == -1)
-	{
-		g_warning ("locale_to_utf8: %s", strerror (errno));
-		g_free (utfstr);
-		return g_strdup (locstr);
-	}
-	return utfstr;
-}
-
-/**
- * utf8_to_locale
- */
-gchar *utf8_to_locale (const gchar *utfstr)
-{
-	gchar *locstr;
-	size_t utflen, loclen;
-	gchar *fsave, *tsave;
-	static char *charset = NULL;
-	size_t count;
-	static iconv_t cd = (iconv_t)(-1);
-
-	if (charset == NULL)
-	{
-		charset = get_locale_charset ();
-	}
-
-	if (cd == (iconv_t)(-1))
-	{
-		cd = iconv_open (charset, "UTF-8");
-		if (cd == (iconv_t)(-1))
-		{
-			g_warning ("iconv_open: %s", strerror (errno));
-			cd = iconv_open ("US-ASCII", "UTF-8");
-			g_assert (cd != (iconv_t)(-1));
-		}
-	}
-
-	if (utfstr == NULL)
-	{
-		return NULL;
-	}
-
-	utflen = strlen (utfstr);
-	loclen = MB_LEN_MAX * utflen;
-	locstr = g_new0 (gchar, loclen + 1);
-	fsave = (gchar *)utfstr;
-	tsave = locstr;
-	count = iconv (cd, &fsave, &utflen, &tsave, &loclen);
-	if (count == -1)
-	{
-		/* warning removed since this can happen quite innocently
-		 * and isn't really a problem -- which just can't display
-		 * the characters properly in the current locale. No need
-		 * to panic though -- MattA
-		g_warning ("utf8_to_locale: %s", strerror (errno));
- 		 */
-		free (locstr);
-		return g_strdup (utfstr);
-	}
-	return locstr;
-}
-
-/**
- * xmlSetPropLocale: set a property in an XML file which has value
- * encoded as Locale. This works around the strictness of libxml1.
- */
-xmlAttrPtr
-xmlSetPropLocale (xmlNodePtr node, 
-		  const xmlChar *name, 
-		  const xmlChar *value)
-{
-	xmlChar *buffer;
-	xmlAttrPtr ret;
-
-	/* check for nullification */
-	if (value == NULL)
-	{
-		return xmlSetProp (node, name, NULL);
-	}
-
-	/* do the conversion */
-	buffer = locale_to_utf8 (value);
-	
-	/* store in the node */
-	ret = xmlSetProp (node, name, buffer);
-
-	/* done with this */
-	g_free (buffer);
-
-	/* result */
-	return ret;
-}
-
-/**
- * xmlGetPropLocale: get a property in an XML file which has value
- * encoded as UTF8 but was originally local charset. This works around the
- * strictness of libxml1.
- */
-xmlChar *
-xmlGetPropLocale (xmlNodePtr node, const xmlChar *name)
-{
-	xmlChar *value;
-	xmlChar *buffer;
-
-	/* get the value */
-	value = xmlGetProp (node, name);
-	if (value == NULL)
-	{
-		return NULL;
-	}
-
-	/* do the conversion */
-	buffer = utf8_to_locale (value);
-
-	/* done with this */
-	xmlFree (value);
-
-	/* converted result */
-	return buffer;
-}
-
-/**
- * xmlSetRawPropLocale:
- */
-xmlAttrPtr
-xmlSetRawPropLocale (xmlNodePtr node, 
-		  const xmlChar *name, 
-		  const xmlChar *value)
-{
-	xmlChar *buffer;
-	xmlAttrPtr ret;
-
-	/* check for nullification */
-	if (value == NULL)
-	{
-		return xmlSetRawProp (node, name, NULL);
-	}
-
-	/* do the conversion */
-	buffer = locale_to_utf8 (value);
-	
-	/* store in the node */
-	ret = xmlSetRawProp (node, name, buffer);
-
-	/* done with this */
-	g_free (buffer);
-
-	/* result */
-	return ret;
-}
-
-/**
- * xmlGetRawPropLocale:
- */
-xmlChar *
-xmlGetRawPropLocale (xmlNodePtr node, const xmlChar *name)
-{
-	xmlChar *value;
-	xmlChar *buffer;
-
-	/* get the value */
-	value = xmlGetRawProp (node, name);
-	if (value == NULL)
-	{
-		return NULL;
-	}
-
-	/* do the conversion */
-	buffer = utf8_to_locale (value);
-
-	/* done with this */
-	xmlFree (value);
-
-	/* result */
-	return buffer;
-}
-
 /**
  * xmlSetRawProp: set a property in an XML file which has value encoded.
  * This works around the newline-losingness and broken entitiy handling 
@@ -1038,7 +869,7 @@ LinkState mouse_state_to_link_state (guint button, guint evstate)
 }
 
 PixmapData *
-pixmap_data_new_from_file (const gchar *file)
+pixmap_data_new_from_file (const gchar *file, gboolean prelight)
 {
 	GdkPixbuf *pixbuf;
 	PixmapData *pixmap_data;
@@ -1063,7 +894,7 @@ pixmap_data_new_from_file (const gchar *file)
 	    gdk_pixbuf_get_colorspace      (pixbuf) == GDK_COLORSPACE_RGB &&
 	    gdk_pixbuf_get_n_channels      (pixbuf) == 4)
 	{
-		pixbuf_threshold_alpha (pixbuf);
+		pixbuf_threshold_alpha (pixbuf, prelight);
 	}
 	
 	/* allocate space for a new pixmap */
@@ -1087,7 +918,7 @@ pixmap_data_new_from_file (const gchar *file)
  * most flat, gradient and pixmap themes
  */
 static void
-pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
+pixbuf_threshold_alpha (GdkPixbuf *pixbuf, gboolean prelight)
 {
 	gint width, height, rowstride;
 	guchar r, g, b, a;
@@ -1095,7 +926,19 @@ pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
 	guchar *buffer;
 	gint x, y, i;
 	GdkColor bg;
-	
+	GtkWidget *dummy = gtk_button_new ();
+	GtkStyle *rcstyle = gtk_rc_get_style (dummy);
+
+	/* this is needed for some (broken?) themes */
+	if (rcstyle == NULL)
+	{
+		rcstyle = gtk_style_new ();
+	}
+	else
+	{
+		gtk_style_ref (rcstyle);
+	}
+
 	/* get pixbuf properties */
 	width  = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
@@ -1103,7 +946,7 @@ pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
 	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 	
 	/* get target colour to blend with based on style background */
-	bg = loading_text_style->bg[0];
+	bg = prelight ? rcstyle->bg[2]: rcstyle->bg[0];
 	tr = bg.red   >> 8;
 	tg = bg.green >> 8;
 	tb = bg.blue  >> 8;
@@ -1141,6 +984,10 @@ pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
 			buffer[i + 3] = a;
 		}
 	}
+
+	gtk_widget_destroy (dummy);
+	gtk_style_unref (rcstyle);
+
 }
 
 /* NOTE: str must be at least 256 chars long */
@@ -1400,6 +1247,379 @@ gtk_radio_button_set (GtkRadioButton *radio_button, gint index)
 	}
 }
 
+/**
+ * simplify_uri: Simplify a local filesystem uri, eg /home/jorn/blah/../txt
+ * to /home/jorn/txt. Caller should free string.
+ */
+gchar *
+simplify_uri (const gchar *uri)
+{
+	gchar **chars, *ret;
+	GString *gstring;
+	gint i;
+	GList *l, *list = NULL;
+	gboolean leading_slash = FALSE;
+
+	if (uri[0] == '/') leading_slash = TRUE;
+	
+	chars = g_strsplit (uri, "/", 0);
+
+	for (i = 0; chars[i] != NULL; i++)
+	{
+		if (strcmp (chars[i], ".") == 0)
+		{
+			g_free (chars[i]);
+			chars[i] = g_strdup ("");
+		}
+		else if (strcmp (chars[i], "..") == 0)
+		{
+		 	GList *l;
+			gint c;
+
+			for (c = i - 1; c > 0 && chars[c] != NULL; c--)
+			{
+				if (strcmp (chars[c], "..") != 0 &&
+				    strlen (chars[c]) > 0)
+				{
+					break;
+				}
+			}
+
+			g_assert (chars[c] != NULL);
+
+			l = g_list_find (list, chars[c]);
+			if (l)
+			{
+				list = g_list_remove (list, l->data);
+				g_free (chars[c]);
+				chars[c] = g_strdup ("");
+			}
+		}
+		else
+		{
+			list = g_list_append (list, chars[i]);
+		}
+	}
+
+	if (leading_slash)
+		gstring = g_string_new ("/");
+	else
+		gstring = g_string_new ("");
+
+	for (l = list; l; l = g_list_next (l))
+	{
+		if (strcmp ((gchar *) l->data, "") == 0) continue;
+
+		if (strcmp (gstring->str,  "") != 0 &&
+		    strcmp (gstring->str, "/") != 0)
+		{
+			g_string_append_c (gstring, '/');
+		}
+		g_string_append (gstring, (gchar *) l->data);
+	}
+
+	ret = gstring->str;
+	
+	g_string_free (gstring, FALSE);
+	g_list_free (list);
+	g_strfreev (chars);
+	  
+	return ret;
+}
+
+/**
+ * xml_decode_entity: translate &amp; et al and return a new expanded string 
+ */
+gchar *
+xml_decode_entity (const unsigned char *encoded)
+{
+	gchar *buffer;
+	gint i, j, length;
+	
+	if (encoded == NULL)
+	{
+		return NULL;
+	}
+
+	buffer = g_strdup (encoded);
+
+	/* Optimize for case where no escape codes are found, otherwise
+	 * use the first found code as the starting index */
+	if ((i = (char *) strstr (buffer, "&") - buffer) < 0)
+	{
+		return buffer;
+	}
+	
+	length = strlen (buffer);
+	for (j = i; i < length; i++)
+	{
+		if (encoded[i] == '&')
+		{
+			if (strncmp (encoded + i + 1, "amp;", 4) == 0)
+			{
+				buffer[j++] = '&';
+				i += 4;
+			}
+			else if (strncmp (encoded + i + 1, "lt;", 3) == 0)
+			{
+				buffer[j++] = '<';
+				i += 3;
+			}
+			else if (strncmp (encoded + i + 1, "gt;", 3) == 0)
+			{
+				buffer[j++] = '>';
+				i += 3;
+			}
+			else if (strncmp (encoded + i + 1, "quot;", 5) == 0)
+			{
+				buffer[j++] = '"';
+				i += 5;
+			}
+			else if (strncmp (encoded + i + 1, "apos;", 5) == 0)
+			{
+				buffer[j++] = '\'';
+				i += 5;
+			}
+			else if (encoded[i + 1] == '#')
+			{
+				buffer[j++] = atoi (encoded + i + 2);
+				i += 5;
+			}
+		}
+		else
+		{
+			buffer[j++] = encoded[i];
+		}
+	}
+
+	buffer[j] = '\0';
+	return buffer;
+}
+
+/**
+ * toolbar_append_separator: build a separator with correct dimensions
+ */
+GtkWidget *
+toolbar_append_separator (GtkToolbar *toolbar)
+{
+	GtkWidget *vsep = gtk_vseparator_new (), *hsep = gtk_hseparator_new ();
+
+	gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), vsep, NULL, NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), hsep, NULL, NULL);
+	
+	gtk_signal_connect_after (GTK_OBJECT (toolbar), "orientation_changed", 
+			    GTK_SIGNAL_FUNC
+			   	(separator_toolbar_orientation_changed_cb),
+			    vsep);
+	gtk_signal_connect_after (GTK_OBJECT (toolbar), "orientation_changed", 
+			    GTK_SIGNAL_FUNC
+			   	(separator_toolbar_orientation_changed_cb),
+			    hsep);
+	
+	if (GTK_TOOLBAR (toolbar)->orientation == GTK_ORIENTATION_HORIZONTAL)
+	{
+		gtk_widget_show (vsep);
+		return vsep;
+	}
+	else
+	{
+		gtk_widget_show (hsep);
+		return hsep;
+	}
+}
+
+/*************************************************************************/
+/* original code stolen from gtkaccellabel code, GTK+ CREDITS            */
+/* modification necessary because gtkaccellabel doesn't work for what we */
+/* want to do in the bookmark menu code                     -- jorn      */
+/* N.B. Strings are not translated here as they aren't in original gtk   */
+/* either                                                                */
+/*************************************************************************/ 
+gchar *
+string_from_accel_data (GtkWidget *widget, guint accel_key, 
+			GdkModifierType accel_mods)
+{
+	GtkAccelLabel *al = GTK_ACCEL_LABEL (gtk_accel_label_new (""));
+	GtkAccelLabelClass *class = 
+		GTK_ACCEL_LABEL_CLASS (GTK_OBJECT (al)->klass);
+	gchar *str = NULL;
+      
+	GString *gstring;
+	gboolean had_mod = FALSE;
+	
+	gstring = g_string_new ("   ");
+
+	if (accel_mods & GDK_SHIFT_MASK)
+	{
+		g_string_append (gstring, class->mod_name_shift);
+		had_mod = TRUE;
+	}
+	if (accel_mods & GDK_CONTROL_MASK)
+	{
+		if (had_mod)
+	    		g_string_append (gstring, class->mod_separator);
+		g_string_append (gstring, class->mod_name_control);
+		had_mod = TRUE;
+	}
+	if (accel_mods & GDK_MOD1_MASK)
+	{
+		if (had_mod)
+			g_string_append (gstring, class->mod_separator);
+		g_string_append (gstring, class->mod_name_alt);
+		had_mod = TRUE;
+	}
+	      
+	if (had_mod)
+		g_string_append (gstring, class->mod_separator);
+      	if (accel_key < 0x80 || (accel_key > 0x80 && 
+	    accel_key <= 0xff && class->latin1_to_char))
+	{
+		switch (accel_key)
+	    	{
+		case ' ':
+			g_string_append (gstring, "Space");
+			break;
+		case '\\':
+			g_string_append (gstring, "Backslash");
+			break;
+		default:
+			g_string_append_c (gstring, 
+					   toupper (accel_key));
+			break;
+		}
+	}
+	else
+	{
+		gchar *tmp;
+	  
+		tmp = gtk_accelerator_name (accel_key, 0);
+		if (tmp[0] != 0 && tmp[1] == 0)
+			tmp[0] = toupper (tmp[0]);
+		g_string_append (gstring, tmp);
+		g_free (tmp);
+	}
+
+	str = gstring->str;
+	g_string_free (gstring, FALSE);
+
+	return str;
+}
+
+/*************************************************************************/
+/* gnome_dialog_run code stolen from libgnomeui, modified because the    */
+/* original is buggy.                        -- jorn                     */
+/*************************************************************************/
+
+struct GnomeDialogRunInfo {
+  gint button_number;
+  gint close_id, clicked_id, destroy_id;
+  gboolean destroyed;
+};
+
+static void
+gnome_dialog_shutdown_run(GnomeDialog* dialog,
+                          struct GnomeDialogRunInfo* runinfo)
+{
+  if (!runinfo->destroyed) 
+    {
+      
+      gtk_signal_disconnect(GTK_OBJECT(dialog),
+                            runinfo->close_id);
+      gtk_signal_disconnect(GTK_OBJECT(dialog),
+                            runinfo->clicked_id);
+  
+      runinfo->close_id = runinfo->clicked_id = -1;
+    }
+
+    gtk_main_quit();
+}
+
+static void
+gnome_dialog_setbutton_callback(GnomeDialog *dialog,
+				gint button_number,
+				struct GnomeDialogRunInfo *runinfo)
+{
+  if(runinfo->close_id < 0)
+    return;
+
+  runinfo->button_number = button_number;
+
+  gnome_dialog_shutdown_run(dialog, runinfo);
+}
+
+static gboolean
+gnome_dialog_quit_run(GnomeDialog *dialog,
+		      struct GnomeDialogRunInfo *runinfo)
+{
+  if(runinfo->close_id < 0)
+    return FALSE;
+
+  gnome_dialog_shutdown_run(dialog, runinfo);
+
+  return FALSE;
+}
+
+static void
+gnome_dialog_mark_destroy(GnomeDialog* dialog,
+                          struct GnomeDialogRunInfo* runinfo)
+{
+  runinfo->destroyed = TRUE;
+
+  if(runinfo->close_id < 0)
+    return;
+  else gnome_dialog_shutdown_run(dialog, runinfo);
+}
+
+gint
+dialog_run(GnomeDialog* dialog, gboolean close_after)
+{
+  struct GnomeDialogRunInfo ri = {-1,-1,-1,-1,FALSE};
+
+  g_return_val_if_fail(dialog != NULL, -1);
+  g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);
+
+  /* There are several things that could happen to the dialog, and we
+     need to handle them all: click, delete_event, close, destroy */
+
+  ri.clicked_id =
+    gtk_signal_connect(GTK_OBJECT(dialog), "clicked",
+		       GTK_SIGNAL_FUNC(gnome_dialog_setbutton_callback),
+		       &ri);
+
+  ri.close_id =
+    gtk_signal_connect(GTK_OBJECT(dialog), "close",
+		       GTK_SIGNAL_FUNC(gnome_dialog_quit_run),
+		       &ri);
+
+  ri.destroy_id = 
+    gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
+                       GTK_SIGNAL_FUNC(gnome_dialog_mark_destroy),
+                       &ri);
+
+  if ( ! GTK_WIDGET_VISIBLE(GTK_WIDGET(dialog)) )
+    gtk_widget_show(GTK_WIDGET(dialog));
+
+  gtk_main();
+  
+  if(!ri.destroyed) {
+
+    gtk_signal_disconnect(GTK_OBJECT(dialog), ri.destroy_id);
+
+    if(ri.close_id >= 0) /* We didn't shut down the run? */
+      {
+	gtk_signal_disconnect(GTK_OBJECT(dialog), ri.close_id);
+	gtk_signal_disconnect(GTK_OBJECT(dialog), ri.clicked_id);
+      }
+
+    if (close_after)
+      {
+        gnome_dialog_close(dialog);
+      }
+  }
+
+  return ri.button_number;
+}
+
 /*************************************************************************/
 /* tree utility functions stolen from gtkctree code, GTK+ CREDITS        */
 /* these are necessary because the original GtkCTree is a total f*ck-up, */
