@@ -27,8 +27,11 @@
 #include "session.h"
 #include "toolbar.h"
 #include "prefs.h"
+#include "themes.h"
 
 #include <gtk/gtkfilesel.h>
+#include <gtk/gtkvseparator.h>
+#include <gtk/gtkhseparator.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
 #include <libgnomeui/gnome-dialog-util.h>
@@ -241,6 +244,7 @@ context_bm_toolbar_appearance_menu_cb (GtkRadioMenuItem *menuitem,
 
 	/* button group is in reverse order! */
 	bm->toolbar_style = radio_menuitem_index (menuitem);
+	bookmarks_tb_dirty = TRUE;
 	bookmarks_updated ();
 }
 
@@ -261,6 +265,7 @@ context_main_toolbar_appearance_menu_cb (GtkRadioMenuItem *menuitem,
 	/* get new value */
 	eel_gconf_set_integer (CONF_TOOLBAR_STYLE, 
 			      radio_menuitem_index (menuitem));
+	window_update_ui ();
 }
 
 /**
@@ -291,3 +296,68 @@ radio_menuitem_index (GtkRadioMenuItem *menuitem)
 	/* reverse order! */
 	return ((length - 1) - index);
 }
+
+void
+separator_toolbar_orientation_changed_cb (GtkToolbar *tb,
+					  GtkOrientation o,
+					  GtkWidget *sep)
+{
+	gboolean vsep = GTK_IS_VSEPARATOR (sep);
+	GtkRequisition r;
+
+	gtk_widget_size_request (GTK_WIDGET (tb), &r);
+	
+	if (o == GTK_ORIENTATION_HORIZONTAL)
+	{
+		if (vsep)
+		{
+			gtk_widget_set_usize (sep, 5, r.height / 2);
+			gtk_widget_show (sep);
+		}
+		else
+			gtk_widget_hide (sep);
+	}
+	else
+	{
+		if (!vsep)
+		{
+			gtk_widget_set_usize (sep, r.width / 2, 5);
+			gtk_widget_show (sep);
+		}
+		else
+			gtk_widget_hide (sep);
+	}
+}
+
+/**
+ * gtk_theme_changed_cb: called when a Gtk+ theme change is detected, update the
+ * tab label styles and aa pixmaps
+ * FIXME this seems to be called too many times
+ */
+GdkFilterReturn
+gtk_theme_changed_cb (GdkXEvent *xevent, GdkEvent *event, gpointer data)
+{
+	GList *l;
+	
+	/* reload colour styles, reload pixmaps */
+	initialise_colours ();
+	reset_theme_pixmap_cache ();
+	bookmarks_load_icons ();
+	bookmarks_tb_dirty = TRUE;
+	bookmarks_updated ();
+	for (l = all_windows; l != NULL; l = g_list_next (l))
+	{
+		GaleonWindow *window = (GaleonWindow *) l->data;
+
+		window_save_layout (window);
+		toolbar_recreate (window);
+		window_restore_layout (window);
+		window_update_tabs_appearance (window);
+		window_statusbar_set_security_icon (window,
+			window->active_embed->secure);
+		bookmarks_menu_recreate (window);
+	}
+
+	/* process other GDK events */
+	return GDK_FILTER_CONTINUE;
+}
