@@ -29,6 +29,7 @@
 #include "context.h"
 #include "prefs.h"
 #include "downloader.h"
+#include "menubar.h"
 
 #include <time.h>
 #include <string.h>
@@ -37,13 +38,11 @@
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-triggers.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-util.h>
 #include <gdk/gdkkeysyms.h>
 
 /* Flag to enable/disable mouse motion listener */
 static gboolean mouse_listener_active = FALSE;
-static gboolean 
-handle_key_without_menubar (GaleonWindow *window, 
-			    gulong key, GdkModifierType ac_mods);
 
 typedef struct
 {
@@ -83,7 +82,8 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	}
 
 	/* create a new browser */
-	new_embed = embed_create (embed, !open_in_tab, FALSE, chrome_mask);
+	new_embed = embed_create (embed, !open_in_tab, FALSE, 
+				  TRUE, chrome_mask);
 
 	/* set the popup flag */
 	if (new_embed->parent_window != embed->parent_window)
@@ -102,7 +102,8 @@ void new_window_orphan_cb (GtkMozEmbedSingle *embed,
 	GaleonEmbed *new_embed;
 
 	/* create a new browser */
-	new_embed = embed_create (NULL, FALSE, FALSE, chrome_mask);
+	new_embed = embed_create (NULL, FALSE, FALSE, TRUE,
+				  chrome_mask);
 
 	/* set the popup flag */
 	new_embed->parent_window->is_popup = TRUE;
@@ -172,7 +173,7 @@ mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	embed_update_page_location (embed);
 
 	/* update the buttons view */
-	if (embed == window->active_embed)
+	if (embed->is_active)
 	{
 		window_update_nav_controls (window);
 	}
@@ -190,18 +191,20 @@ mozembed_title_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	/* free existing titles */
 	g_free (embed->site_title);
 	g_free (embed->site_title_utf8);
-	
+	embed->site_title = NULL;
+	embed->site_title_utf8 = NULL;
+
 	/* get both versions of new title */
 	embed->site_title = 
 		mozilla_get_document_title (embed, &(embed->site_title_utf8));
-
+	
 	/* reset to untitled if this title is nonsense */
 	if (embed->site_title == NULL || strlen (embed->site_title) == 0)
 	{
 		g_free (embed->site_title);
 		g_free (embed->site_title_utf8);
 		embed->site_title = g_strdup (_("Untitled"));
-		embed->site_title_utf8 = locale_to_utf8 (embed->site_title);
+		embed->site_title_utf8 = mozilla_locale_to_utf8 (embed->site_title);
 	}
 
 	/* update in gui */
@@ -228,18 +231,26 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	 * load_started tag, and clear the info about the progress */
 	if (embed->load_started == 1)
 	{
-		spinner_start (window);
 		embed_update_tab_status (embed);
 		embed_progress_clear (embed);
 		embed->when_started = time (NULL);
+
+		/* start spinner if this is the active embed */
+		if (embed->is_active)
+			spinner_start (window);
 	}
 
 	/* update the buttons view */
-	if (embed == window->active_embed)
+	if (embed->is_active)
 	{
 		window_update_nav_controls (window);
-		window_update_status_bar (window);
+		window_update_statusbar (window);
+		if (window->csslist != NULL)
+			gtk_widget_set_sensitive (window->csslist, FALSE);
 	}
+
+	/* no user sheet has been applied yet */
+	embed->usercss_applied = FALSE;
 }
 
 /**
@@ -261,9 +272,11 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	embed->load_started--;
 	embed_progress_clear (embed);
 
-	/* stop the window spinner */
 	if (embed->load_started == 0)
 	{
+		gint index;
+		AlternateStyleSheet *a = NULL;
+	
 		/* check zoom setting for the new site, unless it has 
 		   a javascript URI */
 		if (eel_gconf_get_boolean (CONF_HISTORY_AUTOZOOM) &&
@@ -282,8 +295,9 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 			}
 		}
 
-		/* stop spinner (actually reduces start count) */
-		spinner_stop (window);
+		/* stop spinner if this is the active embed */
+		if (embed->is_active)
+			spinner_stop (window);
 
 		/* new content, so only viewed if this is already active */
 		embed->has_been_viewed = embed->is_active;
@@ -291,13 +305,76 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		/* update tab title and status */
 		embed_update_tab_title (embed);
 		embed_update_tab_status (embed);
+		
+		/* load saved alternate stylesheet */
+		index = history_get_selected_authorcss (embed->site_location);
+		if (index != -1)
+		{
+			GList *csslist =
+				mozilla_get_alternate_stylesheets (embed);
+			GList *l;
+			gint i;
+			gboolean found = FALSE;
+
+			for (i = 1, l = csslist; l; l = g_list_next (l), i++)
+			{
+				a = (AlternateStyleSheet *) l->data;
+				if (index == i)
+				{
+					found = TRUE;
+					break;
+				}
+				g_free (a->name);
+				g_free (a);
+			}
+
+			if (index == 0 && g_list_length (csslist) > 0)
+			{
+				a = g_new0 (AlternateStyleSheet, 1);
+				a->name = g_strdup ("");
+				a->sheet = NULL;
+				found = TRUE;
+			}
+			
+			g_list_free (csslist);
+
+			if (found)
+			{
+				mozilla_set_alternate_stylesheet (embed, a);
+				g_free (a->name);
+				g_free (a);
+			}
+		}
+
+		/* load user stylesheet */
+		if (embed->usercss_applied == FALSE)
+		{
+			gchar *usersheet;
+
+			usersheet = eel_gconf_get_string
+					(CONF_RENDERING_USER_STYLESHEET);
+
+			if (usersheet && strcmp (usersheet, "") != 0)
+			{
+				if (g_file_exists (usersheet))
+				{
+					embed->usersheet =
+						mozilla_set_user_sheet
+							(embed, usersheet);
+				}
+				g_free (usersheet);
+			}
+
+			embed->usercss_applied = TRUE;
+		}
 	}
 
 	/* update the buttons view */
-	if (embed == window->active_embed)
+	if (embed->is_active)
 	{
 		window_update_nav_controls (window);
-		window_update_status_bar (window);
+		window_update_statusbar (window);
+		embed_init_css_menu (embed);
 	}
 }
 
@@ -331,7 +408,7 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 	}*/
 
 	/* clear temporary message, if any */
-	if (embed == window->active_embed)
+	if (embed->is_active)
 	{
 		window_update_temp_message (window, NULL);
 	}
@@ -402,7 +479,7 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 	}
 
 	/* display the update */
-	window_update_status_bar (window);
+	window_update_statusbar (window);
 }
 
 /**
@@ -443,9 +520,9 @@ void mozembed_progress_change_cb (GtkMozEmbed *dummy, gint cur, gint max,
 	}
 
 	/* update view */
-	if (embed == window->active_embed)
+	if (embed->is_active)
 	{
-		window_update_status_bar (window);
+		window_update_statusbar (window);
 	}
 }
 
@@ -464,8 +541,7 @@ mozembed_link_message_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	return_if_not_window (window);
 	
 	/* get the link message */
-	message = gtk_moz_embed_get_link_message 
-		(GTK_MOZ_EMBED(embed->mozEmbed));
+	message = mozilla_get_link_message (embed, NULL);
 
 	/* update browser message */
 	window_update_temp_message (window, message);
@@ -494,8 +570,7 @@ mozembed_js_status_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		/* get the javascript message */
 		if (embed->is_active)
 		{
-			message = gtk_moz_embed_get_js_status 
-				(GTK_MOZ_EMBED(embed->mozEmbed));
+			message = mozilla_get_js_status (embed, NULL);
 			
 			/* update the status bar message */
 			if (message != NULL)
@@ -507,18 +582,6 @@ mozembed_js_status_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	}
 }
 
-/**
- * mozembed_open_uri_cb: GTKMOZEMBED SIGNAL, emitted when the document 
- * tries to open a new document
- */
-gint 
-mozembed_open_uri_cb (GtkMozEmbed *dummy, const char *uri, GaleonEmbed *embed)
-{
-	return_val_if_not_embed (embed, TRUE);
-
-	return FALSE;
-}
-
 /**
  * mozembed_dom_mouse_down_cb: emitted on a button press event
  */
@@ -637,20 +700,33 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 
 		handled = TRUE;
 		break;
+
 	case 1:
 		if (!(info->ctx.context & CONTEXT_LINK))
 		{
-			if (eel_gconf_get_integer(
-				CONF_MOUSE_MMB_ACTION) == 0)
+			gint mmb_action;
 
-				/* popup a bookmarks menu */
-				context_show_bookmark_menu (embed);
-			else {
-				gtk_selection_convert(window->WMain,
-						      GDK_SELECTION_PRIMARY,
-						      GDK_SELECTION_TYPE_STRING,
-						      GDK_CURRENT_TIME);
+			mmb_action = eel_gconf_get_integer (
+				CONF_MOUSE_MMB_ACTION);
+
+			switch (mmb_action)
+			{
+				/* show the bookmarks menu */
+				case 0:
+					context_show_bookmark_menu (embed);
+					break;
+				/* load the url from the clipboard */
+				case 1:
+					gtk_selection_convert(window->WMain,
+						GDK_SELECTION_PRIMARY,
+						GDK_SELECTION_TYPE_STRING,
+						GDK_CURRENT_TIME);
+					break;
+				/* do nothing */
+				default:
+					break;
 			}
+
 			handled = TRUE;
 		}
 		break;
@@ -663,6 +739,18 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			handled = TRUE;
 		}
 		break;
+
+	/* perform back/forward for extra buttons */
+	case 5:
+		gtk_moz_embed_go_back (GTK_MOZ_EMBED (embed->mozEmbed));
+		handled = TRUE;
+		break;
+
+	case 6:
+		gtk_moz_embed_go_forward (GTK_MOZ_EMBED (embed->mozEmbed));
+		handled = TRUE;
+		break;
+
 	default:
 		break;
 	}
@@ -731,12 +819,16 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		switch(info->key)
 		{
 		case DOM_VK_BACK_SPACE:
-			if ((info->ctx.context & CONTEXT_INPUT) == 0)
+			/* FIXME disabled because at some point of the load
+			   the embed get the focus, so if users try to correct
+			   the url they wrote it goes back ... very confusing,
+			   we got a lot of bug reports */
+/*			if ((info->ctx.context & CONTEXT_INPUT) == 0)
 			{
 				gtk_moz_embed_go_back
 					(GTK_MOZ_EMBED (embed->mozEmbed));
 				handled = TRUE;
-			}
+				}*/
 			break;
 		default:
 			break;
@@ -817,7 +909,7 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		case DOM_VK_G:
 		case DOM_VK_l:
 		case DOM_VK_L:
-			if (window->toolBarOn && window->location_entry != NULL)
+			if (window->location_entry != NULL)
 			{
 				gtk_editable_select_region
 					(GTK_EDITABLE(window->location_entry),
@@ -841,6 +933,9 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		/* FIXME cleanup */
 		switch(info->key)
 		{
+		case DOM_VK_ESCAPE:
+			info->key = GDK_Escape;
+			break;
 		case DOM_VK_PAGE_UP:
 			info->key = GDK_Page_Up;
 			break;
@@ -856,9 +951,6 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		case DOM_VK_INSERT:
 			info->key = GDK_Insert;
 			break;
-		case DOM_VK_ESCAPE:
-			info->key = GDK_Escape;
-			break;
 		case DOM_VK_F12:
 			info->key = GDK_F12;
 			break;
@@ -904,18 +996,10 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	   main window in case it is a menu accelerator key */
 	if (!handled)
 	{
-		if (window->menuBarOn)
-		{
-			handled = gtk_accel_groups_activate 
-				(GTK_OBJECT (window->WMain), info->key, state);
-		}
-		else
-		{
-			handled = handle_key_without_menubar (window, 
-					info->key, (GdkModifierType)state);
-		}
+		handled = gtk_accel_groups_activate 
+			(GTK_OBJECT (window->WMain), info->key, state);
 	}
-
+	
 	mozilla_free_context_info_sub(&info->ctx);
 	g_free (info);
 				
@@ -923,40 +1007,6 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	return handled;
 }
 
-/**
- * handle_key_without_menubar: parse menubar uiinfo to handle key presses
- * when the menubar is not created. FIXME accels configurability is broken
- */
-gboolean
-handle_key_without_menubar (GaleonWindow *window, gulong key, 
-			    GdkModifierType ac_mods)
-{
-	extern GnomeUIInfo *all_menus[];
-	extern gint num_menus;
-	GnomeUIInfo *uiinfo;
-	int i;
-
-	for (i=0; i<num_menus; i++) 
-	{
-		uiinfo = all_menus[i];
-
-		for (; uiinfo->type != GNOME_APP_UI_ENDOFINFO; uiinfo++)
-		{
-			if (uiinfo->accelerator_key != 0 &&
-			    (uiinfo->accelerator_key == key ||
-			     uiinfo->accelerator_key == key - 32) && 
-			    uiinfo->ac_mods == ac_mods && 
-			    uiinfo->moreinfo)
-			{
-				GTK_SIGNAL_FUNC(uiinfo->moreinfo)(NULL, window);
-				return TRUE;
-			}
-		}
-	}
-	return FALSE;
-}
-
-
 /**
  * mozembed_size_to_cb: GTKMOZEMBED SIGNAL, emitted when a  size change 
  * is requested
@@ -989,6 +1039,26 @@ mozembed_size_to_cb (GtkMozEmbed *dummy, gint width, gint height,
 	}
 }
 
+void mozembed_security_change_cb (GtkMozEmbed *dummy, gpointer request,
+				  guint state, GaleonEmbed *embed)
+{	
+	GaleonWindow *window;
+	gboolean security_state = (state!=0);
+
+	/* get parent window */
+	return_if_not_embed (embed);
+	window = embed->parent_window;
+	return_if_not_window (window);
+
+	if (embed->secure != security_state)
+	{
+		embed->secure = security_state;
+		window_statusbar_set_security_icon 
+			(window, security_state);
+	}
+}
+
+
 /**
  * mozembed_destroy_cb: gtkmozembed component destroying
  */
