@@ -35,8 +35,6 @@ void mozembed_progress_change_cb (GtkMozEmbed *dummy, gint cur, gint max,
 				  GaleonEmbed *embed);
 void mozembed_link_message_cb (GtkMozEmbed *dummy, GaleonEmbed *embed);
 void mozembed_js_status_cb (GtkMozEmbed *dummy, GaleonEmbed *embed);
-gint mozembed_open_uri_cb (GtkMozEmbed *dummy, const char *uri, 
-			   GaleonEmbed *embed);
 void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 				    guint status, GaleonEmbed *embed);
 gint mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event, 
@@ -55,4 +53,5 @@ gboolean mozembed_drag_drop_cb (GtkWidget *widget, GdkDragContext *context,
 				gint x, gint y, 
 				GtkSelectionData *selection_data, 
 				guint info, guint time);
-
+void mozembed_security_change_cb (GtkMozEmbed *dummy, gpointer request,
+				  guint state, GaleonEmbed *embed);
