@@ -32,6 +32,7 @@ const char *lgroups[] = {
 	N_("Hebrew"),
 	N_("Indian"),
 	N_("Japanese"), 
+	N_("Korean"),
 	N_("Turkish"),
 	N_("Unicode"),
 	N_("Vietnamese"),
@@ -90,7 +91,9 @@ const CharsetInfo charset_trans_array[] = {
 	{N_("Japanese (EUC-JP)"),                 "EUC-JP",                LG_JAPANESE},
 	{N_("Japanese (ISO-2022-JP)"),            "ISO-2022-JP",           LG_JAPANESE},
 	{N_("Japanese (Shift_JIS)"),              "Shift_JIS",             LG_JAPANESE},
-	{N_("Korean (EUC-KR)"),                   "EUC-KR",                LG_OTHER},
+	{N_("Korean (EUC-KR)"),                   "EUC-KR",                LG_KOREAN},
+	{N_("Korean (JOHAB)"),                    "x-johab",               LG_KOREAN},
+	{N_("Korean (UHC)"),                      "x-windows-949",         LG_KOREAN},
 	{N_("Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER},
 	{N_("Romanian (MacRomanian)"),            "x-mac-romanian",        LG_OTHER},
 	{N_("South European (ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER},
