@@ -22,13 +22,14 @@
 #include "mozilla_i18n.h"
 #include "mozilla.h"
 #include "prefs.h"
+#include "nsBuildID.h"
 
 #include <string.h>
 #include <stdlib.h>
 #include <libgnome/gnome-defs.h>
 #include <locale.h>
 #include <libgnome/gnome-i18n.h>
-
+#include <sys/utsname.h>
 
 void mozilla_prefs_set_default_font (void);
 static int
@@ -60,6 +61,9 @@ mozilla_prefs_set (void)
 	mozilla_preference_set ("keyword.URL",_("http://www.google.com/search?q="));
 	mozilla_preference_set_boolean ("security.checkloaduri", FALSE); 
 
+	/* while we have no UI */
+ 	mozilla_preference_set_boolean ("wallet.captureForms", FALSE);
+
 	mozilla_save_prefs ();
 }
 
@@ -197,6 +201,8 @@ mozilla_prefs_set_font (gchar *name, gchar *key, int lang_enc)
 	gchar key_buf[128];
 	int size;
 
+	if (!name) return 0;
+
 	font = g_strsplit (name,"-",-1);
 	font_name = g_strconcat(font[1],"-",font[2],"-",
 				font[13],"-",font[14], NULL);
@@ -225,3 +231,59 @@ mozilla_prefs_set_font_size (gchar *key, int size, int lang_enc)
 	mozilla_preference_set_int (key_buf, size);
 }
 
+void
+mozilla_prefs_set_user_agent ()
+{
+	static gchar *default_user_agent = NULL;
+	gchar *value;
+	gchar *user_agent = NULL;
+	struct utsname name;
+	gchar *system;
+
+	if (!default_user_agent)
+	{
+		if (uname (&name) == 0)
+		{
+			system = g_strdup_printf ("%s %s",
+						  name.sysname, 
+						  name.machine);
+		}
+		else
+		{
+			system = g_strdup ("Unknown");
+		}
+		
+		default_user_agent = g_strdup_printf
+			("Mozilla/5.0 Galeon/" VERSION " (%s) Gecko/%d", 
+			 system,
+			 NS_BUILD_ID/100);
+
+		g_free (system);
+	}
+
+	value = eel_gconf_get_string (CONF_NETWORK_USER_AGENT);
+	
+	/* now, build a valid user agent string */
+	if (!value || !strcmp ("", value) 
+		   || !strcmp ("default", value) 
+	 	   || !strcmp ("Galeon", value)
+	 	   || !strcmp (_("Default (recommended)"), value)
+		   || !strcmp ("Default (recommended)", value))
+	{
+		user_agent = g_strdup (default_user_agent);
+	}
+	else
+	{
+		user_agent = g_strdup (value);
+	}
+				
+	if (user_agent)
+	{
+		mozilla_preference_set ("general.useragent.override",
+					user_agent);
+		g_free (user_agent);
+	}
+
+	g_free (value);
+}
+
