@@ -26,6 +26,7 @@
 #include "bookmarks.h"
 #include "misc.h"
 #include "prefs.h"
+#include "mozilla.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -42,7 +43,7 @@ typedef struct
 	gint startup_page;
 	gint toolbar_style;
 	gboolean enable_java, enable_javascript;
-	gchar *http_proxy, *ftp_proxy, *no_proxies;
+	gchar *proxy_auto_url, *http_proxy, *ftp_proxy, *no_proxies;
 	gint proxy_type, http_proxy_port, ftp_proxy_port;
 } ns_prefs;
 
@@ -55,16 +56,18 @@ static GTime ns_get_bookmark_date (const char *line, const char *search);
 static char *ns_parse_bookmark_item (GString *string);
 static ns_prefs *get_ns_prefs(gchar *);
 
-void netscape_export_bookmarks_item (FILE *file, BookmarkItem *b);
+static void netscape_export_bookmarks_item (FILE *file, BookmarkItem *b, 
+					    gboolean use_locale);
 
 
 /**
  * updates list, tree widget, and updates menu 
  * as the intelligence of this has improved, it's also gotten big and complex.
  * FIXME: needs modularization - Nate
+ * FIXME: needs a total rewrite, using libxml - Ricardo
  */
 gint 
-netscape_import_bookmarks (GString *filename)
+netscape_import_bookmarks (GString *filename, gboolean use_locale)
 {
 	BookmarkItem *parent, *newbm = NULL;
 	FILE *bf;  /* bookmark file */
@@ -76,6 +79,9 @@ netscape_import_bookmarks (GString *filename)
 	GSList *urls, *l;
 	int dup = 0;
 	GSList * folders;
+	gchar *str1, *str2;
+	gchar * (*to_utf8) (const gchar *) = 
+		use_locale ? mozilla_locale_to_utf8 : g_strdup;
 	
 	if (!(bf = fopen (filename->str, "r"))) {
 		g_warning ("Failed to open file: %s\n", filename->str);
@@ -107,8 +113,9 @@ netscape_import_bookmarks (GString *filename)
 			{    /* use existing folder if it's already there */
 				if (l->data)
 				{
-					gchar *stripped = strip_uline_accel (
-						((BookmarkItem *)(l->data))->name);
+					gchar *str = to_utf8 (((BookmarkItem *)(l->data))->name);
+					gchar *stripped = strip_uline_accel (str);
+					g_free (str);
 					if (!g_strcasecmp(stripped, name->str))
 					{
 						g_free (stripped);
@@ -121,9 +128,11 @@ netscape_import_bookmarks (GString *filename)
 					
 			if (!newbm)
 			{
+				gchar *str = to_utf8 (name->str);
 				newbm = bookmarks_new_bookmark 
-					(BM_FOLDER, TRUE, name->str, 
+					(BM_FOLDER, TRUE, str, 
 					 NULL, NULL, NULL, NULL);
+				g_free (str);
 				newbm->parent = parent;
 				newbm->create_toolbar = strcmp
 					(name->str, 
@@ -153,9 +162,16 @@ netscape_import_bookmarks (GString *filename)
 			if (dup) break;
 			
 			parsedname = ns_parse_bookmark_item (name);
+			str1 = to_utf8 (parsedname);
+			if (nick->str) 
+				str2 = to_utf8 (nick->str);
+			else
+				str2 = NULL;
 			newbm = bookmarks_new_bookmark 
-				(BM_SITE, TRUE, parsedname, url->str, 
-				 nick->str, NULL, NULL);
+				(BM_SITE, TRUE, str1, url->str, 
+				 str2, NULL, NULL);
+			g_free (str2);
+			g_free (str1);
 			g_free (parsedname);
 			newbm->parent = parent;
 			parent->list = g_list_insert(parent->list, newbm, -1);
@@ -167,7 +183,11 @@ netscape_import_bookmarks (GString *filename)
 			if (newbm)
 			{
 				char *tmpstr = newbm->notes;
-				newbm->notes = g_strconcat (tmpstr, name->str, NULL);
+				str1 = to_utf8 (tmpstr);
+				str2 = to_utf8 (name->str);
+				newbm->notes = g_strconcat (str1, str2, NULL);
+				g_free (str2);
+				g_free (str1);
 				g_free (tmpstr);
 			}
 			break;
@@ -197,6 +217,7 @@ netscape_import_bookmarks (GString *filename)
 	g_slist_free(urls);
 	g_slist_free(folders);
 		
+	bookmarks_tb_dirty = TRUE;
 	bookmarks_updated ();
 
 	return 0;	
@@ -343,7 +364,8 @@ ns_parse_bookmark_item (GString *string)
  * Will overwrite the file!
  */
 gint 
-netscape_export_bookmarks (gchar *filename, BookmarkItem *root)
+netscape_export_bookmarks (gchar *filename, BookmarkItem *root, 
+			   gboolean use_locale)
 {
 	FILE *file;
 	GList *l;
@@ -363,6 +385,8 @@ netscape_export_bookmarks (gchar *filename, BookmarkItem *root)
 	fputs ("<!-- This file was automatically generated by Galeon\n", file);
 	fputs ("It will be read and overwritten.\n", file);
 	fputs ("Do Not Edit! -->\n", file);
+	if (!use_locale)
+		fputs ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\">\n", file);
 	fputs ("<TITLE>Bookmarks</TITLE>\n", file);
 	fputs ("<H1>Bookmarks</H1>\n", file);
 	fputs ("\n", file);
@@ -370,7 +394,7 @@ netscape_export_bookmarks (gchar *filename, BookmarkItem *root)
 
 	/* write the bookmarks */
 	for (l = root->list; l != NULL; l = l->next) 
-		netscape_export_bookmarks_item (file, l->data);
+		netscape_export_bookmarks_item (file, l->data, use_locale);
 	
 	/* finish */
 	fputs ("</DL><p>\n", file);
@@ -379,11 +403,15 @@ netscape_export_bookmarks (gchar *filename, BookmarkItem *root)
 	return 0;
 }
 
-void 
-netscape_export_bookmarks_item (FILE *file, BookmarkItem *b)
+static void 
+netscape_export_bookmarks_item (FILE *file, BookmarkItem *b, gboolean use_locale)
 {
 	GList *l;
+	gchar *str;
 	gchar *strippedname;
+	gchar * (*from_utf8) (const gchar *) = 
+		use_locale ? mozilla_utf8_to_locale : g_strdup;
+
 	switch (b->type) {
 	case BM_SITE:
 		fputs ("\t<DT><A HREF=\"", file);
@@ -398,36 +426,46 @@ netscape_export_bookmarks_item (FILE *file, BookmarkItem *b)
 		/* support a mozilla-only feature, but won't harm netscape */
 		if (strlen (b->nick) > 0) {
 			fputs (" SHORTCUTURL=\"", file);
-			fputs (b->nick, file);
+			str = from_utf8 (b->nick);
+			fputs (str, file);
+			g_free (str);
 			fputs ("\"", file);
 		}
 		fputs (">", file);
 		strippedname = strip_uline_accel (b->name);
-		fputs (strippedname, file);
+		str = from_utf8 (strippedname);
+		fputs (str, file);
+		g_free (str);
 		g_free(strippedname);
 		fputs ("</A>\n", file);
 		if (strlen (b->notes) > 0) {
 			fputs ("<DD>", file);
-			fputs (b->notes, file);
+			str = from_utf8 (b->notes);
+			fputs (str, file);
+			g_free (str);
 			fputs ("\n", file);
 		}
 		break;
 	case BM_FOLDER:
 		fputs ("<DT><H3 ADD_DATE=\"0\">", file); 
 		strippedname = strip_uline_accel (b->name);
-		fputs (strippedname, file);
+		str = from_utf8 (strippedname);
+		fputs (str, file);
+		g_free (str);
 		g_free(strippedname);
 		fputs ("</H3>\n", file);
 		if (strlen (b->notes) > 0) {
 			fputs ("<DD>", file);
-			fputs (b->notes, file);
+			str = from_utf8 (b->notes);
+			fputs (str, file);
+			g_free (str);
 			fputs ("\n", file);
 		}
 		fputs ("<DL><p>\n", file);
 		/* FIXME: Alias catogories are not exported */
 		if (!b->alias_of)
 			for (l = b->list; l != NULL; l = l->next) 
-				netscape_export_bookmarks_item (file, l->data);
+				netscape_export_bookmarks_item (file, l->data, use_locale);
 		else
 			g_warning ("Detected a bookmark item type that I can't export!");
 		fputs ("</DL><p>\n", file);
@@ -544,6 +582,8 @@ get_ns_prefs(gchar *nsprefsfile)
 					 * 2: auto proxy setup
 					 */
 				}
+				else if (strcmp(pkey->str, "network.proxy.autoconfig_url") == 0)
+					prefs.proxy_auto_url = g_strdup(value->str);
 				else if (strcmp(pkey->str, "network.proxy.http_port") == 0)
 					prefs.http_proxy_port = atoi(value->str);
 				else if (strcmp(pkey->str, "network.proxy.ftp_port") == 0)
@@ -587,21 +627,22 @@ netscape_import_prefs(void)
 		case 1: buffer = 0; break;
 		case 2: buffer = 1; break;
 	}
-	if (prefs->proxy_type == 1)  /* if manual proxy (auto not yet supported) */
-	{
-		eel_gconf_set_boolean(CONF_NETWORK_PROXY_ENABLED, 
-				      prefs->proxy_type > 0);
-		eel_gconf_set_string(CONF_NETWORK_HTTP_PROXY, 
-				     prefs->http_proxy);
-		eel_gconf_set_string(CONF_NETWORK_FTP_PROXY, 
-				     prefs->ftp_proxy);
-	        eel_gconf_set_integer(CONF_NETWORK_HTTP_PROXY_PORT, 
-				      prefs->http_proxy_port);
-		eel_gconf_set_integer(CONF_NETWORK_FTP_PROXY_PORT, 
-				      prefs->ftp_proxy_port);
-		eel_gconf_set_string(CONF_NETWORK_NO_PROXIES_FOR, 
-				     prefs->no_proxies);
-	}
+
+	eel_gconf_set_boolean(CONF_NETWORK_PROXY_MODE, 
+			      prefs->proxy_type);
+	eel_gconf_set_string(CONF_NETWORK_PROXY_AUTO_URL,
+			     prefs->proxy_auto_url);
+	eel_gconf_set_string(CONF_NETWORK_HTTP_PROXY, 
+			     prefs->http_proxy);
+	eel_gconf_set_string(CONF_NETWORK_FTP_PROXY, 
+			     prefs->ftp_proxy);
+        eel_gconf_set_integer(CONF_NETWORK_HTTP_PROXY_PORT, 
+			      prefs->http_proxy_port);
+	eel_gconf_set_integer(CONF_NETWORK_FTP_PROXY_PORT, 
+			      prefs->ftp_proxy_port);
+	eel_gconf_set_string(CONF_NETWORK_NO_PROXIES_FOR, 
+			     prefs->no_proxies);
+
 	eel_gconf_set_integer(CONF_GENERAL_STARTPAGE_TYPE, 
 			     buffer);
 	eel_gconf_set_integer(CONF_TOOLBAR_STYLE, 
