@@ -22,6 +22,7 @@
 #include "glade.h"
 #include "bookmarks.h"
 #include "prefs.h"
+#include "mozilla.h"
 
 #include <string.h>
 #include <gtk/gtktogglebutton.h>
@@ -66,9 +67,13 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	gboolean rpmfind_checked;
 	gboolean debian_packages_checked;
 	gboolean bugzilla_checked;
+	gboolean stock_quotes_checked;
+	gboolean imdb_movies_checked;
+	gboolean google_images_checked;
 	gboolean bookmarklets_checked;
 	gboolean http_checked;
 	gboolean https_checked;
+	gboolean ghelp_checked;
 	gboolean ftp_checked;
 	gboolean file_checked;
 	gchar *http_proxy_site;
@@ -106,6 +111,12 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		  &debian_packages_checked, },
 		{ "new_user_druid_bugzilla_check_button",
 		  &bugzilla_checked, },
+		{ "new_user_druid_stock_quotes_check_button",
+		  &stock_quotes_checked, },
+		{ "new_user_druid_imdb_movies_check_button",
+		  &imdb_movies_checked, },
+		{ "new_user_druid_google_images_check_button",
+		  &google_images_checked, },
 		{ "new_user_druid_bookmarklets_check_button",
 		  &bookmarklets_checked, },
 		{ "new_user_druid_http_check_button",
@@ -114,6 +125,8 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		  &https_checked, },
 		{ "new_user_druid_ftp_check_button",
 		  &ftp_checked, },
+		{ "new_user_druid_ghelp_check_button",
+		  &ghelp_checked, },
 		{ "new_user_druid_file_check_button",
 		  &file_checked, },
 		{ NULL, NULL }/* terminator, must be last */
@@ -185,11 +198,14 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	if (create_smart_bookmarks_checked)
 	{
 		BookmarkItem *toolbar;
-		
+		gchar* utfstr;
+
 		/* create the toolbar */
+		utfstr = mozilla_locale_to_utf8(_("Smart Bookmarks Toolbar"));
 		toolbar = bookmarks_new_bookmark 
-			(BM_FOLDER, TRUE, _("Smart Bookmarks Toolbar"),
+			(BM_FOLDER, TRUE, utfstr,
 			 NULL, NULL, NULL, NULL);
+		g_free(utfstr);
 		toolbar->create_toolbar = TRUE;
 		toolbar->toolbar_style = TOOLBAR_STYLE_HORIZONTAL;
 		
@@ -213,7 +229,8 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		{
 			add_smart (toolbar, "Usenet", "news",
 				   "http://groups.google.com/groups?q=%s"
-				   "&meta=site%3Dgroups", NULL, NULL);
+				   "&meta=site%3Dgroups", 
+				   SHARE_DIR "/google_groups.png", NULL);
 		}
 		if (dogpile_checked)
 		{
@@ -274,6 +291,25 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 				   "release=all", 
 				   SHARE_DIR "/debian.png", NULL);
 		}
+		if (google_images_checked)
+		{
+			add_smart (toolbar, "Images", "img",
+				   "http://images.google.com/images?q=%s",
+				   SHARE_DIR "/google_images.png", NULL);
+		}
+		if (stock_quotes_checked)
+		{
+			add_smart (toolbar, "Stocks", "st",
+				   "http://finance.yahoo.com/q?s=%s"
+				   "&d=c&k=c4&t=1d",
+				   SHARE_DIR "/stocks.png", NULL);
+		}
+		if (imdb_movies_checked)
+		{
+			add_smart (toolbar, "IMDb", "imdb",
+				   "http://www.imdb.com/Find?%s", 
+				   SHARE_DIR "/imdb_movies.png", NULL);
+		}
 		if (bookmarklets_checked)
 		{
 			add_bookmarklets (toolbar);
@@ -302,6 +338,10 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	{
 		gnome_config_set_string ("https-show", galeon_show);
 	}
+	if (ghelp_checked)
+	{
+		gnome_config_set_string ("ghelp-show", galeon_show);
+	}
 	if (ftp_checked)
 	{
 		gnome_config_set_string ("ftp-show", galeon_show);
@@ -315,8 +355,8 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	/* setup proxies */
 	if (http_proxy_site != NULL && strlen (http_proxy_site) > 0)
 	{
-		eel_gconf_set_boolean(CONF_NETWORK_PROXY_ENABLED
-				      , TRUE);
+		eel_gconf_set_integer(CONF_NETWORK_PROXY_MODE
+				      , 1);
 		eel_gconf_set_string (CONF_NETWORK_HTTP_PROXY
 				      , http_proxy_site);
 		eel_gconf_set_string (CONF_NETWORK_SSL_PROXY
@@ -336,6 +376,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	
 	/* save all the initial settings */
+	bookmarks_tb_dirty = TRUE;
 	bookmarks_updated ();
 	
 	/* create an initial window if needed */
@@ -414,10 +455,13 @@ add_smart (BookmarkItem *parent, gchar *name, gchar *nick,
 	   gchar *url, gchar *pixmap_file, gchar * notes)
 {
 	BookmarkItem *site;
+	gchar* utfstr;
 	
+	utfstr = mozilla_locale_to_utf8(name);
 	/* create the bookmarks */
-	site = bookmarks_new_bookmark (BM_SITE, TRUE, name, url, nick, notes, 
+	site = bookmarks_new_bookmark (BM_SITE, TRUE, utfstr, url, nick, notes, 
 				       pixmap_file);
+	g_free(utfstr);
 	
 	/* add it to the toolbar */
 	parent->list = g_list_append (parent->list, site);
@@ -443,7 +487,7 @@ add_bookmarklets (BookmarkItem *parent)
 		{ N_("More Info About..."), "javascript:if(frames.length>0){F=' (Open frame in new window first.)'}else{F=''}Q=document.getSelection();if(!Q){void(Q=prompt('No text selected on page.'+F+'\\n\\nKeywords...?',''))};if(Q)location.href='http://bookmarklets.com/moreinfo.phtml?q='+escape(Q)" },
 		{ "separator", NULL },
 		{ N_("Zoom on First Image..."), "javascript:if(document.images.length<1){alert('No image!')}else{D0iB=document.images[0];sZ2T=prompt('Resize by..','1.5');if(sZ2T){with(document){write('<img src='+D0iB.src+' width='+sZ2T*D0iB.width+' height='+sZ2T*D0iB.height+'>');close()}}}void(null)" },
-		{ N_("Page Color..."), "javascript:if(frames.length<1){void(document.bgColor=prompt('Change to which background color?',''))}else{alert('Page has frames. Use the version of this bookmarklet for frames. (bookmarklets.com)')}" },
+		{ N_("Page Color..."), "javascript:if(frames.length<1){var c0l=prompt('Change to which background color?','');if (c0l != null) void(document.bgColor=c0l)}else{alert('Page has frames. Use the version of this bookmarklet for frames. (bookmarklets.com)')}" },
 		{ "separator", NULL },
 		{ N_("Scroll Page (very slow)"), "javascript:var wN2sc8Rl;Sa5gNA9k=new Function('clearTimeout(wN2sc8Rl)');document.onkeydown=Sa5gNA9k;Sa5gNA9k();void(wN2sc8Rl=setInterval('if(pageYOffset<document.height-innerHeight){window.scrollBy(0,1)}else{Sa5gNA9k()}',150))" },
 		{ N_("Scroll Page (slow)"), "javascript:var wN2scRl;Sa5gNA9k=new Function('clearTimeout(wN2scRl)');document.onkeydown=Sa5gNA9k;Sa5gNA9k();void(wN2scRl=setInterval('if(pageYOffset<document.height-innerHeight){window.scrollBy(0,1)}else{Sa5gNA9k()}',50))" },
@@ -461,9 +505,12 @@ add_bookmarklets (BookmarkItem *parent)
 		{ N_("AutoFill Anonymous"), "javascript:function ROIoiW(){var i=0,j,A='anonymous',D,E,F=document.forms;while(i<F.length){E=F[i].elements;for(j=0;j<E.length;j++){D=E[j];if(D.type=='text'){D.value=(D.name.toUpperCase().indexOf('MAIL')!=-1)?A+'@example.com':A}}i++}}ROIoiW();void(null)" },
 		{ NULL, NULL}
 	};
+	gchar* utfstr;
 
-	root = bookmarks_new_bookmark (BM_FOLDER, TRUE, _("Bookmarklets"),
+	utfstr = mozilla_locale_to_utf8(_("Bookmarklets"));
+	root = bookmarks_new_bookmark (BM_FOLDER, TRUE, utfstr,
 				       NULL, NULL, NULL, NULL);
+	g_free(utfstr);
 	
 	parent->list = g_list_append (parent->list, root);
 	root->parent = parent;
@@ -479,9 +526,11 @@ add_bookmarklets (BookmarkItem *parent)
 		}
 		else
 		{
+			utfstr= mozilla_locale_to_utf8(_(bookmarklets[i].name));
 			item = bookmarks_new_bookmark
-				(BM_SITE, TRUE, _(bookmarklets[i].name), 
+				(BM_SITE, TRUE, utfstr,
 				 bookmarklets[i].url, NULL, NULL, NULL);
+			g_free(utfstr);
 		}
 
 		item->parent = root;
