@@ -16,6 +16,20 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/*
+
+  - Panel applet TODOs:
+M   - bookmarks resync
+    - dont quit on exit
+    - restarting properly
+    - tidy up compositing
+M   - enable DnD
+    - shift click
+    - pasting
+    - shift pasting
+
+*/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -24,6 +38,7 @@
 /* Galeon includes */
 #include "galeon.h"
 #include "embed.h"
+#include "misc.h"
 
 /* GNOME includes */
 #include <applet-widget.h>
@@ -242,6 +257,7 @@ panel_applet_build_bookmarks_menu (AppletWidget *applet, BookmarkItem *bi,
 {
 	gchar *name;
 	GList *l;
+	gchar *loc_str;
 
 	/* build the menu path name */
 	name = g_strdup_printf ("%s/%ld", path, bi->id);
@@ -251,9 +267,11 @@ panel_applet_build_bookmarks_menu (AppletWidget *applet, BookmarkItem *bi,
 	{
 	case BM_SITE:
 		/* a site is an entry with a callback */
+		loc_str = mozilla_utf8_to_locale (bi->name);
 		applet_widget_register_callback	
-			(applet, name, bi->name, (AppletCallbackFunc)
+			(applet, name, loc_str, (AppletCallbackFunc)
 			 panel_applet_bookmark_activate_cb, bi);
+		g_free (loc_str);
 		break;
 
 	case BM_FOLDER:
@@ -264,12 +282,14 @@ panel_applet_build_bookmarks_menu (AppletWidget *applet, BookmarkItem *bi,
 			return;
 		}
 		/* a folder builds recursively */
-		applet_widget_register_callback_dir (applet, name, bi->name);
+		loc_str = mozilla_utf8_to_locale (bi->name);
+		applet_widget_register_callback_dir (applet, name, loc_str);
 		for (l = bi->list; l != NULL; l = g_list_next (l))
 		{
 			BookmarkItem *bi = (BookmarkItem *)(l->data);
 			panel_applet_build_bookmarks_menu (applet, bi, name);
 		}
+		g_free (loc_str);
 		break;
 
 	case BM_SEPARATOR:
