@@ -33,7 +33,6 @@
 
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtknotebook.h>
-#include <gtk/gtktogglebutton.h>
 #include <gtk/gtkentry.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomeui/gnome-propertybox.h>
@@ -46,7 +45,6 @@ static GtkWidget *stored_host;
 static GtkWidget *stored_path;
 static GtkWidget *stored_secure;
 static GtkWidget *stored_expire;
-static GtkWidget *stored_block;
 
 #define foreach_list_elem(elem, list) \
 	for (elem = list; elem != NULL; elem = g_list_next(elem))
@@ -62,7 +60,7 @@ typedef enum {
 typedef struct {
 	GList		*keep, *remove;
 	GtkWidget	*clist;
-	GtkWidget	*remove_button, *remove_all_button;
+	GtkWidget	*remove_button, *block_button, *remove_all_button;
 	gint		last_sort_column;
 	GtkSortType	last_sort_type;
 	PDMViewType     type;
@@ -95,7 +93,6 @@ view_unselect_cb (GtkCList *clist, gint row, gint column,
 {
 	GList *entry = gtk_clist_get_row_data (clist, row);
 	CookieBase *c = (CookieBase*)entry->data;
-	c->block = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(stored_block));
 	c->remove = FALSE;
 }
 
@@ -104,7 +101,6 @@ view_remove_cb (GtkWidget *widget, gpointer data)
 {
 	PDMView *view = data;
 	GList *entry, *next, *remove = NULL;
-	gboolean block = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(stored_block));
 
 	for (entry = view->keep; entry != NULL; entry = next)
 	{
@@ -117,7 +113,7 @@ view_remove_cb (GtkWidget *widget, gpointer data)
 		
 		// This will be used by the cookie remover to know if it should be 
 		// completely blocked next time.
-		b->block = block;
+		b->block = FALSE;
 
 		rownum = gtk_clist_find_row_from_data (GTK_CLIST(view->clist),
 						       entry);
@@ -128,6 +124,7 @@ view_remove_cb (GtkWidget *widget, gpointer data)
 		view->remove = g_list_concat (view->remove, entry);
 		
 		gnome_property_box_changed (GNOME_PROPERTY_BOX(dialog));
+
 	}
 
 	cookies_clear_stored_fields ();
@@ -135,6 +132,47 @@ view_remove_cb (GtkWidget *widget, gpointer data)
 	foreach_list_elem (entry, remove)
 		gtk_clist_remove (GTK_CLIST(view->clist),
 				  GPOINTER_TO_INT(entry->data));
+
+	g_list_free (remove);
+}
+
+static void
+view_block_cb (GtkWidget *widget, gpointer data)
+{
+	PDMView *view = data;
+	GList *entry, *next, *remove = NULL;
+
+	for (entry = view->keep; entry != NULL; entry = next)
+	{
+		CookieBase *b = (CookieBase*)entry->data;
+		gint rownum;
+
+		next = g_list_next (entry);
+
+		if (b->remove == FALSE) continue;
+		
+		// This will be used by the cookie remover to know if it should be 
+		// completely blocked next time.
+		b->block = TRUE;
+
+		rownum = gtk_clist_find_row_from_data (GTK_CLIST(view->clist),
+						       entry);
+
+		remove = g_list_append (remove, GINT_TO_POINTER(rownum));
+
+		view->keep = g_list_remove_link (view->keep, entry);
+		view->remove = g_list_concat (view->remove, entry);
+		
+		gnome_property_box_changed (GNOME_PROPERTY_BOX(dialog));
+
+	}
+
+	cookies_clear_stored_fields ();
+
+	foreach_list_elem (entry, remove)
+		gtk_clist_remove (GTK_CLIST(view->clist),
+				  GPOINTER_TO_INT(entry->data));
+
 	g_list_free (remove);
 }
 
@@ -208,6 +246,11 @@ cookie_view_init (PDMView *view)
 			    view_unselect_cb, view);
 	gtk_signal_connect (GTK_OBJECT(view->remove_button), "clicked",
 			    GTK_SIGNAL_FUNC(view_remove_cb), view);
+	if (view->block_button)
+	{
+		gtk_signal_connect (GTK_OBJECT(view->block_button), "clicked",
+				    GTK_SIGNAL_FUNC(view_block_cb), view);
+	}
 	gtk_signal_connect (GTK_OBJECT(view->remove_all_button), "clicked",
 			    GTK_SIGNAL_FUNC(view_remove_all_cb), view);
 	gtk_signal_connect (GTK_OBJECT(view->clist), "key_press_event",
@@ -233,8 +276,6 @@ stored_select_cb (GtkCList *clist, gint row, gint column,
 		gtk_entry_set_text (GTK_ENTRY(stored_path), c->path);
 		gtk_entry_set_text (GTK_ENTRY(stored_secure), c->secure);
 		gtk_entry_set_text (GTK_ENTRY(stored_expire), c->expire);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(stored_block),
-					      c->base.block);
 	}
 }
 
@@ -489,8 +530,8 @@ dialog_help_cb (GtkWidget *widget, gpointer data)
 {
 }
 
-GtkWidget *
-pdm_show_manager (GaleonWindow *window)
+static GtkWidget *
+pdm_create_manager (GaleonWindow *window)
 {
 	GladeXML *gxml;
 
@@ -505,6 +546,7 @@ pdm_show_manager (GaleonWindow *window)
 	stored = g_new0 (PDMView, 1);
 	stored->clist = glade_lookup_widget (dialog, "stored_clist");
 	stored->remove_button = glade_lookup_widget (dialog, "stored_remove");
+	stored->block_button = glade_lookup_widget (dialog, "stored_block");
 	stored->remove_all_button = glade_lookup_widget (dialog, 
 							 "stored_remove_all");
 	stored->type = PDM_COOKIE;
@@ -517,7 +559,6 @@ pdm_show_manager (GaleonWindow *window)
 	stored_path = glade_lookup_widget (dialog, "stored_path");
 	stored_secure = glade_lookup_widget (dialog, "stored_secure");
 	stored_expire = glade_lookup_widget (dialog, "stored_expire");
-	stored_block = glade_lookup_widget (dialog, "stored_block");
 
 	cookie_view_init (stored);
 
@@ -580,18 +621,13 @@ pdm_show_manager (GaleonWindow *window)
 			    dialog_help_cb, NULL);
 	gtk_signal_connect (GTK_OBJECT(dialog), "destroy", 
 			    dialog_destroy_cb, NULL);
-	
+
 	gtk_window_set_title (GTK_WINDOW(dialog), 
 			      _("Persistent Data Manager"));
 	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
 
 	init_all_clists ();
 
-	gtk_widget_show (dialog);
-	if (window != NULL)
-	{
-		dialog_set_parent (dialog, window->WMain);
-	}
 	gtk_object_destroy (GTK_OBJECT(gxml));
 
 	return dialog;
@@ -602,10 +638,17 @@ pdm_show_page (GaleonWindow *window, PDMPage page)
 {
 	GtkWidget *dialog, *notebook;
 
-	dialog = pdm_show_manager (window);
+	dialog = pdm_create_manager (window);
 
 	notebook = GNOME_PROPERTY_BOX (dialog)->notebook;
 	gtk_notebook_set_page (GTK_NOTEBOOK(notebook), page);
 
+	if (window != NULL)
+	{
+		dialog_set_parent (dialog, window->WMain);
+	}
+
+	gtk_widget_show_all (dialog);
+
 	return dialog;
 }
