@@ -47,6 +47,7 @@
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-icon-list.h>
 #include <libgnomeui/gnome-font-picker.h>
 #include <libgnomeui/gnome-color-picker.h>
 #include <libgnomeui/gnome-preferences.h>
@@ -112,8 +113,9 @@ void prefs_fonts_free_all (PreferencesDialog *pd);
 void prefs_fonts_save (PreferencesDialog *pd);
 static int get_int_from_optionmenu (GtkWidget *optionmenu);
 static void preferences_theme_selector_init (PreferencesDialog *pd);
-static void theme_selector_fill_themelist_from_dir (GtkCList *list, gchar *base);
-static void free_theme_selector (GtkCList *list);
+static void theme_selector_fill_themelist_from_dir (GnomeIconList *list, 
+						    gchar *base);
+static void free_theme_selector (GnomeIconList *list);
 
 /* local variables */
 static const char invalid_chars[] = "\t\r\n\"$&<>,+=#!()'|{}[]?~`;%\\" "/:";
@@ -187,6 +189,9 @@ preferences_open (GaleonWindow *window)
 	/* set values in dialog to stored values in prefs */
 	preferences_set_dialog_to_config (pd);
 
+	/* set mouse wheel prefs sensitivity */
+	mouse_wheel_controls_set_sensitive (pd);
+
 	/* display it */
 	gtk_widget_show (GTK_WIDGET (pd->dialog));
 	dialog_set_parent (GTK_WIDGET (pd->dialog), window->WMain);
@@ -230,6 +235,10 @@ void
 preferences_close (PreferencesDialog *pd)
 {
 	prefs_dialog_save_geometry (pd);
+	/* This unselect is necessary because for some reason, gtk is
+	 * unselecting the row without sending the unselect signal when the
+	 * window is hidden.  -Daniel */
+	gtk_clist_unselect_all (GTK_CLIST (pd->mime_list));
 	gtk_widget_hide (GTK_WIDGET (pd->dialog));
 	prefs_fonts_free_all (pd);
 }
@@ -262,6 +271,9 @@ preferences_apply (PreferencesDialog *pd)
 {
 	preferences_set_config_to_dialog (pd);
 
+	/* update the window ui */
+	window_update_ui ();
+
 	/* save changed fonts values */
 	prefs_fonts_remember (pd);
 	prefs_fonts_save (pd);
@@ -273,12 +285,12 @@ preferences_apply (PreferencesDialog *pd)
 void
 preferences_help (void)
 {
-	GnomeHelpMenuEntry pref_help = { "galeon", "preferences.html" };
+	GnomeHelpMenuEntry pref_help = { "galeon-manual", "preferences.html" };
 	gchar *file = NULL;
 
 	/* Try to find the preferences help page.  If we fail, use ghelp
 	   to load the index, instead */
-	file = gnome_help_file_find_file("galeon", "preferences.html");
+	file = gnome_help_file_find_file("galeon-manual", "preferences.html");
 
 	if (file)
 	{
@@ -286,7 +298,7 @@ preferences_help (void)
 	}
 	else
 	{
-		gnome_help_goto (NULL, "ghelp:galeon");
+		gnome_help_goto (NULL, "ghelp:galeon-manual");
 	}
 
 	g_free(file);
@@ -300,9 +312,15 @@ preferences_build_dialog (void)
 	GList *li;
 	GtkWidget *menu;
 
+	/* clear up any existing display, makes the app seem more responsive */
+	while (gtk_events_pending ())
+	{
+		gtk_main_iteration ();
+	}
+
 	/* not been generated yet... */
 	pd = g_new0 (PreferencesDialog, 1);
-	
+
 	/* build the widgets */
 	gxml = glade_widget_new ("preferences.glade", 
 				 "preferences_dialog", &(pd->dialog), pd);
@@ -337,6 +355,44 @@ preferences_build_dialog (void)
 	} 
 	pd->last_language = 0;
 
+	/* attach mouse wheel sensitivity callbacks */
+	gtk_signal_connect(
+		GTK_OBJECT(GTK_OPTION_MENU(pd->wheel_nokey_action_optionmenu)->menu),
+		"deactivate", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(GTK_OPTION_MENU(pd->wheel_alt_action_optionmenu)->menu),
+		"deactivate", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(GTK_OPTION_MENU(pd->wheel_ctrl_action_optionmenu)->menu),
+		"deactivate", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(GTK_OPTION_MENU(pd->wheel_shift_action_optionmenu)->menu),
+		"deactivate", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(pd->wheel_nokey_default_step_checkbutton),
+		"toggled", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(pd->wheel_alt_default_step_checkbutton),
+		"toggled", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(pd->wheel_ctrl_default_step_checkbutton),
+		"toggled", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+        gtk_signal_connect(
+		GTK_OBJECT(pd->wheel_shift_default_step_checkbutton),
+		"toggled", GTK_SIGNAL_FUNC(prefs_wheel_changed_cb),
+		pd);
+
+#ifdef ENABLE_EXPERIMENTAL_FILTERS
+	gtk_widget_hide (pd->allow_popups_checkbutton);
+#endif
+	
 	/* return completed dialog */
 	return pd;
 }
@@ -479,7 +535,8 @@ preferences_build_sidebar (PreferencesDialog *pd)
 			gtk_widget_show (vbox);
 
 
-			icon_data = pixmap_data_new_from_file (icon->icon);
+			icon_data = pixmap_data_new_from_file (icon->icon,
+							       FALSE);
 
 			if (NULL != icon_data)
 			{
@@ -502,11 +559,6 @@ preferences_build_sidebar (PreferencesDialog *pd)
 
 		gtk_object_set_data (GTK_OBJECT (panel_button), 
 				     "group", button_group);
-
-		/* force Gtk+ to align the panel buttons to the top */
-		widget = gtk_invisible_new ();
-		gtk_box_pack_end (GTK_BOX (panel), widget, TRUE, TRUE, 0);
-		gtk_widget_show (widget);
 	}
 }
 
@@ -622,7 +674,7 @@ preferences_set_config_to_dialog (PreferencesDialog *pd)
 				     CONF_TABS_TABBED_INSERT_NEW_TABS);
 	set_config_from_optionmenu (pd->tabbed_position_optionmenu, 
 				   CONF_TABS_TABBED_POSITION);
-        set_config_from_radiobuttongroup (pd->tabbed_style_radiobutton1, 
+        set_config_from_radiobuttongroup (pd->shorten_target_radiobutton, 
                                    CONF_TABS_TABBED_SHORTEN_STYLE);
 	set_config_from_togglebutton (pd->tabbed_always_show_check, 
 				     CONF_TABS_TABBED_ALWAYS_SHOW);
@@ -656,6 +708,18 @@ preferences_set_config_to_dialog (PreferencesDialog *pd)
 				     CONF_WINDOWS_FS_SHOW_MENUBAR);
 	set_config_from_togglebutton (pd->show_statusbar_in_fullscreen_check,
 				     CONF_WINDOWS_FS_SHOW_STATUSBAR);
+	set_config_from_togglebutton (pd->show_toolbars_check,
+				     CONF_WINDOWS_SHOW_TOOLBARS);
+	set_config_from_togglebutton (pd->show_bookmarks_check,
+				     CONF_WINDOWS_SHOW_BOOKMARKS);
+	set_config_from_togglebutton (pd->show_menubar_check,
+				     CONF_WINDOWS_SHOW_MENUBAR);
+	set_config_from_togglebutton (pd->show_statusbar_check,
+				     CONF_WINDOWS_SHOW_STATUSBAR);
+	set_config_from_togglebutton (pd->show_history_dock_check,
+				     CONF_WINDOWS_SHOW_HISTORY_DOCK);
+	set_config_from_togglebutton (pd->show_bookmarks_dock_check,
+				     CONF_WINDOWS_SHOW_BOOKMARKS_DOCK);
 	set_config_from_editable (pd->windows_title_entry, 
 				  CONF_WINDOWS_TITLE);
 	set_config_from_togglebutton (pd->fullscreen_stacking_checkbutton,
@@ -682,10 +746,14 @@ preferences_set_config_to_dialog (PreferencesDialog *pd)
 				      CONF_BOOKMARKS_GOTO_BUTTON);
 	set_config_from_togglebutton (pd->smart_bm_history_checkbutton,
 				      CONF_BOOKMARKS_HISTORY);
+	set_config_from_togglebutton (pd->smart_bm_clear_checkbutton,
+				      CONF_BOOKMARKS_CLEAR);
 	set_config_from_color (pd->tabbed_loading_color, 
 			       CONF_TABS_TABBED_LOADING_COLOR);
 	set_config_from_color (pd->tabbed_new_color, 
 			       CONF_TABS_TABBED_NEW_COLOR);
+	set_config_from_editable (pd->user_stylesheet_entry,
+				  CONF_RENDERING_USER_STYLESHEET);
 
 	/* process mouse options */
 	set_config_from_togglebutton (pd->history_popup_checkbutton,
@@ -709,10 +777,18 @@ preferences_set_config_to_dialog (PreferencesDialog *pd)
 				      CONF_PROGRAMS_USE_EXTERNAL_SOURCE_VIEWER);
 	set_config_from_editable (pd->external_source_viewer_entry, 
 				  CONF_PROGRAMS_EXTERNAL_SOURCE_VIEWER);
+
+	set_config_from_editable (pd->legacy_mailer_entry, 
+				  CONF_PROGRAMS_MAILER);
+
+	set_config_from_togglebutton (pd->use_internal_ghelp_check,
+				      CONF_GHELP_USE_GALEON);
  
         /* process advanced preferences */
-	set_config_from_togglebutton (pd->proxy_use_checkbutton, 
-				      CONF_NETWORK_PROXY_ENABLED);
+	set_config_from_radiobuttongroup (pd->proxy_none_radiobutton, 
+					  CONF_NETWORK_PROXY_MODE);
+	set_config_from_editable (pd->proxy_auto_url_entry,
+				  CONF_NETWORK_PROXY_AUTO_URL);
 	set_config_from_editable (pd->http_proxy_entry, 
 				  CONF_NETWORK_HTTP_PROXY);
 	set_config_from_editable (pd->ssl_proxy_entry, 
@@ -866,7 +942,7 @@ preferences_set_dialog_to_config (PreferencesDialog *pd)
 				     CONF_TABS_TABBED_INSERT_NEW_TABS);
 	set_optionmenu_from_config (pd->tabbed_position_optionmenu, 
 				   CONF_TABS_TABBED_POSITION);
-        set_radiobuttongroup_from_config (pd->tabbed_style_radiobutton1, 
+        set_radiobuttongroup_from_config (pd->shorten_target_radiobutton, 
                                    CONF_TABS_TABBED_SHORTEN_STYLE);
 	set_togglebutton_from_config (pd->tabbed_always_show_check, 
 				   CONF_TABS_TABBED_ALWAYS_SHOW);
@@ -900,6 +976,18 @@ preferences_set_dialog_to_config (PreferencesDialog *pd)
 				     CONF_WINDOWS_FS_SHOW_MENUBAR);
 	set_togglebutton_from_config (pd->show_statusbar_in_fullscreen_check,
 				     CONF_WINDOWS_FS_SHOW_STATUSBAR);
+	set_togglebutton_from_config (pd->show_toolbars_check,
+				     CONF_WINDOWS_SHOW_TOOLBARS);
+	set_togglebutton_from_config (pd->show_bookmarks_check,
+				     CONF_WINDOWS_SHOW_BOOKMARKS);
+	set_togglebutton_from_config (pd->show_menubar_check,
+				     CONF_WINDOWS_SHOW_MENUBAR);
+	set_togglebutton_from_config (pd->show_statusbar_check,
+				     CONF_WINDOWS_SHOW_STATUSBAR);
+	set_togglebutton_from_config (pd->show_history_dock_check,
+				     CONF_WINDOWS_SHOW_HISTORY_DOCK);
+	set_togglebutton_from_config (pd->show_bookmarks_dock_check,
+				     CONF_WINDOWS_SHOW_BOOKMARKS_DOCK);
 	set_editable_from_config (pd->windows_title_entry,
 				 CONF_WINDOWS_TITLE);
 	set_togglebutton_from_config (pd->fullscreen_stacking_checkbutton,
@@ -926,6 +1014,8 @@ preferences_set_dialog_to_config (PreferencesDialog *pd)
 				      CONF_BOOKMARKS_GOTO_BUTTON);
 	set_togglebutton_from_config (pd->smart_bm_history_checkbutton,
 				      CONF_BOOKMARKS_HISTORY);
+	set_togglebutton_from_config (pd->smart_bm_clear_checkbutton,
+				      CONF_BOOKMARKS_CLEAR);
 	smart_bm_entry_width = gtk_spin_button_get_value_as_int
 		(GTK_SPIN_BUTTON (pd->smart_bm_entry_width_spin));
 	gtk_widget_set_usize (pd->smart_bm_entry_width_spin,
@@ -934,6 +1024,8 @@ preferences_set_dialog_to_config (PreferencesDialog *pd)
 			       CONF_TABS_TABBED_LOADING_COLOR);
 	set_color_from_config (pd->tabbed_new_color, 
 			       CONF_TABS_TABBED_NEW_COLOR);
+	set_editable_from_config (pd->user_stylesheet_entry,
+				  CONF_RENDERING_USER_STYLESHEET);
 
 	/* process mouse options */
 	set_togglebutton_from_config (pd->history_popup_checkbutton,
@@ -957,10 +1049,18 @@ preferences_set_dialog_to_config (PreferencesDialog *pd)
 				      CONF_PROGRAMS_USE_EXTERNAL_SOURCE_VIEWER);
 	set_editable_from_config (pd->external_source_viewer_entry, 
 				  CONF_PROGRAMS_EXTERNAL_SOURCE_VIEWER);
+
+	set_editable_from_config (pd->legacy_mailer_entry, 
+				  CONF_PROGRAMS_MAILER);
+
+	set_togglebutton_from_config (pd->use_internal_ghelp_check,
+				      CONF_GHELP_USE_GALEON);
  
         /* process advanced preferences */
-	set_togglebutton_from_config (pd->proxy_use_checkbutton,
-				     CONF_NETWORK_PROXY_ENABLED);
+	set_radiobuttongroup_from_config (pd->proxy_none_radiobutton,
+					  CONF_NETWORK_PROXY_MODE);
+	set_editable_from_config (pd->proxy_auto_url_entry,
+				  CONF_NETWORK_PROXY_AUTO_URL);
 	set_editable_from_config (pd->http_proxy_entry, 
 				  CONF_NETWORK_HTTP_PROXY);
 	set_editable_from_config (pd->ssl_proxy_entry, 
@@ -1132,6 +1232,11 @@ preferences_mime_set_dialog_to_config (PreferencesDialog *pd)
 		g_free (text[2]);
 	}
 
+	/* sort */
+	gtk_clist_set_sort_column (GTK_CLIST(clist), 0);
+	gtk_clist_set_sort_type (GTK_CLIST(clist), GTK_SORT_ASCENDING);
+	gtk_clist_sort (GTK_CLIST(clist));
+
 	/* thaw the list -- make changes visible */
 	gtk_clist_thaw (GTK_CLIST(clist));	
 }
@@ -1144,8 +1249,8 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 { "notebook",                       &(pd->notebook)                          },
 { "sidebar",                        &(pd->sidebar)                           },
 { "themes_page",                    &(pd->themes_page)                       },
-{ "spinner_iconlist",               &(pd->iconlist)                          },
-{ "theme_list",                     &(pd->themelist)                         },
+{ "spinner_iconlist",               &(pd->spinner_iconlist)                  },
+{ "theme_iconlist",                 &(pd->theme_iconlist)                    },
 { "theme_dir_entry",                &(pd->theme_dir_entry)                   },
 { "smart_bm_example_entry",         &(pd->smart_bm_example_entry)            },
 { "lang_entry",                     &(pd->lang_entry)                        },
@@ -1164,6 +1269,7 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 { "smart_bm_entry_width_spin",      &(pd->smart_bm_entry_width_spin)         },
 { "smart_bm_goto_button_check",     &(pd->smart_bm_goto_button_checkbutton)  },
 { "smart_bm_history_check",         &(pd->smart_bm_history_checkbutton)      },
+{ "smart_bm_clear_check",           &(pd->smart_bm_clear_checkbutton)        },
 { "java_checkbutton",               &(pd->java_checkbutton)                  },
 { "javascript_checkbutton",         &(pd->javascript_checkbutton)            },
 { "user_agent_entry",               &(pd->user_agent_entry)                  },
@@ -1178,17 +1284,20 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 
 /* mime page */
 { "mime_list",                      &(pd->mime_list)                         },
-{ "mime_name_entry",                &(pd->mime_name_entry)                   },
+{ "mime_type_label",                &(pd->mime_type_label)                   },
+{ "mime_type_entry",                &(pd->mime_type_entry)                   },
 { "mime_save_to_disk",	            &(pd->mime_save_to_disk)		     },
 { "mime_helper_frame",	            &(pd->mime_helper_frame)		     },
 { "mime_helper_entry",	            &(pd->mime_helper_entry)		     },
 { "mime_always_use",	            &(pd->mime_always_use)		     },
 { "mime_terminal",	            &(pd->mime_terminal)		     },
 { "mime_url_helper",	            &(pd->mime_url_helper)		     },
+{ "mime_delete_button",	            &(pd->mime_delete_button)		     },
 
 /* downloader page */
 { "download_use_builtin",           &(pd->download_use_builtin)              },
 { "download_command_entry",         &(pd->download_command_entry)            },
+{ "download_command_label",         &(pd->download_command_label)            },
 { "download_command_combo",         &(pd->download_command_combo)            },
 { "download_run_in_terminal",       &(pd->download_run_in_terminal)          },
 { "download_autodownload",          &(pd->download_autodownload)             },
@@ -1202,11 +1311,12 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 { "tabbed_autojump_check",          &(pd->tabbed_autojump_check)             },
 { "tabbed_insert_new_tabs_check",   &(pd->tabbed_insert_new_tabs_check)      },
 { "tabbed_position_optionmenu",     &(pd->tabbed_position_optionmenu)        },
-{ "radiobutton1", /* FIXME */       &(pd->tabbed_style_radiobutton1)         },
+{ "shorten_target_radiobutton",     &(pd->shorten_target_radiobutton)        },
 { "tabbed_always_show_check",       &(pd->tabbed_always_show_check)          },
 { "tabbed_closebutton_check",       &(pd->tabbed_closebutton_check)          },
 { "tabbed_closebutton_insensitive_check",
 				 &(pd->tabbed_closebutton_insensitive_check) },
+{ "tabbed_shorten_name_vbox",       &(pd->tabbed_shorten_name_vbox)          },
 { "tabbed_shorten_spin",            &(pd->tabbed_shorten_spin)               },
 { "tabbed_shorten_spin2",/* FIXME */&(pd->tabbed_shorten_spin2)              },
 { "tabbed_separator_check",         &(pd->tabbed_separator_check)            },
@@ -1232,6 +1342,7 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 { "fantasy_font_fpicker",           &(pd->fantasy_font_fpicker)              },
 { "monospace_font_fpicker",         &(pd->monospace_font_fpicker)            },
 { "minimum_font_size_spin",         &(pd->minimum_font_size_spin)            },
+{ "user_stylesheet_entry",          &(pd->user_stylesheet_entry)             },
 { "use_own_fonts_checkbutton",      &(pd->use_own_fonts_checkbutton)         },
 { "default_font_optionmenu",        &(pd->default_font_optionmenu)           },
 { "history_expire_spin",            &(pd->history_expire_spin)               },
@@ -1244,9 +1355,17 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
                                   &(pd->show_menubar_in_fullscreen_check)    },
 { "show_statusbar_in_fullscreen_check",
                                   &(pd->show_statusbar_in_fullscreen_check)  },
+{ "show_toolbars_check",            &(pd->show_toolbars_check)               },
+{ "show_bookmarks_check",           &(pd->show_bookmarks_check)              },
+{ "show_menubar_check",             &(pd->show_menubar_check)                },
+{ "show_statusbar_check",           &(pd->show_statusbar_check)              },
+{ "show_history_dock_check",        &(pd->show_history_dock_check)           },
+{ "show_bookmarks_dock_check",      &(pd->show_bookmarks_dock_check)         },
 { "use_external_source_viewer_check",
                                   &(pd->use_external_source_viewer_check)    },
 { "external_source_viewer_entry",   &(pd->external_source_viewer_entry)      },
+{ "legacy_mailer_entry",            &(pd->legacy_mailer_entry)               },
+{ "use_internal_ghelp_check",	    &(pd->use_internal_ghelp_check)	     },
 { "crash_optionmenu",               &(pd->crash_optionmenu)                  },
 { "allow_popups_checkbutton",       &(pd->allow_popups_checkbutton)          },
 { "windows_title_entry",            &(pd->windows_title_entry)               },
@@ -1263,7 +1382,11 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 { "use_own_colors_checkbutton",     &(pd->use_own_colors_checkbutton)        },
 { "underline_links_checkbutton",    &(pd->underline_links_checkbutton)       },
 { "proxy_keepalive_checkbutton",    &(pd->proxy_keepalive_checkbutton)       },
-{ "proxy_use_checkbutton",          &(pd->proxy_use_checkbutton)             },
+{ "proxy_manual_radiobutton",	    &(pd->proxy_manual_radiobutton)	     },
+{ "proxy_auto_radiobutton",	    &(pd->proxy_auto_radiobutton)	     },
+{ "proxy_none_radiobutton",	    &(pd->proxy_none_radiobutton)	     },
+{ "proxy_auto_url_entry",	    &(pd->proxy_auto_url_entry)		     },
+{ "proxy_auto_reload_button",	    &(pd->proxy_auto_reload_button)	     },
 { "http_version_optionmenu",        &(pd->http_version_optionmenu)           },
 { "history_popup_checkbutton",      &(pd->history_popup_checkbutton)         },
 { "middle_button_optionmenu",       &(pd->middle_button_optionmenu)          },
@@ -1294,6 +1417,7 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 { "statusbar_rewrite_check",        &(pd->statusbar_rewrite_check)           },
 { "lang_encoding",                  &(pd->lang_encoding_optionmenu)          },
 { "autozoom_checkbutton",           &(pd->autozoom_checkbutton)              },
+{ "mime_action_frame",              &(pd->mime_action_frame)                 },
 
 { NULL, NULL } /* terminator, must be last */
         };
@@ -1305,19 +1429,40 @@ preferences_lookup_widgets (PreferencesDialog *pd, GladeXML *gxml)
 void
 set_config_from_editable (GtkWidget *editable, gchar *config_name)
 {
-	gchar *value;
-	gchar *old_value = NULL;
+	GConfValue *gcvalue;
+	gchar *value, *old_value;
+	gint ivalue;
+	gfloat fvalue;
+
+	gcvalue = eel_gconf_get_value (config_name);
+	if (gcvalue == NULL)
+		return;
 
-	old_value = eel_gconf_get_string(config_name);
 	/* get all the text into a new string */	
 	value = gtk_editable_get_chars (GTK_EDITABLE(editable), 0, -1);
 	
-	if ((old_value == NULL)||(strcmp(old_value, value) != 0)) {
-		eel_gconf_set_string (config_name, value);
+	switch (gcvalue->type) {
+	case GCONF_VALUE_STRING:
+		old_value = eel_gconf_get_string (config_name);
+		if ((old_value == NULL)||(strcmp(old_value, value) != 0)) {
+			eel_gconf_set_string (config_name, value);
+		}
+		g_free (old_value);
+		break;
+	case GCONF_VALUE_INT:
+		ivalue = atoi (value);
+		if (ivalue != eel_gconf_get_integer (config_name))
+			eel_gconf_set_integer (config_name, ivalue);
+		break;
+	case GCONF_VALUE_FLOAT:
+		fvalue = strtod (value, (char**)NULL);
+		if (fvalue != eel_gconf_get_float (config_name))
+			eel_gconf_set_float (config_name, fvalue);
+		break;
+	default:
 	}
 
 	/* free the allocated strings */
-	g_free (old_value);
 	g_free (value);
 }
 
@@ -1429,10 +1574,26 @@ set_config_from_color (GtkWidget *colorpicker, gchar *config_name)
 void
 set_editable_from_config (GtkWidget *editable, gchar *config_name)
 {
+	GConfValue *gcvalue;
 	gchar *value;
 
-	/* get the current value from the configuration space */
-	value = eel_gconf_get_string (config_name);
+	gcvalue = eel_gconf_get_value (config_name);
+	if (gcvalue == NULL)
+		return;
+	
+	switch (gcvalue->type) {
+	case GCONF_VALUE_STRING:
+		value = eel_gconf_get_string (config_name);
+		break;
+	case GCONF_VALUE_INT:
+		value = g_strdup_printf ("%d",eel_gconf_get_integer (config_name));
+		break;
+	case GCONF_VALUE_FLOAT:
+		value = g_strdup_printf ("%.2f",eel_gconf_get_float (config_name));
+		break;
+	default:
+		value = NULL;
+	}
 	
 	/* set this string value in the widget */
 	if (value)
@@ -1567,41 +1728,6 @@ set_color_from_config (GtkWidget *colorpicker, gchar *config_name)
 	}
 }
 
-void
-mouse_wheel_controls_set_sensitive (PreferencesDialog *pd, WheelModKey key,
-				    gboolean sensitive)
-{
-	GtkWidget *spinbutton = NULL, *checkbutton = NULL;
-	
-	switch (key)
-	{
-	case WHEEL_MOD_NOKEY:
-		spinbutton = pd->wheel_nokey_step_spinbutton;
-		checkbutton = pd->wheel_nokey_default_step_checkbutton;
-		break;
-	case WHEEL_MOD_ALT:
-		spinbutton = pd->wheel_alt_step_spinbutton;
-		checkbutton = pd->wheel_alt_default_step_checkbutton;
-		break;
-	case WHEEL_MOD_CTRL:
-		spinbutton = pd->wheel_ctrl_step_spinbutton;
-		checkbutton = pd->wheel_ctrl_default_step_checkbutton;
-		break;
-	case WHEEL_MOD_SHIFT:
-		spinbutton = pd->wheel_shift_step_spinbutton;
-		checkbutton = pd->wheel_shift_default_step_checkbutton;
-		break;
-	default:
-		break;
-	}
-
-	if (spinbutton && checkbutton)
-	{
-		gtk_widget_set_sensitive (spinbutton, sensitive);
-		gtk_widget_set_sensitive (checkbutton, sensitive);
-	}
-}
-
 static void
 prefs_dialog_save_geometry (PreferencesDialog *pd)
 {
@@ -1610,12 +1736,15 @@ prefs_dialog_save_geometry (PreferencesDialog *pd)
 }
 
 gboolean
-preferences_galeon_used_by_gnome_for_ftp (void)
+preferences_galeon_used_by_gnome_for_protocol (const char *protocol)
 {
-	gchar *handler;
 	gboolean uses_galeon;
 
-	handler = gnome_config_get_string ("/Gnome/URL Handlers/ftp-show");
+	gchar *path = g_strconcat ("/Gnome/URL Handlers/", protocol,
+				   "-show", NULL);
+	gchar *handler = gnome_config_get_string (path);
+	g_free (path);
+
 	if (handler == NULL)
 	{
 		handler = gnome_config_get_string 
@@ -1773,67 +1902,104 @@ int get_int_from_optionmenu (GtkWidget *optionmenu)
 static void
 preferences_theme_selector_init (PreferencesDialog *pd)
 {
-	GtkCList *list = GTK_CLIST (pd->themelist);
+	GnomeIconList *list = GNOME_ICON_LIST (pd->theme_iconlist);
 	gchar *home_theme_dir, *home_nautilus_theme_dir;
-	gchar *row[1];
 	gchar *pref_theme_path = NULL;
+	g_assert (list!=NULL);
 
-	gtk_clist_freeze (list);
-
-	gtk_clist_set_sort_column (list, 0);
-	gtk_clist_set_sort_type (list, GTK_SORT_ASCENDING);
+	gnome_icon_list_freeze (list);
 
 	/* clear the list out */
 	free_theme_selector (list);
-	gtk_clist_clear (list);
+	gnome_icon_list_clear (list);
 
 	/* fill list */
-	row[0] = g_strdup (_("GNOME default"));
-	gtk_clist_append (list, row);
-	g_free (row[0]);
-	gtk_clist_set_row_data (list, 0, g_strdup (""));
+	gnome_icon_list_append (list, SHARE_DIR "/gnome_icons.png", 
+				_("GNOME default"));
+	gnome_icon_list_set_icon_data (list, 0, g_strdup (""));
 
-	row[0] = g_strdup (_("Nautilus theme"));
-	gtk_clist_append (list, row);
-	g_free (row[0]);
-	gtk_clist_set_row_data (list, 1, 
-				g_strdup ("NAUTILUS_THEME"));
+	gnome_icon_list_append (list, SHARE_DIR "/nautilus_icons.png", 
+				_("Nautilus theme"));
+	gnome_icon_list_set_icon_data (list, 1, g_strdup ("NAUTILUS_THEME"));
 
 	/* select default row */
-	gtk_clist_select_row (list, 0, 0);
+	gnome_icon_list_select_icon (list, 0);
 
 	/* select theme */
 	pref_theme_path = eel_gconf_get_string (CONF_TOOLBAR_THEME_DIR);
 	if (!pref_theme_path || strcmp (pref_theme_path, "") == 0)
 	{
-		gtk_clist_select_row (list, 0, 0);
+		gnome_icon_list_select_icon (list, 0);
 	}
 	if (pref_theme_path && strcmp (pref_theme_path, "NAUTILUS_THEME") == 0)
 	{
-		gtk_clist_select_row (list, 1, 0);
+		gnome_icon_list_select_icon (list, 1);
 	}
 	if (pref_theme_path) g_free (pref_theme_path);
 
 	/* load all the galeon themes */
-	theme_selector_fill_themelist_from_dir (GTK_CLIST (pd->themelist),
-		SHARE_DIR "/themes");
+	theme_selector_fill_themelist_from_dir (list, SHARE_DIR "/themes");
 	home_theme_dir =
 		g_strconcat (g_get_home_dir (), "/.galeon/themes", NULL);
-	theme_selector_fill_themelist_from_dir (GTK_CLIST (pd->themelist),
-		home_theme_dir);
+	theme_selector_fill_themelist_from_dir (list, home_theme_dir);
 	g_free (home_theme_dir);
 
 	/* load all the nautilus themes */
-	theme_selector_fill_themelist_from_dir (GTK_CLIST (pd->themelist),
-		SHARE_DIR "/../pixmaps/nautilus");
+	theme_selector_fill_themelist_from_dir (list, SHARE_DIR 
+						"/../pixmaps/nautilus");
 	home_nautilus_theme_dir =
 		g_strconcat (g_get_home_dir (), "/.nautilus/themes", NULL);
-	theme_selector_fill_themelist_from_dir (GTK_CLIST (pd->themelist),
-		home_nautilus_theme_dir);
+	theme_selector_fill_themelist_from_dir (list, home_nautilus_theme_dir);
 	g_free (home_nautilus_theme_dir);
 
-	gtk_clist_sort (list);
-	gtk_clist_thaw (list);
+	gnome_icon_list_thaw (list);
+}
+
+void mouse_wheel_controls_set_sensitive (PreferencesDialog *pd)
+{
+	gint item_index;
+        GtkWidget *active_item;
+	GtkMenuShell *menushell;
+	gboolean checked;
+	int i;
+	struct 
+	{
+		GtkWidget *menu;
+		GtkWidget *spinbutton;
+		GtkWidget *checkbox;
+	}
+	widgets [] =
+	{
+		{ pd->wheel_nokey_action_optionmenu, 
+		  pd->wheel_nokey_step_spinbutton, 
+		  pd->wheel_nokey_default_step_checkbutton},
+		{ pd->wheel_ctrl_action_optionmenu, 
+		  pd->wheel_ctrl_step_spinbutton, 
+		  pd->wheel_ctrl_default_step_checkbutton},
+		{ pd->wheel_alt_action_optionmenu, 
+		  pd->wheel_alt_step_spinbutton, 
+		  pd->wheel_alt_default_step_checkbutton},
+		{ pd->wheel_shift_action_optionmenu, 
+		  pd->wheel_shift_step_spinbutton, 
+		  pd->wheel_shift_default_step_checkbutton},
+		{ NULL, NULL, NULL }
+	};
+
+	for (i = 0; widgets[i].menu != NULL; i++)
+	{
+		menushell = GTK_MENU_SHELL(GTK_OPTION_MENU(widgets[i].menu)->menu);
+		active_item = gtk_menu_get_active (GTK_MENU(menushell));
+				  
+		item_index = g_list_index (menushell->children, active_item);
+
+		checked = gtk_toggle_button_get_active 
+			(GTK_TOGGLE_BUTTON (widgets[i].checkbox));
+
+		gtk_widget_set_sensitive (widgets[i].spinbutton, 
+					  ((item_index == 0) && !checked));
+		gtk_widget_set_sensitive (widgets[i].checkbox, 
+					  (item_index == 0));
+	}
 }
 
 /**
@@ -1841,7 +2007,7 @@ preferences_theme_selector_init (PreferencesDialog *pd)
  * starting from base
  */
 static void
-theme_selector_fill_themelist_from_dir (GtkCList *theme_list, gchar *base)
+theme_selector_fill_themelist_from_dir (GnomeIconList *theme_list, gchar *base)
 {
 	gchar *icon;
 	gchar *pref_theme_path; 	
@@ -1849,8 +2015,7 @@ theme_selector_fill_themelist_from_dir (GtkCList *theme_list, gchar *base)
 	GList *list = NULL;
 	GnomeVFSResult rc;
 
-	pref_theme_path = eel_gconf_get_string (CONF_TOOLBAR_THEME_DIR);	
-
+	pref_theme_path = eel_gconf_get_string (CONF_TOOLBAR_THEME_DIR);
 	rc = gnome_vfs_directory_list_load
 		(&list, base, (GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
 			       GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
@@ -1870,19 +2035,20 @@ theme_selector_fill_themelist_from_dir (GtkCList *theme_list, gchar *base)
 		path = g_concat_dir_and_file (base, info->name);
 
 		/* Check to see if there is a Refresh.png */
-		icon = g_concat_dir_and_file (path, "Refresh.png");
+		icon = g_concat_dir_and_file (path, "Forward.png");
 		if (g_file_exists (icon))
 		{
-			gchar *row[1] = { g_basename (path) };
-			gint index = gtk_clist_append (theme_list, row);
-			gtk_clist_set_row_data (theme_list, index,
-						g_strdup (path));
+			gint index = gnome_icon_list_append (theme_list, icon,
+							     g_basename (path));
+			gnome_icon_list_set_icon_data (theme_list, index,
+						       g_strdup (path));
 
 			/* Select the icon configured in prefs */
 			if (pref_theme_path && 
 			    strcmp (pref_theme_path, path) == 0)
 			{
-				gtk_clist_select_row (theme_list, index, 0);
+				gnome_icon_list_select_icon (theme_list, 
+							     index);
 			}
 		}
 
@@ -1898,10 +2064,12 @@ theme_selector_fill_themelist_from_dir (GtkCList *theme_list, gchar *base)
  * free_themes_list: Free any existing themes list.
  */
 static void
-free_theme_selector (GtkCList *list)
+free_theme_selector (GnomeIconList *list)
 {
 	gint i;
 
-	for (i = 0; i < list->rows; i++)
-		g_free (gtk_clist_get_row_data (list, i));
+	for (i = 0; i < list->icons; i++)
+	{
+		g_free (gnome_icon_list_get_icon_data (list, i));
+	}
 }
