@@ -18,10 +18,10 @@
 
 #include "galeon.h"
 #include "mime.h"
+#include "prefs.h"
 #include "misc.h"
 #include "spinner.h"
 #include "history.h"
-#include "prefs.h"
 #include "window.h"
 #include "mozilla.h"
 #include "dialog.h"
@@ -38,6 +38,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-config.h>
+#include <libgnomeui/gnome-icon-list.h>
 #include <libgnomeui/gnome-font-picker.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-color-picker.h>
@@ -58,17 +59,27 @@ void prefs_apply_button_clicked_cb (GtkButton *button, PreferencesDialog *pd);
 void prefs_cancel_button_clicked_cb (GtkButton *button, PreferencesDialog *pd);
 gboolean prefs_close_cb (GnomeDialog *dialog, PreferencesDialog *pd);
 void prefs_help_button_clicked_cb (GtkButton *button, PreferencesDialog *pd);
-void prefs_mime_add_button_clicked_cb (GtkButton *button, 
-				       PreferencesDialog *pd);
-void prefs_mime_delete_button_clicked_cb (GtkButton *button, 
-					  PreferencesDialog *pd);
-void prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd);
+
+/* mime prototypes */
 void prefs_mime_list_select_row_cb (GtkCList *clist, gint row, gint column, 
 				    GdkEventButton *event, 
 				    PreferencesDialog *pd);
 void prefs_mime_list_unselect_row_cb (GtkCList *clist, gint row, gint column,
 				      GdkEventButton *event, 
 				      PreferencesDialog *pd);
+void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
+			      PreferencesDialog *pd);
+void prefs_mime_type_changed_cb (GtkWidget *entry, PreferencesDialog *pd);
+void prefs_mime_action_clicked_cb (GtkWidget *button, PreferencesDialog *pd); 
+void prefs_mime_helper_changed_cb (GtkWidget *entry, PreferencesDialog *pd); 
+void prefs_mime_helper_buttons_toggled_cb (GtkWidget *button,
+					   PreferencesDialog *pd); 
+void prefs_mime_new_clicked_cb (GtkButton *button, 
+				PreferencesDialog *pd);
+void prefs_mime_delete_clicked_cb (GtkButton *button, 
+				   PreferencesDialog *pd);
+
+/* other prototypes */
 void prefs_homepage_my_portal_button_clicked_cb (GtkButton *button,
 						 PreferencesDialog *pd);
 void prefs_homepage_current_button_clicked_cb (GtkButton *button,
@@ -112,27 +123,33 @@ void prefs_password_sites_button_clicked_cb (GtkWidget *button,
 					     PreferencesDialog *pd);
 void prefs_download_use_command_toggled_cb (GtkWidget *button, 
 					    PreferencesDialog *pd);
-void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
-			      PreferencesDialog *pd);
-void prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, PreferencesDialog *pd); 
-void prefs_theme_select_row_cb (GtkCList *list, gint row, gint column,
-				GdkEvent *event, PreferencesDialog *pd);
+void prefs_cookie_policy_clicked_cb (GtkWidget *button,
+				     PreferencesDialog *pd);
+void prefs_theme_select_icon_cb (GnomeIconList *list, gint row,
+				 GdkEvent *event, PreferencesDialog *pd);
+void prefs_javascript_checkbutton_toggled_cb (GtkWidget *button,
+					      PreferencesDialog *pd);
+void
+prefs_autobookmarks_enabled_check_toggled_cb (GtkWidget *button, 
+					      PreferencesDialog *pd);
+void
+prefs_use_external_source_viewer_check_toggled_cb (GtkWidget *button, 
+						   PreferencesDialog *pd);
+void
+prefs_shorten_target_toggled_cb (GtkWidget *button, 
+				 PreferencesDialog *pd);
+void
+prefs_proxy_radiogroup_toggle_cb (GtkRadioButton *radioButton,
+				  PreferencesDialog *pd);
+void
+prefs_proxy_auto_url_reload_cb (GtkButton *button,
+				PreferencesDialog *pd);
+void prefs_enable_history_dock_clicked_cb (GtkToggleButton *check,
+					   PreferencesDialog *pd);
+void prefs_enable_bookmarks_dock_clicked_cb (GtkToggleButton *check,
+					     PreferencesDialog *pd);
 
-/**
- * prefs_mime_list_sort_cb:
- */
-void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
-			      PreferencesDialog *pd)
-{
-	if (column == pd->mime_last_sort_column)
-		pd->mime_last_sort_type = !pd->mime_last_sort_type;
-	else
-		pd->mime_last_sort_type = GTK_SORT_ASCENDING;
-	gtk_clist_set_sort_type (clist, pd->mime_last_sort_type);
-	pd->mime_last_sort_column = column;
-	gtk_clist_set_sort_column (clist, column);
-	gtk_clist_sort (clist);
-}
+static gboolean block_dock_toggle = FALSE;
 
 /**
  * prefs_ok_button_clicked_cb: preferences dialog ok button clicked
@@ -186,7 +203,6 @@ void
 prefs_clear_disk_cache_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 {
 	mozilla_clear_cache (STORE_ON_DISK);
-	mozilla_clear_cache (STORE_ON_DISK_AS_FILE);
 }
 
 /**
@@ -199,14 +215,12 @@ prefs_clear_memory_cache_button_clicked_cb (GtkButton *button,
 	mozilla_clear_cache (STORE_IN_MEMORY);
 }
 
-/**
- * on_prefs_add_button_clicked: mime types add button clicked
- */
+/* new mime type button clicked */
 void
-prefs_mime_add_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
+prefs_mime_new_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 {
 	MimeItem *newitem = g_new0 (MimeItem, 1);
-	GtkWidget *clist = pd->mime_list;
+	GtkCList *clist = GTK_CLIST (pd->mime_list);
 	gchar *text[5];
 	gint row;
 
@@ -221,17 +235,17 @@ prefs_mime_add_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 	text[2] = newitem->helperApp;
 	text[3] = _("False");
 	text[4] = NULL;
-	row = gtk_clist_append (GTK_CLIST (clist), text);
-	gtk_clist_set_row_data (GTK_CLIST (clist), row, newitem);
-	gtk_clist_select_row (GTK_CLIST (clist), row, 0);
+	row = gtk_clist_append (clist, text);
+	gtk_clist_set_row_data (clist, row, newitem);
+	gtk_clist_select_row (clist, row, 0);
+	gtk_clist_moveto (clist, row, 0, 0, 0);
 }
 
-/**
- * on_prefs_delete_button_clicked: mime types delete button clicked
- */
+/* mime delete button clicked */
 void
-prefs_mime_delete_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
+prefs_mime_delete_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 {
+	GtkCList *clist = GTK_CLIST (pd->mime_list);
 	gint row;
 	
 	if (pd->selected_mime == NULL)
@@ -239,18 +253,17 @@ prefs_mime_delete_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 		return;
 	}
 
-	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
-					    pd->selected_mime);
-	gtk_clist_remove (GTK_CLIST (pd->mime_list), row);
+	row = gtk_clist_find_row_from_data (clist, pd->selected_mime);
+	gtk_clist_remove (clist, row);
 	mime_item_free (pd->selected_mime);
 	pd->selected_mime = NULL;
-	gtk_clist_select_row (GTK_CLIST (pd->mime_list), row, 0);
+
+	if (row >= clist->rows) row = clist->rows - 1;
+	gtk_clist_select_row (clist, row, 0);
+	gtk_clist_moveto (clist, row, 0, 0, 0);
 }
 
-
-/**
- * prefs_mime_list_select_row_cb: mime type selected
- */
+/* mime type selected */
 void prefs_mime_list_select_row_cb (GtkCList *clist, 
 				    gint row, 
 				    gint column, 
@@ -278,9 +291,15 @@ void prefs_mime_list_select_row_cb (GtkCList *clist,
 		break;
 	}
 
+	gtk_widget_set_sensitive (pd->mime_action_frame, TRUE);
+	gtk_widget_set_sensitive (pd->mime_delete_button, TRUE);
+	gtk_widget_set_sensitive (pd->mime_type_label, TRUE);
+	gtk_widget_set_sensitive (pd->mime_type_entry, TRUE);
+	gtk_widget_set_sensitive (pd->mime_helper_frame, act_num);
+
 	gtk_radio_button_set (GTK_RADIO_BUTTON (pd->mime_save_to_disk),
 			      act_num);
-	gtk_entry_set_text (GTK_ENTRY (pd->mime_name_entry), 
+	gtk_entry_set_text (GTK_ENTRY (pd->mime_type_entry), 
 			    pd->selected_mime->name);
 	gtk_entry_set_text (GTK_ENTRY (pd->mime_helper_entry),
 			    pd->selected_mime->helperApp);
@@ -290,12 +309,11 @@ void prefs_mime_list_select_row_cb (GtkCList *clist,
 				      pd->selected_mime->terminalHelper);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_url_helper),
 				      pd->selected_mime->urlHelper);
+
 	pd->enable_edit_mime = TRUE;
 }
 
-/**
- * prefs_mime_list_unselect_row_cb: mime type unselected
- */
+/* mime type unselected */
 void 
 prefs_mime_list_unselect_row_cb (GtkCList *clist, gint row, gint column,
 				 GdkEventButton *event, PreferencesDialog *pd)
@@ -304,7 +322,8 @@ prefs_mime_list_unselect_row_cb (GtkCList *clist, gint row, gint column,
 	pd->enable_edit_mime = FALSE;
 
 	/* The default is Ask */
-	gtk_entry_set_text (GTK_ENTRY (pd->mime_name_entry), "");
+	gtk_radio_button_set (GTK_RADIO_BUTTON (pd->mime_save_to_disk), 2);
+	gtk_entry_set_text (GTK_ENTRY (pd->mime_type_entry), "");
 	gtk_entry_set_text (GTK_ENTRY (pd->mime_helper_entry), "");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_always_use),
 				      FALSE);
@@ -312,32 +331,97 @@ prefs_mime_list_unselect_row_cb (GtkCList *clist, gint row, gint column,
 				      FALSE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_url_helper),
 				      FALSE);
-				      
+	gtk_widget_set_sensitive (pd->mime_action_frame, FALSE);
+	gtk_widget_set_sensitive (pd->mime_helper_frame, FALSE);
+	gtk_widget_set_sensitive (pd->mime_type_entry, FALSE);
+	gtk_widget_set_sensitive (pd->mime_delete_button, FALSE);
+	gtk_widget_set_sensitive (pd->mime_type_label, FALSE);
 }
 
-/* FIXME! */
+/* sort the mime listing */
+void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
+			      PreferencesDialog *pd)
+{
+	if (column == pd->mime_last_sort_column)
+		pd->mime_last_sort_type = !pd->mime_last_sort_type;
+	else
+		pd->mime_last_sort_type = GTK_SORT_ASCENDING;
+	gtk_clist_set_sort_type (clist, pd->mime_last_sort_type);
+	pd->mime_last_sort_column = column;
+	gtk_clist_set_sort_column (clist, column);
+	gtk_clist_sort (clist);
+}
+
+/* handles changes in the mime type entry */
 void
-prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
+prefs_mime_type_changed_cb (GtkWidget *entry, PreferencesDialog *pd)
 {
-        gint action_index;
-	gchar *text[4];
 	gint row;
-	
-        /* extract the action menu index */
-	action_index = gtk_radio_button_get 
-		(GTK_RADIO_BUTTON (pd->mime_save_to_disk));
 
 	if (pd->selected_mime == NULL || !pd->enable_edit_mime)
-	{
 		return;
-	}
 
 	if (pd->selected_mime->name)
-	{
 		g_free (pd->selected_mime->name);
-	}
 	pd->selected_mime->name = gtk_editable_get_chars 
-		(GTK_EDITABLE (pd->mime_name_entry), 0, -1);
+		(GTK_EDITABLE (pd->mime_type_entry), 0, -1);
+
+	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
+					    pd->selected_mime);
+	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 0,
+			    pd->selected_mime->name);
+}
+
+/* handles changes in the mime action */
+void
+prefs_mime_action_clicked_cb (GtkWidget *button, PreferencesDialog *pd)
+{
+	gint state;
+	gchar *name;
+	gint row;
+
+	if (pd->selected_mime == NULL || !pd->enable_edit_mime)
+		return;
+
+	state = gtk_radio_button_get (GTK_RADIO_BUTTON (
+				      pd->mime_save_to_disk));
+	switch (state)
+	{
+		case 0:
+			pd->selected_mime->action = MIME_SAVE_TO_DISK;
+			name = _("Save to disk");
+			break;
+		case 1:
+			pd->selected_mime->action = MIME_RUN_PROGRAM;
+			name = _("Run with Helper App");
+			break;
+		case 2:
+			pd->selected_mime->action = MIME_ASK_ACTION;
+			name = _("Ask the user");
+			break;
+		default:
+			pd->selected_mime->action = MIME_UNKNOWN;
+			name = _("Unknown");
+			break;
+	}
+
+	/* update the clist */
+	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
+					    pd->selected_mime);
+	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 1, name);
+
+	/* set the helper frame sensitive if the type is run or ask */
+	gtk_widget_set_sensitive (pd->mime_helper_frame, state);
+}
+
+/* handles changes in the helper app entry */
+void
+prefs_mime_helper_changed_cb (GtkWidget *entry, PreferencesDialog *pd)
+{
+	gint row;
+
+	if (pd->selected_mime == NULL || !pd->enable_edit_mime)
+		return;
 
 	if (pd->selected_mime->helperApp)
 	{
@@ -346,21 +430,23 @@ prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 	pd->selected_mime->helperApp = gtk_editable_get_chars 
 		(GTK_EDITABLE (pd->mime_helper_entry), 0, -1);
 
-	switch (action_index)
-	{
-	case 0:
-		pd->selected_mime->action = MIME_SAVE_TO_DISK;
-		break;
-	case 1:
-		pd->selected_mime->action = MIME_RUN_PROGRAM;
-		break;
-	case 2:
-		pd->selected_mime->action = MIME_ASK_ACTION;
-		break;
-	default:
-		pd->selected_mime->action = MIME_UNKNOWN;
-		break;
-	}
+	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
+					    pd->selected_mime);
+	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 2,
+			    pd->selected_mime->helperApp);
+}
+
+/* handles changes in the mime helper buttons' states */
+void
+prefs_mime_helper_buttons_toggled_cb (GtkWidget *button,
+				      PreferencesDialog *pd)
+{
+	gchar *text;
+	gint row;
+
+	if (pd->selected_mime == NULL || !pd->enable_edit_mime)
+		return;
+
 	pd->selected_mime->terminalHelper = gtk_toggle_button_get_active
 				(GTK_TOGGLE_BUTTON(pd->mime_terminal));
 	pd->selected_mime->urlHelper = gtk_toggle_button_get_active
@@ -368,37 +454,15 @@ prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 	pd->selected_mime->alwaysUse = gtk_toggle_button_get_active
 				(GTK_TOGGLE_BUTTON(pd->mime_always_use));
 
+	if (pd->selected_mime->alwaysUse)
+		text = _("True");
+	else
+		text = _("False");
+
+	/* update clist */
 	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
 					    pd->selected_mime);
-
-	text[0] = pd->selected_mime->name;
-	switch (pd->selected_mime->action)
-	{
-	case MIME_SAVE_TO_DISK:
-		text[1] = _("Save to disk");
-		break;
-	case MIME_RUN_PROGRAM:  
-		text[1] = _("Run with Helper App");
-		break;
-	case MIME_UNKNOWN:
-		text[1] = _("Unknown");
-		break;
-	case MIME_ASK_ACTION:
-		text[1] = _("Ask the user");
-		break;
-	case MIME_IGNORE:
-		break;
-	}
-	text[2] = pd->selected_mime->helperApp;
-	if (pd->selected_mime->alwaysUse)
-		text[3] = _("True");
-	else
-		text[3] = _("False");
-
-	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 0, text[0]);
-	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 1, text[1]);
-	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 2, text[2]);
-	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 3, text[3]);
+	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 3, text);
 }
 
 /**
@@ -509,7 +573,7 @@ prefs_clear_history_button_clicked_cb (GtkButton *button,
 		   "Are you sure you want to do this?"),
 		 (GnomeReplyCallback)prefs_clear_history_question_cb, pd,
 		 GTK_WINDOW (pd->dialog));
-	
+
 	dialog_set_parent (dialog, pd->dialog);
 	gnome_dialog_run (GNOME_DIALOG (dialog));
 }
@@ -560,12 +624,12 @@ prefs_spinner_select_icon_cb (GnomeIconList *gil, gint num,
 }
 
 void
-prefs_theme_select_row_cb (GtkCList *list, gint row, gint column,
-			   GdkEvent *event, PreferencesDialog *pd)
+prefs_theme_select_icon_cb (GnomeIconList *list, gint row, GdkEvent *event, 
+			    PreferencesDialog *pd)
 {
 	const gchar *path;
 
-	path = gtk_clist_get_row_data (list, row);
+	path = gnome_icon_list_get_icon_data (list, row);
 	gtk_entry_set_text (GTK_ENTRY (pd->theme_dir_entry), path);
 }
 
@@ -595,7 +659,7 @@ preferences_switch_page_cb (GtkWidget *notebook,
 	 * this specific page */
 	if (notebook_page->child == pd->themes_page)
 	{
-		spinner_fill_iconlist (GNOME_ICON_LIST (pd->iconlist));
+		spinner_fill_iconlist (GNOME_ICON_LIST (pd->spinner_iconlist));
 	}
 }
 
@@ -661,21 +725,54 @@ prefs_download_use_command_toggled_cb (GtkWidget *button,
 	gboolean active;
 
 	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-	gtk_widget_set_sensitive (pd->download_command_combo,   active);
+	gtk_widget_set_sensitive (pd->download_command_combo, active);
+	gtk_widget_set_sensitive (pd->download_command_label, active);
 	gtk_widget_set_sensitive (pd->download_run_in_terminal, active);
 }
 
-/**
- * prefs_mime_save_to_disk_toggled_cb: called whenever "save to disk" is
- * toggled, make the helper chooser frame sensitive or not accordingly
- */
+/* toggle the sensitivity of the "warn before accepting cookie"
+ * checkbutton, depending on whether all cookies are being denied or not */
 void
-prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, PreferencesDialog *pd)
+prefs_cookie_policy_clicked_cb (GtkWidget *button, PreferencesDialog *pd)
+{
+	gint state;
+
+	state = gtk_radio_button_get (GTK_RADIO_BUTTON (button));
+	gtk_widget_set_sensitive (pd->warn_cookie_checkbutton,
+				  (state == 0 || state == 1));
+}
+
+void
+prefs_autobookmarks_enabled_check_toggled_cb (GtkWidget *button, 
+					      PreferencesDialog *pd)
 {
 	gboolean active;
-
+	
 	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-	gtk_widget_set_sensitive (pd->mime_helper_frame, !active);
+	gtk_widget_set_sensitive (pd->autobookmarks_count_spin, active);
+	gtk_widget_set_sensitive (pd->autobookmarks_shorten_spin, active);
+}
+
+void
+prefs_use_external_source_viewer_check_toggled_cb (GtkWidget *button, 
+						   PreferencesDialog *pd)
+{
+	gboolean active;
+	
+	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	gtk_widget_set_sensitive (pd->external_source_viewer_entry, active);
+}
+
+void
+prefs_shorten_target_toggled_cb (GtkWidget *button, 
+				 PreferencesDialog *pd)
+{
+	gboolean active;
+	
+	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	gtk_widget_set_sensitive (pd->tabbed_shorten_spin, active);
+	gtk_widget_set_sensitive (pd->tabbed_shorten_spin2, !active);
+	gtk_widget_set_sensitive (pd->tabbed_shorten_name_vbox, active);
 }
 
 void
@@ -685,3 +782,101 @@ prefs_encoding_activated_cb (GtkWidget *button,
 	prefs_fonts_remember (pd);
 	prefs_fonts_language_init (pd);
 }
+
+void
+prefs_javascript_checkbutton_toggled_cb (GtkWidget *button,
+					 PreferencesDialog *pd)
+{
+	gboolean active;
+
+	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	gtk_widget_set_sensitive (pd->statusbar_rewrite_check, active);
+}
+
+
+
+void
+prefs_wheel_changed_cb (GtkWidget *widget,  PreferencesDialog *pd)
+{
+	mouse_wheel_controls_set_sensitive (pd);
+}
+
+void
+prefs_proxy_radiogroup_toggle_cb (GtkRadioButton *radioButton,
+				  PreferencesDialog *pd)
+{
+	/* get value from radio button group */
+	gint index = gtk_radio_button_get (radioButton);
+	switch (index)
+	{
+	 case 0:gtk_widget_set_sensitive (pd->proxy_auto_url_entry, FALSE);
+		gtk_widget_set_sensitive (pd->proxy_auto_reload_button, FALSE);
+		gtk_widget_set_sensitive (pd->http_proxy_entry, FALSE);
+		gtk_widget_set_sensitive (pd->ssl_proxy_entry, FALSE);
+		gtk_widget_set_sensitive (pd->ftp_proxy_entry, FALSE);
+		gtk_widget_set_sensitive (pd->http_proxy_port_spin, FALSE);
+		gtk_widget_set_sensitive (pd->ssl_proxy_port_spin, FALSE);
+		gtk_widget_set_sensitive (pd->ftp_proxy_port_spin, FALSE);
+		gtk_widget_set_sensitive (pd->no_proxies_for_entry, FALSE);
+		break;
+	 case 1:gtk_widget_set_sensitive (pd->proxy_auto_url_entry, FALSE);
+		gtk_widget_set_sensitive (pd->proxy_auto_reload_button, FALSE);
+		gtk_widget_set_sensitive (pd->http_proxy_entry, TRUE);
+		gtk_widget_set_sensitive (pd->ssl_proxy_entry, TRUE);
+		gtk_widget_set_sensitive (pd->ftp_proxy_entry, TRUE);
+		gtk_widget_set_sensitive (pd->http_proxy_port_spin, TRUE);
+		gtk_widget_set_sensitive (pd->ssl_proxy_port_spin, TRUE);
+		gtk_widget_set_sensitive (pd->ftp_proxy_port_spin, TRUE);
+		gtk_widget_set_sensitive (pd->no_proxies_for_entry, TRUE);
+		break; 
+	 case 2:gtk_widget_set_sensitive (pd->proxy_auto_url_entry, TRUE);
+		gtk_widget_set_sensitive (pd->proxy_auto_reload_button, TRUE);
+		gtk_widget_set_sensitive (pd->http_proxy_entry, FALSE);
+		gtk_widget_set_sensitive (pd->ssl_proxy_entry, FALSE);
+		gtk_widget_set_sensitive (pd->ftp_proxy_entry, FALSE);
+		gtk_widget_set_sensitive (pd->http_proxy_port_spin, FALSE);
+		gtk_widget_set_sensitive (pd->ssl_proxy_port_spin, FALSE);
+		gtk_widget_set_sensitive (pd->ftp_proxy_port_spin, FALSE);
+		gtk_widget_set_sensitive (pd->no_proxies_for_entry, FALSE);
+		break; 
+	}
+}
+void
+prefs_proxy_auto_url_reload_cb (GtkButton *button,
+				PreferencesDialog *pd)
+{
+	gchar *url = gtk_entry_get_text (GTK_ENTRY(pd->proxy_auto_url_entry));
+	mozilla_reload_proxy_autoconfiguration (url);
+}
+
+void
+prefs_enable_history_dock_clicked_cb (GtkToggleButton *check,
+				      PreferencesDialog *pd)
+{
+	if (block_dock_toggle) return;
+
+	if (GTK_TOGGLE_BUTTON (pd->show_bookmarks_dock_check)->active)
+	{
+		block_dock_toggle = TRUE;
+		gtk_toggle_button_set_active
+			(GTK_TOGGLE_BUTTON (pd->show_bookmarks_dock_check),
+			 FALSE);
+		block_dock_toggle = FALSE;
+	}
+}
+
+void
+prefs_enable_bookmarks_dock_clicked_cb (GtkToggleButton *check,
+					PreferencesDialog *pd)
+{
+	if (block_dock_toggle) return;
+
+	if (GTK_TOGGLE_BUTTON (pd->show_history_dock_check)->active)
+	{
+		block_dock_toggle = TRUE;
+		gtk_toggle_button_set_active
+			(GTK_TOGGLE_BUTTON (pd->show_history_dock_check),
+			 FALSE);
+		block_dock_toggle = FALSE;
+	}	
+}
