@@ -24,6 +24,7 @@
 #include "window.h"
 #include "bookmarks.h"
 #include "eel-gconf-extensions.h"
+#include "themes.h"
 
 #include <string.h>
 #include <gtk/gtkspinbutton.h>
@@ -61,9 +62,17 @@ static void toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 static GtkWidget *toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox, 
 				      gchar *name, gint band_num);
 static void toolbar_set_style (GtkWidget *toolbar, GtkToolbarStyle final_style,
-			       GtkReliefStyle relief);
-static GtkWidget *toolbar_get_theme_pixmap (const gchar *pixmap);
-static char *toolbar_get_nautilus_theme_path (void);
+			       GtkReliefStyle relief, GtkToolbarSpaceStyle sep);
+static GtkWidget *toolbar_get_theme_pixmap (const ToolbarItem *item);
+static void toolbar_create_separator (GtkWidget *toolbar);
+
+/* directory where to load theme pixmaps from */
+static gchar *theme = NULL;
+
+/* the widgets created with horizontal styling which size we want to sync
+ * later on */
+static GList *resized_later = NULL;
+static gint button_height = 0;
 
 /* array of available toolbar items */
 const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
@@ -128,9 +137,9 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 		FALSE, /* priority? */
 		FALSE,
 		700,
-		window_refresh_button_clicked_cb,
+		NULL,
 		window_refresh_button_press_event_cb,
-		NULL, /* release_event */
+		window_refresh_button_release_event_cb,
 		NULL /* drag_data_received */
 	},
 	{
@@ -238,7 +247,7 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 		NULL /* drag_data_received */
 	},
 	{
-		N_("Find"), 
+		N_("Find entry"), 
 		N_("Find in page"), 
 		GNOME_STOCK_PIXMAP_SEARCH, 
 		"Search",
@@ -252,9 +261,9 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 	},
 	{
 		N_("Bookmarks"), 
-		N_("Toggle Bookmarks Sidebar"), 
+		N_("Toggle bookmarks sidebar"), 
 		GNOME_STOCK_PIXMAP_BOOK_OPEN, 
-		NULL,
+		"Bookmarks",
 		FALSE, /* priority? */
 		TRUE,
 		1000,
@@ -265,9 +274,9 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 	},
 	{
 		N_("History"), 
-		N_("Toggle History Sidebar"), 
+		N_("Toggle history sidebar"), 
 		GNOME_STOCK_PIXMAP_INDEX, 
-		NULL,
+		"History",
 		FALSE, /* priority? */
 		TRUE,
 		900,
@@ -278,9 +287,9 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 	},
 	{
 		N_("Print"), 
-		N_("Print Current Page"), 
+		N_("Print current page"), 
 		GNOME_STOCK_PIXMAP_PRINT, 
-		NULL,
+		"Print",
 		FALSE, /* priority? */
 		FALSE,
 		550,
@@ -288,6 +297,45 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 		window_generic_button_press_event_cb,
 		NULL,
 		NULL /* drag_data_received */
+	},
+	{
+		N_("Back history"), 
+		N_("Go back several pages"), 
+		NULL,  /* no stock pixmap */
+		"BackHistory",
+		FALSE, /* priority? */
+		TRUE,
+		201,
+		NULL, /* clicked */
+		window_back_history_button_press_cb,
+		NULL,
+		NULL /* drag_data_received */
+	},
+	{
+		N_("Forward history"), 
+		N_("Go forward several pages"), 
+		NULL, /* no stock pixmap */
+		"ForwardHistory",
+		FALSE, /* priority? */
+		TRUE,
+		401,
+		NULL, /* clicked */
+		window_forward_history_button_press_cb,
+		NULL,
+		NULL /* drag_data_received */
+	},
+	{
+		N_("Clear location"), 
+		N_("Clear the location entry field"), 
+		GNOME_STOCK_PIXMAP_CLEAR,
+		"ClearLocation",
+		FALSE, /* priority? */
+		FALSE,
+		1250,
+		NULL, /* clicked */
+		window_generic_button_press_event_cb,
+		window_clear_location_button_release_cb,
+		NULL /* drag_data_received */
 	}
 };
 
@@ -303,6 +351,7 @@ toolbar_create (GaleonWindow *window, gboolean show)
 	/* read prefs */
 	location_on_main_toolbar = eel_gconf_get_boolean
 		(CONF_TOOLBAR_URL_LOCATION);
+	theme = get_theme_dir ();
 
 	/* build the overall box to pack the toolbar into */
 	hbox = gtk_hbox_new (FALSE, 2);
@@ -340,6 +389,8 @@ toolbar_create (GaleonWindow *window, gboolean show)
 
 	/* update the toolbar appearance */
 	toolbar_update (window);
+
+	if (theme) g_free (theme);
 }
 
 /**
@@ -354,8 +405,7 @@ toolbar_recreate (GaleonWindow *window)
 	/* get current text in url entry if any */
 	/* NB: this MUST be strdup'd here since it's only a reference
 	 * and we're about to destroy the toolbar (D'Oh! -- MattA) */
-	if (window->toolBarOn && window->location_gnomeentry && 
-	    window->location_entry)
+	if (window->location_gnomeentry && window->location_entry)
 	{
 		text = g_strdup (gtk_entry_get_text 
 				 (GTK_ENTRY (window->location_entry)));
@@ -388,21 +438,24 @@ toolbar_recreate (GaleonWindow *window)
 
 	/* zero out the toolbar widgets, these should have already
 	 * been destroyed by the destruction of the toolbars */
-	window->new_button          = NULL;
-	window->back_button         = NULL;
-	window->up_button           = NULL;
-	window->forward_button      = NULL;
-	window->refresh_button      = NULL;
-	window->home_button         = NULL;
-	window->stop_button         = NULL;
-	window->go_button           = NULL;
-	window->zoom_spin           = NULL;
-	window->drag_event_box      = NULL;
-	window->drag_pixmap         = NULL;
-	window->location_gnomeentry = NULL;
-	window->location_entry      = NULL;
-	window->history_button      = NULL;
-	window->bookmarks_button    = NULL;
+	window->new_button             = NULL;
+	window->back_button            = NULL;
+	window->up_button              = NULL;
+	window->forward_button         = NULL;
+	window->refresh_button         = NULL;
+	window->home_button            = NULL;
+	window->stop_button            = NULL;
+	window->go_button              = NULL;
+	window->zoom_spin              = NULL;
+	window->drag_event_box         = NULL;
+	window->drag_pixmap            = NULL;
+	window->location_gnomeentry    = NULL;
+	window->location_entry         = NULL;
+	window->history_button         = NULL;
+	window->bookmarks_button       = NULL;
+	window->back_history_button    = NULL;
+	window->forward_history_button = NULL;
+	window->clear_location_button  = NULL;
 
 	/* create new toolbar */
 	toolbar_create (window, show);
@@ -411,8 +464,7 @@ toolbar_recreate (GaleonWindow *window)
 	window_update_nav_controls (window);
 
 	/* restore location entry text and history */
-	if (window->toolBarOn && window->location_gnomeentry && 
-	    window->location_entry)
+	if (window->location_gnomeentry && window->location_entry)
 	{
 		gnome_entry_load_history  
 			(GNOME_ENTRY (window->location_gnomeentry));
@@ -427,6 +479,13 @@ toolbar_recreate (GaleonWindow *window)
 	window_update_bm_and_hist_buttons (window,
 		(window->dock_type == DOCK_BOOKMARKS),
 		(window->dock_type == DOCK_HISTORY));
+
+	/* get spinner going if necessary */
+	window->spinner_running = FALSE;
+	if (window->active_embed->load_started)
+	{
+		spinner_start (window);
+	}
 }
 
 /**
@@ -438,18 +497,18 @@ toolbar_update (GaleonWindow *window)
 	ToolbarStyle style;
 	GtkReliefStyle relief;
 	GtkToolbarStyle final_style;
-	gboolean toolbar_button_relief;
-
-	/* skip if no toolbar */
-	if (!(window->toolBarOn))
-	{
-		return;
-	}
+	GtkToolbarSpaceStyle sep;
+	gboolean toolbar_button_relief, toolbar_space_style;
 
 	/* get relief */
 	toolbar_button_relief = gnome_preferences_get_toolbar_relief_btn ();
 	relief = (toolbar_button_relief ? GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
 
+	/* get space style */
+	toolbar_space_style = gnome_preferences_get_toolbar_lines ();
+	sep = (toolbar_space_style ? GTK_TOOLBAR_SPACE_LINE : 
+	       GTK_TOOLBAR_SPACE_EMPTY);
+	
 	/* get real style */
 	style = eel_gconf_get_integer (CONF_TOOLBAR_STYLE);
 
@@ -472,9 +531,9 @@ toolbar_update (GaleonWindow *window)
 	}
 
 	/* set toolbar styles */
-	toolbar_set_style (window->main_toolbar,     final_style, relief);
-	toolbar_set_style (window->right_toolbar,    final_style, relief);
-	toolbar_set_style (window->location_toolbar, final_style, relief);
+	toolbar_set_style (window->main_toolbar,     final_style, relief, sep);
+	toolbar_set_style (window->right_toolbar,    final_style, relief, sep);
+	toolbar_set_style (window->location_toolbar, final_style, relief, sep);
 }
 
 /**
@@ -487,6 +546,8 @@ toolbar_create_widgets (GaleonWindow *window,
 {
 	GtkWidget *spinner;
 	GSList *icon_list, *tmp;
+	GList *l;
+	gint toolbars = 0;
 	
 	icon_list = eel_gconf_get_integer_list(CONF_TOOLBAR_ICON_LIST);
 	tmp = icon_list;
@@ -550,8 +611,8 @@ toolbar_create_widgets (GaleonWindow *window,
 			break;
 
 		case TOOLBAR_ITEM_SEPARATOR:
-			/* append some space */
-			gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+			/* append separator */
+			toolbar_create_separator (toolbar);
 			break;
 
 		case TOOLBAR_ITEM_GO:
@@ -576,6 +637,72 @@ toolbar_create_widgets (GaleonWindow *window,
 	}
 
 	g_slist_free(icon_list);
+
+	/* fetch the size of the highest button, and resize all horizontally
+	 * styled buttons to match this size */
+	l = GTK_TOOLBAR (window->main_toolbar)->children;
+	while (l)
+	{
+		GtkWidget *icon = ((GtkToolbarChild *) l->data)->icon;
+		GtkWidget *label = ((GtkToolbarChild *) l->data)->label;
+		GtkWidget *button;
+		GtkRequisition r;
+		GtkRequisition iconr;
+		GtkRequisition labelr;
+		gint height = 0;
+
+		/* skip any non-buttons */
+		if (!GTK_IS_BUTTON (((GtkToolbarChild *) l->data)->widget))
+		{
+			l = g_list_next (l); 
+			continue;
+		}
+
+		button = GTK_BIN (((GtkToolbarChild *) l->data)->widget)->child;
+		if (button) gtk_widget_size_request (button, &r);
+		if (icon) gtk_widget_size_request (icon, &iconr);
+		if (label) gtk_widget_size_request (label, &labelr);
+		switch (eel_gconf_get_integer (CONF_TOOLBAR_STYLE))
+		{
+		case TOOLBAR_STYLE_HORIZONTAL:
+			if (button) height = r.height;
+			break;
+		case TOOLBAR_STYLE_ICONS_ONLY:
+			if (icon) height = iconr.height;
+			break;
+		case TOOLBAR_STYLE_TEXT_ONLY:
+			if (label) height = labelr.height;
+			break;
+		case TOOLBAR_STYLE_VERTICAL:
+			if (icon && label) height =
+				iconr.height + labelr.height;
+			break;
+		}
+
+		if (height > button_height) button_height = height;
+
+		l = g_list_next (l);
+
+		if (!l && toolbars == 0 &&
+		    GTK_IS_TOOLBAR (window->right_toolbar))
+		{
+			l = GTK_TOOLBAR (window->right_toolbar)->children;
+			toolbars++;
+		}
+	}
+
+	/* sync sizes */
+	for (l = resized_later; l; l = g_list_next (l))
+	{
+		GtkWidget *w = GTK_WIDGET (l->data);
+		GtkRequisition r;
+		gtk_widget_size_request (w, &r);
+		gtk_widget_set_usize (w, r.width, button_height);
+	}
+
+	g_list_free (resized_later);
+	resized_later = NULL;
+	button_height = 0;
 }
 
 /**
@@ -585,16 +712,13 @@ toolbar_create_widgets (GaleonWindow *window,
 static void
 toolbar_create_drag_handle (GaleonWindow *window, GtkWidget *toolbar)
 {
-	PixmapData *pixmap = NULL;
 	GtkWidget *event_box;
 
 	/* build suitable pixmap */
-	pixmap = pixmap_data_new_from_file (SHARE_DIR "/i-bookmark.xpm");
-	if (pixmap != NULL)
+	if (site_pixmap_data != NULL)
 	{
-		window->drag_pixmap = gtk_pixmap_new (pixmap->pixmap, 
-						      pixmap->mask);
-		g_free (pixmap);
+		window->drag_pixmap = gtk_pixmap_new (site_pixmap_data->pixmap, 
+						      site_pixmap_data->mask);
 	}
 
 	/* build event box (for capturing drag signals */
@@ -630,13 +754,29 @@ toolbar_create_location_widgets (GaleonWindow *window,
 	GtkWidget *popwin;
 	GtkWidget *gnomeentry;
 	gboolean location_on_main_toolbar;
-
+	gboolean build_go = FALSE, build_handle = FALSE;
+	GSList *icon_list, *tmp;
+	
 	/* read prefs */
+	icon_list = eel_gconf_get_integer_list(CONF_TOOLBAR_ICON_LIST);
+	tmp = icon_list;
+	
+	while (tmp != NULL) 
+	{
+		ToolbarItemType item = GPOINTER_TO_INT (tmp->data);
+		if (item == TOOLBAR_ITEM_GO) build_go = TRUE;
+		if (item == TOOLBAR_ITEM_DRAG_HANDLE) build_handle = TRUE;
+		tmp = g_slist_next (tmp);
+	}
+
+	g_slist_free (icon_list);
+
 	location_on_main_toolbar = eel_gconf_get_boolean
 		(CONF_TOOLBAR_URL_LOCATION);
+	
 
 	/* create drag handle if on separate toolbar */
-	if (!location_on_main_toolbar)
+	if (!location_on_main_toolbar && build_handle)
 	{
 		toolbar_create_drag_handle (window, toolbar);
 	}
@@ -658,7 +798,7 @@ toolbar_create_location_widgets (GaleonWindow *window,
 				  GTK_SIGNAL_FUNC 
 				  (window_location_entry_key_press_cb), 
 				  window);
-	gtk_signal_connect (GTK_OBJECT (entry), "drag_data_received", 
+	gtk_signal_connect (GTK_OBJECT (gnomeentry), "drag_data_received", 
 			    GTK_SIGNAL_FUNC 
 			    (window_location_entry_drag_data_received_cb), 
 			    window);
@@ -683,7 +823,7 @@ toolbar_create_location_widgets (GaleonWindow *window,
 			    FALSE, FALSE, 0);
 
 	/* add in Go button if on separate toolbar */
-	if (!location_on_main_toolbar)
+	if (!location_on_main_toolbar && build_go)
 	{
 		toolbar_create_button (window, window->right_toolbar,
 				       TOOLBAR_ITEM_GO);
@@ -735,19 +875,36 @@ toolbar_create_find_entry (GaleonWindow *window, GtkWidget *toolbar,
 			   ToolbarItemType type)
 {
 	const ToolbarItem *item;
-	GtkWidget *entry;
+	GtkWidget *entry, *invisible, *gnomeentry;
 
 	/* get item from array */
 	item = &(toolbar_items[type]);
 
-	/* create the entry, pack it */
-	entry = gtk_entry_new ();
+	/* prepend some space */
+	invisible = gtk_label_new ("");
+	gtk_widget_set_usize (invisible, 3, -2);
+	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
+				    GTK_TOOLBAR_CHILD_WIDGET,
+				    GTK_WIDGET (invisible), NULL,
+				    NULL, NULL, NULL, NULL, NULL);
+	
+	/* create gnomeentry, pack it */
+	gnomeentry = gnome_entry_new ("ToolbarFindEntry");
+	entry = gnome_entry_gtk_entry (GNOME_ENTRY (gnomeentry));
 	gtk_widget_set_usize (entry, 100, -2);	
 	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar), 
 				    GTK_TOOLBAR_CHILD_WIDGET,
-				    GTK_WIDGET (entry), NULL, 
+				    GTK_WIDGET (gnomeentry), NULL, 
 				    _(item->tooltip), NULL, NULL, NULL, NULL);
 
+	/* append some space */
+	invisible = gtk_label_new ("");
+	gtk_widget_set_usize (invisible, 3, -2);
+	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
+				    GTK_TOOLBAR_CHILD_WIDGET,
+				    GTK_WIDGET (invisible), NULL,
+				    NULL, NULL, NULL, NULL, NULL);
+
 	/* connect signal (Return should search for inserted text) */
 	gtk_signal_connect (GTK_OBJECT (entry), "activate",
 			    GTK_SIGNAL_FUNC (window_find_entry_activate_cb), 
@@ -798,53 +955,34 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 {
 	GtkWidget *button, *label, *box;
 	GtkWidget *pixmap;
-	gchar *theme, *filename;
 	const ToolbarItem *item;
+	gboolean resizable_widget = FALSE;
+
+	/* If the item is too high in number, it must be from a later version of
+	 * Galeon.  Gracefully ignore it, instead of crashing */
+	if (type >= TOOLBAR_ITEM_COUNT) return;
 
 	/* get item from array */
 	item = &(toolbar_items[type]);
 	
 	pixmap = NULL;
 	
-	/* which theme to use */
-	theme = eel_gconf_get_string (CONF_TOOLBAR_THEME_DIR);
-
-	/* get default nautilus theme when needed */
-	if (strcmp (theme, "NAUTILUS_THEME") == 0)
+	/* use generic icon if no stock (e.g. back/forward history icons) */
+	if (pixmap == NULL && item->stock_icon == NULL)
 	{
-		g_free (theme);
-		theme = toolbar_get_nautilus_theme_path ();
+		pixmap = toolbar_get_theme_pixmap (item);
+		resizable_widget = TRUE;
 	}
 
-	if (theme && item->theme_icon)
+	if (pixmap == NULL && theme && item->theme_icon)
 	{
-		filename = g_strconcat (theme, "/", item->theme_icon, ".png", 
-					NULL);
-		pixmap = toolbar_get_theme_pixmap (filename);
-		g_free (filename);
+		pixmap = toolbar_get_theme_pixmap (item);
 	}
 
 	/* use GNOME stock icon if not found or not requested */
-	if (pixmap == NULL)
+	if (pixmap == NULL && item->stock_icon != NULL)
 	{
-		GdkPixbuf *pixbuf;
-		gint height = 24;
-
-		filename = NULL;
-
-		/* we take Refresh because this is the highest icon in 
-		 * nautilus themes... nautilus icons haven't got the same 
-		 * height. Also, we only take height because they aren't
-		 * square either :-/ */
-		if (theme)
-			filename = g_concat_dir_and_file (theme, "Refresh.png");
-		if (filename && g_file_exists (filename))
-		{
-			pixbuf = gdk_pixbuf_new_from_file (filename);
-			height = gdk_pixbuf_get_height (pixbuf);
-			gdk_pixbuf_unref (pixbuf);
-		}
-		if (filename) g_free (filename);
+		gint height = get_theme_pixmap_height ();
 		
 		if (height != 24)
 		{ 
@@ -859,17 +997,25 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 		}
 	}
 
-	if (theme) g_free (theme);
-
 	/* make the toolbar button and add it to the toolbar */
 	if (eel_gconf_get_integer (CONF_TOOLBAR_STYLE) ==
-	    TOOLBAR_STYLE_HORIZONTAL)
+	    TOOLBAR_STYLE_HORIZONTAL || resizable_widget)
 	{
+		gboolean button_relief;
+		GtkWidget *a;
+
+		if (resizable_widget)
+		{
+			a = gtk_alignment_new (0.50, 1.00, 1.00, 0.00);
+			gtk_container_add (GTK_CONTAINER (a), pixmap);
+		}
+
 		/* we create a horizontal toolbar style ourselves */
 		box = gtk_hbox_new (FALSE, 2);
 
 		/* pack the pixmap first */
-		gtk_box_pack_start (GTK_BOX (box), pixmap, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (box),
+				resizable_widget ? a : pixmap, FALSE, FALSE, 0);
 
 		/* if priority text for this button, pack that too */
 		if (item->priority)
@@ -897,8 +1043,14 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 					   _(item->tooltip), NULL);
 
 		/* set it up */
-		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+		button_relief = gnome_preferences_get_toolbar_relief_btn ();
+		gtk_button_set_relief (GTK_BUTTON (button), button_relief ?
+				       GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+	
+		/* we want to sync sizes of all horizontally-styled buttons
+		 * later */
+		resized_later = g_list_append (resized_later, box);
 	}
 	else
 	{
@@ -942,6 +1094,16 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 				   GDK_ACTION_COPY);
 	}
 
+	/* connect enter/leave signals for updating prelight icons */
+	gtk_object_set_data (GTK_OBJECT (button), "pixmap", pixmap);
+
+	gtk_signal_connect (GTK_OBJECT (button), "enter", GTK_SIGNAL_FUNC
+			    (window_toolbar_button_enter_event_cb),
+			    (gpointer *) item);
+	gtk_signal_connect (GTK_OBJECT (button), "leave", GTK_SIGNAL_FUNC
+			    (window_toolbar_button_leave_event_cb),
+			    (gpointer *) item);
+
 	/* set the window->XXX_button appropiately */
 	switch (type)
 	{
@@ -975,6 +1137,15 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 	case TOOLBAR_ITEM_BOOKMARKS:
 		window->bookmarks_button = button;
 		break;
+	case TOOLBAR_ITEM_BACK_HISTORY:
+		window->back_history_button = button;
+		break;
+	case TOOLBAR_ITEM_FORWARD_HISTORY:
+		window->forward_history_button = button;
+		break;
+	case TOOLBAR_ITEM_CLEAR_LOCATION:
+		window->clear_location_button = button;
+		break;
 	default:
 		break;
 	}
@@ -1031,7 +1202,7 @@ toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox,
  */	
 static void
 toolbar_set_style (GtkWidget *toolbar, GtkToolbarStyle final_style,
-		   GtkReliefStyle relief)
+		   GtkReliefStyle relief, GtkToolbarSpaceStyle sep)
 {
 	/* skip if not there */
 	if (toolbar == NULL)
@@ -1042,28 +1213,43 @@ toolbar_set_style (GtkWidget *toolbar, GtkToolbarStyle final_style,
 	/* otherwise set style */
 	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), final_style);
 	gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), relief);
+	gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), sep);
 }
 
 /**
- * toolbar_get_theme_pixmap: FIXME
+ * toolbar_get_theme_pixmap: retreive appriopate pixmap and build a widget
+ * with it.
  */
 static GtkWidget *
-toolbar_get_theme_pixmap (const gchar *pixmap)
+toolbar_get_theme_pixmap (const ToolbarItem *item)
 {
 	PixmapData *icon;
 	GtkWidget *widget;
+	gchar *tmp, *filename;
 
-	if (!g_file_exists (pixmap))
+	filename = g_strconcat (item->theme_icon, ".png", NULL);
+	
+	tmp = g_concat_dir_and_file (theme, filename);
+	if (!g_file_exists (tmp))
 	{
-		return NULL;
+		g_free (tmp);
+		tmp = g_concat_dir_and_file (SHARE_DIR, filename);
+		if (!g_file_exists (tmp) || item->stock_icon != NULL)
+		{
+			g_free (tmp);
+			g_free (filename);
+			return NULL;
+		}
 	}
-
-	icon = pixmap_data_new_from_file (pixmap);
+	g_free (tmp);
+	
+	icon = get_theme_pixmap (filename, FALSE);
+	
+	g_free (filename);
 
 	if (icon != NULL)
 	{
 		widget = gtk_pixmap_new (icon->pixmap, icon->mask);
-		g_free (icon); /* FIXME: better to cache these? */
 		return GTK_WIDGET (widget);
 	}
 	else
@@ -1072,66 +1258,29 @@ toolbar_get_theme_pixmap (const gchar *pixmap)
 	}
 }
 
-/**
- * toolbar_get_nautilus_theme_path: get the path of the 
- * current nautilus theme
-*/
-static char *
-toolbar_get_nautilus_theme_path (void)
+static void
+toolbar_create_separator (GtkWidget *toolbar)
 {
-	int i;
-	char *theme;
-	char *icon, *tmp;
-	char *result = NULL;
-
-	static gchar *path[] =
+	if (eel_gconf_get_integer (CONF_TOOLBAR_STYLE) ==
+	    TOOLBAR_STYLE_HORIZONTAL && gnome_preferences_get_toolbar_lines ())
 	{
-		SHARE_DIR "/../pixmaps/nautilus/%s",
-		SHARE_DIR "/../pixmaps/nautilus/%s/fake-aa-navibar",
-		NULL,
-		NULL,
-		SHARE_DIR "/../pixmaps/nautilus/",
-		NULL
-	};
+		/* this is hacky. because horizontal is
+		 * our own style, standard gtk toolbar
+		 * seps don't know how high they should
+		 * be. because of that, we build our own
+		 * sep here  -- jorn */
+		GtkWidget *sep = gtk_vseparator_new ();
+		gint icon_height = get_theme_pixmap_height ();
 
-	theme = eel_gconf_get_string(CONF_NAUTILUS_THEME);
-
-	if (!theme || 
-	    strcmp (theme, "default") == 0)
-	{
-		g_free (theme);
-		theme = g_strdup ("");
+		gtk_toolbar_append_element 
+			(GTK_TOOLBAR (toolbar), 
+			 GTK_TOOLBAR_CHILD_WIDGET,
+			 GTK_WIDGET (sep),
+			 NULL, NULL, NULL, NULL, NULL, NULL);
+		gtk_widget_set_usize (sep, 5, icon_height / 2);
 	}
 	else
-	if (strcmp (theme, "gnome") == 0) /* use standard icons */
 	{
-		g_free (theme);
-		return NULL;
+		gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 	}
-
-	path[2] = g_strconcat (g_get_home_dir (), 
-			       "/.nautilus/themes/%s", NULL);
-	path[3] = g_strconcat (g_get_home_dir (), 
-			       "/.nautilus/themes/fake-aa-navibar/%s",
-			       NULL);
-
-	for (i = 0; path[i] != NULL && !result; i++)
-	{
-		tmp = g_strdup_printf (path[i], theme);
-		icon = g_strconcat (tmp, "/Refresh.png", NULL);
-
-		if (g_file_exists (icon))
-		{
-			result = g_strdup (tmp);
-		}
-		g_free (tmp);
-		g_free (icon);
-	}
-
-	g_free (path [2]);
-	g_free (path [3]);
-	g_free (theme);
-
-	return result;
 }
-
