@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-void toolbar_create (GaleonWindow *window);
+void toolbar_create (GaleonWindow *window, gboolean show);
 void toolbar_recreate (GaleonWindow *window);
 
 extern const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT];
