@@ -34,6 +34,7 @@
 #include "mozilla_i18n.h"
 #include "spinner.h"
 #include "window_notifiers.h"
+#include "themes.h"
 
 #include <string.h>
 #include <time.h>
@@ -54,6 +55,13 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
+#ifdef HAVE_XINERAMA
+#include <X11/X.h>
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
+#include <X11/extensions/Xinerama.h>
+#endif
+
 /* indicates a window is in fullscreen mode */
 gboolean fullscreen_active = FALSE;
 
@@ -61,7 +69,6 @@ gboolean fullscreen_active = FALSE;
 /* local function prototypes */
 static void window_init_data (GaleonWindow *window);
 static void window_set_menu_data (GaleonWindow *window);
-static void window_set_chrome (GaleonWindow *window, guint32 chrome_mask);
 static void window_menubar_create (GaleonWindow *window);
 static void window_toolbars_create (GaleonWindow *window);
 static void window_set_selection_signals (GaleonWindow *window);
@@ -137,7 +144,7 @@ window_load_url (GaleonWindow *window, const gchar *url)
  */
 void window_clear_url_entry (GaleonWindow *window)
 {
-	if (window->toolBarOn && window->location_entry != NULL)
+	if (window->location_entry != NULL)
 	{	
 		gtk_editable_delete_text 
 			(GTK_EDITABLE (window->location_entry), 0, -1);
@@ -149,7 +156,7 @@ void window_clear_url_entry (GaleonWindow *window)
  */
 void window_set_url_entry (GaleonWindow *window, const gchar *location)
 {
-	if (window->toolBarOn && window->location_entry != NULL)
+	if (window->location_entry != NULL)
 	{
 		gtk_entry_set_text 
 			(GTK_ENTRY (window->location_entry), location);
@@ -166,8 +173,7 @@ window_create (guint32 chrome_mask)
 {
         GaleonWindow *window;
 	gchar *file;
-	gboolean ignore_layout;
-
+	
 	/* allocate and initialise the GaleonWindow structure */
 	window = g_new0 (GaleonWindow, 1);
 
@@ -187,16 +193,8 @@ window_create (guint32 chrome_mask)
 	/* initialize GaleonWindow data */
 	window_init_data (window);
 
-	/* ignore layout when on a popup */
-	ignore_layout = !((window->statusBarOn) &&
-			  (window->menuBarOn) &&
-			  (window->toolBarOn));
-	
 	/* create the statusbar */
-	if (window->statusBarOn)
-	{
-		window_statusbar_create (window);
-	}
+	window_statusbar_create (window);
 
 	/* get the dock */
 	window->dock = GNOME_APP (window->WMain)->dock;
@@ -206,32 +204,23 @@ window_create (guint32 chrome_mask)
 	window->drag_motion_signal = 0;
 	window->drag_release_signal = 0;
 
+	/* the spinner isn't running yet */
+	window->spinner_running = 0;
+
 	/* save the toolbar and menubar layout */
-	if (!ignore_layout)
-	{
-		window_save_layout (window);
-	}
+	window_save_layout (window);
 
 	/* create the toolbar */
-	if (window->toolBarOn)
-	{
-		window_toolbars_create (window);
-	}
+	window_toolbars_create (window);
 
 	/* create the menubar */
-	if (window->menuBarOn)
-	{
-		window_menubar_create (window);
-	}
+	window_menubar_create (window);
 
 	/* add notifiers for the window */
 	window_init_notifiers(window);
 
 	/* restore the toolbar and menubar layout */
-	if (!ignore_layout)
-	{
-		window_restore_layout (window);
-	}
+	window_restore_layout (window);
 
 	/* set mini icon */
 	file = gnome_pixmap_file ("galeon.png");
@@ -269,6 +258,40 @@ window_create (guint32 chrome_mask)
 	/* create widgets used to pack embeds in */
 	window_create_embed_container (window);
 
+	/* setup visibility according to config */
+	if (!eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS) ||
+	    !window->toolBarOn)
+	{
+		window_toolbar_hide (window);
+	}
+	if (!eel_gconf_get_boolean (CONF_WINDOWS_SHOW_MENUBAR) ||
+	    !window->menuBarOn)
+	{
+		window_menubar_hide (window);
+	}
+	if (!eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR) ||
+	    !window->statusBarOn)
+	{
+		window_statusbar_hide (window);
+	}
+	if (!eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS) ||
+	    !window->toolBarOn)
+	{
+		bookmarks_toolbars_set_visibility (window, FALSE);
+		if (window->hbox)
+		{
+			gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
+		}
+	}
+	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_HISTORY_DOCK))
+	{
+		history_show_dock (window);
+	}
+	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_DOCK))
+	{
+		bookmarks_editor_show_dock (window);
+	}
+	
 	/* return the completed GaleonWindow */
 	return window;
 }
@@ -314,6 +337,7 @@ window_init_menu_data (GaleonWindow *window)
 	window->view_statusbar = view_menu_uiinfo[3].widget;
 	window->view_fullscreen = view_menu_uiinfo[4].widget;
 	window->encoding = view_menu_uiinfo[7].widget;
+	window->csslist = view_menu_uiinfo[8].widget;
 
 	/* tab menu */
 	window->detach_tab = tab_menu_uiinfo[0].widget;
@@ -333,17 +357,34 @@ window_init_menu_data (GaleonWindow *window)
 	window->animate_once_through = animate_always_uiinfo[1].widget;
 	window->animate_never = animate_always_uiinfo[2].widget;
 
-	window->use_own_fonts =  settings_menu_uiinfo[3].widget;
-	window->use_own_colors = settings_menu_uiinfo[4].widget;
-	window->enable_java = settings_menu_uiinfo[5].widget;
-	window->enable_javascript = settings_menu_uiinfo[6].widget;
-	window->enable_proxy = settings_menu_uiinfo[7].widget;
-	window->autojump = settings_menu_uiinfo[8].widget;
+	window->no_proxy = proxies_always_uiinfo[0].widget;
+	window->manual_proxy = proxies_always_uiinfo[1].widget;
+	window->autoproxy = proxies_always_uiinfo[2].widget;
+	  
+	window->use_own_fonts =  settings_menu_uiinfo[4].widget;
+	window->use_own_colors = settings_menu_uiinfo[5].widget;
+	window->enable_java = settings_menu_uiinfo[6].widget;
+	window->enable_javascript = settings_menu_uiinfo[7].widget;
+	window->allow_popups = settings_menu_uiinfo[8].widget;
 
 	/* bookmarks menu */
 	window->add_bookmark = bookmarks_menu_uiinfo[1].widget;
 	window->bookmarks_separator = bookmarks_menu_uiinfo[3].widget;
 
+	/* go menu */
+	window->back_menuitem = go_menu_uiinfo[0].widget;
+	window->forward_menuitem = go_menu_uiinfo[1].widget;
+	window->up_menuitem = go_menu_uiinfo[2].widget;
+	window->refresh_menuitem = go_menu_uiinfo[3].widget;
+	window->homepage_menuitem = go_menu_uiinfo[4].widget;
+
+	/* tools menu */
+#ifdef ENABLE_EXPERIMENTAL_FILTERS
+	window->java_console = tools_menu_uiinfo[7].widget;
+#else
+	window->java_console = tools_menu_uiinfo[6].widget;
+#endif
+
 	window->open_url_entry = NULL;
 
 	/* connect signal for the move_tab_to_window_menu */
@@ -354,6 +395,71 @@ window_init_menu_data (GaleonWindow *window)
 		(GTK_MENU_ITEM (window->move_tab_to_window_menu),
 		 gtk_menu_new ());
 
+	/* connect signals for go menu */
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[0].widget),
+			    "button-press-event",
+			    GTK_SIGNAL_FUNC
+			   	(window_back_forward_button_press_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[1].widget),
+			    "button-press-event",
+			    GTK_SIGNAL_FUNC
+			   	(window_back_forward_button_press_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[2].widget),
+			    "button-press-event",
+			    GTK_SIGNAL_FUNC
+			   	(window_up_button_press_event_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[3].widget),
+			    "button-press-event",
+			    GTK_SIGNAL_FUNC
+			   	(window_refresh_button_press_event_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[4].widget),
+			    "button-press-event",
+			    GTK_SIGNAL_FUNC
+			   	(window_home_button_press_event_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[0].widget),
+			    "button-release-event",
+			    GTK_SIGNAL_FUNC
+			   	(window_back_forward_button_release_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[1].widget),
+			    "button-release-event",
+			    GTK_SIGNAL_FUNC
+			    	(window_back_forward_button_release_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[2].widget),
+			    "button-release-event",
+			    GTK_SIGNAL_FUNC
+			    	(window_up_button_release_event_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[3].widget),
+			    "button-release-event",
+			    GTK_SIGNAL_FUNC
+			    	(window_refresh_button_release_event_cb),
+			    window);
+	gtk_signal_connect (GTK_OBJECT (go_menu_uiinfo[4].widget),
+			    "button-release-event",
+			    GTK_SIGNAL_FUNC
+			    	(window_home_button_release_event_cb),
+			    window);
+
+	/* connect signals for help menu */
+	gtk_signal_connect (GTK_OBJECT (help_menu_uiinfo[1].widget),
+			    "button-release-event",
+			    window_menu_help_homepage_release_cb,
+			    window);
+	gtk_signal_connect (GTK_OBJECT (help_menu_uiinfo[2].widget),
+			    "button-release-event",
+			    window_menu_help_bugreport_release_cb,
+			    window);
+	gtk_signal_connect (GTK_OBJECT (help_menu_uiinfo[3].widget),
+			    "button-release-event",
+			    window_menu_help_about_plugins_release_cb,
+			    window);
 }
 
 static void
@@ -371,23 +477,32 @@ window_menubar_create (GaleonWindow *window)
 
 	window_init_menu_data (window);
 
-	if (window->statusBarOn)
-	{
-		gnome_app_install_menu_hints (GNOME_APP(window->WMain), 
-					      menubar_uiinfo);
-	}
+	gnome_app_install_menu_hints (GNOME_APP(window->WMain), 
+				      menubar_uiinfo);
 
 	bookmarks_menu_create (window);
 	window_session_history_menu_create (window);
 
 	gtk_check_menu_item_set_active 
-		(GTK_CHECK_MENU_ITEM (window->view_menubar), TRUE);
+		(GTK_CHECK_MENU_ITEM (window->view_menubar),
+		 window->menuBarOn ?
+		 eel_gconf_get_boolean (CONF_WINDOWS_SHOW_MENUBAR) :
+		 FALSE);
 	gtk_check_menu_item_set_active 
-		(GTK_CHECK_MENU_ITEM (window->view_toolbar), TRUE);
+		(GTK_CHECK_MENU_ITEM (window->view_toolbar),
+		 window->toolBarOn ?
+		 eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS) :
+		 FALSE);
 	gtk_check_menu_item_set_active 
-		(GTK_CHECK_MENU_ITEM (window->view_bookmarks), TRUE);
+		(GTK_CHECK_MENU_ITEM (window->view_bookmarks),
+		 window->toolBarOn ?
+		 eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS) :
+		 FALSE);
 	gtk_check_menu_item_set_active 
-		(GTK_CHECK_MENU_ITEM (window->view_statusbar), TRUE);
+		(GTK_CHECK_MENU_ITEM (window->view_statusbar),
+		 window->statusBarOn ?
+		 eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR) :
+		 FALSE);
 
 	/* connect state-changed on the econding menuitem to catch the
 	 * first entry and create the submenu, not using entry-notify-event
@@ -399,6 +514,8 @@ window_menubar_create (GaleonWindow *window)
 	/* also create a dummy submenu sa that the submenu arrow is shown */
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->encoding), 
 				   gtk_menu_new ());
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->csslist),
+				   gtk_menu_new ());
 
 	/* handle expose for the edit menu, used to set the edit menuitems 
 	 * sensitivity according to the current selection */
@@ -407,6 +524,11 @@ window_menubar_create (GaleonWindow *window)
 			    GTK_SIGNAL_FUNC (window_menu_edit_expose_event_cb),
 			    window);
 
+#ifdef ENABLE_EXPERIMENTAL_FILTERS
+	/* FIXME fix this one up to work nicely with the filters*/
+	gtk_widget_hide (window->allow_popups);
+#endif
+
 	/* setup view menu and install notifiers*/
 	window_menubar_init_notifiers (window);
 }
@@ -417,9 +539,15 @@ window_toolbars_create (GaleonWindow *window)
 	/* create the toolbars */
 	toolbar_create (window, TRUE);
 	bookmarks_toolbars_create (window);
+}
 
-	/* add notifiers for the toolbars */
-	window_toolbars_init_notifiers(window);
+void window_grab_location_focus (GaleonWindow *window)
+{
+	if (window->location_entry)
+	{
+		gtk_widget_grab_focus (
+			GTK_WIDGET (window->location_entry));
+	}
 }
 
 void
@@ -494,6 +622,45 @@ window_statusbar_create (GaleonWindow *window)
 		gnome_appbar_new (TRUE, TRUE,
 				  GNOME_PREFERENCES_NEVER));
 	window->appbar = GNOME_APP(window->WMain)->statusbar;
+
+	window->security_frame = gtk_frame_new(NULL);
+	gtk_frame_set_shadow_type(GTK_FRAME(window->security_frame), 
+				  GTK_SHADOW_IN);
+
+	window_statusbar_set_security_icon (window, FALSE);
+
+	gtk_box_pack_start(GTK_BOX(window->appbar), 
+			   GTK_WIDGET(window->security_frame), 
+			   FALSE, TRUE, 0);
+	gtk_widget_show(window->security_frame);
+}
+
+void
+window_statusbar_set_security_icon (GaleonWindow *window, gboolean state)
+{
+	PixmapData *pixmap = NULL;
+	return_if_not_window (window);
+	
+	if (!state)
+		pixmap = get_theme_pixmap ("insecure.xpm", FALSE);
+	else
+		pixmap = get_theme_pixmap ("secure.xpm", FALSE);
+
+
+	if (!window->security_icon)
+	{
+
+	  	window->security_icon = gtk_pixmap_new (pixmap->pixmap,
+						        pixmap->mask);
+		gtk_container_add (GTK_CONTAINER(window->security_frame), 
+				   GTK_WIDGET(window->security_icon));
+		gtk_widget_show (window->security_icon);
+	}
+	else
+	{
+		gtk_pixmap_set (window->security_icon, pixmap->pixmap,
+			        pixmap->mask);
+	}
 }
 
 static void
@@ -594,10 +761,10 @@ window_set_chrome (GaleonWindow *window, guint32 chrome_mask)
 }
 
 /**
- * window_update_status_bar: update the status bar of the toplevel window
+ * window_update_statusbar: update the status bar of the toplevel window
  */
 void
-window_update_status_bar (GaleonWindow *window)
+window_update_statusbar (GaleonWindow *window)
 {
 	GaleonEmbed *embed;
 	GtkWidget *progress;
@@ -616,9 +783,6 @@ window_update_status_bar (GaleonWindow *window)
 	/* still check the magic number */
 	return_if_not_embed (embed);
 
-	/* check if the statusbar exist */
-	if (!window->statusBarOn) return;
-
 	/* get progress bar widget */
 	progress = GTK_WIDGET (gnome_appbar_get_progress 
 			       (GNOME_APPBAR(window->appbar)));
@@ -727,9 +891,6 @@ window_update_status_bar (GaleonWindow *window)
 void
 window_update_temp_message(GaleonWindow *window, const char *message)
 {
-	/* check if the statusbar exist */
-	if (!window->statusBarOn) return;
-
 	/* free the previous message */
 	if (window->tempMessage) 
 	{
@@ -759,7 +920,7 @@ window_update_temp_message(GaleonWindow *window, const char *message)
 	{	
 		window->tempMessage = NULL;
 	}
-	window_update_status_bar (window);
+	window_update_statusbar (window);
 }
 
 /**
@@ -770,7 +931,7 @@ void
 window_update_tab_controls (GaleonWindow *window)
 {
 	gboolean tabs_to_left, tabs_to_right;
-	gboolean detachable, show_tabs;
+	gboolean tabs, show_tabs;
 	GaleonEmbed *embed;
 	gint index, size;
 
@@ -784,20 +945,15 @@ window_update_tab_controls (GaleonWindow *window)
 	index = gtk_notebook_page_num (GTK_NOTEBOOK (window->notebook), 
 				       GTK_WIDGET (embed->mozEmbed));
 	size = g_list_length (GTK_NOTEBOOK (window->notebook)->children);
-	detachable = (size > 1);
+	tabs = (size > 1);
 	tabs_to_left = (index > 0);
 	tabs_to_right = (index < (size - 1));
 
-	/* make sure the menubar is visible */
-	if (window->menuBarOn)
-	{
-		gtk_widget_set_sensitive (window->detach_tab, detachable);
-		gtk_widget_set_sensitive (window->prev_tab, tabs_to_left);
-		gtk_widget_set_sensitive (window->next_tab, tabs_to_right);
-		gtk_widget_set_sensitive (window->move_tab_left, tabs_to_left);
-		gtk_widget_set_sensitive (window->move_tab_right,
-					  tabs_to_right);
-	}
+	gtk_widget_set_sensitive (window->detach_tab, tabs);
+	gtk_widget_set_sensitive (window->prev_tab, tabs);
+	gtk_widget_set_sensitive (window->next_tab, tabs);
+	gtk_widget_set_sensitive (window->move_tab_left, tabs_to_left);
+	gtk_widget_set_sensitive (window->move_tab_right, tabs_to_right);
 
 	/* update tab display */
 	show_tabs = eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW);
@@ -836,62 +992,65 @@ window_update_nav_controls (GaleonWindow *window)
 		gtk_moz_embed_can_go_forward (GTK_MOZ_EMBED(embed->mozEmbed));
 	can_stop = (embed->load_started > 0);
 
-	if (window->menuBarOn)
+	/* gets gnome detachabilty of menus so we set sensitivity on
+	   the right item */
+	if (gnome_preferences_get_menus_have_tearoff ())
 	{
-		/* gets gnome detachabilty of menus so we set sensitivity on
-		   the right item */
-		if (gnome_preferences_get_menus_have_tearoff ())
-		{
-			tearoff++;
-		}
-		
-		/* get the go_menu widgets */
-		menu = window->go_menu;
-		children = gtk_container_children (GTK_CONTAINER(menu));
-		
-		/* set go_menu buttons */
-		gtk_widget_set_sensitive (
-			GTK_WIDGET(g_list_nth_data(children, 0 + tearoff)),
-			can_go_back);
-		gtk_widget_set_sensitive (
-			GTK_WIDGET(g_list_nth_data(children, 1 + tearoff)),
-			can_go_forward);
-		gtk_widget_set_sensitive (
-			GTK_WIDGET(g_list_nth_data(children, 2 + tearoff)),
-			can_go_up);
-		gtk_widget_set_sensitive (
-			GTK_WIDGET(g_list_nth_data(children, 5 + tearoff)),
-			can_stop);
-
-		/* free the list of children */
-		g_list_free (children);
-
+		tearoff++;
 	}
+		
+	/* get the go_menu widgets */
+	menu = window->go_menu;
+	children = gtk_container_children (GTK_CONTAINER(menu));
+		
+	/* set go_menu buttons */
+	gtk_widget_set_sensitive (
+		GTK_WIDGET(g_list_nth_data(children, 0 + tearoff)),
+		can_go_back);
+	gtk_widget_set_sensitive (
+		GTK_WIDGET(g_list_nth_data(children, 1 + tearoff)),
+		can_go_forward);
+	gtk_widget_set_sensitive (
+		GTK_WIDGET(g_list_nth_data(children, 2 + tearoff)),
+		can_go_up);
+	gtk_widget_set_sensitive (
+		GTK_WIDGET(g_list_nth_data(children, 5 + tearoff)),
+		can_stop);
 
-        /* check if the toolbar exists and then set buttons */
-	if (window->toolBarOn) 
+	/* free the list of children */
+	g_list_free (children);
+
+	/* update back, forward, up, stop and refresh buttons */
+	if (window->back_button != NULL)
 	{
-		/* update back, forward, up, stop and refresh buttons */
-		if (window->back_button != NULL)
-		{
-			gtk_widget_set_sensitive 
-				(window->back_button, can_go_back);
-		}
-		if (window->up_button != NULL)
-		{
-			gtk_widget_set_sensitive
-				(window->up_button, can_go_up);
-		}
-		if (window->forward_button != NULL)
-		{
-			gtk_widget_set_sensitive
-				(window->forward_button, can_go_forward);
-		}
-		if (window->stop_button != NULL)
-		{
-			gtk_widget_set_sensitive 
-				(window->stop_button, can_stop);
-		}
+		gtk_widget_set_sensitive 
+			(window->back_button, can_go_back);
+	}
+	if (window->back_history_button != NULL)
+	{
+		gtk_widget_set_sensitive 
+			(window->back_history_button, can_go_back);
+	}
+	if (window->up_button != NULL)
+	{
+		gtk_widget_set_sensitive
+			(window->up_button, can_go_up);
+	}
+	if (window->forward_button != NULL)
+	{
+		gtk_widget_set_sensitive
+			(window->forward_button, can_go_forward);
+	}
+	if (window->forward_history_button != NULL)
+	{
+		gtk_widget_set_sensitive
+			(window->forward_history_button, 
+			 can_go_forward);
+	}
+	if (window->stop_button != NULL)
+	{
+		gtk_widget_set_sensitive 
+			(window->stop_button, can_stop);
 	}
 }
 
@@ -916,7 +1075,6 @@ void window_show_open_dialog (GaleonWindow *window)
 {
 	gchar *dir, *retDir;
 	gchar *file = NULL;
-	gchar *url;
 	GnomeVFSURI *uri;
 
 	dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
@@ -928,12 +1086,9 @@ void window_show_open_dialog (GaleonWindow *window)
 		uri = gnome_vfs_uri_new (file);
 		if (uri)
 		{
-			retDir = gnome_vfs_uri_extract_dirname (uri);
+			window_load_url(window, file);
 
-			/* open the file */
-			url = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
-			window_load_url(window, url);
-			g_free (url);
+			retDir = gnome_vfs_uri_extract_dirname (uri);
 
 			/* set default open dir */
 			eel_gconf_set_string (CONF_STATE_OPEN_DIR, 
@@ -989,16 +1144,14 @@ void window_show_openurl_dialog (GaleonWindow *window)
  */
 void window_toolbar_show (GaleonWindow *window)
 {
-	if (!(window->toolBarOn))
-	{
-		return;
-	}
-
 	gtk_widget_show (window->main_dockitem);	
 	if (!eel_gconf_get_boolean (CONF_TOOLBAR_URL_LOCATION))
 	{
 		gtk_widget_show (window->location_dockitem);
 	}
+
+	if (!GTK_CHECK_MENU_ITEM(window->view_fullscreen)->active)
+		window->show_toolbar = TRUE;
 }
 
 /**
@@ -1006,15 +1159,15 @@ void window_toolbar_show (GaleonWindow *window)
  */
 void window_toolbar_hide (GaleonWindow *window)
 {
-	if (window->toolBarOn) 
+	gtk_widget_hide (GTK_WIDGET (window->main_dockitem));
+	if (window->location_dockitem)
 	{
-		gtk_widget_hide (GTK_WIDGET (window->main_dockitem));
-		if (window->location_dockitem)
-		{
-			gtk_widget_hide (GTK_WIDGET (window->location_dockitem));
-		}
-		gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
+		gtk_widget_hide (GTK_WIDGET (window->location_dockitem));
 	}
+	gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
+
+	if (!GTK_CHECK_MENU_ITEM(window->view_fullscreen)->active)
+		window->show_toolbar = FALSE;
 }
 
 /**
@@ -1022,10 +1175,10 @@ void window_toolbar_hide (GaleonWindow *window)
  */
 void window_menubar_show (GaleonWindow *window)
 {
-	if (window->menuBarOn) 
-	{
-		gtk_widget_show (GTK_WIDGET (window->menubar->parent));	
-	}
+	gtk_widget_show (GTK_WIDGET (window->menubar->parent));	
+
+	if (!GTK_CHECK_MENU_ITEM(window->view_fullscreen)->active)
+		window->show_menubar = TRUE;
 }
 
 /**
@@ -1033,11 +1186,11 @@ void window_menubar_show (GaleonWindow *window)
  */
 void window_menubar_hide (GaleonWindow *window)
 {
-	if (window->menuBarOn) 
-	{
-		gtk_widget_hide (GTK_WIDGET (window->menubar->parent));
-		gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
-	}
+	gtk_widget_hide (GTK_WIDGET (window->menubar->parent));
+	gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
+
+	if (!GTK_CHECK_MENU_ITEM(window->view_fullscreen)->active)
+		window->show_menubar = FALSE;
 }
 
 /**
@@ -1045,10 +1198,10 @@ void window_menubar_hide (GaleonWindow *window)
  */
 void window_statusbar_show (GaleonWindow *window)
 {
-	if (window->statusBarOn) 
-	{
-		gtk_widget_show (GTK_WIDGET (window->appbar->parent));
-	}
+	gtk_widget_show (GTK_WIDGET (window->appbar->parent));
+
+	if (!GTK_CHECK_MENU_ITEM(window->view_fullscreen)->active)
+		window->show_statusbar = TRUE;
 }
 
 /**
@@ -1056,10 +1209,11 @@ void window_statusbar_show (GaleonWindow *window)
  */
 void window_statusbar_hide (GaleonWindow *window)
 {
-	if (window->statusBarOn) {
-		gtk_widget_hide (window->appbar->parent);
-		gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
-	}
+	gtk_widget_hide (window->appbar->parent);
+	gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
+
+	if (!GTK_CHECK_MENU_ITEM(window->view_fullscreen)->active)
+		window->show_statusbar = FALSE;
 }
 
 /**
@@ -1144,11 +1298,14 @@ window_close (GaleonWindow *window)
 		/* remove window from session */
 		session_remove_window (window);
 
+		/* destroy dummy widgets list */
+		g_list_foreach (window->dummy_widgets, 
+				(GFunc)gtk_widget_unref, NULL);
+		g_list_free (window->dummy_widgets);
+		
 		/* scrub and free the window structure */
-		/* FIXME doesnt seem to crash anymore, try to
-		   remove comments soon */
-/*		memset (window, 0, sizeof (GaleonWindow));
-		g_free (window);*/
+		memset (window, 0, sizeof (GaleonWindow));
+		g_free (window);
 	}
 }
 
@@ -1213,66 +1370,164 @@ window_toggle_fullscreen_mode (GaleonWindow *window)
 void
 window_set_fullscreen_mode (GaleonWindow *window, gboolean active)
 {
-	GdkWindow *gdk_window = window->WMain->window;
-	gint client_x, client_y, root_x, root_y;
-	gint width, height;
+	GdkWindow *gdk_window;
 
 	return_if_not_window (window);
 
-	if (window->menuBarOn)
-	{
-		GTK_CHECK_MENU_ITEM (window->view_fullscreen)->active = active;
+	GTK_CHECK_MENU_ITEM (window->view_fullscreen)->active = active;
 
-		if (!eel_gconf_get_boolean
-		    (CONF_WINDOWS_FS_SHOW_MENUBAR)) 
-		{
+	if (active)
+	{
+		gboolean bool;
+
+		bool = eel_gconf_get_boolean
+			(CONF_WINDOWS_FS_SHOW_MENUBAR);
+		if (bool != window->show_menubar) 
+ 		{
+ 			gtk_check_menu_item_set_active
+ 				(GTK_CHECK_MENU_ITEM
+				 (window->view_menubar),
+				 bool);
+ 		}
+		bool = eel_gconf_get_boolean
+			(CONF_WINDOWS_FS_SHOW_TOOLBARS);
+		if (bool != window->show_toolbar) 
+ 		{
+ 			gtk_check_menu_item_set_active
+ 				(GTK_CHECK_MENU_ITEM
+				 (window->view_toolbar),
+				 bool);
+ 		}
+		bool = eel_gconf_get_boolean
+			(CONF_WINDOWS_FS_SHOW_BOOKMARKS);
+		if (bool != window->show_bookmarks) 
+ 		{
 			gtk_check_menu_item_set_active
-				(GTK_CHECK_MENU_ITEM (window->view_menubar),
-				 !active);
-		}
-		if (!eel_gconf_get_boolean
-		    (CONF_WINDOWS_FS_SHOW_TOOLBARS)) 
-		{
-			gtk_check_menu_item_set_active
-				(GTK_CHECK_MENU_ITEM (window->view_toolbar),
-				 !active);
-		}
-		if (!eel_gconf_get_boolean
-		    (CONF_WINDOWS_FS_SHOW_BOOKMARKS)) 
-		{
-			gtk_check_menu_item_set_active
-				(GTK_CHECK_MENU_ITEM (window->view_bookmarks),
-				 !active);
-		}
-		if (!eel_gconf_get_boolean
-		    (CONF_WINDOWS_FS_SHOW_STATUSBAR))
-		{
-			gtk_check_menu_item_set_active
-				(GTK_CHECK_MENU_ITEM (window->view_statusbar),
-				 !active);
-		}
+ 				(GTK_CHECK_MENU_ITEM
+				 (window->view_bookmarks),
+				 bool);
+ 		}
+		bool = eel_gconf_get_boolean
+			(CONF_WINDOWS_FS_SHOW_STATUSBAR);
+		if (bool != window->show_statusbar)
+ 		{
+ 			gtk_check_menu_item_set_active
+ 				(GTK_CHECK_MENU_ITEM
+				 (window->view_statusbar),
+				 bool);
+ 		}
+	}
+	else
+	{
+		/* set things back to the state they were before 
+		 * fullscreen */
+		gtk_check_menu_item_set_active
+			(GTK_CHECK_MENU_ITEM (window->view_menubar),
+			 window->show_menubar);
+		gtk_check_menu_item_set_active
+			(GTK_CHECK_MENU_ITEM (window->view_toolbar),
+			 window->show_toolbar);
+		gtk_check_menu_item_set_active
+			(GTK_CHECK_MENU_ITEM (window->view_bookmarks),
+			 window->show_bookmarks);
+		gtk_check_menu_item_set_active
+			(GTK_CHECK_MENU_ITEM (window->view_statusbar),
+			 window->show_statusbar);
 	}
 
 	fullscreen_active = active;
+	gdk_window = window->WMain->window;
 	window_set_layer (window);
 
-	if (fullscreen_active)
+	if (active)
 	{
+		gint client_x, client_y, root_x, root_y;
+		gint width, height;
+		gint screen_width, screen_height;
+
+		/* get window position and geometry */
 		gdk_window_get_origin (gdk_window, &root_x, &root_y);
 		gdk_window_get_geometry (gdk_window, &client_x, &client_y,
 					 &width, &height, NULL);
 
-		gdk_window_move_resize (gdk_window, -client_x, -client_y,
-					gdk_screen_width () + 1,
-					gdk_screen_height () + 1);
-		
+		/* store original position and size */
 		window->x = root_x - client_x;
 		window->y = root_y - client_y;
 		window->width = width;
 		window->height = height;
+
+		/* get the size of the screen */
+		screen_width = gdk_screen_width ();
+		screen_height = gdk_screen_height ();
+
+#ifdef HAVE_XINERAMA
+		/* if the user is using Xinerama, adjust the area and
+		 * positioning of the fullscreen window so it's not
+		 * occupying any dead area or covering all of the heads */
+		{
+		Display *dpy;
+
+		dpy = GDK_DISPLAY ();
+		if (XineramaIsActive (dpy))
+		{
+			int num_heads, i;
+			int max_area = 0, max_area_head = 0;
+			int int_width, int_height, int_area;
+			XineramaScreenInfo *head_info;
+
+			/* query heads */
+			head_info = XineramaQueryScreens (dpy, &num_heads);
+			g_return_if_fail (num_heads != 0);
+
+			/* find the head with the largest area of the
+			 * window in it */
+			for (i = 0; i < num_heads; i++)
+			{
+				/* find the width and height of the
+				 * intersecting area */
+				int_width = MIN (head_info[i].x_org +
+					head_info[i].width,
+					window->x + window->width) -
+					MAX (head_info[i].x_org, window->x);
+				int_height = MIN (head_info[i].y_org +
+					head_info[i].height,
+					window->y + window->height) -
+					MAX (head_info[i].y_org, window->y);
+
+				/* find the area of the intersection */
+				if (int_width < 0 || int_height < 0)
+					int_area = 0;
+				else
+					int_area = int_width * int_height;
+
+				/* see if this head has a larger area */
+				if (int_area > max_area)
+				{
+					max_area_head = i;
+					max_area = int_area;
+				}
+			}
+
+			/* update screen size and position offset */
+			screen_width = head_info[max_area_head].width;
+			screen_height = head_info[max_area_head].height;
+			client_x -= head_info[max_area_head].x_org;
+			client_y -= head_info[max_area_head].y_org;
+
+			XFree (head_info);
+		}
+		}
+#endif
+
+		/* move and resize the window (+1s are to get rid of the
+		 * crappy one-pixel-border along the right edge and bottom
+		 * that makes it hard to hit the scroll buttons) */
+		gdk_window_move_resize (gdk_window, -client_x, -client_y,
+					screen_width + 1, screen_height + 1);
 	}
 	else
 	{
+		/* restore the initial position and size */
 		gdk_window_move_resize (gdk_window, 
 					window->x, window->y,
 					window->width, 
@@ -1351,7 +1606,8 @@ window_reload_all (GaleonWindow *window)
 	/* reload all tabs in the window */
 	for (l = window->embed_list; l != NULL; l = g_list_next (l))
 	{
-		embed_reload ((GaleonEmbed *)(l->data));
+		embed_reload ((GaleonEmbed *)(l->data), 
+			      GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
 	}
 }
 
@@ -1465,6 +1721,7 @@ window_create_charset_submenus (GaleonWindow *window,
 	for (i = 0;i < lgroups_count;i++) {
 		tw = gtk_menu_item_new_with_label (_(lgroups[i]));
 		gtk_menu_append (GTK_MENU(encoding_menu), tw);
+		gtk_widget_lock_accelerators (tw);
 		gtk_widget_show (tw);
 		
 		tw2 = gtk_menu_new();
@@ -1487,6 +1744,7 @@ window_create_charset_submenus (GaleonWindow *window,
 				tw3 = gtk_menu_item_new_with_label (
 				       _(charset_trans_array[j].charset_title));
 				gtk_menu_append (GTK_MENU (tw2), tw3);
+				gtk_widget_lock_accelerators (tw3);
 				gtk_widget_show (tw3);
 				gtk_signal_connect (
 					GTK_OBJECT (tw3), 
@@ -1499,6 +1757,7 @@ window_create_charset_submenus (GaleonWindow *window,
 	while (tl != NULL) { 
 		tw = gtk_menu_item_new_with_label (tl->data);
 		gtk_menu_append (GTK_MENU(tw2), tw);
+		gtk_widget_lock_accelerators (tw);
 		gtk_widget_show (tw);
 		gtk_signal_connect (GTK_OBJECT (tw), "activate",
 				    window_menu_encoding_activate_cb, window);
@@ -1516,7 +1775,7 @@ strcasestr_cb (gconstpointer a, gconstpointer b)
 
 void
 window_add_embed (GaleonWindow *window, GaleonEmbed *embed, 
-		  gboolean force_jump)
+		  gboolean force_jump, gboolean raise_window)
 {
 	gboolean auto_jump;
 	GtkWidget *tab;
@@ -1559,7 +1818,8 @@ window_add_embed (GaleonWindow *window, GaleonEmbed *embed,
 		gtk_widget_show (embed->mozEmbed);
 		gtk_notebook_set_page (GTK_NOTEBOOK (window->notebook), page);
 		embed->focus_type = FOCUS_ON_CREATE;
-		if (GTK_WIDGET (window->WMain)->window != NULL)
+		if (raise_window &&
+		    GTK_WIDGET (window->WMain)->window != NULL)
 		{
 			gdk_window_raise (GTK_WIDGET (window->WMain)->window);
 		}
@@ -1621,6 +1881,14 @@ window_notebook_tab_new (GaleonEmbed *embed, const gchar *title)
 	embed->notebook_hbox = hbox = gtk_hbox_new (FALSE, 0);
 	embed->notebook_label = label = gtk_label_new (NULL);
 
+	/* set up drag target */
+	gtk_signal_connect (GTK_OBJECT (hbox), "drag_data_received",
+			    GTK_SIGNAL_FUNC (embed_drag_data_received_cb),
+			    embed);
+	gtk_drag_dest_set (hbox, GTK_DEST_DEFAULT_ALL, embed_drop_types,
+			   embed_drop_types_num_items, GDK_ACTION_COPY |
+			   GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+
 	/* setup label */
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
 	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
@@ -1630,9 +1898,10 @@ window_notebook_tab_new (GaleonEmbed *embed, const gchar *title)
 	embed_update_tab_title (embed);
 	embed_update_tab_close_button (embed);
 	embed_update_tab_status (embed);
+	embed_update_tab_size (embed);
 
-	/* show everything */
-	gtk_widget_show_all (hbox);
+	/* show label */
+	gtk_widget_show (embed->notebook_label);
 
 	/* return overall widget */
 	return hbox;
@@ -1775,3 +2044,77 @@ window_update_bm_and_hist_buttons (GaleonWindow *window,
 		}
 	}
 }
+
+/*
+ * window_update_ui: update user interface according to current settings
+ * the hack is due to a bug in gconf: the values are correctly updated
+ * only in notifiers. The bug has been already reported.
+ */
+void window_update_ui (void)
+{
+	eel_gconf_set_boolean ("/apps/galeon/updateuihack", FALSE);
+}
+
+void window_update_ui_hack (GaleonWindow *window)
+{
+	if (window->to_update & MAIN_TOOLBAR)
+	{
+		window_save_layout (window);
+		toolbar_recreate (window);
+		window_restore_layout (window);
+	}
+	if (window->to_update & BOOKMARKS_TOOLBAR)
+	{
+		bookmarks_tb_dirty = TRUE;
+		bookmarks_updated ();
+	}
+	if (window->to_update & TABS_APPEAREANCE)
+	{
+		window_update_tabs_appearance (window);
+	}
+
+	window->to_update = 0;
+}
+
+void 
+window_update_tabs_appearance (GaleonWindow *window)
+{
+	gint n_embeds, always_show, popup = 0;
+	GList *w, *e;
+
+	initialise_colours ();
+
+	always_show =
+		eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW);
+
+	for (w = all_windows; w != NULL; w = g_list_next (w))
+	{
+		GaleonWindow *window = (GaleonWindow *)(w->data);
+
+		/* check if the tab(s) should be visible */
+		n_embeds = g_list_length (window->embed_list);
+		if (n_embeds == 1)
+		{
+			GaleonEmbed *embed;
+
+			e = window->embed_list;
+ 			embed = (GaleonEmbed *) (e->data);
+			popup = window->is_popup;
+		}
+
+		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->notebook), 
+					    (n_embeds > 1) | 
+					    (always_show && !popup));
+
+		/* update notebook tabs */
+		for (e = window->embed_list; e != NULL; e = g_list_next (e))
+		{
+			GaleonEmbed *embed = (GaleonEmbed *) (e->data);
+
+			embed_update_tab_title (embed);
+			embed_update_tab_close_button (embed);
+			embed_update_tab_status (embed);
+			embed_update_tab_size (embed);
+		}
+	}
+}
