@@ -29,11 +29,14 @@
 #include "toolbar.h"
 #include "find.h"
 #include "session.h"
+#include "spinner.h"
+#include "themes.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-app.h>
 #include <libgnomeui/gnome-popup-menu.h>
 #include <libgnomeui/gnome-stock.h>
@@ -51,6 +54,7 @@ void window_location_gnomeentry_popwin_cb (GtkWidget *widget,
 static void window_handle_go_button (GaleonWindow *window, LinkState state);
 static void window_move_tab_to_window_menu_item_cb (GtkMenuItem *mi, 
 						   GaleonWindow *w);
+static gchar *guess_protocol (gchar *text);
 
 GTimer *zoom_timer = NULL;
 #define ZOOM_DELAY 0.20
@@ -109,9 +113,6 @@ window_delete_cb (GtkWidget *widget, GdkEventAny *event, GaleonWindow *window)
 	 * chose to exit Galeon (and do automatic session saving if the
 	 * pref is set */
 
-	/* FIXME: I think this is broken. Should call always window_close, 
-	 * window_close already takes care of this (ricardo) 
-	 */
 	if (g_list_length (all_windows) == 1 && 
 	    !galeon_server_mode && !galeon_panel_mode)
 	{
@@ -144,11 +145,13 @@ window_back_forward_button_press_cb (GtkButton *button,
 
 	if (event->button == 3)
 	{
-		if (GTK_WIDGET (button) == window->forward_button)
+		if (GTK_WIDGET (button) == window->forward_button ||
+		    GTK_WIDGET (button) == window->forward_menuitem)
 		{
 			menu = embed_create_forward_menu (embed);
 		}
-		else if (GTK_WIDGET (button) == window->back_button)
+		else if (GTK_WIDGET (button) == window->back_button ||
+			 GTK_WIDGET (button) == window->back_menuitem)
 		{
 			menu = embed_create_back_menu (embed);
 		}
@@ -156,12 +159,6 @@ window_back_forward_button_press_cb (GtkButton *button,
 		{
 			g_assert_not_reached ();
 		}
-#if 0		/* FIXME: removed this since people objected to it --
-		 * MattA 12/07/01 */
-		context_menu_add_seperator (GTK_MENU (menu));
-		context_show_appearance_menu (window, GTK_MENU (menu), 
-					      NULL, TRUE, TRUE);
-#endif
 		gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu), NULL,
 						 NULL, event, NULL);
 		gtk_widget_unref (GTK_WIDGET(menu));
@@ -170,6 +167,60 @@ window_back_forward_button_press_cb (GtkButton *button,
 	return FALSE;
 }
 
+/** 
+ * window_back_history_button_press_cb:
+ */
+gboolean
+window_back_history_button_press_cb (GtkButton *button,
+				     GdkEventButton *event,
+				     GaleonWindow *window)
+{
+	GtkMenu *menu = NULL;
+	GaleonEmbed *embed;
+	
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+
+	menu = embed_create_back_menu (embed);
+	gtk_signal_emit_stop_by_name (GTK_OBJECT (button),
+				      "button-press-event");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu), 
+					 menu_position_under_widget,
+					 button, event, NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+	gtk_widget_unref (GTK_WIDGET(menu));
+	return TRUE;
+}
+
+/** 
+ * window_forward_history_button_press_cb:
+ */
+gboolean
+window_forward_history_button_press_cb (GtkButton *button,
+					GdkEventButton *event,
+					GaleonWindow *window)
+{
+	GtkMenu *menu = NULL;
+	GaleonEmbed *embed;
+	
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+
+	menu = embed_create_forward_menu (embed);
+	gtk_signal_emit_stop_by_name (GTK_OBJECT (button),
+				      "button-press-event");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu), 
+					 menu_position_under_widget,
+					 button, event, NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+	gtk_widget_unref (GTK_WIDGET(menu));
+	return TRUE;
+}
+
 /** 
  * window_home_button_release_event_cb: home button clicked
  */
@@ -177,6 +228,8 @@ gboolean
 window_home_button_release_event_cb (GtkButton *button, GdkEventButton *event,
 				     GaleonWindow *window)
 {
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
 	if (GTK_WIDGET (button)->window != event->window)
 		return FALSE;
 	return_val_if_not_window (window, FALSE);
@@ -184,6 +237,12 @@ window_home_button_release_event_cb (GtkButton *button, GdkEventButton *event,
 	{
 		window_go_home (window, mouse_state_to_link_state 
 						(event->button, event->state));
+		if (GTK_WIDGET (button) == window->homepage_menuitem)
+		{
+			GtkMenuShell *shell =
+				GTK_MENU_SHELL (GTK_WIDGET (button)->parent);
+			gtk_menu_shell_deactivate (shell);
+		}
 		return TRUE;
 	}
 	return FALSE;
@@ -221,9 +280,12 @@ window_home_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	{
 		/* show context menu */
 		menu = gnome_popup_menu_new (menu_uiinfo);
-		context_menu_add_seperator (GTK_MENU (menu));
-		context_show_appearance_menu (window, GTK_MENU (menu), 
-					      NULL, TRUE, TRUE);
+		if (GTK_WIDGET (button) != window->homepage_menuitem)
+		{
+			context_menu_add_seperator (GTK_MENU (menu));
+			context_show_appearance_menu (window, GTK_MENU (menu), 
+						      NULL, TRUE, TRUE);
+		}
 		action = gnome_popup_menu_do_popup_modal 
 			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
 
@@ -302,6 +364,8 @@ window_new_button_release_cb (GtkButton *button, GdkEventButton *event,
 	GaleonEmbed *embed;
 	GaleonEmbed *new_embed;
 
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
 	if (GTK_WIDGET (button)->window != event->window)
 		return FALSE;
 	return_val_if_not_window (window, FALSE);
@@ -318,17 +382,15 @@ window_new_button_release_cb (GtkButton *button, GdkEventButton *event,
 							  shift_modifier));
 		/* FIXME: why this, why here? */
 		window_update_nav_controls (window);
-		if (new_embed->parent_window->location_entry != NULL)
-		{
-			gtk_widget_grab_focus (GTK_WIDGET (
-				new_embed->parent_window->location_entry));
-		}
+
+		window_grab_location_focus (window);
 		return TRUE;
 		
 	case 2:
 		/* hmmm, this is probably a bit hacky ;-) -- MattA */
 		embed = embed_create (embed, (tabbed_mode ^ shift_modifier), 
-				      TRUE, GTK_MOZ_EMBED_FLAG_ALLCHROME);
+				      TRUE, TRUE, GTK_MOZ_EMBED_FLAG_ALLCHROME);
+		embed_set_visibility (embed, TRUE);		
 		gtk_selection_convert (embed->parent_window->WMain,
 				       GDK_SELECTION_PRIMARY,
 				       GDK_SELECTION_TYPE_STRING,
@@ -352,6 +414,7 @@ window_new_button_drag_data_received_cb (GtkWidget *widget,
 					 GaleonWindow *window)
 {
 	gboolean tabbed_mode;
+	gchar **tmp;
 
 	/* check args */
 	g_return_if_fail (selection_data != NULL);
@@ -366,6 +429,22 @@ window_new_button_drag_data_received_cb (GtkWidget *widget,
 	switch (info)
 	{
 	case DND_TARGET_NETSCAPE_URL:
+		tmp = g_strsplit(selection_data->data, "\n", 2);
+		if (tmp)
+		{
+			embed_create_from_url (window->active_embed,
+					       tmp[0],
+					       TRUE, !tabbed_mode);
+		}
+		else
+		{
+			embed_create_from_url (window->active_embed,
+					       selection_data->data,
+					       TRUE, !tabbed_mode);
+		}
+		window_update_nav_controls (window);
+		g_strfreev (tmp);
+		break;
 	case DND_TARGET_GALEON_URL:
 	case DND_TARGET_STRING:
 		embed_create_from_url (window->active_embed,
@@ -391,6 +470,8 @@ window_back_forward_button_release_cb (GtkButton *button,
 	char *url = NULL;
 	GaleonEmbed *embed;
 	
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
 	if (GTK_WIDGET (button)->window != event->window)
 		return FALSE;
 	return_val_if_not_window (window, FALSE);
@@ -402,12 +483,14 @@ window_back_forward_button_release_cb (GtkButton *button,
 	embed = window->active_embed;
 	if (event->button == 1 && !(event->state & GDK_SHIFT_MASK))
 	{
-		if (GTK_WIDGET (button) == window->forward_button)
+		if (GTK_WIDGET (button) == window->forward_button ||
+		    GTK_WIDGET (button) == window->forward_menuitem)
 		{
 			gtk_moz_embed_go_forward 
 				(GTK_MOZ_EMBED (embed->mozEmbed));
 		}
-		else if (GTK_WIDGET (button) == window->back_button)
+		else if (GTK_WIDGET (button) == window->back_button ||
+			 GTK_WIDGET (button) == window->back_menuitem)
 		{
 			gtk_moz_embed_go_back 
 				(GTK_MOZ_EMBED (embed->mozEmbed));
@@ -416,14 +499,23 @@ window_back_forward_button_release_cb (GtkButton *button,
 		{
 			g_assert_not_reached ();
 		}
+		if (GTK_WIDGET (button) == window->back_menuitem ||
+	    	    GTK_WIDGET (button) == window->forward_menuitem)
+		{
+			GtkMenuShell *shell =
+				GTK_MENU_SHELL (GTK_WIDGET (button)->parent);
+			gtk_menu_shell_deactivate (shell);
+		}
 		return TRUE;
 	}
 	
-	if (GTK_WIDGET (button) == window->forward_button)
+	if (GTK_WIDGET (button) == window->forward_button ||
+	    GTK_WIDGET (button) == window->forward_menuitem)
 	{
 		url = mozilla_session_history_get_url_relative (embed, 1);
 	}
-	else if (GTK_WIDGET (button) == window->back_button)
+	else if (GTK_WIDGET (button) == window->back_button ||
+		 GTK_WIDGET (button) == window->back_menuitem)
 	{
 		url = mozilla_session_history_get_url_relative (embed, -1);
 	}
@@ -433,6 +525,14 @@ window_back_forward_button_release_cb (GtkButton *button,
 	}
 	embed_activate_link_mouse (window->active_embed, NULL, url, event);
 	if (url != NULL) free (url);
+
+	if (GTK_WIDGET (button) == window->back_menuitem ||
+	    GTK_WIDGET (button) == window->forward_menuitem)
+	{
+		GtkMenuShell *shell =
+			GTK_MENU_SHELL (GTK_WIDGET (button)->parent);
+		gtk_menu_shell_deactivate (shell);
+	}
 	
 	return TRUE;
 }
@@ -458,9 +558,6 @@ window_up_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 
 	/* make menu */
 	menu = embed_create_up_menu (window->active_embed);
-	context_menu_add_seperator (GTK_MENU (menu));
-	context_show_appearance_menu (window, GTK_MENU (menu), 
-				      NULL, TRUE, TRUE);
 	if (menu == NULL)
 	{
 		return TRUE;
@@ -484,6 +581,8 @@ gboolean
 window_up_button_release_event_cb (GtkButton *button, GdkEventButton *event,
 				   GaleonWindow *window)
 {
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
 	if (GTK_WIDGET (button)->window != event->window)
 		return FALSE;
 	return_val_if_not_window (window, FALSE);
@@ -492,17 +591,60 @@ window_up_button_release_event_cb (GtkButton *button, GdkEventButton *event,
 		return FALSE;
 	embed_go_up (window->active_embed, 0, 
 		mouse_state_to_link_state (event->button, event->state));
+	if (GTK_WIDGET (button) == window->up_menuitem)
+	{
+		GtkMenuShell *shell =
+			GTK_MENU_SHELL (GTK_WIDGET (button)->parent);
+		gtk_menu_shell_deactivate (shell);
+	}
 	return TRUE;
 }
 
 /** 
- * window_refresh_button_clicked_cb: refresh button clicked
+ * window_refresh_button_release_event_cb: refresh button released
  */
-void
-window_refresh_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+gboolean
+window_refresh_button_release_event_cb (GtkButton *button, 
+					GdkEventButton *event,
+					GaleonWindow *window)
 {
-	return_if_not_window (window);
-	embed_reload (window->active_embed);
+	GaleonEmbed *embed;
+
+	/* check args */
+	return_val_if_not_window (window, FALSE);
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
+
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, TRUE);
+
+	if (GTK_WIDGET (button)->window != event->window)
+		return FALSE;
+
+	switch (event->button)
+	{
+	case 1:
+		if (event->state & GDK_SHIFT_MASK)
+		{
+			embed_reload (embed, 
+				      GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE);	
+		}
+		else
+		{
+			embed_reload (embed, 
+				      GTK_MOZ_EMBED_FLAG_RELOADNORMAL);	
+		}
+		return FALSE;
+		break;
+	}
+
+	if (GTK_WIDGET (button) == window->refresh_menuitem)
+	{
+		GtkMenuShell *shell =
+			GTK_MENU_SHELL (GTK_WIDGET (button)->parent);
+		gtk_menu_shell_deactivate (shell);
+	}
+	return TRUE;
 }
 
 /** 
@@ -515,6 +657,7 @@ window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	GtkWidget *menu;
 	gint action;
 	GList *l;
+	GaleonEmbed *embed;
 
 	/* reload context menu */
 	static GnomeUIInfo menu_uiinfo[] =
@@ -523,12 +666,22 @@ window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
 		GNOMEUIINFO_ITEM_STOCK (N_("Reload all windows"),
 					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
+		GNOMEUIINFO_SEPARATOR,
+		GNOMEUIINFO_ITEM_STOCK (N_("Bypass cache"),
+					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
+		GNOMEUIINFO_ITEM_STOCK (N_("Bypass proxy"),
+					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
+		GNOMEUIINFO_ITEM_STOCK (N_("Bypass cache and proxy"),
+					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
 		GNOMEUIINFO_END
 	};
-	
+
 	/* check args */
 	return_val_if_not_window (window, FALSE);
 
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, TRUE);
+
 	/* handle button accordingly */	
 	switch (event->button)
 	{
@@ -543,9 +696,12 @@ window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	case 3:
 		/* show context menu */
 		menu = gnome_popup_menu_new (menu_uiinfo);
-		context_menu_add_seperator (GTK_MENU (menu));
-		context_show_appearance_menu (window, GTK_MENU (menu), 
-					      NULL, TRUE, TRUE);
+		if (GTK_WIDGET (button) != window->refresh_menuitem)
+		{
+			context_menu_add_seperator (GTK_MENU (menu));
+			context_show_appearance_menu (window, GTK_MENU (menu), 
+						      NULL, TRUE, TRUE);
+		}
 		action = gnome_popup_menu_do_popup_modal 
 			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
 
@@ -562,6 +718,16 @@ window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 				window_reload_all ((GaleonWindow *)(l->data));
 			}
 			break;
+		case 3: 
+			embed_reload (embed, GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
+			break;
+		case 4: 
+			embed_reload (embed, GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXY);
+			break;
+		case 5: 
+			embed_reload (embed, 
+				      GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE);
+			break;
 		}
 
 		/* destroy the popup menu */
@@ -740,17 +906,18 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 			gtk_entry_set_text (entry, before_completion);
 			g_free (before_completion);
 			before_completion = NULL;
+			return TRUE;
 		}
 		else
 		{
 			gtk_entry_set_text (entry, embed->site_location);
 		}
-		return TRUE;
+		break;
 	default:
 		break;
 	}
 
-	if (event->string[0] > 32 && event->string[0] < 126) 
+	if ((event->string[0] > 32) && (event->string[0] < 126)) 
 	{
 		if (before_completion != NULL) 
 		{
@@ -801,8 +968,11 @@ window_handle_go_button (GaleonWindow *window, LinkState state)
 	gchar *text2;
 	GList *wl;
 	GnomeEntry *ge;
+	GaleonEmbed *embed;
 
 	return_if_not_window (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
 
 	if (window->location_entry == NULL) return;
 	if (window->location_gnomeentry == NULL) return;
@@ -841,6 +1011,15 @@ window_handle_go_button (GaleonWindow *window, LinkState state)
 			g_free (text);
 			text = text2;
 		}
+		else
+		{
+			text2 = guess_protocol (text);
+			if (text2)
+			{
+				g_free (text);
+				text = text2;
+			}
+		}
 	}
 	/* empty string -- load a blank page */
 	else
@@ -849,11 +1028,44 @@ window_handle_go_button (GaleonWindow *window, LinkState state)
 		text = g_strdup ("about:blank");
 	}
 
-	embed_activate_link (window->active_embed, NULL, text, state);
+	embed_activate_link (embed, NULL, text, state);
+
+	embed_grab_focus (embed);
 
 	g_free (text);
 }
 
+/* is the URI does not specify a protocol, try to
+ * guess one ourselves looking in the history before
+ * giving it to mozilla */
+static gchar *
+guess_protocol (gchar *text)
+{
+	
+	/* this is to check that the url does not already 
+	   specify a protocol. Obviously, this check is not 
+	   very good... */
+	gchar *colon = strchr (text, ':');
+	if (!colon || ((colon - text) < 12))
+	{
+		static const gchar *guess_protocols[] = {
+			"", "https://", "http://", "ftp://", NULL };
+		int i;
+		for (i = 0; guess_protocols[i] != NULL; i++)
+		{
+			gchar *guess = g_strconcat 
+				(guess_protocols[i], text, NULL);
+			if (history_is_visited (guess))
+			{
+				return guess;
+			}
+			g_free (guess);
+		}
+	}
+	/* nothing better can be found */
+	return NULL;
+}
+
 /** 
  * window_go_button_release_cb: go button released
  */
@@ -861,6 +1073,8 @@ gboolean
 window_go_button_release_cb (GtkButton *button, GdkEventButton *event,
 			     GaleonWindow *window)
 {
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
 	if (GTK_WIDGET (button)->window != event->window)
 		return FALSE;
 	return_val_if_not_window (window, FALSE);
@@ -871,6 +1085,21 @@ window_go_button_release_cb (GtkButton *button, GdkEventButton *event,
 	return TRUE;
 }
 
+/**
+ * window_clear_location_button_release_cb
+ */
+gboolean
+window_clear_location_button_release_cb (GtkButton *button,
+					 GdkEventButton *event,
+					 GaleonWindow *window)
+{
+	if (event->button == 1 && !GTK_BUTTON (button)->button_down)
+		return FALSE;
+	return_val_if_not_window (window, FALSE);
+	window_clear_url_entry (window);
+	return TRUE;
+}
+
 /**
  * Changes the zoom if enough time time has passed 
  */
@@ -978,7 +1207,7 @@ window_drag_pixmap_drag_data_get_cb (GtkWidget *widget,
 	switch (info)
 	{
 	case DND_TARGET_GALEON_BOOKMARK:
-		b = bookmarks_new_bookmark (BM_SITE, TRUE, embed->site_title,
+		b = bookmarks_new_bookmark (BM_SITE, TRUE, embed->site_title_utf8,
 					    embed->site_location,
 					    NULL, NULL, NULL);
 		mem = bookmarks_item_to_string (b);
@@ -1011,16 +1240,34 @@ window_location_entry_drag_data_received_cb (GtkWidget *widget,
 				 guint time, GaleonWindow *window)
 {
 	gchar *url = selection_data->data;
+	gchar **tmp;
 	gint i;
 
+	window_clear_url_entry (window);
+
 	switch (info) {
-	case DND_TARGET_STRING:
 	case DND_TARGET_NETSCAPE_URL:
+		tmp = g_strsplit(selection_data->data, "\n", 2);
+		if (tmp)
+		{
+			gtk_editable_insert_text 
+				(GTK_EDITABLE (window->location_entry),
+				 tmp[0], strlen (tmp[0]), &i);
+			g_strfreev (tmp);
+		}
+		else
+		{
+			gtk_editable_insert_text 
+				(GTK_EDITABLE (window->location_entry),
+				 url, strlen (url), &i);
+		}
+		break;
+	case DND_TARGET_STRING:
 	case DND_TARGET_GALEON_URL:
 	case DND_TARGET_TEXT_URI_LIST:/* FIXME: won't work right for a list */
-		window_clear_url_entry (window);
-		gtk_editable_insert_text (GTK_EDITABLE (window->location_entry),
-					  url, strlen (url), &i);
+		gtk_editable_insert_text 
+			(GTK_EDITABLE (window->location_entry),
+			 url, strlen (url), &i);
 		break;
 	default:
 		g_warning ("Unknown DND type");
@@ -1098,7 +1345,7 @@ window_location_gnomeentry_popwin_cb (GtkWidget *widget, GaleonWindow *window)
 			 GTK_SIGNAL_FUNC (window_entry_changed_cb),
 			 &selection_made);
 
-		if (window->go_button != NULL && selection_made)
+		if (selection_made)
 		{
 			window_handle_go_button (window, 0);
 		}
@@ -1281,7 +1528,7 @@ window_notebook_switch_page_cb (GtkNotebook *notebook,
 		/* save edited site location information */
 		save_site_location (old_embed);
 
-		/* no longer the active window */
+		/* no longer the active embed */
 		old_embed->is_active = FALSE;
 		embed_update_tab_status (old_embed);
 	}
@@ -1294,6 +1541,10 @@ window_notebook_switch_page_cb (GtkNotebook *notebook,
 	window->embed_list = g_list_remove (window->embed_list, embed);
 	window->embed_list = g_list_prepend (window->embed_list, embed);
 
+	/* update security icon */
+	if (!(old_embed && embed->secure == old_embed->secure))
+		window_statusbar_set_security_icon (window, embed->secure);
+
 	/* update tab label colour and closebutton status */
 	embed_update_tab_status (embed);
 
@@ -1305,24 +1556,25 @@ window_notebook_switch_page_cb (GtkNotebook *notebook,
 	window_update_nav_controls (window);
 	window_update_tab_controls (window);
 
-	/* update the status bar */
-	window_update_status_bar (window);
+	/* update the statusbar */
+	window_update_statusbar (window);
 
 	/* update the zoom control */
 	window_update_zoom (window);
 
+	/* update the css menu */
+	embed_init_css_menu (embed);
+
+	/* update the spinner animation state */
+	if (embed->load_started) spinner_start (window);
+	else spinner_stop (window);
+
 	/* set focus to the embed when clicking on a tab */
-	/* it's not grabbing the focus when opening a new tab
-	   from context menu, actually there is not a good way
-	   to do that */
+	/* FIXME I test for embed->wrapper to not grab on
+	   new windows/tabs, it's obviously hacky */
 	if (embed->wrapper && window->visible_embeds > 1)
 	{
-		if (old_embed != NULL)
-		{
-			mozilla_deactivate (old_embed);
-		}
-		
-		mozilla_activate (embed);
+		embed_grab_focus (embed);
 	}
 
 	/* we assume the user requested this focus by default */
@@ -1637,6 +1889,9 @@ gboolean window_key_press_event (GtkWidget *widget,
 {
 	int page;
 
+	if ((event->state & GDK_Shift_L) || (event->state & GDK_Shift_R))
+		return TRUE;
+
 	if ((event->state & GDK_Alt_L) || (event->state & GDK_Alt_R))
 	{
 		page = event->keyval - GDK_0 -1;
@@ -1744,3 +1999,55 @@ window_find_entry_activate_cb (GtkWidget *widget, GaleonWindow *window)
 
 	return TRUE;
 }
+
+/**
+ * window_toolbar_button_enter_event_cb: Called when the mouse enters the
+ * button; show prelight icon.
+ */
+void
+window_toolbar_button_enter_event_cb (GtkWidget *widget, ToolbarItem *item)
+{
+	GtkWidget *pixmap = gtk_object_get_data (GTK_OBJECT (widget), "pixmap");
+	const PixmapData *data;
+	gchar *filename, *tmp;
+	gchar *themedir;
+	
+	if (!GTK_IS_PIXMAP (pixmap)) return;
+	
+	themedir = get_theme_dir ();
+
+	filename = g_strconcat (item->theme_icon, "-prelight.png", NULL);
+	tmp = g_concat_dir_and_file (themedir, filename);
+	if (!g_file_exists (tmp))
+	{
+		g_free (filename);
+		filename = g_strconcat (item->theme_icon, ".png", NULL);
+	}
+	g_free (tmp);
+	g_free (themedir);
+
+	data = get_theme_pixmap (filename, TRUE);
+	g_free (filename);
+
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), data->pixmap, data->mask);
+}
+
+/**
+ * window_toolbar_button_leave_event_cb: Called when the mouse leaves the
+ * button; put original icon back
+ */
+void
+window_toolbar_button_leave_event_cb (GtkWidget *widget, ToolbarItem *item)
+{
+	GtkWidget *pixmap = gtk_object_get_data (GTK_OBJECT (widget), "pixmap");
+	const PixmapData *data;
+	gchar *filename;
+
+	if (!GTK_IS_PIXMAP (pixmap)) return;
+
+	filename = g_strconcat (item->theme_icon, ".png", NULL);
+	data = get_theme_pixmap (filename, FALSE);
+	g_free (filename);
+
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), data->pixmap, data->mask);
+}
